@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: DOMCount.cpp,v 1.27 2002/07/17 18:58:35 tng Exp $
+ * $Id: DOMCount.cpp,v 1.35 2003/01/09 22:43:43 tng Exp $
  */
 
 // ---------------------------------------------------------------------------
@@ -78,8 +78,6 @@
 #include <fstream.h>
 
 
-
-
 // ---------------------------------------------------------------------------
 //  This is a simple program which invokes the DOMParser to build a DOM
 //  tree for the specified input file. It then walks the tree and counts
@@ -98,7 +96,8 @@ static void usage()
             "    -n          Enable namespace processing. Defaults to off.\n"
             "    -s          Enable schema processing. Defaults to off.\n"
             "    -f          Enable full schema constraint checking. Defaults to off.\n"
-		      "    -?          Show this help.\n\n"
+            "    -locale=ll_CC specify the locale, default: en_US.\n"
+		    "    -?          Show this help.\n\n"
             "  * = Default if not provided explicitly.\n"
          << endl;
 }
@@ -123,7 +122,6 @@ static int countChildElements(DOMNode *n)
     return count;
 }
 
-
 // ---------------------------------------------------------------------------
 //
 //   main
@@ -131,24 +129,11 @@ static int countChildElements(DOMNode *n)
 // ---------------------------------------------------------------------------
 int main(int argC, char* argV[])
 {
-    // Initialize the XML4C system
-    try
-    {
-        XMLPlatformUtils::Initialize();
-    }
-
-    catch (const XMLException& toCatch)
-    {
-         cerr << "Error during initialization! :\n"
-              << StrX(toCatch.getMessage()) << endl;
-         return 1;
-    }
 
     // Check command line and extract arguments.
     if (argC < 2)
     {
         usage();
-        XMLPlatformUtils::Terminate();
         return 1;
     }
 
@@ -159,6 +144,9 @@ int main(int argC, char* argV[])
     bool                       schemaFullChecking = false;
     bool                       doList = false;
     bool                       errorOccurred = false;
+    bool                       recognizeNEL = false;
+    char                       localeStr[64];
+    memset(localeStr, 0, sizeof localeStr);
 
     int argInd;
     for (argInd = 1; argInd < argC; argInd++)
@@ -171,7 +159,6 @@ int main(int argC, char* argV[])
         if (!strcmp(argV[argInd], "-?"))
         {
             usage();
-            XMLPlatformUtils::Terminate();
             return 2;
         }
          else if (!strncmp(argV[argInd], "-v=", 3)
@@ -188,7 +175,6 @@ int main(int argC, char* argV[])
             else
             {
                 cerr << "Unknown -v= value: " << parm << endl;
-                XMLPlatformUtils::Terminate();
                 return 2;
             }
         }
@@ -218,8 +204,14 @@ int main(int argC, char* argV[])
             // it will recognize the unicode character 0x85 as new line character
             // instead of regular character as specified in XML 1.0
             // do not turn this on unless really necessary
-            XMLPlatformUtils::recognizeNEL(true);
+
+             recognizeNEL = true;
         }
+         else if (!strncmp(argV[argInd], "-locale=", 8))
+        {
+             // Get out the end of line
+             strcpy(localeStr, &(argV[argInd][8]));
+        }			
          else
         {
             cerr << "Unknown option '" << argV[argInd]
@@ -234,10 +226,34 @@ int main(int argC, char* argV[])
     if (argInd != argC - 1)
     {
         usage();
-        XMLPlatformUtils::Terminate();
         return 1;
     }
 
+    // Initialize the XML4C system
+    try
+    {
+        if (strlen(localeStr))
+        {
+            XMLPlatformUtils::Initialize(localeStr);
+        }
+        else
+        {
+            XMLPlatformUtils::Initialize();
+        }
+
+        if (recognizeNEL)
+        {
+            XMLPlatformUtils::recognizeNEL(recognizeNEL);
+        }
+    }
+
+    catch (const XMLException& toCatch)
+    {
+         cerr << "Error during initialization! :\n"
+              << StrX(toCatch.getMessage()) << endl;
+         return 1;
+    }
+
     // Instantiate the DOM parser.
     static const XMLCh gLS[] = { chLatin_L, chLatin_S, chNull };
     DOMImplementation *impl = DOMImplementationRegistry::getDOMImplementation(gLS);
@@ -280,6 +296,11 @@ int main(int argC, char* argV[])
     if (doList)
         fin.open(argV[argInd]);
 
+    if (fin.fail()) {
+        cerr <<"Cannot open the list file: " << argV[argInd] << endl;
+        return 2;
+    }
+
     while (more)
     {
         char fURI[1000];
@@ -330,9 +351,15 @@ int main(int argC, char* argV[])
         }
         catch (const DOMException& toCatch)
         {
+            const unsigned int maxChars = 2047;
+            XMLCh errText[maxChars + 1];
+
             cerr << "\nDOM Error during parsing: '" << xmlFile << "'\n"
-                 << "DOMException code is:  \n"
-                 << toCatch.code << "\n" << endl;
+                 << "DOMException code is:  " << toCatch.code << endl;
+
+            if (DOMImplementation::loadDOMExceptionMsg(toCatch.code, errText, maxChars))
+                 cerr << "Message is: " << StrX(errText) << endl;
+
             errorOccurred = true;
             continue;
         }
@@ -421,7 +448,7 @@ bool DOMCountErrorHandler::handleError(const DOMError& domError)
          << ", char " << domError.getLocation()->getColumnNumber()
          << "\n  Message: " << StrX(domError.getMessage()) << endl;
 
-    return false;
+    return true;
 }
 
 void DOMCountErrorHandler::resetErrors()
