@@ -56,6 +56,12 @@
 
 /*
  * $Log: EnumVal.cpp,v $
+ * Revision 1.19  2003/02/05 18:53:23  tng
+ * [Bug 11915] Utility for freeing memory.
+ *
+ * Revision 1.18  2002/11/05 21:46:19  tng
+ * Explicit code using namespace in application.
+ *
  * Revision 1.17  2002/04/17 20:18:08  tng
  * [Bug 7493] The word "occured" is misspelled and it is a global error.
  *
@@ -125,6 +131,8 @@
 #include <string.h>
 
 
+XERCES_CPP_NAMESPACE_USE
+
 // ---------------------------------------------------------------------------
 //  Forward references
 // ---------------------------------------------------------------------------
@@ -150,7 +158,7 @@ public :
 
     ~StrX()
     {
-        delete [] fLocalForm;
+        XMLString::release(&fLocalForm);
     }
 
 
