@@ -56,6 +56,24 @@
 
 /*
 * $Log: SAX2Count.cpp,v $
+* Revision 1.24  2002/12/10 13:34:51  tng
+* Samples minor update in usage information.
+*
+* Revision 1.23  2002/11/08 16:18:50  peiyongz
+* no message
+*
+* Revision 1.22  2002/11/07 18:30:42  peiyongz
+* command line option for "locale"
+*
+* Revision 1.21  2002/11/04 14:09:06  tng
+* [Bug 14201] use of ios::nocreate breaks build.
+*
+* Revision 1.20  2002/11/01 22:05:57  tng
+* Samples/Test update: Issue error if the list file failed to open.
+*
+* Revision 1.19  2002/09/27 19:24:57  tng
+* Samples Fix: wrong length in memset
+*
 * Revision 1.18  2002/07/17 18:58:35  tng
 * samples update: for testing special encoding purpose.
 *
@@ -144,7 +162,8 @@ void usage()
             "                NOTE: THIS IS OPPOSITE FROM OTHER SAMPLES.\n"
             "    -s          Disable schema processing. Defaults to on.\n"
             "                NOTE: THIS IS OPPOSITE FROM OTHER SAMPLES.\n"
-		      "    -?          Show this help.\n\n"
+            "    -locale=ll_CC specify the locale, default: en_US.\n"
+            "    -?          Show this help.\n\n"
             "  * = Default if not provided explicitly.\n"
          << endl;
 }
@@ -155,24 +174,11 @@ void usage()
 // ---------------------------------------------------------------------------
 int main(int argC, char* argV[])
 {
-    // Initialize the XML4C2 system
-    try
-    {
-        XMLPlatformUtils::Initialize();
-    }
-
-    catch (const XMLException& toCatch)
-    {
-        cerr << "Error during initialization! Message:\n"
-            << StrX(toCatch.getMessage()) << endl;
-        return 1;
-    }
 
     // Check command line and extract arguments.
     if (argC < 2)
     {
         usage();
-        XMLPlatformUtils::Terminate();
         return 1;
     }
 
@@ -184,6 +190,9 @@ int main(int argC, char* argV[])
     bool                         doList = false;
     bool                         errorOccurred = false;
     bool                         namespacePrefixes = false;
+    bool                         recognizeNEL = false;
+    char                         localeStr[64];
+    memset(localeStr, 0, sizeof localeStr);
 
     int argInd;
     for (argInd = 1; argInd < argC; argInd++)
@@ -196,7 +205,6 @@ int main(int argC, char* argV[])
         if (!strcmp(argV[argInd], "-?"))
         {
             usage();
-            XMLPlatformUtils::Terminate();
             return 2;
         }
          else if (!strncmp(argV[argInd], "-v=", 3)
@@ -213,7 +221,6 @@ int main(int argC, char* argV[])
             else
             {
                 cerr << "Unknown -v= value: " << parm << endl;
-                XMLPlatformUtils::Terminate();
                 return 2;
             }
         }
@@ -248,8 +255,13 @@ int main(int argC, char* argV[])
             // it will recognize the unicode character 0x85 as new line character
             // instead of regular character as specified in XML 1.0
             // do not turn this on unless really necessary
-            XMLPlatformUtils::recognizeNEL(true);
+             recognizeNEL = true;
         }
+         else if (!strncmp(argV[argInd], "-locale=", 8))
+        {
+             // Get out the end of line
+             strcpy(localeStr, &(argV[argInd][8]));
+        }			
         else
         {
             cerr << "Unknown option '" << argV[argInd]
@@ -264,7 +276,31 @@ int main(int argC, char* argV[])
     if (argInd != argC - 1)
     {
         usage();
-        XMLPlatformUtils::Terminate();
+        return 1;
+    }
+
+    // Initialize the XML4C2 system
+    try
+    {
+        if (strlen(localeStr))
+        {
+            XMLPlatformUtils::Initialize(localeStr);
+        }
+        else
+        {
+            XMLPlatformUtils::Initialize();
+        }
+
+        if (recognizeNEL)
+        {
+            XMLPlatformUtils::recognizeNEL(recognizeNEL);
+        }
+    }
+
+    catch (const XMLException& toCatch)
+    {
+        cerr << "Error during initialization! Message:\n"
+            << StrX(toCatch.getMessage()) << endl;
         return 1;
     }
 
@@ -314,6 +350,11 @@ int main(int argC, char* argV[])
     if (doList)
         fin.open(argV[argInd]);
 
+    if (fin.fail()) {
+        cerr <<"Cannot open the list file: " << argV[argInd] << endl;
+        return 2;
+    }
+
     while (more)
     {
         char fURI[1000];
