@@ -58,13 +58,16 @@
  */
 
 /*
- * $Id: DOMBuilder.hpp,v 1.8 2002/08/22 15:04:57 tng Exp $
+ * $Id: DOMBuilder.hpp,v 1.10 2002/12/23 15:23:17 knoaman Exp $
  *
  */
 
 
 #include <xercesc/util/XercesDefs.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
+
 class DOMErrorHandler;
 class DOMEntityResolver;
 class DOMInputSource;
@@ -697,9 +700,18 @@ public:
       */
     virtual void resetCachedGrammarPool() = 0;
 
+    /**
+      * Returns the current src offset within the input source.
+      *
+      * @return offset within the input source
+      */
+    virtual unsigned int getSrcOffset() const = 0;
+
     //@}
 
 };
 
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
