@@ -58,11 +58,13 @@
  */
 
 /*
- * $Id: DOMNode.hpp,v 1.11 2002/08/22 15:04:57 tng Exp $
+ * $Id: DOMNode.hpp,v 1.13 2002/11/04 15:09:24 tng Exp $
  */
 
 #include <xercesc/util/XercesDefs.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 
 class DOMDocument;
 class DOMNamedNodeMap;
@@ -665,7 +667,7 @@ public:
      *   <code>false</code> otherwise.
      * @since DOM Level 3
      */
-    virtual bool              isSameNode(const DOMNode* other) = 0;
+    virtual bool              isSameNode(const DOMNode* other) const = 0;
 
     /**
      * Tests whether two nodes are equal.
@@ -709,7 +711,7 @@ public:
      *   <code>true</code> otherwise <code>false</code>.
      * @since DOM Level 3
      */
-    virtual bool              isEqualNode(const DOMNode* arg) = 0;
+    virtual bool              isEqualNode(const DOMNode* arg) const = 0;
 
 
     /**
@@ -791,7 +793,7 @@ public:
      *   node.
      * @since DOM Level 3
      */
-    virtual short                  compareTreePosition(DOMNode* other) = 0;
+    virtual short                  compareTreePosition(const DOMNode* other) const = 0;
 
     /**
      * This attribute returns the text content of this node and its
@@ -983,5 +985,7 @@ public:
 };
 
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
 
