@@ -58,8 +58,14 @@
  */
 
 /*
- * $Id: DOMWriter.hpp,v 1.7 2002/08/22 15:04:57 tng Exp $
+ * $Id: DOMWriter.hpp,v 1.9 2002/11/27 18:46:39 tng Exp $
  * $Log: DOMWriter.hpp,v $
+ * Revision 1.9  2002/11/27 18:46:39  tng
+ * Documentation: clarify that DOMWriter::writeToString always returns string in UTF-16
+ *
+ * Revision 1.8  2002/11/04 15:09:25  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.7  2002/08/22 15:04:57  tng
  * Remove unused parameter variables in inline functions.
  *
@@ -308,6 +314,8 @@
 #include <xercesc/dom/DOMErrorHandler.hpp>
 #include <xercesc/framework/XMLFormatter.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 class CDOM_EXPORT DOMWriter {
 protected :
     // -----------------------------------------------------------------------
@@ -571,7 +579,8 @@ public:
      * @param nodeToWrite  The node to be written.
      * @return  Returns the serialized data, or <code>null</code> in case a
      *   failure occured and the failure wasn't canceled by the error
-     *   handler.
+     *   handler.   The returned string is always in UTF-16.
+     *   The encoding information available in DOMWriter is ignored in writeToString().
      * @since DOM Level 3
      */
     virtual XMLCh*     writeToString(const DOMNode &nodeToWrite) = 0;
@@ -595,4 +604,6 @@ public:
 
 };
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
