@@ -55,14 +55,13 @@
  */
 
 /*
- * $Id: DOMParser.hpp,v 1.10 2002/08/14 15:20:37 knoaman Exp $
+ * $Id: DOMParser.hpp,v 1.16 2003/01/09 19:07:44 tng Exp $
  *
  */
 
 #if !defined(DOMPARSER_HPP)
 #define DOMPARSER_HPP
 
-
 #include <xercesc/framework/XMLDocumentHandler.hpp>
 #include <xercesc/framework/XMLErrorReporter.hpp>
 #include <xercesc/framework/XMLEntityHandler.hpp>
@@ -73,13 +72,16 @@
 #include "DOM_Document.hpp"
 #include "DOM_DocumentType.hpp"
 
+XERCES_CPP_NAMESPACE_BEGIN
+
+
 class EntityResolver;
 class ErrorHandler;
 class XMLPScanToken;
 class XMLScanner;
 class XMLValidator;
 class Grammar;
-
+class GrammarResolver;
 
 /**
   * This class implements the Document Object Model (DOM) interface.
@@ -398,6 +400,31 @@ public :
       */
     bool isUsingCachedGrammarInParse() const;
 
+    /**
+      * Get the 'calculate src offset flag'
+      *
+      * This method returns the state of the parser's src offset calculation
+      * when parsing an XML document.
+      *
+      * @return true, if the parser is currently configured to
+      *         calculate src offsets, false otherwise.
+      *
+      * @see #setCalculateSrcOfs
+      */
+    bool getCalculateSrcOfs() const;
+
+    /**
+      * Get the 'force standard uri flag'
+      *
+      * This method returns the state if the parser forces standard uri
+      *
+      * @return true, if the parser is currently configured to
+      *         force standard uri, i.e. malformed uri will be rejected.
+      *
+      * @see #setStandardUriConformant
+      */
+    bool getStandardUriConformant() const;
+
     /**
       * Retrieve the grammar that is associated with the specified namespace key
       *
@@ -418,8 +445,15 @@ public :
       *
       * @param uriId id of the string in the URI string pool.
       * @return URI string corresponding to the URI id.
-      */    
-    const XMLCh* getURIText(unsigned int uriId);
+      */
+    const XMLCh* getURIText(unsigned int uriId) const;
+
+    /**
+      * Returns the current src offset within the input source.
+      *
+      * @return offset within the input source
+      */
+    unsigned int getSrcOffset() const;
 
     //@}
 
@@ -723,6 +757,41 @@ public :
       */
     void useCachedGrammarInParse(const bool newState);
 
+    /** Enable/disable src offset calculation
+      *
+      * This method allows users to enable/disable src offset calculation.
+      * Disabling the calculation will improve performance.
+      *
+      * The parser's default state is: false.
+      *
+      * @param newState The value specifying whether we should enable or
+      *                 disable src offset calculation
+      *
+      * @see #getCalculateSrcOfs
+      */
+    void setCalculateSrcOfs(const bool newState);
+
+    /** Force standard uri
+      *
+      * This method allows users to tell the parser to force standard uri conformance.
+      *
+      * The parser's default state is: false.
+      *
+      * @param newState The value specifying whether the parser should reject malformed URI.
+      *
+      * @see #getStandardUriConformant
+      */
+    void setStandardUriConformant(const bool newState);
+
+    /** Set the scanner to use when scanning the XML document
+      *
+      * This method allows users to set the scanner to use
+      * when scanning a given XML document.
+      *
+      * @param scannerName The name of the desired scanner
+      */
+    void useScanner(const XMLCh* const scannerName);
+
     //@}
 
 
@@ -1609,6 +1678,12 @@ protected :
 
 
 private :
+    // -----------------------------------------------------------------------
+    //  Protected setter methods
+    // -----------------------------------------------------------------------
+    void initialize();
+    void cleanUp();
+
     // -----------------------------------------------------------------------
     //  Private data members
     //
@@ -1658,23 +1733,25 @@ private :
     //      This is an extension to xerces implementation
     //
     // -----------------------------------------------------------------------
+    bool                    fToCreateXMLDeclTypeNode;
+    bool                    fCreateEntityReferenceNodes;
+    bool                    fIncludeIgnorableWhitespace;
+    bool                    fParseInProgress;
+    bool                    fWithinElement;
     DOM_Node                fCurrentParent;
     DOM_Node                fCurrentNode;
     DOM_Document            fDocument;
     EntityResolver*         fEntityResolver;
     ErrorHandler*           fErrorHandler;
-    bool                    fCreateEntityReferenceNodes;
-    bool                    fIncludeIgnorableWhitespace;
     ValueStackOf<DOM_Node>* fNodeStack;
-    bool                    fParseInProgress;
     XMLScanner*             fScanner;
-    bool                    fWithinElement;
     DocumentTypeImpl*       fDocumentType;
-    bool                    fToCreateXMLDeclTypeNode;
+    GrammarResolver*        fGrammarResolver;
+    XMLStringPool*          fURIStringPool;
+    XMLValidator*           fValidator;
 };
 
 
-
 // ---------------------------------------------------------------------------
 //  DOMParser: Handlers for the XMLEntityHandler interface
 // ---------------------------------------------------------------------------
@@ -1785,7 +1862,6 @@ inline DOM_Node DOMParser::getCurrentNode()
     return fCurrentNode;
 }
 
-
 // ---------------------------------------------------------------------------
 //  DOMParser: Protected setter methods
 // ---------------------------------------------------------------------------
@@ -1799,4 +1875,6 @@ inline void DOMParser::setDocument(DOM_Document toSet)
     fDocument = toSet;
 }
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
