@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: DOM_Document.hpp,v 1.2 2002/05/21 20:15:01 tng Exp $
+ * $Id: DOM_Document.hpp,v 1.3 2002/11/04 15:04:44 tng Exp $
 */
 
 #ifndef DOM_Document_HEADER_GUARD_
@@ -81,6 +81,9 @@
 #include "DOM_XMLDecl.hpp"
 #include "DOM_Range.hpp"
 
+XERCES_CPP_NAMESPACE_BEGIN
+
+
 class DocumentImpl;
 class NodeIteratorImpl;
 
@@ -610,4 +613,6 @@ protected:
 };
 
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
