@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: DocumentImpl.cpp,v 1.3 2002/07/16 18:22:45 tng Exp $
+ * $Id: DocumentImpl.cpp,v 1.5 2002/12/20 22:10:57 tng Exp $
  */
 
 //
@@ -92,6 +92,9 @@
 #include "RangeImpl.hpp"
 #include "DOM_Document.hpp"
 
+XERCES_CPP_NAMESPACE_BEGIN
+
+
 static DOMString *nam = 0;  // will be lazily initialized to "#document"
 static XMLRegisterCleanup namCleanup;
 
@@ -445,24 +448,7 @@ NodeImpl *DocumentImpl::insertBefore(NodeImpl *newChild, NodeImpl *refChild)
 
 bool DocumentImpl::isXMLName(const DOMString &s)
 {
-    const XMLCh   *nam;
-    int           length;
-    int           i;
-
-    length = s.length();
-    if (length == 0)
-        return false;
-
-    nam = s.rawBuffer();
-    if (!XMLReader::isFirstNameChar(nam[0]))
-        return false;
-
-    for (i=1; i<length; i++)
-    {
-        if (!XMLReader::isNameChar(nam[i]))
-            return false;
-    }
-    return true;
+    return XMLChar1_0::isValidName(s.rawBuffer(),s.length());
 };
 
 
@@ -874,3 +860,6 @@ void DocumentImpl::reinitDocumentImpl() {
 	delete nam;
 	nam = 0;
 }
+
+XERCES_CPP_NAMESPACE_END
+
