@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: RangeImpl.cpp,v 1.2 2002/05/21 20:15:01 tng Exp $
+ * $Id: RangeImpl.cpp,v 1.3 2002/11/04 15:04:44 tng Exp $
  */
 
 #include <xercesc/util/RefVectorOf.hpp>
@@ -71,6 +71,8 @@
 #include "DOM_DOMException.hpp"
 #include "DOM_Text.hpp"
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 
 //---------------------
 // C'tor and D'tor
@@ -1692,4 +1694,6 @@ void RangeImpl::updateSplitInfo(TextImpl* oldNode, TextImpl* startNode, unsigned
 }
 
 
+XERCES_CPP_NAMESPACE_END
+
 
