@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: DOMDocumentImpl.cpp,v 1.21 2002/08/21 20:58:31 tng Exp $
+ * $Id: DOMDocumentImpl.cpp,v 1.31 2003/02/06 14:08:39 tng Exp $
  */
 
 #include <xercesc/util/XMLUniDefs.hpp>
@@ -94,6 +94,9 @@
 #include <xercesc/internal/XMLReader.hpp>
 #include <xercesc/util/HashPtr.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
+
 //
 //   Constructors.   Warning - be very careful with the ordering of initialization
 //                             of the heap.  Ordering depends on the order of declaration
@@ -115,6 +118,7 @@ DOMDocumentImpl::DOMDocumentImpl()
       fNamePool(0),
       fNodeIDMap(0),
       fRanges(0),
+      fNodeIterators(0),
       fChanges(0),
       fNodeListPool(0),
       fActualEncoding(0),
@@ -145,6 +149,7 @@ DOMDocumentImpl::DOMDocumentImpl(const XMLCh *fNamespaceURI,
       fNamePool(0),
       fNodeIDMap(0),
       fRanges(0),
+      fNodeIterators(0),
       fChanges(0),
       fNodeListPool(0),
       fActualEncoding(0),
@@ -204,6 +209,9 @@ DOMDocumentImpl::~DOMDocumentImpl()
     if (fRanges)
         fRanges->cleanup();
 
+    if (fNodeIterators)
+        fNodeIterators->cleanup();
+
     if (fUserDataTable)
         fUserDataTable->cleanup();
 
@@ -261,7 +269,7 @@ DOMDocument * DOMDocumentImpl::getOwnerDocument() const {
 
 DOMAttr *DOMDocumentImpl::createAttribute(const XMLCh *nam)
 {
-    if(!isXMLName(nam))
+    if(!nam || !isXMLName(nam))
         throw DOMException(DOMException::INVALID_CHARACTER_ERR,0);
     return new (this, DOMDocumentImpl::ATTR_OBJECT) DOMAttrImpl(this,nam);
 };
@@ -290,7 +298,7 @@ DOMDocumentFragment *DOMDocumentImpl::createDocumentFragment()
 
 DOMDocumentType *DOMDocumentImpl::createDocumentType(const XMLCh *nam)
 {
-    if (!isXMLName(nam))
+    if (!nam || !isXMLName(nam))
         throw DOMException(
         DOMException::INVALID_CHARACTER_ERR, 0);
 
@@ -304,7 +312,7 @@ DOMDocumentType *
                                      const XMLCh *publicId,
                                      const XMLCh *systemId)
 {
-    if (!isXMLName(qualifiedName))
+    if (!qualifiedName || !isXMLName(qualifiedName))
         throw DOMException(
         DOMException::INVALID_CHARACTER_ERR, 0);
 
@@ -315,7 +323,7 @@ DOMDocumentType *
 
 DOMElement *DOMDocumentImpl::createElement(const XMLCh *tagName)
 {
-    if(!isXMLName(tagName))
+    if(!tagName || !isXMLName(tagName))
         throw DOMException(DOMException::INVALID_CHARACTER_ERR,0);
 
     return new (this, DOMDocumentImpl::ELEMENT_OBJECT) DOMElementImpl(this,tagName);
@@ -332,7 +340,7 @@ DOMElement *DOMDocumentImpl::createElementNoCheck(const XMLCh *tagName)
 
 DOMEntity *DOMDocumentImpl::createEntity(const XMLCh *nam)
 {
-    if (!isXMLName(nam))
+    if (!nam || !isXMLName(nam))
         throw DOMException(
         DOMException::INVALID_CHARACTER_ERR, 0);
 
@@ -343,7 +351,7 @@ DOMEntity *DOMDocumentImpl::createEntity(const XMLCh *nam)
 
 DOMEntityReference *DOMDocumentImpl::createEntityReference(const XMLCh *nam)
 {
-    if (!isXMLName(nam))
+    if (!nam || !isXMLName(nam))
         throw DOMException(
         DOMException::INVALID_CHARACTER_ERR, 0);
 
@@ -354,7 +362,7 @@ DOMEntityReference *DOMDocumentImpl::createEntityReference(const XMLCh *nam)
 
 DOMNotation *DOMDocumentImpl::createNotation(const XMLCh *nam)
 {
-    if (!isXMLName(nam))
+    if (!nam || !isXMLName(nam))
         throw DOMException(
         DOMException::INVALID_CHARACTER_ERR, 0);
 
@@ -366,7 +374,7 @@ DOMNotation *DOMDocumentImpl::createNotation(const XMLCh *nam)
 DOMProcessingInstruction *DOMDocumentImpl::createProcessingInstruction(
                                           const XMLCh *target, const XMLCh *data)
 {
-    if(!isXMLName(target))
+    if(!target || !isXMLName(target))
         throw DOMException(DOMException::INVALID_CHARACTER_ERR,0);
     return new (this, DOMDocumentImpl::PROCESSING_INSTRUCTION_OBJECT) DOMProcessingInstructionImpl(this,target,data);
 };
@@ -383,12 +391,49 @@ DOMText *DOMDocumentImpl::createTextNode(const XMLCh *data)
 DOMNodeIterator* DOMDocumentImpl::createNodeIterator (
           DOMNode *root, unsigned long whatToShow, DOMNodeFilter* filter, bool entityReferenceExpansion)
 {
-    return new (this) DOMNodeIteratorImpl(root, whatToShow, filter, entityReferenceExpansion);
+    if (!root) {
+        throw DOMException(DOMException::NOT_SUPPORTED_ERR, 0);
+        return 0;
+    }
+
+    DOMNodeIteratorImpl* nodeIterator = new (this) DOMNodeIteratorImpl(this, root, whatToShow, filter, entityReferenceExpansion);
+
+    if (fNodeIterators == 0L) {
+        fNodeIterators = new (this) NodeIterators(1, false);
+    }
+    fNodeIterators->addElement(nodeIterator);
+
+    return nodeIterator;
 }
 
 
+NodeIterators* DOMDocumentImpl::getNodeIterators() const
+{
+    return fNodeIterators;
+}
+
+void DOMDocumentImpl::removeNodeIterator(DOMNodeIteratorImpl* nodeIterator)
+{
+    if (fNodeIterators != 0) {
+        XMLSize_t sz = fNodeIterators->size();
+        if (sz !=0) {
+            for (XMLSize_t i =0; i<sz; i++) {
+                if (fNodeIterators->elementAt(i) == nodeIterator) {
+                    fNodeIterators->removeElementAt(i);
+                    break;
+                }
+            }
+        }
+    }
+}
+
 DOMTreeWalker* DOMDocumentImpl::createTreeWalker (DOMNode *root, unsigned long whatToShow, DOMNodeFilter* filter, bool entityReferenceExpansion)
 {
+    if (!root) {
+        throw DOMException(DOMException::NOT_SUPPORTED_ERR, 0);
+        return 0;
+    }
+
     return new (this) DOMTreeWalkerImpl(root, whatToShow, filter, entityReferenceExpansion);
 }
 
@@ -431,6 +476,10 @@ DOMNode *DOMDocumentImpl::insertBefore(DOMNode *newChild, DOMNode *refChild)
         )
         throw DOMException(DOMException::HIERARCHY_REQUEST_ERR,0);
 
+    // if the newChild is a documenttype node created from domimplementation, set the ownerDoc first
+    if ((newChild->getNodeType() == DOMNode::DOCUMENT_TYPE_NODE) && !newChild->getOwnerDocument())
+        ((DOMDocumentTypeImpl*)newChild)->setOwnerDocument(this);
+
     fParent.insertBefore(newChild,refChild);
 
     // If insert succeeded, cache the kid appropriately
@@ -443,20 +492,38 @@ DOMNode *DOMDocumentImpl::insertBefore(DOMNode *newChild, DOMNode *refChild)
 };
 
 
+DOMNode* DOMDocumentImpl::replaceChild(DOMNode *newChild, DOMNode *oldChild) {
+    DOMDocumentType* tempDocType = fDocType;
+    DOMElement* tempDocElement = fDocElement;
+
+    if(oldChild->getNodeType() == DOMNode::DOCUMENT_TYPE_NODE)
+        fDocType=0;
+    else if(oldChild->getNodeType() == DOMNode::ELEMENT_NODE)
+        fDocElement=0;
+
+    try {
+        insertBefore(newChild, oldChild);
+        // changed() already done.
+
+        if((oldChild->getNodeType() == DOMNode::DOCUMENT_TYPE_NODE)
+        || (oldChild->getNodeType() == DOMNode::ELEMENT_NODE))
+            return fParent.removeChild(oldChild);
+        else
+            return removeChild(oldChild);
+    }
+    catch(...) {
+        fDocType = tempDocType;
+        fDocElement = tempDocElement;
+        throw;
+    }
+}
 
 bool DOMDocumentImpl::isXMLName(const XMLCh *s)
 {
-    // revist.  This function probably already exists in the scanner.
-    if (!XMLReader::isFirstNameChar(s[0]))
-        return false;
-
-    const XMLCh *p;
-    for (p=s+1; *p!=0; p++)
-    {
-        if (!XMLReader::isNameChar(*p))
-            return false;
-    }
-    return true;
+    if (XMLString::equals(fVersion, XMLUni::fgVersion1_1))
+        return XMLChar1_1::isValidName(s, XMLString::stringLen(s));
+    else
+        return XMLChar1_0::isValidName(s, XMLString::stringLen(s));
 };
 
 
@@ -493,7 +560,7 @@ DOMNode *DOMDocumentImpl::importNode(DOMNode *source, bool deep)
 DOMElement *DOMDocumentImpl::createElementNS(const XMLCh *fNamespaceURI,
     const XMLCh *qualifiedName)
 {
-    if(!isXMLName(qualifiedName))
+    if(!qualifiedName || !isXMLName(qualifiedName))
         throw DOMException(DOMException::INVALID_CHARACTER_ERR,0);
     //XMLCh * pooledTagName = this->fNamePool->getPooledString(qualifiedName);
     return new (this, DOMDocumentImpl::ELEMENT_NS_OBJECT) DOMElementNSImpl(this, fNamespaceURI, qualifiedName);
@@ -504,7 +571,7 @@ DOMElement *DOMDocumentImpl::createElementNS(const XMLCh *fNamespaceURI,
                                               const XMLSSize_t lineNo,
                                               const XMLSSize_t columnNo)
 {
-    if(!isXMLName(qualifiedName))
+    if(!qualifiedName || !isXMLName(qualifiedName))
         throw DOMException(DOMException::INVALID_CHARACTER_ERR,0);
 
     return new (this) XSDElementNSImpl(this, fNamespaceURI, qualifiedName, lineNo, columnNo);
@@ -514,7 +581,7 @@ DOMElement *DOMDocumentImpl::createElementNS(const XMLCh *fNamespaceURI,
 DOMAttr *DOMDocumentImpl::createAttributeNS(const XMLCh *fNamespaceURI,
     const XMLCh *qualifiedName)
 {
-    if(!isXMLName(qualifiedName))
+    if(!qualifiedName || !isXMLName(qualifiedName))
         throw DOMException(DOMException::INVALID_CHARACTER_ERR,0);
     return new (this, DOMDocumentImpl::ATTR_NS_OBJECT) DOMAttrNSImpl(this, fNamespaceURI, qualifiedName);
 }
@@ -560,6 +627,11 @@ int DOMDocumentImpl::indexofQualifiedName(const XMLCh * qName)
 }
 
 
+const XMLCh*     DOMDocumentImpl::getBaseURI() const
+{
+	  return fDocumentURI;
+};
+
 
 DOMRange* DOMDocumentImpl::createRange()
 {
@@ -667,19 +739,16 @@ int             DOMDocumentImpl::changes() const{
            DOMNode*         DOMDocumentImpl::getPreviousSibling() const              {return fNode.getPreviousSibling (); };
            bool             DOMDocumentImpl::hasChildNodes() const                   {return fParent.hasChildNodes (); };
            void             DOMDocumentImpl::normalize()                             {fParent.normalize (); };
-           DOMNode*         DOMDocumentImpl::replaceChild(DOMNode *newChild, DOMNode *oldChild)
-                                                                                     {return fParent.replaceChild (newChild, oldChild); };
            bool             DOMDocumentImpl::isSupported(const XMLCh *feature, const XMLCh *version) const
                                                                                      {return fNode.isSupported (feature, version); };
            void             DOMDocumentImpl::setPrefix(const XMLCh  *prefix)         {fNode.setPrefix(prefix); };
            bool             DOMDocumentImpl::hasAttributes() const                   {return fNode.hasAttributes(); };
-           bool             DOMDocumentImpl::isSameNode(const DOMNode* other)        {return fNode.isSameNode(other);};
-           bool             DOMDocumentImpl::isEqualNode(const DOMNode* arg)         {return fParent.isEqualNode(arg);};
+           bool             DOMDocumentImpl::isSameNode(const DOMNode* other) const  {return fNode.isSameNode(other);};
+           bool             DOMDocumentImpl::isEqualNode(const DOMNode* arg) const   {return fParent.isEqualNode(arg);};
            void*            DOMDocumentImpl::setUserData(const XMLCh* key, void* data, DOMUserDataHandler* handler)
                                                                                      {return fNode.setUserData(key, data, handler); };
            void*            DOMDocumentImpl::getUserData(const XMLCh* key) const     {return fNode.getUserData(key); };
-           const XMLCh*     DOMDocumentImpl::getBaseURI() const                      {return fNode.getBaseURI(); };
-           short            DOMDocumentImpl::compareTreePosition(DOMNode* other)     {return fNode.compareTreePosition(other); };
+           short            DOMDocumentImpl::compareTreePosition(const DOMNode* other) const {return fNode.compareTreePosition(other); };
            const XMLCh*     DOMDocumentImpl::getTextContent() const                  {return fNode.getTextContent(); };
            void             DOMDocumentImpl::setTextContent(const XMLCh* textContent){fNode.setTextContent(textContent); };
            const XMLCh*     DOMDocumentImpl::lookupNamespacePrefix(const XMLCh* namespaceURI, bool useDefault) const  {return fNode.lookupNamespacePrefix(namespaceURI, useDefault); };
@@ -713,7 +782,8 @@ XMLCh * DOMDocumentImpl::cloneString(const XMLCh *src)
 
 const XMLCh *  DOMDocumentImpl::getPooledString(const XMLCh *src)
 {
-    return this->fNamePool->getPooledString(src);
+    if (!src) return 0;
+    else return this->fNamePool->getPooledString(src);
 }
 
 static const int kHeapAllocSize = 0x10000;    // The chunk size to allocate from the
@@ -871,19 +941,27 @@ const XMLCh* DOMDocumentImpl::getVersion() const {
 }
 
 void DOMDocumentImpl::setVersion(const XMLCh* version){
-    if (XMLString::stringLen(version) &&
-        XMLString::compareString(version, XMLUni::fgSupportedVersion))
+    if ((version && *version) &&
+        !XMLString::equals(version, XMLUni::fgVersion1_0) &&
+        !XMLString::equals(version, XMLUni::fgVersion1_1))
         throw DOMException(DOMException::NOT_SUPPORTED_ERR, 0);
 
     fVersion = cloneString(version);
 }
 
-const XMLCh* DOMDocumentImpl::getDocumentURI() const {
+const XMLCh* DOMDocumentImpl::getDocumentURI() const
+{
     return fDocumentURI;
 }
 
 void DOMDocumentImpl::setDocumentURI(const XMLCh* documentURI){
-    fDocumentURI = cloneString(documentURI);
+    if (documentURI && *documentURI) {
+        XMLCh* temp = (XMLCh*) this->allocate((XMLString::stringLen(documentURI) + 9)*sizeof(XMLCh));
+        XMLString::fixURI(documentURI, temp);
+        fDocumentURI = temp;
+    }
+    else
+        fDocumentURI = 0;
 }
 
 bool DOMDocumentImpl::getStrictErrorChecking() const {
@@ -1283,3 +1361,5 @@ void * DOMDocumentImpl::allocate(size_t amount, NodeObjectType type)
 
 }
 
+XERCES_CPP_NAMESPACE_END
+
