@@ -55,44 +55,48 @@
  */
 
 /*
- * $Id: DOMDocumentTypeImpl.cpp,v 1.13 2002/08/16 19:20:28 tng Exp $
+ * $Id: DOMDocumentTypeImpl.cpp,v 1.20 2002/12/20 22:10:58 tng Exp $
  */
 
 #include "DOMDocumentTypeImpl.hpp"
 #include <xercesc/dom/DOMNode.hpp>
 #include <xercesc/dom/DOMException.hpp>
+#include <xercesc/util/XMLUniDefs.hpp>
+#include <xercesc/util/XMLChar.hpp>
 
 #include "DOMNamedNodeMapImpl.hpp"
 #include "DOMDocumentImpl.hpp"
 #include "DOMCasts.hpp"
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 
 DOMDocumentTypeImpl::DOMDocumentTypeImpl(DOMDocument *ownerDoc,
                                    const XMLCh *dtName,
                                    bool heap)
     : fNode(ownerDoc),
     fParent(ownerDoc),
-    publicId(0),
-    systemId(0),
-    name(0),
-    intSubsetReading(false),
-    entities(0),
-    notations(0),
-    elements(0),
-    internalSubset(0),
+    fPublicId(0),
+    fSystemId(0),
+    fName(0),
+    fIntSubsetReading(false),
+    fEntities(0),
+    fNotations(0),
+    fElements(0),
+    fInternalSubset(0),
     fIsCreatedFromHeap(heap)
 {
     if (ownerDoc) {
-        name = ((DOMDocumentImpl *)ownerDoc)->getPooledString(dtName);
-        entities = new (ownerDoc) DOMNamedNodeMapImpl(this);
-        notations= new (ownerDoc) DOMNamedNodeMapImpl(this);
-        elements = new (ownerDoc) DOMNamedNodeMapImpl(this);
+        fName = ((DOMDocumentImpl *)ownerDoc)->getPooledString(dtName);
+        fEntities = new (ownerDoc) DOMNamedNodeMapImpl(this);
+        fNotations= new (ownerDoc) DOMNamedNodeMapImpl(this);
+        fElements = new (ownerDoc) DOMNamedNodeMapImpl(this);
     }
     else {
-        name = XMLString::replicate(dtName);
-        entities = new DOMNamedNodeMapImpl(this);
-        notations= new DOMNamedNodeMapImpl(this);
-        elements = new DOMNamedNodeMapImpl(this);
+        fName = XMLString::replicate(dtName);
+        fEntities = new DOMNamedNodeMapImpl(this);
+        fNotations= new DOMNamedNodeMapImpl(this);
+        fElements = new DOMNamedNodeMapImpl(this);
     }
 };
 
@@ -105,37 +109,64 @@ DOMDocumentTypeImpl::DOMDocumentTypeImpl(DOMDocument *ownerDoc,
                                    bool heap)
     : fNode(ownerDoc),
     fParent(ownerDoc),
-    publicId(0),
-    systemId(0),
-    name(0),
-    intSubsetReading(false),
-    entities(0),
-    notations(0),
-    elements(0),
-    internalSubset(0),
+    fPublicId(0),
+    fSystemId(0),
+    fName(0),
+    fIntSubsetReading(false),
+    fEntities(0),
+    fNotations(0),
+    fElements(0),
+    fInternalSubset(0),
     fIsCreatedFromHeap(heap)
 {
-    if (DOMDocumentImpl::indexofQualifiedName(qualifiedName) < 0)
+    int index = DOMDocumentImpl::indexofQualifiedName(qualifiedName);
+    if (index < 0)
         throw DOMException(DOMException::NAMESPACE_ERR, 0);
+    else if (index > 0)
+    {
+        // we have to make sure the qualifiedName has correct prefix and localName
+        // although we don't really to store them separately
+        XMLCh* newName;
+        XMLCh temp[4000];
+        if (index >= 3999)
+            newName = new XMLCh[XMLString::stringLen(qualifiedName)+1];
+        else
+            newName = temp;
+
+        XMLString::copyNString(newName, qualifiedName, index);
+        newName[index] = chNull;
+
+        // Before we carry on, we should check if the prefix or localName are valid XMLName
+        if (ownerDoc) {
+            if (!((DOMDocumentImpl*)ownerDoc)->isXMLName(newName) || !((DOMDocumentImpl*)ownerDoc)->isXMLName(qualifiedName+index+1))
+                throw DOMException(DOMException::NAMESPACE_ERR, 0);
+        }
+        else {
+            // document is not there yet, so assume XML 1.0
+            if (!XMLChar1_0::isValidName(newName, index) || !XMLChar1_0::isValidName(qualifiedName+index+1, XMLString::stringLen(qualifiedName)-index-1))
+                throw DOMException(DOMException::NAMESPACE_ERR, 0);
+        }
+
+        if (index >= 3999)
+            delete[] newName;
+    }
 
     if (ownerDoc) {
         DOMDocumentImpl *docImpl = (DOMDocumentImpl *)ownerDoc;
-        publicId = docImpl->cloneString(pubId);
-        systemId = docImpl->cloneString(sysId);
-        internalSubset = docImpl->cloneString(internalSubset);
-        name = ((DOMDocumentImpl *)ownerDoc)->getPooledString(qualifiedName);
-        entities = new (ownerDoc) DOMNamedNodeMapImpl(this);
-        notations= new (ownerDoc) DOMNamedNodeMapImpl(this);
-        elements = new (ownerDoc) DOMNamedNodeMapImpl(this);
+        fPublicId = docImpl->cloneString(pubId);
+        fSystemId = docImpl->cloneString(sysId);
+        fName = ((DOMDocumentImpl *)ownerDoc)->getPooledString(qualifiedName);
+        fEntities = new (ownerDoc) DOMNamedNodeMapImpl(this);
+        fNotations= new (ownerDoc) DOMNamedNodeMapImpl(this);
+        fElements = new (ownerDoc) DOMNamedNodeMapImpl(this);
     }
     else {
-        publicId = XMLString::replicate(pubId);
-        systemId = XMLString::replicate(sysId);
-        internalSubset = XMLString::replicate(internalSubset);
-        name = XMLString::replicate(qualifiedName);
-        entities = new DOMNamedNodeMapImpl(this);
-        notations= new DOMNamedNodeMapImpl(this);
-        elements = new DOMNamedNodeMapImpl(this);
+        fPublicId = XMLString::replicate(pubId);
+        fSystemId = XMLString::replicate(sysId);
+        fName = XMLString::replicate(qualifiedName);
+        fEntities = new DOMNamedNodeMapImpl(this);
+        fNotations= new DOMNamedNodeMapImpl(this);
+        fElements = new DOMNamedNodeMapImpl(this);
     }
 };
 
@@ -144,35 +175,35 @@ DOMDocumentTypeImpl::DOMDocumentTypeImpl(const DOMDocumentTypeImpl &other, bool
     : fNode(other.fNode),
     fParent(other.fParent),
     fChild(other.fChild),
-    publicId(0),
-    systemId(0),
-    name(0),
-    intSubsetReading(other.intSubsetReading),
-    entities(0),
-    notations(0),
-    elements(0),
-    internalSubset(0),
+    fPublicId(0),
+    fSystemId(0),
+    fName(0),
+    fIntSubsetReading(other.fIntSubsetReading),
+    fEntities(0),
+    fNotations(0),
+    fElements(0),
+    fInternalSubset(0),
     fIsCreatedFromHeap(heap)
 {
     if ((DOMDocumentImpl *)this->fNode.getOwnerDocument()) {
-        name = other.name;
+        fName = other.fName;
         if (deep)
             fParent.cloneChildren(&other);
         //DOM Level 2
-        publicId        = other.publicId;
-        systemId        = other.systemId;
-        internalSubset  = other.internalSubset;
+        fPublicId        = other.fPublicId;
+        fSystemId        = other.fSystemId;
+        fInternalSubset  = other.fInternalSubset;
     }
     else {
-        name = XMLString::replicate(other.name);
-        publicId = XMLString::replicate(other.publicId);
-        systemId = XMLString::replicate(other.systemId);
-        internalSubset = XMLString::replicate(other.internalSubset);
+        fName = XMLString::replicate(other.fName);
+        fPublicId = XMLString::replicate(other.fPublicId);
+        fSystemId = XMLString::replicate(other.fSystemId);
+        fInternalSubset = XMLString::replicate(other.fInternalSubset);
     }
 
-    entities = ((DOMNamedNodeMapImpl *)other.entities)->cloneMap(this);
-    notations= ((DOMNamedNodeMapImpl *)other.notations)->cloneMap(this);
-    elements = ((DOMNamedNodeMapImpl *)other.elements)->cloneMap(this);
+    fEntities = ((DOMNamedNodeMapImpl *)other.fEntities)->cloneMap(this);
+    fNotations= ((DOMNamedNodeMapImpl *)other.fNotations)->cloneMap(this);
+    fElements = ((DOMNamedNodeMapImpl *)other.fElements)->cloneMap(this);
 
 }
 
@@ -180,21 +211,21 @@ DOMDocumentTypeImpl::DOMDocumentTypeImpl(const DOMDocumentTypeImpl &other, bool
 DOMDocumentTypeImpl::~DOMDocumentTypeImpl()
 {
     if (!(castToNodeImpl(this)->getOwnerDocument())) {
-        XMLCh* temp = (XMLCh*) name;  // cast off const
+        XMLCh* temp = (XMLCh*) fName;  // cast off const
         delete [] temp;
 
-        temp = (XMLCh*) publicId;  // cast off const
+        temp = (XMLCh*) fPublicId;  // cast off const
         delete [] temp;
 
-        temp = (XMLCh*) systemId;
+        temp = (XMLCh*) fSystemId;
         delete [] temp;
 
-        temp = (XMLCh*) internalSubset;
+        temp = (XMLCh*) fInternalSubset;
         delete [] temp;
 
-        delete entities;
-        delete notations;
-        delete elements;
+        delete fEntities;
+        delete fNotations;
+        delete fElements;
     }
 }
 
@@ -224,36 +255,36 @@ void DOMDocumentTypeImpl::setOwnerDocument(DOMDocument *doc) {
         if (doc) {
             DOMDocumentImpl *docImpl = (DOMDocumentImpl *)doc;
 
-            XMLCh* temp = (XMLCh*) publicId; // cast off const
-            publicId = docImpl->cloneString(publicId);
+            XMLCh* temp = (XMLCh*) fPublicId; // cast off const
+            fPublicId = docImpl->cloneString(fPublicId);
             delete [] temp;
 
-            temp = (XMLCh*) systemId; // cast off const
-            systemId = docImpl->cloneString(systemId);
+            temp = (XMLCh*) fSystemId; // cast off const
+            fSystemId = docImpl->cloneString(fSystemId);
             delete [] temp;
 
-            temp = (XMLCh*) internalSubset; // cast off const
-            systemId = docImpl->cloneString(internalSubset);
+            temp = (XMLCh*) fInternalSubset; // cast off const
+            fInternalSubset = docImpl->cloneString(fInternalSubset);
             delete [] temp;
 
-            temp = (XMLCh*) name; // cast off const
-            name = docImpl->cloneString(name);
+            temp = (XMLCh*) fName; // cast off const
+            fName = docImpl->getPooledString(fName);
             delete [] temp;
 
             fNode.setOwnerDocument(doc);
             fParent.setOwnerDocument(doc);
 
-            DOMNamedNodeMap* entitiesTemp = ((DOMNamedNodeMapImpl *)entities)->cloneMap(this);
-            DOMNamedNodeMap* notationsTemp = ((DOMNamedNodeMapImpl *)notations)->cloneMap(this);
-            DOMNamedNodeMap* elementsTemp = ((DOMNamedNodeMapImpl *)elements)->cloneMap(this);
+            DOMNamedNodeMap* entitiesTemp = ((DOMNamedNodeMapImpl *)fEntities)->cloneMap(this);
+            DOMNamedNodeMap* notationsTemp = ((DOMNamedNodeMapImpl *)fNotations)->cloneMap(this);
+            DOMNamedNodeMap* elementsTemp = ((DOMNamedNodeMapImpl *)fElements)->cloneMap(this);
 
-            delete entities;
-            delete notations;
-            delete elements;
+            delete fEntities;
+            delete fNotations;
+            delete fElements;
 
-            entities = entitiesTemp;
-            notations = notationsTemp;
-            elements = elementsTemp;
+            fEntities = entitiesTemp;
+            fNotations = notationsTemp;
+            fElements = elementsTemp;
 
         }
     }
@@ -261,7 +292,7 @@ void DOMDocumentTypeImpl::setOwnerDocument(DOMDocument *doc) {
 
 const XMLCh * DOMDocumentTypeImpl::getNodeName() const
 {
-    return name;
+    return fName;
 };
 
 
@@ -272,26 +303,26 @@ short DOMDocumentTypeImpl::getNodeType()  const {
 
 DOMNamedNodeMap *DOMDocumentTypeImpl::getEntities() const
 {
-    return entities;
+    return fEntities;
 };
 
 
 
 const XMLCh * DOMDocumentTypeImpl::getName() const
 {
-    return name;
+    return fName;
 };
 
 
 DOMNamedNodeMap *DOMDocumentTypeImpl::getNotations() const
 {
-    return notations;
+    return fNotations;
 };
 
 
 DOMNamedNodeMap *DOMDocumentTypeImpl::getElements() const
 {
-    return elements;
+    return fElements;
 };
 
 
@@ -304,10 +335,10 @@ void DOMDocumentTypeImpl::setNodeValue(const XMLCh *val)
 void DOMDocumentTypeImpl::setReadOnly(bool readOnl, bool deep)
 {
     fNode.setReadOnly(readOnl,deep);
-    if (entities)
-        ((DOMNamedNodeMapImpl *)entities)->setReadOnly(readOnl,true);
-    if (notations)
-        ((DOMNamedNodeMapImpl *)notations)->setReadOnly(readOnl,true);
+    if (fEntities)
+        ((DOMNamedNodeMapImpl *)fEntities)->setReadOnly(readOnl,true);
+    if (fNotations)
+        ((DOMNamedNodeMapImpl *)fNotations)->setReadOnly(readOnl,true);
 };
 
 
@@ -315,24 +346,24 @@ void DOMDocumentTypeImpl::setReadOnly(bool readOnl, bool deep)
 
 const XMLCh * DOMDocumentTypeImpl::getPublicId() const
 {
-    return publicId;
+    return fPublicId;
 }
 
 
 const XMLCh * DOMDocumentTypeImpl::getSystemId() const
 {
-    return systemId;
+    return fSystemId;
 }
 
 
 const XMLCh * DOMDocumentTypeImpl::getInternalSubset() const
 {
-    return internalSubset;
+    return fInternalSubset;
 }
 
 bool DOMDocumentTypeImpl::isIntSubsetReading() const
 {
-    return intSubsetReading;
+    return fIntSubsetReading;
 }
 
 
@@ -345,11 +376,11 @@ void        DOMDocumentTypeImpl::setPublicId(const XMLCh *value)
         return;
 
     if ((DOMDocumentImpl *)castToNodeImpl(this)->getOwnerDocument())
-        publicId = ((DOMDocumentImpl *)castToNodeImpl(this)->getOwnerDocument())->getPooledString(value);
+        fPublicId = ((DOMDocumentImpl *)castToNodeImpl(this)->getOwnerDocument())->cloneString(value);
     else {
-        XMLCh* temp = (XMLCh*) publicId; // cast off const
+        XMLCh* temp = (XMLCh*) fPublicId; // cast off const
         delete [] temp;
-        publicId = XMLString::replicate(value);
+        fPublicId = XMLString::replicate(value);
     }
 }
 
@@ -359,11 +390,11 @@ void        DOMDocumentTypeImpl::setSystemId(const XMLCh *value)
         return;
 
     if ((DOMDocumentImpl *)castToNodeImpl(this)->getOwnerDocument())
-        systemId = ((DOMDocumentImpl *)castToNodeImpl(this)->getOwnerDocument())->getPooledString(value);
+        fSystemId = ((DOMDocumentImpl *)castToNodeImpl(this)->getOwnerDocument())->cloneString(value);
     else {
-        XMLCh* temp = (XMLCh*) systemId; // cast off const
+        XMLCh* temp = (XMLCh*) fSystemId; // cast off const
         delete [] temp;
-        systemId = XMLString::replicate(value);
+        fSystemId = XMLString::replicate(value);
     }
 }
 
@@ -373,11 +404,11 @@ void        DOMDocumentTypeImpl::setInternalSubset(const XMLCh *value)
         return;
 
     if ((DOMDocumentImpl *)castToNodeImpl(this)->getOwnerDocument())
-        internalSubset = ((DOMDocumentImpl *)castToNodeImpl(this)->getOwnerDocument())->getPooledString(value);
+        fInternalSubset = ((DOMDocumentImpl *)castToNodeImpl(this)->getOwnerDocument())->cloneString(value);
     else {
-        XMLCh* temp = (XMLCh*) internalSubset; // cast off const
+        XMLCh* temp = (XMLCh*) fInternalSubset; // cast off const
         delete [] temp;
-        internalSubset = XMLString::replicate(value);
+        fInternalSubset = XMLString::replicate(value);
     }
 }
 
@@ -428,7 +459,7 @@ void DOMDocumentTypeImpl::release()
      const XMLCh*           DOMDocumentTypeImpl::getNamespaceURI() const                 {return fNode.getNamespaceURI (); };
            DOMNode*         DOMDocumentTypeImpl::getNextSibling() const                  {return fChild.getNextSibling (); };
      const XMLCh*           DOMDocumentTypeImpl::getNodeValue() const                    {return fNode.getNodeValue (); };
-           DOMDocument*     DOMDocumentTypeImpl::getOwnerDocument() const                {return fNode.getOwnerDocument (); };
+           DOMDocument*     DOMDocumentTypeImpl::getOwnerDocument() const                {return fParent.fOwnerDocument; };
      const XMLCh*           DOMDocumentTypeImpl::getPrefix() const                       {return fNode.getPrefix (); };
            DOMNode*         DOMDocumentTypeImpl::getParentNode() const                   {return fChild.getParentNode (this); };
            DOMNode*         DOMDocumentTypeImpl::getPreviousSibling() const              {return fChild.getPreviousSibling (this); };
@@ -443,12 +474,12 @@ void DOMDocumentTypeImpl::release()
                                                                                          {return fNode.isSupported (feature, version); };
            void             DOMDocumentTypeImpl::setPrefix(const XMLCh  *prefix)         {fNode.setPrefix(prefix); };
            bool             DOMDocumentTypeImpl::hasAttributes() const                   {return fNode.hasAttributes(); };
-           bool             DOMDocumentTypeImpl::isSameNode(const DOMNode* other)        {return fNode.isSameNode(other); };
+           bool             DOMDocumentTypeImpl::isSameNode(const DOMNode* other) const  {return fNode.isSameNode(other); };
            void*            DOMDocumentTypeImpl::setUserData(const XMLCh* key, void* data, DOMUserDataHandler* handler)
                                                                                          {return fNode.setUserData(key, data, handler); };
            void*            DOMDocumentTypeImpl::getUserData(const XMLCh* key) const     {return fNode.getUserData(key); };
            const XMLCh*     DOMDocumentTypeImpl::getBaseURI() const                      {return fNode.getBaseURI(); };
-           short            DOMDocumentTypeImpl::compareTreePosition(DOMNode* other)     {return fNode.compareTreePosition(other); };
+           short            DOMDocumentTypeImpl::compareTreePosition(const DOMNode* other) const {return fNode.compareTreePosition(other); };
            const XMLCh*     DOMDocumentTypeImpl::getTextContent() const                  {return fNode.getTextContent(); };
            void             DOMDocumentTypeImpl::setTextContent(const XMLCh* textContent){fNode.setTextContent(textContent); };
            const XMLCh*     DOMDocumentTypeImpl::lookupNamespacePrefix(const XMLCh* namespaceURI, bool useDefault) const  {return fNode.lookupNamespacePrefix(namespaceURI, useDefault); };
@@ -457,8 +488,12 @@ void DOMDocumentTypeImpl::release()
            DOMNode*         DOMDocumentTypeImpl::getInterface(const XMLCh* feature)      {return fNode.getInterface(feature); };
 
 
-bool DOMDocumentTypeImpl::isEqualNode(const DOMNode* arg)
+bool DOMDocumentTypeImpl::isEqualNode(const DOMNode* arg) const
 {
+    if (isSameNode(arg)) {
+        return true;
+    }
+
     if (!fNode.isEqualNode(arg)) {
         return false;
     }
@@ -470,7 +505,7 @@ bool DOMDocumentTypeImpl::isEqualNode(const DOMNode* arg)
             return false;
         }
     }
-    else if (XMLString::compareString(getPublicId(), argDT->getPublicId())) {
+    else if (!XMLString::equals(getPublicId(), argDT->getPublicId())) {
         return false;
     }
 
@@ -479,7 +514,7 @@ bool DOMDocumentTypeImpl::isEqualNode(const DOMNode* arg)
             return false;
         }
     }
-    else if (XMLString::compareString(getSystemId(), argDT->getSystemId())) {
+    else if (!XMLString::equals(getSystemId(), argDT->getSystemId())) {
         return false;
     }
 
@@ -488,7 +523,7 @@ bool DOMDocumentTypeImpl::isEqualNode(const DOMNode* arg)
             return false;
         }
     }
-    else if (XMLString::compareString(getInternalSubset(), argDT->getInternalSubset())) {
+    else if (!XMLString::equals(getInternalSubset(), argDT->getInternalSubset())) {
         return false;
     }
 
@@ -545,3 +580,5 @@ bool DOMDocumentTypeImpl::isEqualNode(const DOMNode* arg)
     return fParent.isEqualNode(arg);
 };
 
+XERCES_CPP_NAMESPACE_END
+
