@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: DOMNodeImpl.cpp,v 1.14 2002/08/21 17:58:42 tng Exp $
+ * $Id: DOMNodeImpl.cpp,v 1.21 2003/01/16 18:56:49 tng Exp $
  */
 
 // This class doesn't support having any children, and implements the behavior
@@ -77,6 +77,10 @@
 #include <xercesc/util/PlatformUtils.hpp>
 #include <stdio.h>
 #include <assert.h>
+
+XERCES_CPP_NAMESPACE_BEGIN
+
+
 const unsigned short DOMNodeImpl::READONLY     = 0x1<<0;
 const unsigned short DOMNodeImpl::SYNCDATA     = 0x1<<1;
 const unsigned short DOMNodeImpl::SYNCCHILDREN = 0x1<<2;
@@ -401,12 +405,12 @@ const XMLCh* DOMNodeImpl::mapPrefix(const XMLCh *prefix,
     if (prefix == 0)
         return namespaceURI;
 
-    if (XMLString::compareString(prefix, s_xml) == 0)  {
-        if (XMLString::compareString(namespaceURI, s_xmlURI) == 0)
+    if (XMLString::equals(prefix, s_xml))  {
+        if (XMLString::equals(namespaceURI, s_xmlURI))
             return s_xmlURI;
         throw DOMException(DOMException::NAMESPACE_ERR, 0);
-    } else if (nType == DOMNode::ATTRIBUTE_NODE && XMLString::compareString(prefix, s_xmlns) == 0) {
-        if (XMLString::compareString(namespaceURI, s_xmlnsURI) == 0)
+    } else if (nType == DOMNode::ATTRIBUTE_NODE && XMLString::equals(prefix, s_xmlns)) {
+        if (XMLString::equals(namespaceURI, s_xmlnsURI))
             return s_xmlnsURI;
         throw DOMException(DOMException::NAMESPACE_ERR, 0);
     } else if (namespaceURI == 0 || *namespaceURI == 0) {
@@ -441,12 +445,12 @@ void DOMNodeImpl::callUserDataHandlers(DOMUserDataHandler::DOMOperationType oper
         ((DOMDocumentImpl*)getOwnerDocument())->callUserDataHandlers(this, operation, src, dst);
 }
 
-bool DOMNodeImpl::isSameNode(const DOMNode* other)
+bool DOMNodeImpl::isSameNode(const DOMNode* other) const
 {
     return (castToNode(this) == other);
 }
 
-bool DOMNodeImpl::isEqualNode(const DOMNode* arg)
+bool DOMNodeImpl::isEqualNode(const DOMNode* arg) const
 {
     if (!arg)
         return false;
@@ -462,31 +466,29 @@ bool DOMNodeImpl::isEqualNode(const DOMNode* arg)
     }
 
     // the compareString will check null string as well
-    if (XMLString::compareString(thisNode->getNodeName(), arg->getNodeName())) {
+    if (!XMLString::equals(thisNode->getNodeName(), arg->getNodeName())) {
         return false;
     }
 
-    if (XMLString::compareString(thisNode->getLocalName(),arg->getLocalName())) {
+    if (!XMLString::equals(thisNode->getLocalName(),arg->getLocalName())) {
         return false;
     }
 
-    if (XMLString::compareString(thisNode->getNamespaceURI(), arg->getNamespaceURI())) {
+    if (!XMLString::equals(thisNode->getNamespaceURI(), arg->getNamespaceURI())) {
         return false;
     }
 
-    if (XMLString::compareString(thisNode->getPrefix(), arg->getPrefix())) {
+    if (!XMLString::equals(thisNode->getPrefix(), arg->getPrefix())) {
         return false;
     }
 
-    if (XMLString::compareString(thisNode->getNodeValue(), arg->getNodeValue())) {
+    if (!XMLString::equals(thisNode->getNodeValue(), arg->getNodeValue())) {
         return false;
     }
 
-// baseURI not suppported yet
-/*
-    if (XMLString::compareString(thisNode->getBaseURI(), arg->getBaseURI())) {
+    if (!XMLString::equals(thisNode->getBaseURI(), arg->getBaseURI())) {
         return false;
-    }*/
+    }
 
     return true;
 }
@@ -550,15 +552,15 @@ DOMNode* DOMNodeImpl::getElementAncestor (const DOMNode* currentNode) const {
 const XMLCh* DOMNodeImpl::lookupNamespacePrefix(const XMLCh* const namespaceURI, bool useDefault, DOMElement *el) const {
     DOMNode *thisNode = castToNode(this);
 
-    const XMLCh* ns = getNamespaceURI();
+    const XMLCh* ns = thisNode->getNamespaceURI();
     // REVISIT: if no prefix is available is it null or empty string, or
     //          could be both?
-    const XMLCh* prefix = getPrefix();
+    const XMLCh* prefix = thisNode->getPrefix();
 
-    if (ns != 0 && (XMLString::compareString(ns,namespaceURI) == 0)) {
+    if (ns != 0 && XMLString::equals(ns,namespaceURI)) {
         if (useDefault || prefix != 0) {
             const XMLCh* foundNamespace =  el->lookupNamespaceURI(prefix);
-            if (foundNamespace != 0 && (XMLString::compareString(foundNamespace, namespaceURI) == 0)) {
+            if (foundNamespace != 0 && XMLString::equals(foundNamespace, namespaceURI)) {
                 return prefix;
             }
         }
@@ -576,14 +578,14 @@ const XMLCh* DOMNodeImpl::lookupNamespacePrefix(const XMLCh* const namespaceURI,
 
                 ns = attr->getNamespaceURI();
 
-                if (ns != 0 && (XMLString::compareString(ns, s_xmlnsURI) == 0)) {
+                if (ns != 0 && XMLString::equals(ns, s_xmlnsURI)) {
                     // DOM Level 2 nodes
-                    if ((useDefault && (XMLString::compareString(attr->getNodeName(), s_xmlns) == 0)) ||
-                        (attrPrefix != 0 && (XMLString::compareString(attrPrefix, s_xmlns) == 0)) &&
-                        (XMLString::compareString(value, namespaceURI) == 0)) {
+                    if ((useDefault && XMLString::equals(attr->getNodeName(), s_xmlns)) ||
+                        (attrPrefix != 0 && XMLString::equals(attrPrefix, s_xmlns)) &&
+                        XMLString::equals(value, namespaceURI)) {
                         const XMLCh* localname= attr->getLocalName();
                         const XMLCh* foundNamespace = el->lookupNamespaceURI(localname);
-                        if (foundNamespace != 0 && (XMLString::compareString(foundNamespace, namespaceURI) == 0)) {
+                        if (foundNamespace != 0 && XMLString::equals(foundNamespace, namespaceURI)) {
                             return localname;
                         }
                     }
@@ -604,14 +606,14 @@ const XMLCh* DOMNodeImpl::lookupNamespaceURI(const XMLCh* specifiedPrefix) const
     short type = thisNode->getNodeType();
     switch (type) {
     case DOMNode::ELEMENT_NODE : {
-        const XMLCh* ns = getNamespaceURI();
-        const XMLCh* prefix = getPrefix();
+        const XMLCh* ns = thisNode->getNamespaceURI();
+        const XMLCh* prefix = thisNode->getPrefix();
         if (ns != 0) {
             // REVISIT: is it possible that prefix is empty string?
             if (specifiedPrefix == 0 && prefix == specifiedPrefix) {
                 // looking for default namespace
                 return ns;
-            } else if (prefix != 0 && (XMLString::compareString(prefix, specifiedPrefix) == 0)) {
+            } else if (prefix != 0 && XMLString::equals(prefix, specifiedPrefix)) {
                 // non default namespace
                 return ns;
             }
@@ -626,15 +628,15 @@ const XMLCh* DOMNodeImpl::lookupNamespaceURI(const XMLCh* specifiedPrefix) const
                     const XMLCh *value = attr->getNodeValue();
                     ns = attr->getNamespaceURI();
 
-                    if (ns != 0 && (XMLString::compareString(ns, s_xmlnsURI) == 0)) {
+                    if (ns != 0 && XMLString::equals(ns, s_xmlnsURI)) {
                         // at this point we are dealing with DOM Level 2 nodes only
                         if (specifiedPrefix == 0 &&
-                            (XMLString::compareString(attr->getNodeName(), s_xmlns) == 0)) {
+                            XMLString::equals(attr->getNodeName(), s_xmlns)) {
                             // default namespace
                             return value;
                         } else if (attrPrefix != 0 &&
-                                   (XMLString::compareString(attrPrefix, s_xmlns) == 0) &&
-                                   (XMLString::compareString(attr->getLocalName(), specifiedPrefix) == 0)) {
+                                   XMLString::equals(attrPrefix, s_xmlns) &&
+                                   XMLString::equals(attr->getLocalName(), specifiedPrefix)) {
                             // non default namespace
                             return value;
                         }
@@ -675,10 +677,15 @@ const XMLCh* DOMNodeImpl::lookupNamespaceURI(const XMLCh* specifiedPrefix) const
 
 
 const XMLCh*     DOMNodeImpl::getBaseURI() const{
-    return 0;
+    DOMNode *thisNode = castToNode(this);
+    DOMNode* parent = thisNode->getParentNode();
+    if (parent)
+        return parent->getBaseURI();
+    else
+        return 0;
 }
 
-short            DOMNodeImpl::compareTreePosition(DOMNode* other){
+short            DOMNodeImpl::compareTreePosition(const DOMNode* other) const {
     // Questions of clarification for this method - to be answered by the
     // DOM WG.   Current assumptions listed - LM
     //
@@ -752,9 +759,9 @@ short            DOMNodeImpl::compareTreePosition(DOMNode* other){
     // We do this now, so that we get this info correct for attribute nodes
     // and their children.
 
-    DOMNode *node;
-    DOMNode *thisAncestor = castToNode(this);
-    DOMNode *otherAncestor = other;
+    const DOMNode *node;
+    const DOMNode *thisAncestor = castToNode(this);
+    const DOMNode *otherAncestor = other;
     int thisDepth=0;
     int otherDepth=0;
     for (node = castToNode(this); node != 0; node = node->getParentNode()) {
@@ -774,7 +781,7 @@ short            DOMNodeImpl::compareTreePosition(DOMNode* other){
     }
 
 
-    DOMNode *otherNode = other;
+    const DOMNode *otherNode = other;
 
     short thisAncestorType = thisAncestor->getNodeType();
     short otherAncestorType = otherAncestor->getNodeType();
@@ -916,11 +923,11 @@ bool DOMNodeImpl::isDefaultNamespace(const XMLCh* namespaceURI) const{
     short type = thisNode->getNodeType();
     switch (type) {
     case DOMNode::ELEMENT_NODE: {
-        //if we dont find a xmlns and we are looking for "" then its true
-        if(thisNode->isSameNode(thisNode->getOwnerDocument()->getDocumentElement())) {
-            if(namespaceURI == 0) {
-                return true;
-            }
+        const XMLCh *prefix = thisNode->getPrefix();
+
+        // REVISIT: is it possible that prefix is empty string?
+        if (prefix == 0 || !*prefix) {
+            return XMLString::equals(namespaceURI, thisNode->getNamespaceURI());
         }
 
         if (thisNode->hasAttributes()) {
@@ -928,7 +935,7 @@ bool DOMNodeImpl::isDefaultNamespace(const XMLCh* namespaceURI) const{
             DOMNode *attr = elem->getAttributeNodeNS(s_xmlnsURI, s_xmlns);
             if (attr != 0) {
                 const XMLCh *value = attr->getNodeValue();
-                return (XMLString::compareString(namespaceURI, value) == 0);
+                return XMLString::equals(namespaceURI, value);
             }
         }
         DOMNode *ancestor = getElementAncestor(thisNode);
@@ -979,3 +986,5 @@ void DOMNodeImpl::release()
     throw DOMException(DOMException::INVALID_ACCESS_ERR,0);
 }
 
+XERCES_CPP_NAMESPACE_END
+
