@@ -55,20 +55,27 @@
  */
 
 /*
- * $Id: DOMRangeImpl.cpp,v 1.4 2002/06/25 20:08:39 tng Exp $
+ * $Id: DOMRangeImpl.cpp,v 1.10 2003/01/29 15:23:35 tng Exp $
  */
 
 #include "DOMRangeImpl.hpp"
 #include "DOMDocumentImpl.hpp"
 #include "DOMDocumentFragmentImpl.hpp"
+#include "DOMCommentImpl.hpp"
+#include "DOMProcessingInstructionImpl.hpp"
 #include "DOMCasts.hpp"
 
 #include <xercesc/dom/DOMException.hpp>
 #include <xercesc/dom/DOMDocument.hpp>
 #include <xercesc/dom/DOMRangeException.hpp>
 #include <xercesc/dom/DOMText.hpp>
+#include <xercesc/dom/DOMProcessingInstruction.hpp>
 
 #include <xercesc/framework/XMLBuffer.hpp>
+#include <xercesc/util/Janitor.hpp>
+
+XERCES_CPP_NAMESPACE_BEGIN
+
 
 //---------------------
 // C'tor and D'tor
@@ -111,21 +118,45 @@ DOMRangeImpl::~DOMRangeImpl()
 
 DOMNode* DOMRangeImpl::getStartContainer() const
 {
+    if (fDetached)
+    {
+        throw DOMException(
+            DOMException::INVALID_STATE_ERR, 0);
+    }
+
     return fStartContainer;
 }
 
 XMLSize_t DOMRangeImpl::getStartOffset() const
 {
+    if (fDetached)
+    {
+        throw DOMException(
+            DOMException::INVALID_STATE_ERR, 0);
+    }
+
     return fStartOffset;
 }
 
 DOMNode* DOMRangeImpl::getEndContainer() const
 {
+    if (fDetached)
+    {
+        throw DOMException(
+            DOMException::INVALID_STATE_ERR, 0);
+    }
+
     return fEndContainer;
 }
 
 XMLSize_t DOMRangeImpl::getEndOffset() const
 {
+    if (fDetached)
+    {
+        throw DOMException(
+            DOMException::INVALID_STATE_ERR, 0);
+    }
+
     return fEndOffset;
 }
 
@@ -144,7 +175,7 @@ bool DOMRangeImpl::getCollapsed() const
 }
 
 //-------------------------------
-// Public getter functions
+// Public setter functions
 //-------------------------------
 
 void DOMRangeImpl::setStartContainer(const DOMNode* node)
@@ -197,15 +228,23 @@ void DOMRangeImpl::setStart(const DOMNode* refNode, XMLSize_t offset)
     validateNode(refNode);
     checkIndex(refNode, offset);
 
+    // error if not the same owner document
+    if (fDocument != refNode->getOwnerDocument()) {
+        if ( refNode != fDocument ) {
+            collapse(true); //collapse the range positions to start
+            fCollapsed = true;
+            throw DOMException(
+                DOMException::WRONG_DOCUMENT_ERR, 0);
+        }
+    }
+
     fStartContainer = (DOMNode*) refNode;
     fStartOffset    = offset;
 
-    if ((fDocument != refNode->getOwnerDocument() )
-        && (refNode->getOwnerDocument() != 0) )
-    {
-        fDocument = refNode->getOwnerDocument();
+    // they may be of same document, but not same root container
+    // collapse if not the same root container
+    if (!commonAncestorOf(refNode, fEndContainer))
         collapse(true);
-    }
 
     //compare the start and end boundary point
     //collapse if start point is after the end point
@@ -220,15 +259,23 @@ void DOMRangeImpl::setEnd(const DOMNode* refNode, XMLSize_t offset)
     validateNode(refNode);
     checkIndex(refNode, offset);
 
+    // error if not the same owner document
+    if (fDocument != refNode->getOwnerDocument()) {
+        if ( refNode != fDocument ) {
+            collapse(false); //collapse the range positions to end
+            fCollapsed = true;
+            throw DOMException(
+                DOMException::WRONG_DOCUMENT_ERR, 0);
+        }
+    }
+
     fEndContainer   = (DOMNode*) refNode;
     fEndOffset      = offset;
 
-    if ((fDocument != refNode->getOwnerDocument() )
-        && (refNode->getOwnerDocument() != 0) )
-    {
-        fDocument = refNode->getOwnerDocument();
+    // they may be of same document, but not same root container
+    // collapse if not the same root container
+    if (!commonAncestorOf(refNode, fStartContainer))
         collapse(false);
-    }
 
     //compare the start and end boundary point
     //collapse if start point is after the end point
@@ -249,6 +296,16 @@ void DOMRangeImpl::setStartBefore(const DOMNode* refNode)
             DOMRangeException::INVALID_NODE_TYPE_ERR, 0);
     }
 
+    // error if not the same owner document
+    if (fDocument != refNode->getOwnerDocument()) {
+        if ( refNode != fDocument ) {
+            collapse(true); //collapse the range positions to start
+            fCollapsed = true;
+            throw DOMException(
+                DOMException::WRONG_DOCUMENT_ERR, 0);
+        }
+    }
+
     fStartContainer = refNode->getParentNode();
    XMLSize_t i = 0;
     for (DOMNode* n = (DOMNode*) refNode; n!=0; n = n->getPreviousSibling()) {
@@ -259,12 +316,10 @@ void DOMRangeImpl::setStartBefore(const DOMNode* refNode)
     else
         fStartOffset = i-1;
 
-    if ((fDocument != refNode->getOwnerDocument())
-        && (refNode->getOwnerDocument() != 0) )
-    {
-        fDocument = refNode->getOwnerDocument();
+    // they may be of same document, but not same root container
+    // collapse if not the same root container
+    if (!commonAncestorOf(refNode, fEndContainer))
         collapse(true);
-    }
 
     //compare the start and end boundary point
     //collapse if start point is after the end point
@@ -285,6 +340,16 @@ void DOMRangeImpl::setStartAfter(const DOMNode* refNode)
             DOMRangeException::INVALID_NODE_TYPE_ERR, 0);
     }
 
+    // error if not the same owner document
+    if (fDocument != refNode->getOwnerDocument()) {
+        if ( refNode != fDocument ) {
+            collapse(true); //collapse the range positions to start
+            fCollapsed = true;
+            throw DOMException(
+                DOMException::WRONG_DOCUMENT_ERR, 0);
+        }
+    }
+
     fStartContainer = refNode->getParentNode();
     XMLSize_t i = 0;
     for (DOMNode* n = (DOMNode*) refNode; n!=0; n = n->getPreviousSibling()) {
@@ -293,12 +358,10 @@ void DOMRangeImpl::setStartAfter(const DOMNode* refNode)
 
     fStartOffset = i;
 
-    if ((fDocument != refNode->getOwnerDocument() )
-        && (refNode->getOwnerDocument() != 0) )
-    {
-        fDocument = refNode->getOwnerDocument();
+    // they may be of same document, but not same root container
+    // collapse if not the same root container
+    if (!commonAncestorOf(refNode, fEndContainer))
         collapse(true);
-    }
 
     //compare the start and end boundary point
     //collapse if start point is after the end point
@@ -319,6 +382,16 @@ void DOMRangeImpl::setEndBefore(const DOMNode* refNode)
             DOMRangeException::INVALID_NODE_TYPE_ERR, 0);
     }
 
+    // error if not the same owner document
+    if (fDocument != refNode->getOwnerDocument()) {
+        if ( refNode != fDocument ) {
+            collapse(false); //collapse the range positions to end
+            fCollapsed = true;
+            throw DOMException(
+                DOMException::WRONG_DOCUMENT_ERR, 0);
+        }
+    }
+
     fEndContainer = refNode->getParentNode();
     XMLSize_t i = 0;
     for (DOMNode* n = (DOMNode*) refNode; n!=0; n = n->getPreviousSibling(), i++) ;
@@ -328,12 +401,10 @@ void DOMRangeImpl::setEndBefore(const DOMNode* refNode)
     else
         fEndOffset = i-1;
 
-    if ((fDocument != refNode->getOwnerDocument() )
-        && (refNode->getOwnerDocument() != 0) )
-    {
-        fDocument = refNode->getOwnerDocument();
-        collapse(true);
-    }
+    // they may be of same document, but not same root container
+    // collapse if not the same root container
+    if (!commonAncestorOf(refNode, fStartContainer))
+        collapse(false);
 
     //compare the start and end boundary point
     //collapse if start point is after the end point
@@ -354,6 +425,16 @@ void DOMRangeImpl::setEndAfter(const DOMNode* refNode)
             DOMRangeException::INVALID_NODE_TYPE_ERR, 0);
     }
 
+    // error if not the same owner document
+    if (fDocument != refNode->getOwnerDocument()) {
+        if ( refNode != fDocument ) {
+            collapse(false); //collapse the range positions to end
+            fCollapsed = true;
+            throw DOMException(
+                DOMException::WRONG_DOCUMENT_ERR, 0);
+        }
+    }
+
     fEndContainer = refNode->getParentNode();
     XMLSize_t i = 0;
     for (DOMNode* n = (DOMNode*) refNode; n!=0; n = n->getPreviousSibling(), i++) ;
@@ -363,12 +444,10 @@ void DOMRangeImpl::setEndAfter(const DOMNode* refNode)
     else
         fEndOffset = i;
 
-    if ((fDocument != refNode->getOwnerDocument() )
-        && (refNode->getOwnerDocument() != 0) )
-    {
-        fDocument = refNode->getOwnerDocument();
-        collapse(true);
-    }
+    // they may be of same document, but not same root container
+    // collapse if not the same root container
+    if (!commonAncestorOf(refNode, fStartContainer))
+        collapse(false);
 
     //compare the start and end boundary point
     //collapse if start point is after the end point
@@ -426,7 +505,11 @@ void DOMRangeImpl::selectNode(const DOMNode* refNode)
             DOMRangeException::INVALID_NODE_TYPE_ERR, 0);
     }
     //First check for the text type node
-    if (refNode->getNodeType() ==  DOMNode::TEXT_NODE)
+    short type = refNode->getNodeType();
+    if((type == DOMNode::TEXT_NODE
+        || type == DOMNode::CDATA_SECTION_NODE
+        || type == DOMNode::COMMENT_NODE
+        || type == DOMNode::PROCESSING_INSTRUCTION_NODE))
     {
         //The node itself is the container.
         fStartContainer = (DOMNode*) refNode;
@@ -434,7 +517,10 @@ void DOMRangeImpl::selectNode(const DOMNode* refNode)
 
         //Select all the contents of the node
         fStartOffset = 0;
-        fEndOffset = ((DOMText *)refNode)->getLength();
+        if (type == DOMNode::PROCESSING_INSTRUCTION_NODE)
+            fEndOffset = XMLString::stringLen(((DOMProcessingInstruction*)refNode)->getData());
+        else
+            fEndOffset = ((DOMText *)refNode)->getLength();
         return;
     }
 
@@ -462,10 +548,19 @@ void DOMRangeImpl::selectNodeContents(const DOMNode* node)
     fEndContainer = (DOMNode*) node;
 
     fStartOffset = 0;
-    if (node->getNodeType() == DOMNode::TEXT_NODE ) {
+    short type = node->getNodeType();
+
+    if((type == DOMNode::TEXT_NODE
+        || type == DOMNode::CDATA_SECTION_NODE
+        || type == DOMNode::COMMENT_NODE)) {
+
         fEndOffset = ((DOMText *)node)->getLength();
         return;
     }
+    if (type == DOMNode::PROCESSING_INSTRUCTION_NODE) {
+        fEndOffset = XMLString::stringLen(((DOMProcessingInstruction*)node)->getData());
+        return;
+    }
 
     DOMNode* first = node->getFirstChild();
     if (first == 0) {
@@ -507,10 +602,18 @@ void DOMRangeImpl::surroundContents(DOMNode* newParent)
     DOMNode* realStart = fStartContainer;
     DOMNode* realEnd = fEndContainer;
 
-    if (fStartContainer->getNodeType() == DOMNode::TEXT_NODE) {
+    type = fStartContainer->getNodeType();
+    if((type == DOMNode::TEXT_NODE
+        || type == DOMNode::CDATA_SECTION_NODE
+        || type == DOMNode::COMMENT_NODE
+        || type == DOMNode::PROCESSING_INSTRUCTION_NODE)) {
         realStart = fStartContainer->getParentNode();
     }
-    if (fEndContainer->getNodeType() == DOMNode::TEXT_NODE) {
+    type = fEndContainer->getNodeType();
+    if((type == DOMNode::TEXT_NODE
+        || type == DOMNode::CDATA_SECTION_NODE
+        || type == DOMNode::COMMENT_NODE
+        || type == DOMNode::PROCESSING_INSTRUCTION_NODE)) {
         realEnd = fEndContainer->getParentNode();
     }
 
@@ -593,17 +696,40 @@ short DOMRangeImpl::compareBoundaryPoints(DOMRange::CompareHow how, const DOMRan
     }
 
     // case 4: preorder traversal of context tree.
-    DOMNode* ancestor = (DOMNode*) commonAncestorOf(pointA, pointB);
-    DOMNode* current = ancestor;
+    // Instead of literally walking the context tree in pre-order,
+    // we use relative node depth walking which is usually faster
 
-    do {
-        if (current == pointA) return -1;
-        if (current == pointB) return 1;
-        current = nextNode(current, true);
+    int depthDiff = 0;
+    DOMNode* n = 0;
+    for ( n = pointB; n != 0; n = n->getParentNode() )
+        depthDiff++;
+    for ( n = pointA; n != 0; n = n->getParentNode() )
+        depthDiff--;
+    while (depthDiff > 0) {
+        pointB = pointB->getParentNode();
+        depthDiff--;
     }
-    while (current!=0 && current!=ancestor);
-
-    return -2; // this should never happen
+    while (depthDiff < 0) {
+        pointA = pointA->getParentNode();
+        depthDiff++;
+    }
+    for (DOMNode* pB = pointB->getParentNode(),
+         *pA = pointA->getParentNode();
+         pB != pA;
+         pB = pB->getParentNode(), pA = pA->getParentNode() )
+    {
+        pointB = pB;
+        pointA = pA;
+    }
+    for ( n = pointB->getNextSibling();
+         n != 0;
+         n = n->getNextSibling() )
+    {
+        if (n == pointA) {
+            return 1;
+        }
+    }
+    return -1;
 }
 
 
@@ -629,25 +755,6 @@ void DOMRangeImpl::insertNode(DOMNode* newNode)
 {
     if (newNode == 0) return; //don't have to do anything
 
-    for (DOMNode* aNode = fStartContainer; aNode!=0; aNode = aNode->getParentNode()) {
-        if (castToNodeImpl(newNode)->isReadOnly()) {
-        throw DOMException(
-            DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
-    }
-    }
-
-    if (fDocument != newNode->getOwnerDocument()) {
-        throw DOMException(
-            DOMException::WRONG_DOCUMENT_ERR, 0);
-    }
-
-    // Prevent cycles in the tree.
-    //isKidOK() is not checked here as its taken care by insertBefore() function
-    if (isAncestorOf( newNode, fStartContainer)) {
-        throw DOMException(
-            DOMException::HIERARCHY_REQUEST_ERR, 0);
-    }
-
     if( fDetached) {
         throw DOMException(
             DOMException::INVALID_STATE_ERR, 0);
@@ -663,18 +770,47 @@ void DOMRangeImpl::insertNode(DOMNode* newNode)
             DOMRangeException::INVALID_NODE_TYPE_ERR, 0);
     }
 
+    // Prevent cycles in the tree.
+    //isKidOK() is not checked here as its taken care by insertBefore() function
+    if (isAncestorOf( newNode, fStartContainer)) {
+        throw DOMException(
+            DOMException::HIERARCHY_REQUEST_ERR, 0);
+    }
+
+    for (DOMNode* aNode = fStartContainer; aNode!=0; aNode = aNode->getParentNode()) {
+        if (castToNodeImpl(newNode)->isReadOnly()) {
+        throw DOMException(
+            DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
+    }
+    }
+
+    if (fDocument != newNode->getOwnerDocument()) {
+        throw DOMException(
+            DOMException::WRONG_DOCUMENT_ERR, 0);
+    }
+
 
     DOMNode* parent;
     DOMNode* next;
 
-    if (fStartContainer->getNodeType() == DOMNode::TEXT_NODE) {
+    type = fStartContainer->getNodeType();
+    if((type == DOMNode::TEXT_NODE
+        || type == DOMNode::CDATA_SECTION_NODE
+        || type == DOMNode::COMMENT_NODE
+        || type == DOMNode::PROCESSING_INSTRUCTION_NODE)) {
 
         //set 'parent' and 'next' here
         parent = fStartContainer->getParentNode();
 
         //split the text nodes
-       if (fStartOffset > 0)
-            ((DOMText*)fStartContainer)->splitText(fStartOffset);
+       if (fStartOffset > 0) {
+           if (type == DOMNode::COMMENT_NODE)
+               ((DOMCommentImpl*)fStartContainer)->splitText(fStartOffset);
+           else if (type == DOMNode::PROCESSING_INSTRUCTION_NODE)
+               ((DOMProcessingInstructionImpl*)fStartContainer)->splitText(fStartOffset);
+           else
+               ((DOMText*)fStartContainer)->splitText(fStartOffset);
+       }
 
         //update the new start information later. After inserting the first newNode
         if (fStartOffset == 0)
@@ -721,34 +857,33 @@ const XMLCh* DOMRangeImpl::toString() const
             DOMException::INVALID_STATE_ERR, 0);
     }
 
+    if ((fStartContainer == fEndContainer) && (fEndOffset == fStartOffset))
+        return XMLUni::fgZeroLenString;
+
     DOMNode* node = fStartContainer;
     DOMNode* stopNode = fEndContainer;
 
     XMLBuffer retStringBuf;
-    if ( (fStartContainer->getNodeType() == DOMNode::TEXT_NODE)
-        || (fStartContainer->getNodeType() == DOMNode::CDATA_SECTION_NODE) ) {
+    short type = fStartContainer->getNodeType();
+    if((type == DOMNode::TEXT_NODE
+        || type == DOMNode::CDATA_SECTION_NODE
+        || type == DOMNode::COMMENT_NODE
+        || type == DOMNode::PROCESSING_INSTRUCTION_NODE)) {
         if (fStartContainer == fEndContainer) {
-            if (fEndOffset == fStartOffset) {
-                return XMLUni::fgZeroLenString;
-            }
-            else {
+            XMLCh* tempString;
+            XMLCh temp[4000];
+            if ((fEndOffset-fStartOffset) >= 3999)
+                tempString = new XMLCh[fEndOffset-fStartOffset+1];
+            else
+                tempString = temp;
 
-                XMLCh* tempString;
-                XMLCh temp[4000];
-                if ((fEndOffset-fStartOffset) >= 3999)
-                    tempString = new XMLCh[fEndOffset-fStartOffset+1];
-                else
-                    tempString = temp;
+            XMLString::subString(tempString, fStartContainer->getNodeValue(), fStartOffset, fEndOffset);
+            const XMLCh* retString = ((DOMDocumentImpl *)fDocument)->getPooledString(tempString);
 
-                XMLString::subString(tempString, fStartContainer->getNodeValue(), fStartOffset, fEndOffset);
-                const XMLCh* retString = ((DOMDocumentImpl *)fDocument)->getPooledString(tempString);
-
-                if ((fEndOffset-fStartOffset) >= 3999)
-                    delete[] tempString;
-
-                return retString;
-            }
+            if ((fEndOffset-fStartOffset) >= 3999)
+                delete[] tempString;
 
+            return retString;
         } else {
             XMLSize_t length = XMLString::stringLen(fStartContainer->getNodeValue());
             if (length != fStartOffset) {
@@ -783,8 +918,11 @@ const XMLCh* DOMRangeImpl::toString() const
         }
     }
 
-    if ( fEndContainer->getNodeType()!= DOMNode::TEXT_NODE &&
-        fEndContainer->getNodeType()!= DOMNode::CDATA_SECTION_NODE ){
+    type = fEndContainer->getNodeType();
+    if((type != DOMNode::TEXT_NODE
+        && type != DOMNode::CDATA_SECTION_NODE
+        && type != DOMNode::COMMENT_NODE
+        && type != DOMNode::PROCESSING_INSTRUCTION_NODE)) {
         int i=fEndOffset;
         stopNode = fEndContainer->getFirstChild();
         while( i>0 && stopNode!=0 ){
@@ -797,15 +935,22 @@ const XMLCh* DOMRangeImpl::toString() const
 
     while (node != stopNode) {  //look into all kids of the Range
         if (node == 0) break;
-        if (node->getNodeType() == DOMNode::TEXT_NODE
-            ||  node->getNodeType() == DOMNode::CDATA_SECTION_NODE) {
+        type = node->getNodeType();
+
+        if((type == DOMNode::TEXT_NODE
+            || type == DOMNode::CDATA_SECTION_NODE
+            || type == DOMNode::COMMENT_NODE
+            || type == DOMNode::PROCESSING_INSTRUCTION_NODE)) {
             retStringBuf.append(node->getNodeValue());
         }
         node = nextNode(node, true);
     }
 
-    if (fEndContainer->getNodeType() == DOMNode::TEXT_NODE
-        || fEndContainer->getNodeType() == DOMNode::CDATA_SECTION_NODE) {
+    type = fEndContainer->getNodeType();
+    if((type == DOMNode::TEXT_NODE
+        || type == DOMNode::CDATA_SECTION_NODE
+        || type == DOMNode::COMMENT_NODE
+        || type == DOMNode::PROCESSING_INSTRUCTION_NODE)) {
 
         if (fEndOffset != 0) {
 
@@ -928,9 +1073,6 @@ const DOMNode* DOMRangeImpl::commonAncestorOf(const DOMNode* pointA, const DOMNo
     if (fDetached)
             throw DOMException(DOMException::INVALID_STATE_ERR, 0);
 
-    if (pointA->getOwnerDocument() != pointB->getOwnerDocument())
-        throw DOMException( DOMException::WRONG_DOCUMENT_ERR, 0 );
-
     //if the containers are same then it itself is its common ancestor.
     if (pointA == pointB)
         return pointA;
@@ -939,12 +1081,12 @@ const DOMNode* DOMRangeImpl::commonAncestorOf(const DOMNode* pointA, const DOMNo
     VectorNodes startV(1, false);
     DOMNode* node;
 
-    for (node=fStartContainer; node != 0; node=node->getParentNode())
+    for (node=(DOMNode*)pointA; node != 0; node=node->getParentNode())
     {
         startV.addElement(node);
     }
     VectorNodes endV(1, false);
-    for (node=fEndContainer; node != 0; node=node->getParentNode())
+    for (node=(DOMNode*)pointB; node != 0; node=node->getParentNode())
     {
         endV.addElement(node);
     }
@@ -952,7 +1094,7 @@ const DOMNode* DOMRangeImpl::commonAncestorOf(const DOMNode* pointA, const DOMNo
     int s = startV.size()-1;
     int e = endV.size()-1;
 
-    DOMNode* commonAncestor;
+    DOMNode* commonAncestor = 0;
 
     while (s>=0 && e>=0) {
         if (startV.elementAt(s) == endV.elementAt(e)) {
@@ -1025,8 +1167,6 @@ DOMNode* DOMRangeImpl::nextNode(const DOMNode* node, bool visitChildren) const
             return result;
         } else {
             parent = parent->getParentNode();
-            if (parent == fEndContainer) return parent;
-
         }
 
     }
@@ -1060,22 +1200,55 @@ DOMDocumentFragment* DOMRangeImpl::traverseContents(TraversalType how)
         return traverseSameContainer( how );
 
     // case 2: Child C of start container is ancestor of end container
-    for (DOMNode* node = fStartContainer->getFirstChild(); node != 0; node=node->getNextSibling()) {
-        if (isAncestorOf(node, fEndContainer))
-            return traverseCommonStartContainer( node, how );
-    }
-
-    // case 3: Child C of end container  is ancestor of start container
-    for (DOMNode* nd = fEndContainer->getFirstChild(); nd != 0; nd=nd->getNextSibling()) {
-        if (isAncestorOf(nd, fStartContainer))
-             return traverseCommonEndContainer( nd, how );
+    // This can be quickly tested by walking the parent chain of
+    // end container
+    int endContainerDepth = 0;
+    for ( DOMNode* c = fEndContainer, *p = c->getParentNode();
+             p != 0;
+             c = p, p = p->getParentNode())
+        {
+            if (p == fStartContainer)
+                return traverseCommonStartContainer( c, how );
+            ++endContainerDepth;
         }
 
-    // case 4: preorder traversal of context tree.
-    // There is a common ancestor container.  Find the
+    // case 3: Child C of end container  is ancestor of start container
+    // This can be quickly tested by walking the parent chain of A
+    int startContainerDepth = 0;
+    for ( DOMNode* c2 = fStartContainer, *p2 = c2->getParentNode();
+         p2 != 0;
+         c2 = p2, p2 = p2->getParentNode())
+    {
+        if (p2 == fEndContainer)
+            return traverseCommonEndContainer( c2, how );
+        ++startContainerDepth;
+    }
+
+    // case 4: There is a common ancestor container.  Find the
     // ancestor siblings that are children of that container.
-    DOMNode* ancestor = (DOMNode*)commonAncestorOf(fStartContainer, fEndContainer);
-    return traverseCommonAncestors( ancestor, ancestor, how );
+    int depthDiff = startContainerDepth - endContainerDepth;
+
+    DOMNode* startNode = fStartContainer;
+    while (depthDiff > 0) {
+        startNode = startNode->getParentNode();
+        depthDiff--;
+    }
+
+    DOMNode* endNode = fEndContainer;
+    while (depthDiff < 0) {
+        endNode = endNode->getParentNode();
+        depthDiff++;
+    }
+
+    // ascend the ancestor hierarchy until we have a common parent.
+    for( DOMNode* sp = startNode->getParentNode(), *ep = endNode->getParentNode();
+         sp!=ep;
+         sp = sp->getParentNode(), ep = ep->getParentNode() )
+    {
+        startNode = sp;
+        endNode = ep;
+    }
+    return traverseCommonAncestors( startNode, endNode, how );
     }
 
 /**
@@ -1097,7 +1270,11 @@ DOMDocumentFragment* DOMRangeImpl::traverseSameContainer( int how )
     DOMNode* cloneCurrent = 0;
 
     // Text node needs special case handling
-    if ( fStartContainer->getNodeType()== DOMNode::TEXT_NODE )
+    short type = fStartContainer->getNodeType();
+    if((type == DOMNode::TEXT_NODE
+        || type == DOMNode::CDATA_SECTION_NODE
+        || type == DOMNode::COMMENT_NODE
+        || type == DOMNode::PROCESSING_INSTRUCTION_NODE))
     {
         cloneCurrent = fStartContainer->cloneNode(false);
         if (fEndOffset == fStartOffset) {
@@ -1119,8 +1296,13 @@ DOMDocumentFragment* DOMRangeImpl::traverseSameContainer( int how )
         }
 
         // set the original text node to its new value
-        if ( how != CLONE_CONTENTS )
-            ((DOMText*)fStartContainer)->deleteData(fStartOffset, fEndOffset-fStartOffset);
+        if ( how != CLONE_CONTENTS ) {
+            if(type == DOMNode::PROCESSING_INSTRUCTION_NODE) {
+                ((DOMProcessingInstructionImpl*)fStartContainer)->deleteData(fStartOffset, fEndOffset-fStartOffset);
+            }
+            else
+                ((DOMCharacterData*)fStartContainer)->deleteData(fStartOffset, fEndOffset-fStartOffset);
+        }
         if ( how != DELETE_CONTENTS)
             frag->appendChild(cloneCurrent);
     }
@@ -1128,7 +1310,7 @@ DOMDocumentFragment* DOMRangeImpl::traverseSameContainer( int how )
         // Copy nodes between the start/end offsets.
         DOMNode* n = getSelectedNode( fStartContainer, fStartOffset );
         int cnt = fEndOffset - fStartOffset;
-        while( cnt > 0 )
+        while( cnt > 0 && n)
         {
             DOMNode* sibling = n->getNextSibling();
             DOMNode* xferNode = traverseFullySelected( n, how );
@@ -1136,7 +1318,7 @@ DOMDocumentFragment* DOMRangeImpl::traverseSameContainer( int how )
                 frag->appendChild( xferNode );
             --cnt;
             n = sibling;
-            }
+        }
     }
 
     // Nothing is partially selected, so collapse to start point
@@ -1442,8 +1624,15 @@ DOMNode* DOMRangeImpl::traverseNode( DOMNode* n, bool isFullySelected, bool isLe
 {
     if ( isFullySelected )
         return traverseFullySelected( n, how );
-    if ( n->getNodeType()== DOMNode::TEXT_NODE )
+
+    short type = n->getNodeType();
+
+    if((type == DOMNode::TEXT_NODE
+        || type == DOMNode::CDATA_SECTION_NODE
+        || type == DOMNode::COMMENT_NODE
+        || type == DOMNode::PROCESSING_INSTRUCTION_NODE))
         return traverseTextNode( n, isLeft, how );
+
     return traversePartiallySelected( n, how );
 }
 
@@ -1460,11 +1649,6 @@ DOMNode* DOMRangeImpl::traverseFullySelected( DOMNode* n, int how )
     case CLONE_CONTENTS:
         return n->cloneNode( true );
     case EXTRACT_CONTENTS:
-        if ( n->getNodeType()== DOMNode::DOCUMENT_TYPE_NODE )
-        {
-            throw DOMException(
-                DOMException::HIERARCHY_REQUEST_ERR, 0);
-        }
         return n;
     case DELETE_CONTENTS:
         // revisit:
@@ -1504,7 +1688,8 @@ DOMNode* DOMRangeImpl::traversePartiallySelected( DOMNode*n, int how )
  */
 DOMNode* DOMRangeImpl::traverseTextNode( DOMNode*n, bool isLeft, int how )
 {
-    const XMLCh* txtValue = n->getNodeValue();
+    XMLCh* txtValue = XMLString::replicate(n->getNodeValue());
+    ArrayJanitor<XMLCh> janValue(txtValue);
 
     if ( isLeft )
     {
@@ -1628,7 +1813,11 @@ DOMNode* DOMRangeImpl::traverseTextNode( DOMNode*n, bool isLeft, int how )
  */
 DOMNode* DOMRangeImpl::getSelectedNode( DOMNode*container, int offset )
 {
-    if ( container->getNodeType() == DOMNode::TEXT_NODE )
+    short type = container->getNodeType();
+    if((type == DOMNode::TEXT_NODE
+        || type == DOMNode::CDATA_SECTION_NODE
+        || type == DOMNode::COMMENT_NODE
+        || type == DOMNode::PROCESSING_INSTRUCTION_NODE))
         return container;
 
     // This case is an important convenience for
@@ -1651,28 +1840,57 @@ void DOMRangeImpl::checkReadOnly(DOMNode* start, DOMNode* end,
                               XMLSize_t startOffset, XMLSize_t endOffset)
 {
     if ((start == 0) || (end == 0) ) return;
-    //if both start and end are text check and return
-    if (start->getNodeType() == DOMNode::TEXT_NODE) {
+    DOMNode*sNode = 0;
+
+    short type = start->getNodeType();
+    if ( type == DOMNode::DOCUMENT_TYPE_NODE )
+    {
+        throw DOMException(
+            DOMException::HIERARCHY_REQUEST_ERR, 0);
+    }
+
+    if((type == DOMNode::TEXT_NODE
+        || type == DOMNode::CDATA_SECTION_NODE
+        || type == DOMNode::COMMENT_NODE
+        || type == DOMNode::PROCESSING_INSTRUCTION_NODE))
+    {
         if (castToNodeImpl(start)->isReadOnly()) {
             throw DOMException(
                 DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
         }
+        //if both start and end are text check and return
         if (start == end)
             return;
+
+        sNode = start;
+    } else {
+        //set the start and end nodes to check
+        sNode = start->getFirstChild();
+        for(XMLSize_t i = 0; i<startOffset; i++)
+            sNode = sNode->getNextSibling();
     }
-    //set the start and end nodes to check
-    DOMNode*sNode = start->getFirstChild();
-    for(XMLSize_t i = 0; i<startOffset; i++)
-        sNode = sNode->getNextSibling();
 
     DOMNode* eNode;
-    if (end->getNodeType() == DOMNode::TEXT_NODE) {
+    type = end->getNodeType();
+    if ( type == DOMNode::DOCUMENT_TYPE_NODE )
+    {
+        throw DOMException(
+            DOMException::HIERARCHY_REQUEST_ERR, 0);
+    }
+
+    if((type == DOMNode::TEXT_NODE
+        || type == DOMNode::CDATA_SECTION_NODE
+        || type == DOMNode::COMMENT_NODE
+        || type == DOMNode::PROCESSING_INSTRUCTION_NODE))
+    {
         eNode = end; //need to check only till this node
     }
     else { //need to check all the kids that fall before the end offset value
         eNode = end->getFirstChild();
-        for (XMLSize_t i = 0; i<endOffset-1; i++)
-            eNode = eNode->getNextSibling();
+        if (endOffset > 0)  {
+            for (XMLSize_t i = 0; i<endOffset-1; i++)
+                eNode = eNode->getNextSibling();
+        }
     }
     //recursivly search if any node is readonly
     recurseTreeAndCheck(sNode, eNode);
@@ -1682,6 +1900,12 @@ void DOMRangeImpl::recurseTreeAndCheck(DOMNode* start, DOMNode* end)
 {
     for(DOMNode* node=start; node != 0 && node !=end; node=node->getNextSibling())
     {
+        if ( node->getNodeType()== DOMNode::DOCUMENT_TYPE_NODE )
+        {
+            throw DOMException(
+                DOMException::HIERARCHY_REQUEST_ERR, 0);
+        }
+
         if (castToNodeImpl(node)->isReadOnly()) {
             throw DOMException(
                 DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
@@ -1710,42 +1934,62 @@ DOMNode* DOMRangeImpl::removeChild(DOMNode* parent, DOMNode* child)
 
 
 /* This function is called from DOM.
-*  The  text has already beeen replaced.
+*  The  text has already been replaced.
 *  Fix-up any offsets.
 */
 void DOMRangeImpl::receiveReplacedText(DOMNode* node)
 {
     if (node == 0) return;
 
+    short type = fStartContainer->getNodeType();
     if (node == fStartContainer
-        && fStartContainer->getNodeType() == DOMNode::TEXT_NODE) {
+        && (type == DOMNode::TEXT_NODE
+        || type == DOMNode::CDATA_SECTION_NODE
+        || type == DOMNode::COMMENT_NODE
+        || type == DOMNode::PROCESSING_INSTRUCTION_NODE))
+    {
         fStartOffset = 0;
     }
+    type = fEndContainer->getNodeType();
     if (node == fEndContainer
-        && fEndContainer->getNodeType() == DOMNode::TEXT_NODE) {
+        && (type == DOMNode::TEXT_NODE
+        || type == DOMNode::CDATA_SECTION_NODE
+        || type == DOMNode::COMMENT_NODE
+        || type == DOMNode::PROCESSING_INSTRUCTION_NODE))
+    {
         fEndOffset = 0;
     }
 }
 
 
 /** This function is called from DOM.
-*  The  text has already beeen inserted.
+*  The  text has already been deleted.
 *  Fix-up any offsets.
 */
 void DOMRangeImpl::updateRangeForDeletedText(DOMNode* node, XMLSize_t offset, int count)
 {
     if (node == 0) return;
 
+    short type = fStartContainer->getNodeType();
     if (node == fStartContainer
-        && fStartContainer->getNodeType() == DOMNode::TEXT_NODE) {
+        && (type == DOMNode::TEXT_NODE
+        || type == DOMNode::CDATA_SECTION_NODE
+        || type == DOMNode::COMMENT_NODE
+        || type == DOMNode::PROCESSING_INSTRUCTION_NODE))
+    {
         if (fStartOffset > offset+count) {
             fStartOffset = fStartOffset-count;
         } else if (fStartOffset > offset) {
             fStartOffset = offset;
         }
     }
+    type = fEndContainer->getNodeType();
     if (node == fEndContainer
-        && fEndContainer->getNodeType() == DOMNode::TEXT_NODE) {
+        && (type == DOMNode::TEXT_NODE
+        || type == DOMNode::CDATA_SECTION_NODE
+        || type == DOMNode::COMMENT_NODE
+        || type == DOMNode::PROCESSING_INSTRUCTION_NODE))
+    {
         if (fEndOffset > offset+count) {
             fEndOffset = fEndOffset-count;
         } else if (fEndOffset > offset) {
@@ -1756,6 +2000,40 @@ void DOMRangeImpl::updateRangeForDeletedText(DOMNode* node, XMLSize_t offset, in
 
 
 
+/** This function is called from DOM.
+*  The  text has already beeen inserted.
+*  Fix-up any offsets.
+*/
+void DOMRangeImpl::updateRangeForInsertedText(DOMNode* node, XMLSize_t offset, int count)
+{
+    if (node == 0) return;
+
+    short type = fStartContainer->getNodeType();
+    if (node == fStartContainer
+        && (type == DOMNode::TEXT_NODE
+        || type == DOMNode::CDATA_SECTION_NODE
+        || type == DOMNode::COMMENT_NODE
+        || type == DOMNode::PROCESSING_INSTRUCTION_NODE))
+    {
+        if (fStartOffset > offset) {
+            fStartOffset = offset;
+        }
+    }
+    type = fEndContainer->getNodeType();
+    if (node == fEndContainer
+        && (type == DOMNode::TEXT_NODE
+        || type == DOMNode::CDATA_SECTION_NODE
+        || type == DOMNode::COMMENT_NODE
+        || type == DOMNode::PROCESSING_INSTRUCTION_NODE))
+    {
+        if (fEndOffset > offset) {
+            fEndOffset = fEndOffset+count;
+        }
+    }
+}
+
+
+
 /** This function must be called by the DOM _BEFORE_
 *  a node is deleted, because at that time it is
 *  connected in the DOM tree, which we depend on.
@@ -1815,20 +2093,38 @@ void DOMRangeImpl::updateRangeForInsertedNode(DOMNode* node) {
 }
 
 
-void DOMRangeImpl::updateSplitInfo(DOMText* oldNode, DOMText* startNode, XMLSize_t offset)
+void DOMRangeImpl::updateSplitInfo(DOMNode* oldNode, DOMNode* startNode, XMLSize_t offset)
 {
     if (startNode == 0) return;
 
-    if (fStartContainer == oldNode && fStartOffset > offset) {
-          fStartOffset = fStartOffset - offset;
-        fStartContainer = startNode;
+    short type = fStartContainer->getNodeType();
+    if (oldNode == fStartContainer
+        && (type == DOMNode::TEXT_NODE
+        || type == DOMNode::CDATA_SECTION_NODE
+        || type == DOMNode::COMMENT_NODE
+        || type == DOMNode::PROCESSING_INSTRUCTION_NODE))
+    {
+        if (fStartOffset > offset) {
+            fStartOffset = fStartOffset - offset;
+            fStartContainer = startNode;
+        }
     }
 
-    if (fEndContainer == oldNode && fEndOffset > offset) {
+    type = fEndContainer->getNodeType();
+    if (oldNode == fEndContainer
+        && (type == DOMNode::TEXT_NODE
+        || type == DOMNode::CDATA_SECTION_NODE
+        || type == DOMNode::COMMENT_NODE
+        || type == DOMNode::PROCESSING_INSTRUCTION_NODE))
+    {
+        if (fEndOffset > offset) {
             fEndContainer = startNode;
-       fEndOffset = fEndOffset - offset;
+           fEndOffset = fEndOffset - offset;
+        }
     }
 }
 
 
 
+XERCES_CPP_NAMESPACE_END
+
