@@ -56,6 +56,15 @@
 
 /*
  * $Log: XMLAttDef.hpp,v $
+ * Revision 1.5  2003/01/29 19:45:35  gareth
+ * added api for DOMTypeInfo
+ *
+ * Revision 1.4  2002/12/10 02:15:55  knoaman
+ * Schema Errata: E1-21.
+ *
+ * Revision 1.3  2002/11/04 15:00:21  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.2  2002/02/20 18:17:01  tng
  * [Bug 5977] Warnings on generating apiDocs.
  *
@@ -112,6 +121,8 @@
 
 #include <xercesc/util/XMLString.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 class XMLAttr;
 
 /** Represents the core information of an atribute definition
@@ -181,9 +192,9 @@ public:
         , Required               = 2
         , Required_And_Fixed     = 3
         , Implied                = 4
-        , ProcessContents_Strict = 5
+        , ProcessContents_Skip   = 5
         , ProcessContents_Lax    = 6
-        , ProcessContents_Skip   = 7
+        , ProcessContents_Strict = 7
         , Prohibited             = 8
 
         , DefAttTypes_Count
@@ -269,6 +280,12 @@ public:
       */
     virtual const XMLCh* getFullName() const = 0;
 
+    /**
+     * The derived class should implement any cleaning up required between
+     * each use of an instance of this class for validation
+     */
+    virtual void reset() = 0;
+
     //@}
 
 
@@ -366,6 +383,17 @@ public:
       */
     bool isExternal() const;
 
+
+    /**
+     * @return the uri part of DOM Level 3 TypeInfo
+     */
+    virtual const XMLCh* getDOMTypeInfoUri() const = 0;
+
+    /**
+     * @return the name part of DOM Level 3 TypeInfo
+     */
+    virtual const XMLCh* getDOMTypeInfoName() const = 0;
+
     //@}
 
 
@@ -633,4 +661,6 @@ inline void XMLAttDef::setExternalAttDeclaration(const bool aValue)
     fExternalAttribute = aValue;
 }
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
