@@ -56,6 +56,12 @@
 
 /*
  * $Log: XMLAttr.hpp,v $
+ * Revision 1.4  2002/11/28 20:12:45  knoaman
+ * Allow creating/setting of XMLAttr using a rawname (i.e. 'prefix:localpart').
+ *
+ * Revision 1.3  2002/11/04 15:00:21  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.2  2002/02/20 18:17:01  tng
  * [Bug 5977] Warnings on generating apiDocs.
  *
@@ -104,6 +110,7 @@
 #include <xercesc/util/QName.hpp>
 #include <xercesc/framework/XMLAttDef.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 /**
  *  This class defines the information about an attribute that will come out
@@ -177,6 +184,38 @@ public:
         , const XMLAttDef::AttTypes type = XMLAttDef::CData
         , const bool                specified = true
     );
+
+    /**
+      * This is the primary constructor which takes all of the information
+      * required to construct a complete attribute object.
+      *
+      * @param  uriId       The id into the validator's URI pool of the URI
+      *                     that the prefix mapped to. Only used if namespaces
+      *                     are enabled/supported.
+      *
+      * @param  rawName     The raw name of the attribute.
+      *
+      * @param  attrValue   The value string of the attribute, which should
+      *                     be fully normalized by XML rules!
+      *
+      * @param  type        The type of the attribute. This will indicate
+      *                     the type of normalization done and constrains
+      *                     the value content. Make sure that the value
+      *                     set meets the constraints!
+      *
+      * @param  specified   Indicates whether the attribute was explicitly
+      *                     specified or not. If not, then it was faulted
+      *                     in from a FIXED or DEFAULT value.
+      */
+    XMLAttr
+    (
+        const unsigned int uriId
+        , const XMLCh* const rawName
+        , const XMLCh* const attrValue
+        , const XMLAttDef::AttTypes type = XMLAttDef::CData
+        , const bool specified = true
+    );
+
     //@}
 
     /** @name Destructor */
@@ -284,6 +323,34 @@ public:
         , const XMLAttDef::AttTypes type = XMLAttDef::CData
     );
 
+    /**
+      * This method is called to set up a default constructed object after
+      * the fact, or to reuse a previously used object.
+      *
+      * @param  uriId       The id into the validator's URI pool of the URI
+      *                     that the prefix mapped to. Only used if namespaces
+      *                     are enabled/supported.
+      *
+      * @param  attrRawName The raw name of the attribute.
+      *
+      * @param  attrValue   The value string of the attribute, which should
+      *                     be fully normalized by XML rules according to the
+      *                     attribute type.
+      *
+      * @param  type        The type of the attribute. This will indicate
+      *                     the type of normalization done and constrains
+      *                     the value content. Make sure that the value
+      *                     set meets the constraints!
+      *
+      */
+    void set
+    (
+        const   unsigned int        uriId
+        , const XMLCh* const        attrRawName
+        , const XMLCh* const        attrValue
+        , const XMLAttDef::AttTypes type = XMLAttDef::CData
+    );
+
     /**
       * This method will update just the name related fields of the
       * attribute object. The other fields are left as is.
@@ -452,6 +519,19 @@ inline void XMLAttr::set(const  unsigned int        uriId
     fType = type;
 }
 
+inline void XMLAttr::set(const  unsigned int        uriId
+                        , const XMLCh* const        attrRawName
+                        , const XMLCh* const        attrValue
+                        , const XMLAttDef::AttTypes type)
+{
+    // Set the name info and the value via their respective calls
+    fAttName->setName(attrRawName, uriId);
+    setValue(attrValue);
+
+    // And store the type
+    fType = type;
+}
+
 inline void XMLAttr::setType(const XMLAttDef::AttTypes newValue)
 {
     fType = newValue;
@@ -462,4 +542,6 @@ inline void XMLAttr::setSpecified(const bool newValue)
     fSpecified = newValue;
 }
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
