@@ -56,6 +56,12 @@
 
 /*
  * $Log: XMLElementDecl.hpp,v $
+ * Revision 1.4  2003/01/29 19:45:35  gareth
+ * added api for DOMTypeInfo
+ *
+ * Revision 1.3  2002/11/04 15:00:21  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.2  2002/08/20 16:55:33  tng
  * [Bug 6251] Info during compilation.
  *
@@ -131,6 +137,8 @@
 #include <xercesc/framework/XMLAttr.hpp>
 #include <xercesc/framework/XMLAttDefList.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 class ContentSpecNode;
 class XMLContentModel;
 
@@ -443,6 +451,18 @@ class XMLPARSER_EXPORT XMLElementDecl
       */
     unsigned int getId() const;
 
+
+    /**
+     * @return the uri part of DOM Level 3 TypeInfo
+     */
+    virtual const XMLCh* getDOMTypeInfoUri() const = 0;
+
+    /**
+     * @return the name part of DOM Level 3 TypeInfo
+     */
+    virtual const XMLCh* getDOMTypeInfoName() const = 0;
+
+
     /** Indicate whether this element type has been declared yet
       *
       * This method returns a boolean that indicates whether this element
@@ -668,4 +688,6 @@ inline void XMLElementDecl::setExternalElemDeclaration(const bool aValue)
     fExternalElement = aValue;
 }
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
