@@ -55,7 +55,7 @@
  */
 
 /**
- * $Id: XMLEntityDecl.cpp,v 1.2 2002/08/22 19:27:41 tng Exp $
+ * $Id: XMLEntityDecl.cpp,v 1.3 2002/11/04 15:00:21 tng Exp $
  */
 
 
@@ -67,6 +67,7 @@
 #include <xercesc/util/XMLString.hpp>
 #include <xercesc/framework/XMLEntityDecl.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  XMLEntityDecl: Constructors and Destructor
@@ -174,3 +175,6 @@ void XMLEntityDecl::cleanUp()
     delete [] fSystemId;
     delete [] fBaseURI;
 }
+
+XERCES_CPP_NAMESPACE_END
+
