@@ -56,6 +56,9 @@
 
  /*
   * $Log: XMLEntityHandler.hpp,v $
+  * Revision 1.5  2002/11/04 15:00:21  tng
+  * C++ Namespace Support.
+  *
   * Revision 1.4  2002/06/06 20:41:51  tng
   * Regression fix: should assign the returned InputSource from resolveEntity to srcToFill.
   *
@@ -103,6 +106,8 @@
 
 #include <xercesc/util/XercesDefs.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 class InputSource;
 class XMLBuffer;
 
@@ -233,4 +238,6 @@ private:
     void operator=(const XMLEntityHandler&);
 };
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
