@@ -56,6 +56,18 @@
 
 /*
  * $Log: XMLFormatter.hpp,v $
+ * Revision 1.7  2003/01/31 00:30:48  jberry
+ * Syntax error in declaration
+ *
+ * Revision 1.6  2003/01/28 18:32:33  peiyongz
+ * Bug#13694: Allow Xerces to write the BOM to XML files
+ *
+ * Revision 1.5  2003/01/24 20:20:22  tng
+ * Add method flush to XMLFormatTarget
+ *
+ * Revision 1.4  2002/11/04 15:00:21  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.3  2002/07/30 16:29:16  tng
  * [Bug 8550] No explanation of XMLFormatter escape options.
  *
@@ -102,6 +114,8 @@
 
 #include <xercesc/util/XercesDefs.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 class XMLFormatTarget;
 class XMLTranscoder;
 
@@ -299,6 +313,10 @@ public:
     (
         const   XMLCh           toFormat
     );
+
+    void writeBOM(const XMLByte* const toFormat
+                , const unsigned int   count);
+
     //@}
 
     // -----------------------------------------------------------------------
@@ -469,6 +487,8 @@ public:
         ,       XMLFormatter* const formatter
     ) = 0;
 
+    virtual void flush() {};
+
 
 protected :
     // -----------------------------------------------------------------------
@@ -519,5 +539,6 @@ inline XMLFormatter& XMLFormatter::operator<<(const UnRepFlags newFlags)
     return *this;
 }
 
+XERCES_CPP_NAMESPACE_END
 
 #endif
