@@ -55,7 +55,7 @@
  */
 
 /**
- *  $Id: XMLRecognizer.cpp,v 1.2 2002/07/16 13:01:12 tng Exp $
+ *  $Id: XMLRecognizer.cpp,v 1.7 2003/02/05 17:30:20 peiyongz Exp $
  */
 
 
@@ -68,6 +68,7 @@
 #include <xercesc/framework/XMLRecognizer.hpp>
 #include <string.h>
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  Local data
@@ -85,6 +86,7 @@ static const XMLCh* gEncodingNameMap[XMLRecognizer::Encodings_Count] =
     , XMLUni::fgUTF8EncodingString
     , XMLUni::fgUTF16BEncodingString
     , XMLUni::fgUTF16LEncodingString
+    , XMLUni::fgXMLChEncodingString
 };
 
 
@@ -147,24 +149,60 @@ XMLRecognizer::basicEncodingProbe(  const   XMLByte* const  rawBuffer
     //
     if (rawByteCount < 2)
         return UTF_8;
+         
+    //  
+    //  We have two to four bytes, so lets check for a UTF-16 BOM. That
+    //  is quick to check and enough to identify two major encodings.   
+    // 
+
+    if (rawByteCount < 4)
+    {
+        if ((rawBuffer[0] == 0xFE) && (rawBuffer[1] == 0xFF))
+            return UTF_16B;
+        else if ((rawBuffer[0] == 0xFF) && (rawBuffer[1] == 0xFE))
+            return UTF_16L;
+        else 
+            return UTF_8;
+    }
+
+    /***
+     *    F.1 Detection Without External Encoding Information
+     *
+     *    Because each XML entity not accompanied by external encoding information and 
+     *    not in UTF-8 or UTF-16 encoding must begin with an XML encoding declaration, 
+     *    in which the first characters must be '<?xml', any conforming processor can detect, 
+     *    after two to four octets of input, which of the following cases apply. 
+     *
+     *    In reading this list, it may help to know that in UCS-4, '<' is "#x0000003C" and 
+     *    '?' is "#x0000003F", and the Byte Order Mark required of UTF-16 data streams is 
+     *    "#xFEFF". The notation ## is used to denote any byte value except that two consecutive 
+     *    ##s cannot be both 00.
+     *
+     *    With a Byte Order Mark:
+     *
+     *    00 00 FE FF           UCS-4,    big-endian machine    (1234 order) 
+     *    FF FE 00 00           UCS-4,    little-endian machine (4321 order) 
+     *    00 00 FF FE           UCS-4,    unusual octet order   (2143) 
+     *    FE FF 00 00           UCS-4,    unusual octet order   (3412) 
+     *    FE FF ## ##           UTF-16,   big-endian 
+     *    FF FE ## ##           UTF-16,   little-endian 
+     *    EF BB BF              UTF-8 
+     *
+     ***/
 
     //
-    //  We know its at least two bytes, so lets check for a UTF-16 BOM. That
-    //  is quick to check and enough to identify two major encodings.
+    //  We have at least four bytes, so we can check all BOM
+    //  for UCS-4BE, UCS-4LE, UTF-16BE and UTF-16LE as well.
     //
-    if ((rawBuffer[0] == 0xFE) && (rawBuffer[1] == 0xFF))
+    if ((rawBuffer[0] == 0x00) && (rawBuffer[1] == 0x00) && (rawBuffer[2] == 0xFE) && (rawBuffer[3] == 0xFF))
+        return UCS_4B;
+    else if ((rawBuffer[0] == 0xFF) && (rawBuffer[1] == 0xFE) && (rawBuffer[2] == 0x00) && (rawBuffer[3] == 0x00))
+        return UCS_4L;
+    else if ((rawBuffer[0] == 0xFE) && (rawBuffer[1] == 0xFF))
         return UTF_16B;
     else if ((rawBuffer[0] == 0xFF) && (rawBuffer[1] == 0xFE))
         return UTF_16L;
 
-    //
-    //  Oh well, not one of those. So now lets see if we have at least 4
-    //  bytes. If not, then we are out of ideas and can return UTF-8 as the
-    //  fallback.
-    //
-    if (rawByteCount < 4)
-        return UTF_8;
-
     //
     //  We have at least 4 bytes. So lets check the 4 byte sequences that
     //  indicate other UTF-16 and UCS encodings.
@@ -203,46 +241,46 @@ XMLRecognizer::Encodings
 XMLRecognizer::encodingForName(const XMLCh* const encName)
 {
     //
-    //  Compare the passed string, case insensitively, to the variations
-    //  that we recognize.
+    //  Compare the passed string, assume input string is already uppercased,
+    //  to the variations that we recognize.
     //
     //  !!NOTE: Note that we don't handle EBCDIC here because we don't handle
     //  that one ourselves. It is allowed to fall into 'other'.
     //
    if (encName == XMLUni::fgXMLChEncodingString ||
-        !XMLString::compareIString(encName, XMLUni::fgXMLChEncodingString))
+        !XMLString::compareString(encName, XMLUni::fgXMLChEncodingString))
    {
-        return XMLRecognizer::OtherEncoding;
+        return XMLRecognizer::XERCES_XMLCH;
    }
-   else if (!XMLString::compareIString(encName, XMLUni::fgUTF8EncodingString)
-    ||  !XMLString::compareIString(encName, XMLUni::fgUTF8EncodingString2))
+   else if (!XMLString::compareString(encName, XMLUni::fgUTF8EncodingString)
+    ||  !XMLString::compareString(encName, XMLUni::fgUTF8EncodingString2))
     {
         return XMLRecognizer::UTF_8;
     }
-     else if (!XMLString::compareIString(encName, XMLUni::fgUSASCIIEncodingString)
-          ||  !XMLString::compareIString(encName, XMLUni::fgUSASCIIEncodingString2)
-          ||  !XMLString::compareIString(encName, XMLUni::fgUSASCIIEncodingString3)
-          ||  !XMLString::compareIString(encName, XMLUni::fgUSASCIIEncodingString4))
+     else if (!XMLString::compareString(encName, XMLUni::fgUSASCIIEncodingString)
+          ||  !XMLString::compareString(encName, XMLUni::fgUSASCIIEncodingString2)
+          ||  !XMLString::compareString(encName, XMLUni::fgUSASCIIEncodingString3)
+          ||  !XMLString::compareString(encName, XMLUni::fgUSASCIIEncodingString4))
     {
         return XMLRecognizer::US_ASCII;
     }
-     else if (!XMLString::compareIString(encName, XMLUni::fgUTF16LEncodingString)
-          ||  !XMLString::compareIString(encName, XMLUni::fgUTF16LEncodingString2))
+     else if (!XMLString::compareString(encName, XMLUni::fgUTF16LEncodingString)
+          ||  !XMLString::compareString(encName, XMLUni::fgUTF16LEncodingString2))
     {
         return XMLRecognizer::UTF_16L;
     }
-     else if (!XMLString::compareIString(encName, XMLUni::fgUTF16BEncodingString)
-          ||  !XMLString::compareIString(encName, XMLUni::fgUTF16BEncodingString2))
+     else if (!XMLString::compareString(encName, XMLUni::fgUTF16BEncodingString)
+          ||  !XMLString::compareString(encName, XMLUni::fgUTF16BEncodingString2))
     {
         return XMLRecognizer::UTF_16B;
     }
-     else if (!XMLString::compareIString(encName, XMLUni::fgUCS4LEncodingString)
-          ||  !XMLString::compareIString(encName, XMLUni::fgUCS4LEncodingString2))
+     else if (!XMLString::compareString(encName, XMLUni::fgUCS4LEncodingString)
+          ||  !XMLString::compareString(encName, XMLUni::fgUCS4LEncodingString2))
     {
         return XMLRecognizer::UCS_4L;
     }
-     else if (!XMLString::compareIString(encName, XMLUni::fgUCS4BEncodingString)
-          ||  !XMLString::compareIString(encName, XMLUni::fgUCS4BEncodingString2))
+     else if (!XMLString::compareString(encName, XMLUni::fgUCS4BEncodingString)
+          ||  !XMLString::compareString(encName, XMLUni::fgUCS4BEncodingString2))
     {
         return XMLRecognizer::UCS_4B;
     }
@@ -260,3 +298,5 @@ XMLRecognizer::nameForEncoding(const XMLRecognizer::Encodings theEncoding)
 
     return gEncodingNameMap[theEncoding];
 }
+
+XERCES_CPP_NAMESPACE_END
