@@ -56,6 +56,15 @@
 
  /*
   * $Log: XMLValidator.hpp,v $
+  * Revision 1.7  2002/11/07 21:59:22  tng
+  * Pass elemDecl to XMLValidator::validateAttrValue so that we can include element name in error message.
+  *
+  * Revision 1.6  2002/11/04 15:00:21  tng
+  * C++ Namespace Support.
+  *
+  * Revision 1.5  2002/09/04 18:17:55  tng
+  * Do not set IDREF to used during prevalidation.
+  *
   * Revision 1.4  2002/08/20 16:54:11  tng
   * [Bug 6251] Info during compilation.
   *
@@ -140,6 +149,8 @@
 #include <xercesc/framework/XMLValidityCodes.hpp>
 #include <xercesc/framework/XMLRefInfo.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 class ReaderMgr;
 class XMLBuffer;
 class XMLBufferMgr;
@@ -277,6 +288,8 @@ public:
     (
         const   XMLAttDef*                  attDef
         , const XMLCh* const                attrValue
+        , bool                              preValidation = false
+        , const XMLElementDecl*             elemDecl = 0
     ) = 0;
 
     /**
@@ -543,5 +556,6 @@ inline XMLScanner* XMLValidator::getScanner()
     return fScanner;
 }
 
+XERCES_CPP_NAMESPACE_END
 
 #endif
