@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: ReaderMgr.cpp,v 1.2 2002/02/28 21:55:57 tng Exp $
+ * $Id: ReaderMgr.cpp,v 1.11 2003/01/06 19:44:57 tng Exp $
  */
 
 // ---------------------------------------------------------------------------
@@ -79,6 +79,8 @@
 #include <xercesc/internal/EndOfEntityException.hpp>
 #include <xercesc/internal/ReaderMgr.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 // ---------------------------------------------------------------------------
 //  ReaderMgr: Constructors and Destructor
 // ---------------------------------------------------------------------------
@@ -91,6 +93,8 @@ ReaderMgr::ReaderMgr() :
     , fNextReaderNum(1)
     , fReaderStack(0)
     , fThrowEOE(false)
+    , fXMLVersion(XMLReader::XMLV1_0)
+    , fStandardUriConformant(false)
 {
 }
 
@@ -351,7 +355,7 @@ XMLCh ReaderMgr::skipUntilInOrWS(const XMLCh* const listToSkip)
         if (!nextCh)
             break;
 
-        if (XMLReader::isWhitespace(nextCh))
+        if (fCurReader->isWhitespace(nextCh))
             break;
 
         if (XMLString::indexOf(listToSkip, nextCh) != -1)
@@ -409,7 +413,8 @@ XMLReader* ReaderMgr::createReader( const   InputSource&        src
                                     , const bool                xmlDecl
                                     , const XMLReader::RefFrom  refFrom
                                     , const XMLReader::Types    type
-                                    , const XMLReader::Sources  source)
+                                    , const XMLReader::Sources  source
+                                    , const bool                calcSrcOfs)
 {
     //
     //  Ask the input source to create us an input stream. The particular
@@ -445,6 +450,9 @@ XMLReader* ReaderMgr::createReader( const   InputSource&        src
                 , refFrom
                 , type
                 , source
+                , false
+                , calcSrcOfs
+                , fXMLVersion
                 );
         }
         else
@@ -457,6 +465,9 @@ XMLReader* ReaderMgr::createReader( const   InputSource&        src
                 , refFrom
                 , type
                 , source
+                , false
+                , calcSrcOfs
+                , fXMLVersion
                 );
         }
     }
@@ -485,7 +496,8 @@ XMLReader* ReaderMgr::createReader( const   XMLCh* const        sysId
                                     , const XMLReader::RefFrom  refFrom
                                     , const XMLReader::Types    type
                                     , const XMLReader::Sources  source
-                                    ,       InputSource*&       srcToFill)
+                                    ,       InputSource*&       srcToFill
+                                    , const bool                calcSrcOfs)
 {
     // Create a buffer for expanding the system id
     XMLBuffer expSysId;
@@ -535,17 +547,24 @@ XMLReader* ReaderMgr::createReader( const   XMLCh* const        sysId
                     , XMLExcepts::URL_NoProtocolPresent
                 );
             }
-            srcToFill = new URLInputSource(urlTmp);
+            else {
+                if (fStandardUriConformant && urlTmp.hasInvalidChar())
+                    ThrowXML(MalformedURLException, XMLExcepts::URL_MalformedURL);
+                srcToFill = new URLInputSource(urlTmp);
+            }
         }
 
-        catch(const MalformedURLException&)
+        catch(const MalformedURLException& e)
         {
-            // Its not a URL, so lets assume its a local file name.
-            srcToFill = new LocalFileInputSource
-            (
-                lastInfo.systemId
-                , expSysId.getRawBuffer()
-            );
+            // Its not a URL, so lets assume its a local file name if non-standard uri is allowed
+            if (!fStandardUriConformant)
+                srcToFill = new LocalFileInputSource
+                (
+                    lastInfo.systemId
+                    , expSysId.getRawBuffer()
+                );
+            else
+                throw e;
         }
     }
 
@@ -563,6 +582,117 @@ XMLReader* ReaderMgr::createReader( const   XMLCh* const        sysId
         , refFrom
         , type
         , source
+        , calcSrcOfs
+    );
+
+    // Either way, we can release the input source now
+    janSrc.orphan();
+
+    // If it failed for any reason, then return zero.
+    if (!retVal)
+        return 0;
+
+    // Give this reader the next available reader number and return it
+    retVal->setReaderNum(fNextReaderNum++);
+    return retVal;
+}
+
+
+XMLReader* ReaderMgr::createReader( const   XMLCh* const        baseURI
+                                    , const XMLCh* const        sysId
+                                    , const XMLCh* const        pubId
+                                    , const bool                xmlDecl
+                                    , const XMLReader::RefFrom  refFrom
+                                    , const XMLReader::Types    type
+                                    , const XMLReader::Sources  source
+                                    ,       InputSource*&       srcToFill
+                                    , const bool                calcSrcOfs)
+{
+    // Create a buffer for expanding the system id
+    XMLBuffer expSysId;
+
+    //
+    //  Allow the entity handler to expand the system id if they choose
+    //  to do so.
+    //
+    if (fEntityHandler)
+    {
+        if (!fEntityHandler->expandSystemId(sysId, expSysId))
+            expSysId.set(sysId);
+    }
+     else
+    {
+        expSysId.set(sysId);
+    }
+
+    // Call the entity resolver interface to get an input source
+    srcToFill = 0;
+    if (fEntityHandler)
+    {
+        srcToFill = fEntityHandler->resolveEntity
+        (
+            pubId
+            , expSysId.getRawBuffer()
+        );
+    }
+
+    //
+    //  If they didn't create a source via the entity resolver, then we
+    //  have to create one on our own.
+    //
+    if (!srcToFill)
+    {
+        LastExtEntityInfo lastInfo;
+        getLastExtEntityInfo(lastInfo);
+
+        try
+        {
+            XMLURL urlTmp((!baseURI || !*baseURI) ? lastInfo.systemId : baseURI, expSysId.getRawBuffer());
+
+            if (urlTmp.isRelative())
+            {
+                ThrowXML
+                (
+                    MalformedURLException
+                    , XMLExcepts::URL_NoProtocolPresent
+                );
+            }
+            else {
+                if (fStandardUriConformant && urlTmp.hasInvalidChar())
+                    ThrowXML(MalformedURLException, XMLExcepts::URL_MalformedURL);
+                srcToFill = new URLInputSource(urlTmp);
+            }
+        }
+
+        catch(const MalformedURLException& e)
+        {
+            // Its not a URL, so lets assume its a local file name if non-standard uri is allowed
+            if (!fStandardUriConformant)
+                srcToFill = new LocalFileInputSource
+                (
+                    lastInfo.systemId
+                    , expSysId.getRawBuffer()
+                );
+            else
+                throw e;
+        }
+    }
+
+    // Put a janitor on the input source
+    Janitor<InputSource> janSrc(srcToFill);
+
+    //
+    //  Now call the other version with the input source that we have, and
+    //  return the resulting reader.
+    //
+    XMLReader* retVal = createReader
+    (
+        *srcToFill
+        , xmlDecl
+        , refFrom
+        , type
+        , source
+        , calcSrcOfs
     );
 
     // Either way, we can release the input source now
@@ -584,7 +714,8 @@ ReaderMgr::createIntEntReader(  const   XMLCh* const        sysId
                                 , const XMLReader::Types    type
                                 , const XMLCh* const        dataBuf
                                 , const unsigned int        dataLen
-                                , const bool                copyBuf)
+                                , const bool                copyBuf
+                                , const bool                calcSrcOfs)
 {
     //
     //  This one is easy, we just create an input stream for the data and
@@ -609,10 +740,13 @@ ReaderMgr::createIntEntReader(  const   XMLCh* const        sysId
         sysId
         , 0
         , newStream
-        , XMLUni::fgXMLChEncodingString
+        , XMLRecognizer::XERCES_XMLCH
         , refFrom
         , type
         , XMLReader::Source_Internal
+        , false
+        , calcSrcOfs
+        , fXMLVersion
     );
 
     // If it failed for any reason, then return zero.
@@ -744,7 +878,7 @@ bool ReaderMgr::pushReader(         XMLReader* const        reader
             const XMLEntityDecl* curDecl = fEntityStack->elementAt(index);
             if (curDecl)
             {
-                if (!XMLString::compareString(theName, curDecl->getName()))
+                if (XMLString::equals(theName, curDecl->getName()))
                 {
                     // Oops, already there so delete reader and return
                     delete reader;
@@ -829,7 +963,7 @@ const XMLCh* ReaderMgr::getSystemId() const
     return getLastExtEntity(theEntity)->getSystemId();
 }
 
-int ReaderMgr::getColumnNumber() const
+XMLSSize_t ReaderMgr::getColumnNumber() const
 {
     if (!fReaderStack && !fCurReader)
         return 0;
@@ -838,7 +972,7 @@ int ReaderMgr::getColumnNumber() const
     return getLastExtEntity(theEntity)->getColumnNumber();
 }
 
-int ReaderMgr::getLineNumber() const
+XMLSSize_t ReaderMgr::getLineNumber() const
 {
     if (!fReaderStack && !fCurReader)
         return 0;
@@ -973,3 +1107,5 @@ bool ReaderMgr::popReader()
     }
     return true;
 }
+
+XERCES_CPP_NAMESPACE_END
