@@ -56,6 +56,18 @@
 
 /*
  * $Log: ReaderMgr.hpp,v $
+ * Revision 1.7  2003/01/03 20:08:39  tng
+ * New feature StandardUriConformant to force strict standard uri conformance.
+ *
+ * Revision 1.6  2002/12/20 22:09:56  tng
+ * XML 1.1
+ *
+ * Revision 1.5  2002/12/03 15:31:19  knoaman
+ * Enable/disable calculation of src offset.
+ *
+ * Revision 1.4  2002/11/04 14:58:18  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.3  2002/08/22 19:28:11  tng
  * [Bug 11448] DomCount has problems with XHTML1.1 DTD.
  *
@@ -133,6 +145,8 @@
 #include <xercesc/framework/XMLBuffer.hpp>
 #include <xercesc/internal/XMLReader.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 class XMLBuffer;
 class XMLEntityDecl;
 class XMLEntityHandler;
@@ -213,6 +227,7 @@ public :
         , const XMLReader::RefFrom  refFrom
         , const XMLReader::Types    type
         , const XMLReader::Sources  source
+        , const bool                calcSrsOfs = true
     );
     XMLReader* createReader
     (
@@ -223,6 +238,7 @@ public :
         , const XMLReader::Types    type
         , const XMLReader::Sources  source
         ,       InputSource*&       srcToFill
+        , const bool                calcSrcOfs = true
     );
     XMLReader* createReader
     (
@@ -234,6 +250,7 @@ public :
         , const XMLReader::Types    type
         , const XMLReader::Sources  source
         ,       InputSource*&       srcToFill
+        , const bool                calcSrcOfs = true
     );
     XMLReader* createIntEntReader
     (
@@ -243,6 +260,7 @@ public :
         , const XMLCh* const        dataBuf
         , const unsigned int        dataLen
         , const bool                copyBuf
+        , const bool                calcSrcOfs = true
     );
     bool isScanningPERefOutOfLiteral() const;
     bool pushReader
@@ -273,7 +291,8 @@ public :
     // -----------------------------------------------------------------------
     void setEntityHandler(XMLEntityHandler* const newHandler);
     void setThrowEOE(const bool newValue);
-
+    void setXMLVersion(const XMLReader::XMLVersion version);
+    void setStandardUriConformant(const bool newValue);
 
     // -----------------------------------------------------------------------
     //  Implement the SAX Locator interface
@@ -328,6 +347,13 @@ private :
     //      end of entity. The scanner doesn't really need to know about ends
     //      of entities in the int/ext subsets, so it will turn this flag off
     //      until it gets into the content usually.
+    //
+    //  fXMLVersion
+    //      Enum to indicate if each Reader should be created as XML 1.1 or
+    //      XML 1.0 conformant
+    //
+    //  fStandardUriConformant
+    //      This flag controls whether we force conformant URI
     // -----------------------------------------------------------------------
     XMLEntityDecl*              fCurEntity;
     XMLReader*                  fCurReader;
@@ -336,6 +362,8 @@ private :
     unsigned int                fNextReaderNum;
     RefStackOf<XMLReader>*      fReaderStack;
     bool                        fThrowEOE;
+    XMLReader::XMLVersion       fXMLVersion;
+    bool                        fStandardUriConformant;
 };
 
 
@@ -391,7 +419,8 @@ inline bool ReaderMgr::lookingAtChar(const XMLCh chToCheck)
 
 inline bool ReaderMgr::lookingAtSpace()
 {
-    return XMLReader::isWhitespace(peekNextChar());
+    XMLCh c = peekNextChar();
+    return fCurReader->isWhitespace(c);
 }
 
 inline void ReaderMgr::setThrowEOE(const bool newValue)
@@ -399,6 +428,11 @@ inline void ReaderMgr::setThrowEOE(const bool newValue)
     fThrowEOE = newValue;
 }
 
+inline void ReaderMgr::setStandardUriConformant(const bool newValue)
+{
+    fStandardUriConformant = newValue;
+}
+
 inline bool ReaderMgr::skippedString(const XMLCh* const toSkip)
 {
     return fCurReader->skippedString(toSkip);
@@ -439,6 +473,11 @@ inline void ReaderMgr::setEntityHandler(XMLEntityHandler* const newHandler)
     fEntityHandler = newHandler;
 }
 
+inline void ReaderMgr::setXMLVersion(const XMLReader::XMLVersion version)
+{
+    fXMLVersion = version;
+    fCurReader->setXMLVersion(version);
+}
 
 //
 //  This is a simple class to temporarily change the 'throw at end of entity'
@@ -480,4 +519,6 @@ private :
     ReaderMgr*  fMgr;
 };
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
