@@ -56,6 +56,38 @@
 
 /*
  * $Log: XMLReader.hpp,v $
+ * Revision 1.12  2003/01/27 16:50:27  knoaman
+ * some cleanup.
+ *
+ * Revision 1.11  2002/12/20 22:09:56  tng
+ * XML 1.1
+ *
+ * Revision 1.10  2002/12/11 22:09:08  knoaman
+ * Performance: reduce instructions count.
+ *
+ * Revision 1.9  2002/12/03 15:31:19  knoaman
+ * Enable/disable calculation of src offset.
+ *
+ * Revision 1.8  2002/12/02 17:20:05  knoaman
+ * Remove unused data member.
+ *
+ * Revision 1.7  2002/11/28 19:19:12  knoaman
+ * Performance: remove unnecessary if condition.
+ *
+ * Revision 1.6  2002/11/28 18:17:22  knoaman
+ * Performance: make getNextChar/peekNextChar inline.
+ *
+ * Revision 1.5  2002/11/25 21:31:08  tng
+ * Performance:
+ * 1. use XMLRecognizer::Encodings enum to make new transcode, faster than comparing the encoding string every time.
+ * 2. Pre uppercase the encodingString before calling encodingForName to avoid calling compareIString
+ *
+ * Revision 1.4  2002/11/04 14:58:19  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.3  2002/09/27 12:56:23  tng
+ * [Bug 12740] Extra include.  By Peter Volchek.
+ *
  * Revision 1.2  2002/05/27 18:42:14  tng
  * To get ready for 64 bit large file, use XMLSSize_t to represent line and column number.
  *
@@ -134,9 +166,11 @@
 #define XMLREADER_HPP
 
 #include <xercesc/util/XMLUniDefs.hpp>
+#include <xercesc/util/XMLChar.hpp>
 #include <xercesc/framework/XMLRecognizer.hpp>
 #include <xercesc/framework/XMLBuffer.hpp>
-#include <xercesc/util/XMLUniDefs.hpp>
+
+XERCES_CPP_NAMESPACE_BEGIN
 
 class InputSource;
 class BinInputStream;
@@ -146,17 +180,6 @@ class XMLScanner;
 class XMLTranscoder;
 
 
-// Masks for the fgCharCharsTable array
-const XMLByte   gBaseCharMask               = 0x1;
-const XMLByte   gSpecialCharDataMask        = 0x2;
-const XMLByte   gNameCharMask               = 0x4;
-const XMLByte   gPlainContentCharMask       = 0x8;
-const XMLByte   gSpecialStartTagCharMask    = 0x10;
-const XMLByte   gLetterCharMask             = 0x20;
-const XMLByte   gXMLCharMask                = 0x40;
-const XMLByte   gWhitespaceCharMask         = 0x80;
-
-
 // ---------------------------------------------------------------------------
 //  Instances of this class are used to manage the content of entities. The
 //  scanner maintains a stack of these, one for each entity (this means entity
@@ -195,39 +218,39 @@ public:
         , RefFrom_NonLiteral
     };
 
+    enum XMLVersion
+    {
+        XMLV1_0
+        , XMLV1_1
+        , XMLV_Unknown
+    };
+
 
     // -----------------------------------------------------------------------
-    //  Public, static methods
+    //  Public, query methods
     // -----------------------------------------------------------------------
-    static bool isAllSpaces
+    bool isAllSpaces
     (
         const   XMLCh* const    toCheck
         , const unsigned int    count
     );
 
-    static bool containsWhiteSpace
+    bool containsWhiteSpace
     (
         const   XMLCh* const    toCheck
         , const unsigned int    count
     );
 
 
-    static bool isBaseChar(const XMLCh toCheck);
-    static bool isFirstNameChar(const XMLCh toCheck);
-    static bool isNameChar(const XMLCh toCheck);
-    static bool isPlainContentChar(const XMLCh toCheck);
-    static bool isPublicIdChar(const XMLCh toCheck);
-    static bool isSpecialCharDataChar(const XMLCh toCheck);
-    static bool isSpecialStartTagChar(const XMLCh toCheck);
-    static bool isXMLLetter(const XMLCh toCheck);
-    static bool isXMLChar(const XMLCh toCheck);
-    static bool isWhitespace(const XMLCh toCheck);
-
-    /**
-      * Return the value of fgNEL flag.
-      */
-    static bool isNELRecognized();
-
+    bool isXMLLetter(const XMLCh toCheck);
+    bool isFirstNameChar(const XMLCh toCheck);
+    bool isNameChar(const XMLCh toCheck);
+    bool isPlainContentChar(const XMLCh toCheck);
+    bool isSpecialStartTagChar(const XMLCh toCheck);
+    bool isXMLChar(const XMLCh toCheck);
+    bool isWhitespace(const XMLCh toCheck);
+    bool isControlChar(const XMLCh toCheck);
+    bool isPublicIdChar(const XMLCh toCheck);
 
     // -----------------------------------------------------------------------
     //  Constructors and Destructor
@@ -241,6 +264,8 @@ public:
         , const Types                       type
         , const Sources                     source
         , const bool                        throwAtEnd = false
+        , const bool                        calculateSrcOfs = true
+        , const XMLVersion                  xmlVersion = XMLV1_0
     );
 
     XMLReader
@@ -253,6 +278,22 @@ public:
         , const Types                       type
         , const Sources                     source
         , const bool                        throwAtEnd = false
+        , const bool                        calculateSrcOfs = true
+        , const XMLVersion                  xmlVersion = XMLV1_0
+    );
+
+    XMLReader
+    (
+        const   XMLCh* const                pubId
+        , const XMLCh* const                sysId
+        ,       BinInputStream* const       streamToAdopt
+        , XMLRecognizer::Encodings          encodingEnum
+        , const RefFrom                     from
+        , const Types                       type
+        , const Sources                     source
+        , const bool                        throwAtEnd = false
+        , const bool                        calculateSrcOfs = true
+        , const XMLVersion                  xmlVersion = XMLV1_0
     );
 
     ~XMLReader();
@@ -309,6 +350,7 @@ public:
     );
     void setReaderNum(const unsigned int newNum);
     void setThrowAtEnd(const bool newValue);
+    void setXMLVersion(const XMLVersion version);
 
 
 private:
@@ -335,21 +377,6 @@ private:
     };
 
 
-    // -----------------------------------------------------------------------
-    //  Private static methods
-    // -----------------------------------------------------------------------
-    static bool checkTable
-    (
-        const   XMLCh* const    theTable
-        , const XMLCh           toCheck
-    );
-
-
-    /**
-      * Method to enable NEL char to be treated as white space char.
-      */
-    static void enableNELWS();
-
     // -----------------------------------------------------------------------
     //  Private helper methods
     // -----------------------------------------------------------------------
@@ -510,6 +537,15 @@ private:
     //      Indicates whether this reader represents a PE or not. If this
     //      flag is true and the fInLiteral flag is false, then we will put
     //      out an extra space at the end.
+    //
+    //  fgCharCharsTable;
+    //      Pointer to XMLChar table, depends on XML version
+    //
+    //  fNEL
+    //      Boolean indicates if NEL and LSEP should be recognized as NEL
+    //
+    //  fXMLVersion
+    //      Enum to indicate if this Reader is conforming to XML 1.0 or XML 1.1
     // -----------------------------------------------------------------------
     unsigned int                fCharIndex;
     XMLCh                       fCharBuf[kCharBufSize];
@@ -529,44 +565,24 @@ private:
     RefFrom                     fRefFrom;
     bool                        fSentTrailingSpace;
     Sources                     fSource;
-    XMLCh                       fSpareCh;
     unsigned int                fSrcOfsBase;
     bool                        fSrcOfsSupported;
+    bool                        fCalculateSrcOfs;
     XMLCh*                      fSystemId;
     BinInputStream*             fStream;
     bool                        fSwapped;
     bool                        fThrowAtEnd;
     XMLTranscoder*              fTranscoder;
     Types                       fType;
-
-
-    // -----------------------------------------------------------------------
-    //  Static data members
-    //
-    //  fgCharCharsTable
-    //      The character characteristics table. Bits in each byte, represent
-    //      the characteristics of each character. It is generated via some
-    //      code and then hard coded into the cpp file for speed.
-    //
-    //  fNEL
-    //      Flag to respresents whether NEL whitespace recognition is enabled
-    //      or disabled
-    // -----------------------------------------------------------------------
-    static XMLByte  fgCharCharsTable[0x10000];
-    static bool     fNEL;
-
-    friend class XMLPlatformUtils;
+    XMLByte*                    fgCharCharsTable;
+    bool                        fNEL;
+    XMLVersion                  fXMLVersion;
 };
 
 
 // ---------------------------------------------------------------------------
-//  XMLReader: Public, static methods
+//  XMLReader: Public, query methods
 // ---------------------------------------------------------------------------
-inline bool XMLReader::isBaseChar(const XMLCh toCheck)
-{
-    return ((fgCharCharsTable[toCheck] & gBaseCharMask) != 0);
-}
-
 inline bool XMLReader::isNameChar(const XMLCh toCheck)
 {
     return ((fgCharCharsTable[toCheck] & gNameCharMask) != 0);
@@ -578,9 +594,9 @@ inline bool XMLReader::isPlainContentChar(const XMLCh toCheck)
 }
 
 
-inline bool XMLReader::isSpecialCharDataChar(const XMLCh toCheck)
+inline bool XMLReader::isFirstNameChar(const XMLCh toCheck)
 {
-    return ((fgCharCharsTable[toCheck] & gSpecialCharDataMask) != 0);
+    return ((fgCharCharsTable[toCheck] & gFirstNameCharMask) != 0);
 }
 
 inline bool XMLReader::isSpecialStartTagChar(const XMLCh toCheck)
@@ -595,8 +611,7 @@ inline bool XMLReader::isXMLChar(const XMLCh toCheck)
 
 inline bool XMLReader::isXMLLetter(const XMLCh toCheck)
 {
-    const XMLByte ourMask = gBaseCharMask | gLetterCharMask;
-    return ((fgCharCharsTable[toCheck] & ourMask) != 0);
+    return ((fgCharCharsTable[toCheck] & gLetterCharMask) != 0);
 }
 
 inline bool XMLReader::isWhitespace(const XMLCh toCheck)
@@ -604,6 +619,11 @@ inline bool XMLReader::isWhitespace(const XMLCh toCheck)
     return ((fgCharCharsTable[toCheck] & gWhitespaceCharMask) != 0);
 }
 
+inline bool XMLReader::isControlChar(const XMLCh toCheck)
+{
+    return ((fgCharCharsTable[toCheck] & gControlCharMask) != 0);
+}
+
 // ---------------------------------------------------------------------------
 //  XMLReader: Buffer management methods
 // ---------------------------------------------------------------------------
@@ -671,11 +691,6 @@ inline XMLReader::Types XMLReader::getType() const
     return fType;
 }
 
-inline bool XMLReader::isNELRecognized() {
-
-    return fNEL;
-}
-
 // ---------------------------------------------------------------------------
 //  XMLReader: Setter methods
 // ---------------------------------------------------------------------------
@@ -689,6 +704,20 @@ inline void XMLReader::setThrowAtEnd(const bool newValue)
     fThrowAtEnd = newValue;
 }
 
+inline void XMLReader::setXMLVersion(const XMLVersion version)
+{
+    fXMLVersion = version;
+    if (version == XMLV1_1) {
+        fNEL = true;
+        fgCharCharsTable = XMLChar1_1::fgCharCharsTable1_1;
+    }
+    else {
+        fNEL = XMLChar1_0::enableNEL;
+        fgCharCharsTable = XMLChar1_0::fgCharCharsTable1_0;
+    }
+
+}
+
 
 
 // ---------------------------------------------------------------------------
@@ -703,30 +732,23 @@ inline void XMLReader::setThrowAtEnd(const bool newValue)
 // ---------------------------------------------------------------------------
 inline void XMLReader::movePlainContentChars(XMLBuffer &dest)
 {
-    int count = 0;
-    XMLCh *pStart = &fCharBuf[fCharIndex];
-    XMLCh *pCurrent = pStart;
-    XMLCh *pEnd     = &fCharBuf[fCharsAvail];
+    unsigned int count = fCharIndex;
 
-
-    while (pCurrent < pEnd)
+    while (fCharIndex < fCharsAvail)
     {
-        if (! XMLReader::isPlainContentChar(*pCurrent++))
+        if (!isPlainContentChar(fCharBuf[fCharIndex]))
             break;
-        count++;
+        fCharIndex++;
     }
 
-    if (count > 0)
+    if (count != fCharIndex)
     {
-        fCharIndex += count;
-        fCurCol    += count;
-        dest.append(pStart, count);
+        fCurCol    += (fCharIndex - count);
+        dest.append(&fCharBuf[count], fCharIndex - count);
     }
 }
 
 
-
-
 // ---------------------------------------------------------------------------
 //  XMLReader: getNextCharIfNot() method inlined for speed
 // ---------------------------------------------------------------------------
@@ -736,40 +758,24 @@ inline bool XMLReader::getNextCharIfNot(const XMLCh chNotToGet, XMLCh& chGotten)
     //  See if there is at least a char in the buffer. Else, do the buffer
     //  reload logic.
     //
-    if (fCharIndex < fCharsAvail)
-    {
-        // Check the next char
-        if (fCharBuf[fCharIndex] == chNotToGet)
-            return false;
-
-        // Its not the one we want to skip so bump the index
-        chGotten = fCharBuf[fCharIndex++];
-    }
-     else
+    if (fCharIndex >= fCharsAvail)
     {
         // If fNoMore is set, then we have nothing else to give
         if (fNoMore)
             return false;
 
-        // If the buffer is empty, then try to refresh
-        if (fCharIndex == fCharsAvail)
-        {
-            if (!refreshCharBuffer())
-            {
-                // If still empty, then return false
-                if (fCharIndex == fCharsAvail)
-                    return false;
-            }
-        }
-
-        // Check the next char
-        if (fCharBuf[fCharIndex] == chNotToGet)
+        // Try to refresh
+        if (!refreshCharBuffer())
             return false;
-
-        // Its not the one we want to skip so bump the index
-        chGotten = fCharBuf[fCharIndex++];
     }
 
+    // Check the next char
+    if (fCharBuf[fCharIndex] == chNotToGet)
+        return false;
+
+    // Its not the one we want to skip so bump the index
+    chGotten = fCharBuf[fCharIndex++];
+
     // Handle end of line normalization and line/col member maintenance.
     if (chGotten == chCR)
     {
@@ -812,7 +818,7 @@ inline bool XMLReader::getNextCharIfNot(const XMLCh chNotToGet, XMLCh& chGotten)
         fCurLine++;
     }
      else if (chGotten == chLF
-              || ((chGotten == chNEL) && fNEL))
+              || ((chGotten == chNEL || chGotten == chLineSeparator) && fNEL))
     {
         chGotten = chLF;
         fCurLine++;
@@ -830,4 +836,114 @@ inline bool XMLReader::getNextCharIfNot(const XMLCh chNotToGet, XMLCh& chGotten)
     return true;
 }
 
+// ---------------------------------------------------------------------------
+//  XMLReader: getNextChar() method inlined for speed
+// ---------------------------------------------------------------------------
+inline bool XMLReader::getNextChar(XMLCh& chGotten)
+{
+    //
+    //  See if there is at least a char in the buffer. Else, do the buffer
+    //  reload logic.
+    //
+    if (fCharIndex >= fCharsAvail)
+    {
+        // If fNoMore is set, then we have nothing else to give
+        if (fNoMore)
+            return false;
+
+        // Try to refresh
+        if (!refreshCharBuffer())
+            return false;
+    }
+
+    chGotten = fCharBuf[fCharIndex++];
+
+    // Handle end of line normalization and line/col member maintenance.
+    if (chGotten == chCR)
+    {
+        //
+        //  Do the normalization. We return chLF regardless of which was
+        //  found. We also eat a chCR followed by an chLF.
+        //
+        //  We only do this if the content being spooled is not already
+        //  internalized.
+        //
+        if (fSource == Source_External)
+        {
+            //
+            //  See if we have another char left. If not, don't bother.
+            //  Else, see if its an chLF to eat. If it is, bump the
+            //  index again.
+            //
+            if ((fCharIndex < fCharsAvail) || refreshCharBuffer())
+            {
+                if (fCharBuf[fCharIndex] == chLF
+                    || ((fCharBuf[fCharIndex] == chNEL) && fNEL))
+                    fCharIndex++;
+            }
+
+            // And return just an chLF
+            chGotten = chLF;
+        }
+
+        // And handle the line/col stuff
+        fCurCol = 1;
+        fCurLine++;
+    }
+     else if (chGotten == chLF
+              || ((chGotten == chNEL || chGotten == chLineSeparator) && fNEL))
+    {
+        chGotten = chLF;
+        fCurLine++;
+        fCurCol = 1;
+    }
+     else if (chGotten)
+    {
+        //
+        //  Only do this is not a null char. Null chars are not part of the
+        //  real content. They are just marker characters inserted into
+        //  the stream.
+        //
+        fCurCol++;
+    }
+    return true;
+}
+
+
+// ---------------------------------------------------------------------------
+//  XMLReader: peekNextChar() method inlined for speed
+// ---------------------------------------------------------------------------
+inline bool XMLReader::peekNextChar(XMLCh& chGotten)
+{
+    //
+    //  If there is something still in the buffer, get it. Else do the reload
+    //  scenario.
+    //
+    if (fCharIndex >= fCharsAvail)
+    {
+        // Try to refresh the buffer
+        if (!refreshCharBuffer())
+        {
+            chGotten = chNull;
+            return false;
+        }
+    }
+
+    chGotten = fCharBuf[fCharIndex];
+
+    //
+    //  Even though we are only peeking, we have to act the same as the
+    //  normal char get method in regards to newline normalization, though
+    //  its not as complicated as the actual character getting method's.
+    //
+    if ((chGotten == chCR || ((chGotten == chNEL || chGotten == chLineSeparator) && fNEL))
+        && (fSource == Source_External))
+        chGotten = chLF;
+
+    return true;
+}
+
+
+XERCES_CPP_NAMESPACE_END
+
 #endif
