@@ -55,13 +55,12 @@
  */
 
 /*
- * $Id: AbstractDOMParser.hpp,v 1.11 2002/08/14 15:20:38 knoaman Exp $
+ * $Id: AbstractDOMParser.hpp,v 1.16 2003/01/03 20:09:36 tng Exp $
  *
  */
 #if !defined(ABSTRACTDOMPARSER_HPP)
 #define ABSTRACTDOMPARSER_HPP
 
-
 #include <xercesc/dom/DOMDocument.hpp>
 #include <xercesc/framework/XMLDocumentHandler.hpp>
 #include <xercesc/framework/XMLErrorReporter.hpp>
@@ -72,12 +71,16 @@
 #include <xercesc/validators/DTD/DTDElementDecl.hpp>
 #include <xercesc/framework/XMLBufferMgr.hpp>
 
+
+XERCES_CPP_NAMESPACE_BEGIN
+
 class XMLPScanToken;
 class XMLScanner;
 class XMLValidator;
 class DOMDocumentImpl;
 class DOMDocumentTypeImpl;
 class DOMElement;
+class GrammarResolver;
 
 
 /**
@@ -359,6 +362,31 @@ public :
       */
     bool  getCreateCommentNodes()const;
 
+    /**
+      * Get the 'calculate src offset flag'
+      *
+      * This method returns the state of the parser's src offset calculation
+      * when parsing an XML document.
+      *
+      * @return true, if the parser is currently configured to
+      *         calculate src offsets, false otherwise.
+      *
+      * @see #setCalculateSrcOfs
+      */
+    bool getCalculateSrcOfs() const;
+
+    /**
+      * Get the 'force standard uri flag'
+      *
+      * This method returns the state if the parser forces standard uri
+      *
+      * @return true, if the parser is currently configured to
+      *         force standard uri, i.e. malformed uri will be rejected.
+      *
+      * @see #setStandardUriConformant
+      */
+    bool getStandardUriConformant() const;
+
     //@}
 
 
@@ -608,6 +636,41 @@ public :
       */
     void setCreateCommentNodes(const bool create);
 
+    /** Enable/disable src offset calculation
+      *
+      * This method allows users to enable/disable src offset calculation.
+      * Disabling the calculation will improve performance.
+      *
+      * The parser's default state is: false.
+      *
+      * @param newState The value specifying whether we should enable or
+      *                 disable src offset calculation
+      *
+      * @see #getCalculateSrcOfs
+      */
+    void setCalculateSrcOfs(const bool newState);
+
+    /** Force standard uri
+      *
+      * This method allows users to tell the parser to force standard uri conformance.
+      *
+      * The parser's default state is: false.
+      *
+      * @param newState The value specifying whether the parser should reject malformed URI.
+      *
+      * @see #getStandardUriConformant
+      */
+    void setStandardUriConformant(const bool newState);
+
+    /** Set the scanner to use when scanning the XML document
+      *
+      * This method allows users to set the the scanner to use
+      * when scanning a given XML document.
+      *
+      * @param scannerName The name of the desired scanner
+      */
+    void useScanner(const XMLCh* const scannerName);
+
     //@}
 
 
@@ -1231,6 +1294,12 @@ protected :
       */
     XMLScanner* getScanner() const;
 
+    /** Get the Grammar resolver
+      *
+      * This provides derived classes with access to the grammar resolver.
+      */
+    GrammarResolver* getGrammarResolver() const;
+
     /** Get the parse in progress flag
       *
       * This provides derived classes with access to the parse in progress
@@ -1283,10 +1352,22 @@ protected :
                                               const XMLCh *qualifiedName);
 
     void resetPool();
+
+    /**
+     * Returns true if the user has adopted the document
+     */
+    bool isDocumentAdopted() const;
+
     //@}
 
 
 private :
+    // -----------------------------------------------------------------------
+    //  Initialize/Cleanup methods
+    // -----------------------------------------------------------------------
+    void initialize();
+    void cleanUp();
+
     // -----------------------------------------------------------------------
     //  Private data members
     //
@@ -1350,6 +1431,8 @@ private :
     bool                          fIncludeIgnorableWhitespace;
     bool                          fWithinElement;
     bool                          fParseInProgress;
+    bool                          fCreateCommentNodes;
+    bool                          fDocumentAdoptedByUser;
     XMLBufferMgr                  fBufMgr;
     XMLScanner*                   fScanner;
     DOMNode*                      fCurrentParent;
@@ -1359,8 +1442,9 @@ private :
     ValueStackOf<DOMNode*>*       fNodeStack;
     DOMDocumentTypeImpl*          fDocumentType;
     RefVectorOf<DOMDocumentImpl>* fDocumentVector;
-    bool                          fCreateCommentNodes;
-    bool                          fDocumentAdoptedByUser;
+    GrammarResolver*              fGrammarResolver;
+    XMLStringPool*                fURIStringPool;
+    XMLValidator*                 fValidator;
     XMLBuffer&                    fInternalSubset;
 };
 
@@ -1393,6 +1477,11 @@ inline XMLScanner* AbstractDOMParser::getScanner() const
     return fScanner;
 }
 
+inline GrammarResolver* AbstractDOMParser::getGrammarResolver() const
+{
+    return fGrammarResolver;
+}
+
 inline bool AbstractDOMParser::getCreateCommentNodes() const
 {
     return fCreateCommentNodes;
@@ -1449,6 +1538,8 @@ inline void AbstractDOMParser::setParseInProgress(const bool toSet)
     fParseInProgress = toSet;
 }
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
 
 
