@@ -59,7 +59,7 @@
 *  handler with the scanner. In these handler methods, appropriate DOM nodes
 *  are created and added to the DOM tree.
 *
-* $Id: DOMBuilderImpl.cpp,v 1.12 2002/07/19 14:58:29 knoaman Exp $
+* $Id: DOMBuilderImpl.cpp,v 1.21 2003/01/23 19:19:50 tng Exp $
 *
 */
 
@@ -78,6 +78,10 @@
 #include <xercesc/sax/SAXParseException.hpp>
 #include <xercesc/internal/XMLScanner.hpp>
 #include <xercesc/framework/Wrapper4DOMInputSource.hpp>
+#include <xercesc/validators/common/GrammarResolver.hpp>
+
+XERCES_CPP_NAMESPACE_BEGIN
+
 
 
 // ---------------------------------------------------------------------------
@@ -92,6 +96,7 @@ AbstractDOMParser(valToAdopt)
 , fEntityResolver(0)
 , fFilter(0)
 , fCharsetOverridesXMLEncoding(true)
+, fUserAdoptsDocument(false)
 {
     // dom spec has different default from scanner's default, so set explicitly
     getScanner()->setNormalizeData(false);
@@ -200,9 +205,12 @@ void DOMBuilderImpl::setFeature(const XMLCh* const name, const bool state)
     {
         setValidationSchemaFullChecking(state);
     }
-    else if ((XMLString::compareIString(name, XMLUni::fgXercesUserAdoptsDOMDocument) == 0) && state)
+    else if (XMLString::compareIString(name, XMLUni::fgXercesUserAdoptsDOMDocument) == 0)
     {
-        adoptDocument();
+        if(state)
+            fUserAdoptsDocument = true;
+        else
+            fUserAdoptsDocument = false;
     }
 
     else if (XMLString::compareIString(name, XMLUni::fgXercesLoadExternalDTD) == 0)
@@ -228,7 +236,15 @@ void DOMBuilderImpl::setFeature(const XMLCh* const name, const bool state)
     else if (XMLString::compareIString(name, XMLUni::fgXercesUseCachedGrammarInParse) == 0)
     {
         if (state || !getScanner()->isCachingGrammarFromParse())
-         getScanner()->useCachedGrammarInParse(state);
+            getScanner()->useCachedGrammarInParse(state);
+    }
+    else if (XMLString::compareIString(name, XMLUni::fgXercesCalculateSrcOfs) == 0)
+    {
+        getScanner()->setCalculateSrcOfs(state);
+    }
+    else if (XMLString::compareIString(name, XMLUni::fgXercesStandardUriConformant) == 0)
+    {
+        getScanner()->setStandardUriConformant(state);
     }
     else {
         throw DOMException(DOMException::NOT_FOUND_ERR, 0);
@@ -301,6 +317,17 @@ bool DOMBuilderImpl::getFeature(const XMLCh* const name) const
     {
         return getScanner()->isUsingCachedGrammarInParse();
     }
+    else if (XMLString::compareIString(name, XMLUni::fgXercesCalculateSrcOfs) == 0)
+    {
+        return getScanner()->getCalculateSrcOfs();
+    }
+    else if (XMLString::compareIString(name, XMLUni::fgXercesStandardUriConformant) == 0)
+    {
+        return getScanner()->getStandardUriConformant();
+    }
+    else if(XMLString::compareIString(name, XMLUni::fgXercesUserAdoptsDOMDocument) == 0) {
+        return fUserAdoptsDocument;
+    }
     else {
         throw DOMException(DOMException::NOT_FOUND_ERR, 0);
     }
@@ -317,7 +344,10 @@ bool DOMBuilderImpl::canSetFeature(const XMLCh* const name, const bool state) co
         (XMLString::compareIString(name, XMLUni::fgDOMValidation) == 0) ||
         (XMLString::compareIString(name, XMLUni::fgDOMValidateIfSchema) == 0) ||
         (XMLString::compareIString(name, XMLUni::fgDOMCharsetOverridesXMLEncoding) == 0) ||
-        (XMLString::compareIString(name, XMLUni::fgDOMWhitespaceInElementContent) == 0)) {
+        (XMLString::compareIString(name, XMLUni::fgDOMWhitespaceInElementContent) == 0) ||
+        (XMLString::compareIString(name, XMLUni::fgXercesUserAdoptsDOMDocument) == 0) ||
+        (XMLString::compareIString(name, XMLUni::fgXercesCalculateSrcOfs) == 0) ||
+        (XMLString::compareIString(name, XMLUni::fgXercesStandardUriConformant) == 0)) {
         return true;
     }
 
@@ -359,6 +389,10 @@ void DOMBuilderImpl::setProperty(const XMLCh* const name, void* value)
 	{
 		setExternalNoNamespaceSchemaLocation((XMLCh*)value);
 	}
+    else if (XMLString::equals(name, XMLUni::fgXercesScannerName))
+    {
+        AbstractDOMParser::useScanner((const XMLCh*) value);
+    }
     else
       throw DOMException(DOMException::NOT_FOUND_ERR, 0);
 }
@@ -395,19 +429,28 @@ DOMDocument* DOMBuilderImpl::parse(const DOMInputSource& source)
     Wrapper4DOMInputSource isWrapper((DOMInputSource*) &source, false);
 
     AbstractDOMParser::parse(isWrapper);
-    return getDocument();
+    if (fUserAdoptsDocument)
+        return adoptDocument();
+    else
+        return getDocument();
 }
 
 DOMDocument* DOMBuilderImpl::parseURI(const XMLCh* const systemId)
 {
     AbstractDOMParser::parse(systemId);
-    return getDocument();
+    if (fUserAdoptsDocument)
+        return adoptDocument();
+    else
+        return getDocument();
 }
 
 DOMDocument* DOMBuilderImpl::parseURI(const char* const systemId)
 {
     AbstractDOMParser::parse(systemId);
-    return getDocument();
+    if (fUserAdoptsDocument)
+        return adoptDocument();
+    else
+        return getDocument();
 }
 
 void DOMBuilderImpl::parseWithContext(const DOMInputSource& source,
@@ -442,7 +485,9 @@ void DOMBuilderImpl::error( const   unsigned int                code
         DOMLocatorImpl location((int)lineNum, (int) colNum, getCurrentNode(), systemId);
         DOMErrorImpl domError(severity, errorText, &location);
 
-        fErrorHandler->handleError(domError);
+        // if user return false, we should stop the process, so throw an error
+        if (!fErrorHandler->handleError(domError) && !getScanner()->getInException())
+            throw (XMLErrs::Codes) code;
     }
 }
 
@@ -574,12 +619,12 @@ Grammar* DOMBuilderImpl::loadGrammar(const DOMInputSource& source,
 
 void DOMBuilderImpl::resetCachedGrammarPool()
 {
-    getScanner()->resetCachedGrammarPool();
+    getGrammarResolver()->resetCachedGrammar();
 }
 
 Grammar* DOMBuilderImpl::getGrammar(const XMLCh* const nameSpaceKey) const
 {
-    return getScanner()->getGrammar(nameSpaceKey);
+    return getGrammarResolver()->getGrammar(nameSpaceKey);
 }
 
 Grammar* DOMBuilderImpl::getRootGrammar() const
@@ -592,3 +637,10 @@ const XMLCh* DOMBuilderImpl::getURIText(unsigned int uriId) const
     return getScanner()->getURIText(uriId);
 }
 
+unsigned int DOMBuilderImpl::getSrcOffset() const
+{
+    return getScanner()->getSrcOffset();
+}
+
+XERCES_CPP_NAMESPACE_END
+
