@@ -56,6 +56,24 @@
 
 /*
  * $Log: SAX2XMLReaderImpl.cpp,v $
+ * Revision 1.16  2003/01/03 20:09:36  tng
+ * New feature StandardUriConformant to force strict standard uri conformance.
+ *
+ * Revision 1.15  2002/12/27 16:16:51  knoaman
+ * Set scanner options and handlers.
+ *
+ * Revision 1.14  2002/12/11 22:14:54  knoaman
+ * Performance: no need to use temporary buffer to hold namespace value.
+ *
+ * Revision 1.13  2002/12/04 01:57:09  knoaman
+ * Scanner re-organization.
+ *
+ * Revision 1.12  2002/11/04 14:57:03  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.11  2002/09/24 20:00:32  tng
+ * Performance: use XMLString::equals instead of XMLString::compareString
+ *
  * Revision 1.10  2002/08/14 15:20:38  knoaman
  * [Bug 3111] Problem with LexicalHandler::startDTD() and LexicalHandler::endDTD().
  *
@@ -218,6 +236,7 @@
 #include <xercesc/util/XMLChTranscoder.hpp>
 #include <xercesc/util/RefStackOf.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
+#include <xercesc/util/Janitor.hpp>
 #include <xercesc/sax2/ContentHandler.hpp>
 #include <xercesc/sax2/LexicalHandler.hpp>
 #include <xercesc/sax2/DeclHandler.hpp>
@@ -226,35 +245,74 @@
 #include <xercesc/sax/EntityResolver.hpp>
 #include <xercesc/sax/SAXParseException.hpp>
 #include <xercesc/sax/SAXException.hpp>
-#include <xercesc/internal/XMLScanner.hpp>
+#include <xercesc/internal/XMLScannerResolver.hpp>
 #include <xercesc/parsers/SAX2XMLReaderImpl.hpp>
+#include <xercesc/validators/common/GrammarResolver.hpp>
 #include <string.h>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
+
 const XMLCh gDTDEntityStr[] =
 {
     chOpenSquare, chLatin_d, chLatin_t, chLatin_d, chCloseSquare, chNull
 };
 
 SAX2XMLReaderImpl::SAX2XMLReaderImpl() :
-    fDocHandler(0)
-    , fDTDHandler(0)
+
+    fNamespacePrefix(false)
+    , fAutoValidation(false)
+    , fValidation(true)
+    , fParseInProgress(false)
+    , fHasExternalSubset(false)
     , fElemDepth(0)
+    , fAdvDHCount(0)
+    , fAdvDHListSize(32)
+    , fDocHandler(0)
+    , fTempAttrVec(0)
+    , fPrefixes(0)
+    , fPrefixCounts(0)
+    , fDTDHandler(0)
     , fEntityResolver(0)
     , fErrorHandler(0)
     , fLexicalHandler(0)
     , fDeclHandler(0)
-    , fAdvDHCount(0)
     , fAdvDHList(0)
-    , fAdvDHListSize(32)
-    , fParseInProgress(false)
     , fScanner(0)
-    , fHasExternalSubset(false)
+    , fGrammarResolver(0)
+    , fURIStringPool(0)
+    , fValidator(0)
 {
-    //
+    try
+    {
+        initialize();
+    }
+    catch(...)
+    {
+        cleanUp();
+        throw;
+    }
+}
+
+SAX2XMLReaderImpl::~SAX2XMLReaderImpl()
+{
+    cleanUp();
+}
+
+// ---------------------------------------------------------------------------
+//  SAX2XMLReaderImpl: Initialize/Cleanup methods
+// ---------------------------------------------------------------------------
+void SAX2XMLReaderImpl::initialize()
+{
+    // Create grammar resolver and string pool that we pass to the scanner
+    fGrammarResolver = new GrammarResolver();
+    fURIStringPool = new XMLStringPool();
+
     //  Create a scanner and tell it what validator to use. Then set us
     //  as the document event handler so we can fill the DOM document.
-    //
-    fScanner = new XMLScanner(0);
+    fScanner = XMLScannerResolver::getDefaultScanner(0);
+    fScanner->setGrammarResolver(fGrammarResolver);
+    fScanner->setURIStringPool(fURIStringPool);
 
     // Create the initial advanced handler list array and zero it out
     fAdvDHList = new XMLDocumentHandler*[fAdvDHListSize];
@@ -263,28 +321,24 @@ SAX2XMLReaderImpl::SAX2XMLReaderImpl() :
 	// SAX2 default is for namespaces (feature http://xml.org/sax/features/namespaces) to be on
 	setDoNamespaces(true) ;
 
-	// SAX2 default is for namespaces-prefixes to be off
-	fnamespacePrefix = false;
-
-	// SAX2 default: validation on, auto-validation off
-	fValidation = true;
-	fautoValidation = false;
-
 	// default: schema is on
 	setDoSchema(true);
 	
 	fPrefixes    = new RefStackOf<XMLBuffer> (10, false) ;
-	tempAttrVec  = new RefVectorOf<XMLAttr>  (10, false) ;
-	prefixCounts = new ValueStackOf<unsigned int>(10) ;
+	fTempAttrVec  = new RefVectorOf<XMLAttr>  (10, false) ;
+	fPrefixCounts = new ValueStackOf<unsigned int>(10) ;
 }
 
-SAX2XMLReaderImpl::~SAX2XMLReaderImpl()
+
+void SAX2XMLReaderImpl::cleanUp()
 {
     delete [] fAdvDHList;
     delete fScanner;
     delete fPrefixes;
-    delete tempAttrVec;
-    delete prefixCounts ;
+    delete fTempAttrVec;
+    delete fPrefixCounts;
+    delete fGrammarResolver;
+    delete fURIStringPool;
 }
 
 // ---------------------------------------------------------------------------
@@ -380,6 +434,7 @@ bool SAX2XMLReaderImpl::removeAdvDocHandler(XMLDocumentHandler* const toRemove)
 // ---------------------------------------------------------------------------
 void SAX2XMLReaderImpl::setValidator(XMLValidator* valueToAdopt)
 {
+    fValidator = valueToAdopt;
 	fScanner->setValidator(valueToAdopt);
 }
 
@@ -748,9 +803,9 @@ void SAX2XMLReaderImpl::resetDocument()
     fElemDepth = 0;
 
     // Pop any prefix buffers left over from previous uses
-    while (!prefixCounts->empty())
+    while (!fPrefixCounts->empty())
     {
-        unsigned int numPrefix = prefixCounts->pop();
+        unsigned int numPrefix = fPrefixCounts->pop();
         for (unsigned int i = 0; i < numPrefix; i++)
         {
             XMLBuffer * buf = fPrefixes->pop() ;
@@ -808,25 +863,25 @@ startElement(   const   XMLElementDecl&         elemDecl
             const XMLCh*   nsURI    = 0;
             const XMLAttr* tempAttr = 0;
 
-            if (!fnamespacePrefix)
+            if (!fNamespacePrefix)
             {
-                tempAttrVec->removeAllElements();
+                fTempAttrVec->removeAllElements();
             }
 
             for (unsigned int i = 0; i < attrCount; i++)
             {
                 tempAttr = attrList.elementAt(i);
-                if (XMLString::compareString(tempAttr->getQName(), nsString) == 0)
+                if (XMLString::equals(tempAttr->getQName(), nsString))
                     nsURI = tempAttr->getValue();
-                if (XMLString::compareString(tempAttr->getPrefix(), nsString) == 0)
+                if (XMLString::equals(tempAttr->getPrefix(), nsString))
                 {
                     nsPrefix = tempAttr->getName();
                     nsURI = tempAttr->getValue();
                 }
-                if (!fnamespacePrefix)
+                if (!fNamespacePrefix)
                 {
                     if (nsURI == 0)
-                        tempAttrVec->addElement((XMLAttr* const)tempAttr);
+                        fTempAttrVec->addElement((XMLAttr* const)tempAttr);
                 }
                 if (nsURI != 0)
                 {
@@ -841,22 +896,20 @@ startElement(   const   XMLElementDecl&         elemDecl
                 nsURI = 0;
                 nsPrefix = 0;
             }
-            prefixCounts->push(numPrefix) ;
-            if (!fnamespacePrefix)
-                fAttrList.setVector(tempAttrVec, tempAttrVec->size(), fScanner);
+            fPrefixCounts->push(numPrefix) ;
+            if (!fNamespacePrefix)
+                fAttrList.setVector(fTempAttrVec, fTempAttrVec->size(), fScanner);
             else
                 fAttrList.setVector(&attrList, attrCount, fScanner);
 
             // call startElement() with namespace declarations
-            XMLBufBid URIBufferBid ( &fStringBuffers ) ;
-            XMLBuffer &URIBuffer = URIBufferBid.getBuffer() ;
-
-            fScanner->getURIText(elemURLId, (XMLBuffer &)URIBuffer);
-
-            fDocHandler->startElement(URIBuffer.getRawBuffer(),
-										elemDecl.getBaseName(),
-										elemQName.getRawBuffer(),
-										fAttrList);
+            fDocHandler->startElement
+            (
+                fScanner->getURIText(elemURLId)
+                , elemDecl.getBaseName()
+                , elemQName.getRawBuffer()
+                , fAttrList
+            );
         }
         else // no namespace
         {
@@ -874,15 +927,14 @@ startElement(   const   XMLElementDecl&         elemDecl
             // call endPrefixMapping appropriately.
             if (getDoNamespaces())
             {
-                XMLBufBid URIBufferBid ( &fStringBuffers ) ;
-                XMLBuffer &URIBuffer = URIBufferBid.getBuffer() ;
-                fScanner->getURIText(elemURLId, (XMLBuffer &)URIBuffer);
+                fDocHandler->endElement
+                (
+                    fScanner->getURIText(elemURLId)
+                    , elemDecl.getBaseName()
+                    , elemQName.getRawBuffer()
+                );
 
-                fDocHandler->endElement(	URIBuffer.getRawBuffer(),
-											elemDecl.getBaseName(),
-											elemQName.getRawBuffer());
-
-                unsigned int numPrefix = prefixCounts->pop();
+                unsigned int numPrefix = fPrefixCounts->pop();
                 for (unsigned int i = 0; i < numPrefix; ++i)
                 {
                     XMLBuffer * buf = fPrefixes->pop() ;
@@ -930,11 +982,6 @@ void SAX2XMLReaderImpl::endElement( const   XMLElementDecl& elemDecl
         // get the prefixes back so that we can call endPrefixMapping()
         if (getDoNamespaces())
         {
-            XMLBufBid URIBufferBid ( &fStringBuffers ) ;
-            XMLBuffer &URIBuffer = URIBufferBid.getBuffer() ;
-
-            fScanner->getURIText(uriId, URIBuffer ) ;
-
             XMLBufBid elemQName( &fStringBuffers ) ;
             if (elemPrefix && *elemPrefix) {
                 elemQName.set(elemPrefix);
@@ -942,11 +989,14 @@ void SAX2XMLReaderImpl::endElement( const   XMLElementDecl& elemDecl
             }
             elemQName.append(elemDecl.getBaseName());
 
-            fDocHandler->endElement(	URIBuffer.getRawBuffer(),
-										elemDecl.getBaseName(),
-										elemQName.getRawBuffer());
+            fDocHandler->endElement
+            (
+                fScanner->getURIText(uriId)
+                , elemDecl.getBaseName()
+                , elemQName.getRawBuffer()
+            );
 
-            unsigned int numPrefix = prefixCounts->pop();
+            unsigned int numPrefix = fPrefixCounts->pop();
             for (unsigned int i = 0; i < numPrefix; i++)
             {
                 XMLBuffer * buf = fPrefixes->pop() ;
@@ -1330,7 +1380,7 @@ void SAX2XMLReaderImpl::setFeature(const XMLCh* const name, const bool value)
     {
         fValidation = value;
         if (fValidation)
-            if (fautoValidation)
+            if (fAutoValidation)
                 setValidationScheme(Val_Auto);
             else
                 setValidationScheme(Val_Always);
@@ -1339,14 +1389,14 @@ void SAX2XMLReaderImpl::setFeature(const XMLCh* const name, const bool value)
     }
     else if (XMLString::compareIString(name, XMLUni::fgSAX2CoreNameSpacePrefixes) == 0)
     {
-        fnamespacePrefix = value;
+        fNamespacePrefix = value;
     }
     else if (XMLString::compareIString(name, XMLUni::fgXercesDynamic) == 0)
     {
-        fautoValidation = value;
+        fAutoValidation = value;
         // for auto validation, the sax2 core validation feature must also be enabled.
         if (fValidation)
-            if (fautoValidation)
+            if (fAutoValidation)
                 setValidationScheme(Val_Auto);
             else
                 setValidationScheme(Val_Always);
@@ -1385,6 +1435,14 @@ void SAX2XMLReaderImpl::setFeature(const XMLCh* const name, const bool value)
         if (value || !fScanner->isCachingGrammarFromParse())
             fScanner->useCachedGrammarInParse(value);
     }
+    else if (XMLString::compareIString(name, XMLUni::fgXercesCalculateSrcOfs) == 0)
+    {
+        fScanner->setCalculateSrcOfs(value);
+    }
+    else if (XMLString::compareIString(name, XMLUni::fgXercesStandardUriConformant) == 0)
+    {
+        fScanner->setStandardUriConformant(value);
+    }
     else
        throw SAXNotRecognizedException("Unknown Feature");
 }
@@ -1396,9 +1454,9 @@ bool SAX2XMLReaderImpl::getFeature(const XMLCh* const name) const
     else if (XMLString::compareIString(name, XMLUni::fgSAX2CoreValidation) == 0)
         return fValidation;
     else if (XMLString::compareIString(name, XMLUni::fgSAX2CoreNameSpacePrefixes) == 0)
-        return fnamespacePrefix;
+        return fNamespacePrefix;
     else if (XMLString::compareIString(name, XMLUni::fgXercesDynamic) == 0)
-        return fautoValidation;
+        return fAutoValidation;
     else if (XMLString::compareIString(name, XMLUni::fgXercesSchema) == 0)
         return getDoSchema();
     else if (XMLString::compareIString(name, XMLUni::fgXercesSchemaFullChecking) == 0)
@@ -1413,6 +1471,10 @@ bool SAX2XMLReaderImpl::getFeature(const XMLCh* const name) const
         return fScanner->isCachingGrammarFromParse();
     else if (XMLString::compareIString(name, XMLUni::fgXercesUseCachedGrammarInParse) == 0)
         return fScanner->isUsingCachedGrammarInParse();
+    else if (XMLString::compareIString(name, XMLUni::fgXercesCalculateSrcOfs) == 0)
+        return fScanner->getCalculateSrcOfs();
+    else if (XMLString::compareIString(name, XMLUni::fgXercesStandardUriConformant) == 0)
+        return fScanner->getStandardUriConformant();
     else
        throw SAXNotRecognizedException("Unknown Feature");
 
@@ -1428,13 +1490,24 @@ void SAX2XMLReaderImpl::setProperty(const XMLCh* const name, void* value)
 	{
 		fScanner->setExternalSchemaLocation((XMLCh*)value);
 	}
-
 	else if (XMLString::compareIString(name, XMLUni::fgXercesSchemaExternalNoNameSpaceSchemaLocation) == 0)
 	{
 		fScanner->setExternalNoNamespaceSchemaLocation((XMLCh*)value);
 	}
+    else if (XMLString::equals(name, XMLUni::fgXercesScannerName))
+    {
+        XMLScanner* tempScanner = XMLScannerResolver::resolveScanner((const XMLCh*) value, fValidator);
 
-   else
+        if (tempScanner) {
+
+            tempScanner->setParseSettings(fScanner);
+            tempScanner->setGrammarResolver(fGrammarResolver);
+            tempScanner->setURIStringPool(fURIStringPool);
+            delete fScanner;
+            fScanner = tempScanner;
+        }
+    }
+    else
        throw SAXNotRecognizedException("Unknown Property");
 }
 
@@ -1445,6 +1518,8 @@ void* SAX2XMLReaderImpl::getProperty(const XMLCh* const name) const
         return (void*)fScanner->getExternalSchemaLocation();
     else if (XMLString::compareIString(name, XMLUni::fgXercesSchemaExternalNoNameSpaceSchemaLocation) == 0)
         return (void*)fScanner->getExternalNoNamespaceSchemaLocation();
+    else if (XMLString::equals(name, XMLUni::fgXercesScannerName))
+        return (void*)fScanner->getName();
     else
         throw SAXNotRecognizedException("Unknown Property");
     return 0;
@@ -1566,5 +1641,14 @@ Grammar* SAX2XMLReaderImpl::loadGrammar(const InputSource& source,
 
 void SAX2XMLReaderImpl::resetCachedGrammarPool()
 {
-    fScanner->resetCachedGrammarPool();
+    fGrammarResolver->resetCachedGrammar();
 }
+
+Grammar* SAX2XMLReaderImpl::getGrammar(const XMLCh* const nameSpaceKey)
+{
+    return fGrammarResolver->getGrammar(nameSpaceKey);
+}
+
+
+XERCES_CPP_NAMESPACE_END
+
