@@ -56,6 +56,19 @@
 
 /*
  * $Log: SAX2XMLReaderImpl.hpp,v $
+ * Revision 1.17  2003/01/09 19:07:08  tng
+ * [Bug 15802] Add "const" qualifier to getURIText.
+ *
+ * Revision 1.16  2002/12/23 15:23:18  knoaman
+ * Added a public api to various parsers to return the src offset within the input
+ * source.
+ *
+ * Revision 1.15  2002/12/04 01:57:09  knoaman
+ * Scanner re-organization.
+ *
+ * Revision 1.14  2002/11/04 14:57:03  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.13  2002/08/14 15:20:38  knoaman
  * [Bug 3111] Problem with LexicalHandler::startDTD() and LexicalHandler::endDTD().
  *
@@ -190,9 +203,13 @@
 #include <xercesc/util/ValueStackOf.hpp>
 #include <xercesc/framework/XMLBufferMgr.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
+
 class ContentHandler;
 class LexicalHandler;
 class DeclHandler;
+class GrammarResolver;
 
 /**
   * This class implements the SAX2 'XMLReader' interface and should be
@@ -636,8 +653,15 @@ public :
       *
       * @param uriId id of the string in the URI string pool.
       * @return URI string corresponding to the URI id.
-      */    
-    virtual const XMLCh* getURIText(unsigned int uriId);
+      */
+    virtual const XMLCh* getURIText(unsigned int uriId) const;
+
+    /**
+      * Returns the current src offset within the input source.
+      *
+      * @return offset within the input source
+      */
+    virtual unsigned int getSrcOffset() const;
 
     //@}
 
@@ -1636,6 +1660,12 @@ private :
     SAX2XMLReaderImpl(const SAX2XMLReaderImpl&);
     void operator=(const SAX2XMLReaderImpl&);
 
+    // -----------------------------------------------------------------------
+    //  Initialize/Cleanup methods
+    // -----------------------------------------------------------------------
+    void initialize();
+    void cleanUp();
+
     // -----------------------------------------------------------------------
     //  Private data members
     //
@@ -1716,30 +1746,30 @@ private :
     //      Indicate if the document has external DTD subset.
     //
     // -----------------------------------------------------------------------
-	VecAttributesImpl		   fAttrList ;
-	ContentHandler*			   fDocHandler ;
-	RefVectorOf<XMLAttr>*      tempAttrVec ;
-
-	bool                       fnamespacePrefix;
-	bool                       fautoValidation;
-	bool                       fValidation;
-
-	XMLBufferMgr			   fStringBuffers ;
-	RefStackOf<XMLBuffer> *    fPrefixes ;
-	ValueStackOf<unsigned int> * prefixCounts ;
-
-    DTDHandler*                fDTDHandler;
-    unsigned int               fElemDepth;
-    EntityResolver*            fEntityResolver;
-    ErrorHandler*              fErrorHandler;
-    LexicalHandler*            fLexicalHandler;
-    DeclHandler*               fDeclHandler;
-    unsigned int               fAdvDHCount;
-    XMLDocumentHandler**       fAdvDHList;
-    unsigned int               fAdvDHListSize;
-    bool                       fParseInProgress;
-    XMLScanner*                fScanner;
-    bool                       fHasExternalSubset;
+    bool                        fNamespacePrefix;
+    bool                        fAutoValidation;
+    bool                        fValidation;
+    bool                        fParseInProgress;
+    bool                        fHasExternalSubset;
+    unsigned int                fElemDepth;
+    unsigned int                fAdvDHCount;
+    unsigned int                fAdvDHListSize;
+    VecAttributesImpl		fAttrList ;
+    ContentHandler*		fDocHandler ;
+    RefVectorOf<XMLAttr>*       fTempAttrVec ;
+    XMLBufferMgr		fStringBuffers ;
+    RefStackOf<XMLBuffer> *     fPrefixes ;
+    ValueStackOf<unsigned int>* fPrefixCounts ;
+    DTDHandler*                 fDTDHandler;
+    EntityResolver*             fEntityResolver;
+    ErrorHandler*               fErrorHandler;
+    LexicalHandler*             fLexicalHandler;
+    DeclHandler*                fDeclHandler;
+    XMLDocumentHandler**        fAdvDHList;
+    XMLScanner*                 fScanner;
+    GrammarResolver*            fGrammarResolver;
+    XMLStringPool*              fURIStringPool;
+    XMLValidator*               fValidator;
 	
     // -----------------------------------------------------------------------
     // internal function used to set the state of the parser
@@ -1749,7 +1779,6 @@ private :
     bool getDoNamespaces() const;
     void setDoSchema(const bool newState);
     bool getDoSchema() const;
-
 };
 
 
@@ -1796,19 +1825,21 @@ inline bool SAX2XMLReaderImpl::getValidationConstraintFatal() const
     return fScanner->getValidationConstraintFatal();
 }
 
-inline Grammar* SAX2XMLReaderImpl::getGrammar(const XMLCh* const nameSpaceKey)
-{
-    return fScanner->getGrammar(nameSpaceKey);
-}
-
 inline Grammar* SAX2XMLReaderImpl::getRootGrammar()
 {
     return fScanner->getRootGrammar();
 }
 
-inline const XMLCh* SAX2XMLReaderImpl::getURIText(unsigned int uriId)
+inline const XMLCh* SAX2XMLReaderImpl::getURIText(unsigned int uriId) const
 {
     return fScanner->getURIText(uriId);
 }
 
+inline unsigned int SAX2XMLReaderImpl::getSrcOffset() const
+{
+    return fScanner->getSrcOffset();
+}
+
+XERCES_CPP_NAMESPACE_END
+
 #endif
