@@ -56,6 +56,53 @@
 
 /*
  * $Log: SAXParser.cpp,v $
+ * Revision 1.15  2003/02/04 19:27:43  knoaman
+ * Performance: use global buffer to eliminate repetitive memory creation/deletion.
+ *
+ * Revision 1.14  2003/01/09 19:07:08  tng
+ * [Bug 15802] Add "const" qualifier to getURIText.
+ *
+ * Revision 1.13  2003/01/03 20:09:36  tng
+ * New feature StandardUriConformant to force strict standard uri conformance.
+ *
+ * Revision 1.12  2002/12/27 16:16:51  knoaman
+ * Set scanner options and handlers.
+ *
+ * Revision 1.11  2002/12/23 15:23:18  knoaman
+ * Added a public api to various parsers to return the src offset within the input
+ * source.
+ *
+ * Revision 1.10  2002/12/04 01:57:09  knoaman
+ * Scanner re-organization.
+ *
+ * Revision 1.9  2002/11/04 14:57:03  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.8  2002/08/14 15:20:38  knoaman
+ * [Bug 3111] Problem with LexicalHandler::startDTD() and LexicalHandler::endDTD().
+ *
+ * Revision 1.7  2002/07/11 18:27:04  knoaman
+ * Grammar caching/preparsing - initial implementation.
+ *
+ * Revision 1.6  2002/05/30 16:20:09  tng
+ * Add feature to optionally ignore external DTD.
+ *
+ * Revision 1.5  2002/05/29 21:37:47  knoaman
+ * Add baseURI to resolveEntity to support DOMInputSource.
+ *
+ * Revision 1.4  2002/05/28 20:44:14  tng
+ * [Bug 9104] prefixes dissapearing when schema validation turned on.
+ *
+ * Revision 1.3  2002/05/27 18:39:21  tng
+ * To get ready for 64 bit large file, use XMLSSize_t to represent line and column number.
+ *
+ * Revision 1.2  2002/05/22 20:53:41  knoaman
+ * Prepare for DOM L3 :
+ * - Make use of the XMLEntityHandler/XMLErrorReporter interfaces, instead of using
+ * EntityHandler/ErrorHandler directly.
+ * - Add 'AbstractDOMParser' class to provide common functionality for XercesDOMParser
+ * and DOMBuilder.
+ *
  * Revision 1.1.1.1  2002/02/01 22:22:07  peiyongz
  * sane_include
  *
@@ -155,48 +202,86 @@
 // ---------------------------------------------------------------------------
 //  Includes
 // ---------------------------------------------------------------------------
+#include <xercesc/parsers/SAXParser.hpp>
+#include <xercesc/internal/XMLScannerResolver.hpp>
+#include <xercesc/framework/XMLValidator.hpp>
 #include <xercesc/util/IOException.hpp>
 #include <xercesc/sax/DocumentHandler.hpp>
 #include <xercesc/sax/DTDHandler.hpp>
 #include <xercesc/sax/ErrorHandler.hpp>
 #include <xercesc/sax/EntityResolver.hpp>
 #include <xercesc/sax/SAXParseException.hpp>
-#include <xercesc/internal/XMLScanner.hpp>
-#include <xercesc/parsers/SAXParser.hpp>
+#include <xercesc/validators/common/GrammarResolver.hpp>
 #include <string.h>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 
 // ---------------------------------------------------------------------------
 //  SAXParser: Constructors and Destructor
 // ---------------------------------------------------------------------------
 SAXParser::SAXParser(XMLValidator* const valToAdopt) :
 
-    fDocHandler(0)
-    , fDTDHandler(0)
+    fParseInProgress(false)
     , fElemDepth(0)
+    , fAdvDHCount(0)
+    , fAdvDHListSize(32)
+    , fDocHandler(0)
+    , fDTDHandler(0)
     , fEntityResolver(0)
     , fErrorHandler(0)
-    , fAdvDHCount(0)
     , fAdvDHList(0)
-    , fAdvDHListSize(32)
-    , fParseInProgress(false)
     , fScanner(0)
+    , fGrammarResolver(0)
+    , fURIStringPool(0)
+    , fValidator(valToAdopt)
 {
+    try
+    {
+        initialize();
+    }
+    catch(...)
+    {
+        cleanUp();
+        throw;
+    }
+}
+
+
+SAXParser::~SAXParser()
+{
+    cleanUp();
+}
+
+// ---------------------------------------------------------------------------
+//  SAXParser: Initialize/CleanUp methods
+// ---------------------------------------------------------------------------
+void SAXParser::initialize()
+{
+    // Create grammar resolver and string pool to pass to scanner
+    fGrammarResolver = new GrammarResolver();
+    fURIStringPool = new XMLStringPool();
+
     // Create our scanner and tell it what validator to use
-    fScanner = new XMLScanner(valToAdopt);
+    fScanner = XMLScannerResolver::getDefaultScanner(fValidator);
+    fScanner->setGrammarResolver(fGrammarResolver);
+    fScanner->setURIStringPool(fURIStringPool);
 
     // Create the initial advanced handler list array and zero it out
     fAdvDHList = new XMLDocumentHandler*[fAdvDHListSize];
     memset(fAdvDHList, 0, sizeof(void*) * fAdvDHListSize);
 }
 
-
-SAXParser::~SAXParser()
+void SAXParser::cleanUp()
 {
     delete [] fAdvDHList;
     delete fScanner;
-}
+    delete fGrammarResolver;
+    delete fURIStringPool;
 
+    if (fValidator)
+        delete fValidator;
+}
 
 
 // ---------------------------------------------------------------------------
@@ -351,6 +436,50 @@ XMLCh* SAXParser::getExternalNoNamespaceSchemaLocation() const
     return fScanner->getExternalNoNamespaceSchemaLocation();
 }
 
+bool SAXParser::getLoadExternalDTD() const
+{
+    return fScanner->getLoadExternalDTD();
+}
+
+bool SAXParser::isCachingGrammarFromParse() const
+{
+    return fScanner->isCachingGrammarFromParse();
+}
+
+bool SAXParser::isUsingCachedGrammarInParse() const
+{
+    return fScanner->isUsingCachedGrammarInParse();
+}
+
+bool SAXParser::getCalculateSrcOfs() const
+{
+    return fScanner->getCalculateSrcOfs();
+}
+
+bool SAXParser::getStandardUriConformant() const
+{
+    return fScanner->getStandardUriConformant();
+}
+
+Grammar* SAXParser::getGrammar(const XMLCh* const nameSpaceKey)
+{
+    return fGrammarResolver->getGrammar(nameSpaceKey);
+}
+
+Grammar* SAXParser::getRootGrammar()
+{
+    return fScanner->getRootGrammar();
+}
+
+const XMLCh* SAXParser::getURIText(unsigned int uriId) const
+{
+    return fScanner->getURIText(uriId);
+}
+
+unsigned int SAXParser::getSrcOffset() const
+{
+    return fScanner->getSrcOffset();
+}
 
 // ---------------------------------------------------------------------------
 //  SAXParser: Setter methods
@@ -411,11 +540,54 @@ void SAXParser::setExternalNoNamespaceSchemaLocation(const char* const noNamespa
     fScanner->setExternalNoNamespaceSchemaLocation(noNamespaceSchemaLocation);
 }
 
+void SAXParser::setLoadExternalDTD(const bool newState)
+{
+    fScanner->setLoadExternalDTD(newState);
+}
+
+void SAXParser::cacheGrammarFromParse(const bool newState)
+{
+    fScanner->cacheGrammarFromParse(newState);
+
+    if (newState)
+        fScanner->useCachedGrammarInParse(newState);
+}
+
+void SAXParser::useCachedGrammarInParse(const bool newState)
+{
+    if (newState || !fScanner->isCachingGrammarFromParse())
+        fScanner->useCachedGrammarInParse(newState);
+}
+
+void SAXParser::setCalculateSrcOfs(const bool newState)
+{
+    fScanner->setCalculateSrcOfs(newState);
+}
+
+void SAXParser::setStandardUriConformant(const bool newState)
+{
+    fScanner->setStandardUriConformant(newState);
+}
+
+void SAXParser::useScanner(const XMLCh* const scannerName)
+{
+    XMLScanner* tempScanner = XMLScannerResolver::resolveScanner(scannerName, fValidator);
+
+    if (tempScanner) {
+
+        tempScanner->setParseSettings(fScanner);
+        tempScanner->setGrammarResolver(fGrammarResolver);
+        tempScanner->setURIStringPool(fURIStringPool);
+        delete fScanner;
+        fScanner = tempScanner;
+    }
+}
+
 
 // ---------------------------------------------------------------------------
 //  SAXParser: Overrides of the SAX Parser interface
 // ---------------------------------------------------------------------------
-void SAXParser::parse(const InputSource& source, const bool reuseGrammar)
+void SAXParser::parse(const InputSource& source)
 {
     // Avoid multiple entrance
     if (fParseInProgress)
@@ -424,7 +596,7 @@ void SAXParser::parse(const InputSource& source, const bool reuseGrammar)
     try
     {
         fParseInProgress = true;
-        fScanner->scanDocument(source, reuseGrammar);
+        fScanner->scanDocument(source);
         fParseInProgress = false;
     }
 
@@ -435,7 +607,7 @@ void SAXParser::parse(const InputSource& source, const bool reuseGrammar)
     }
 }
 
-void SAXParser::parse(const XMLCh* const systemId, const bool reuseGrammar)
+void SAXParser::parse(const XMLCh* const systemId)
 {
     // Avoid multiple entrance
     if (fParseInProgress)
@@ -444,7 +616,7 @@ void SAXParser::parse(const XMLCh* const systemId, const bool reuseGrammar)
     try
     {
         fParseInProgress = true;
-        fScanner->scanDocument(systemId, reuseGrammar);
+        fScanner->scanDocument(systemId);
         fParseInProgress = false;
     }
 
@@ -455,7 +627,7 @@ void SAXParser::parse(const XMLCh* const systemId, const bool reuseGrammar)
     }
 }
 
-void SAXParser::parse(const char* const systemId, const bool reuseGrammar)
+void SAXParser::parse(const char* const systemId)
 {
     // Avoid multiple entrance
     if (fParseInProgress)
@@ -464,7 +636,7 @@ void SAXParser::parse(const char* const systemId, const bool reuseGrammar)
     try
     {
         fParseInProgress = true;
-        fScanner->scanDocument(systemId, reuseGrammar);
+        fScanner->scanDocument(systemId);
         fParseInProgress = false;
     }
 
@@ -532,11 +704,9 @@ void SAXParser::setEntityResolver(EntityResolver* const resolver)
     fEntityResolver = resolver;
     if (fEntityResolver) {
         fScanner->setEntityHandler(this);
-        fScanner->setEntityResolver(fEntityResolver);
     }
     else {
         fScanner->setEntityHandler(0);
-        fScanner->setEntityResolver(0);
     }
 }
 
@@ -546,8 +716,7 @@ void SAXParser::setEntityResolver(EntityResolver* const resolver)
 //  SAXParser: Progressive parse methods
 // ---------------------------------------------------------------------------
 bool SAXParser::parseFirst( const   XMLCh* const    systemId
-                            ,       XMLPScanToken&  toFill
-                            , const bool            reuseGrammar)
+                            ,       XMLPScanToken&  toFill)
 {
     //
     //  Avoid multiple entrance. We cannot enter here while a regular parse
@@ -556,12 +725,11 @@ bool SAXParser::parseFirst( const   XMLCh* const    systemId
     if (fParseInProgress)
         ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
 
-    return fScanner->scanFirst(systemId, toFill, reuseGrammar);
+    return fScanner->scanFirst(systemId, toFill);
 }
 
 bool SAXParser::parseFirst( const   char* const     systemId
-                            ,       XMLPScanToken&  toFill
-                            , const bool            reuseGrammar)
+                            ,       XMLPScanToken&  toFill)
 {
     //
     //  Avoid multiple entrance. We cannot enter here while a regular parse
@@ -570,12 +738,11 @@ bool SAXParser::parseFirst( const   char* const     systemId
     if (fParseInProgress)
         ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
 
-    return fScanner->scanFirst(systemId, toFill, reuseGrammar);
+    return fScanner->scanFirst(systemId, toFill);
 }
 
 bool SAXParser::parseFirst( const   InputSource&    source
-                            ,       XMLPScanToken&  toFill
-                            , const bool            reuseGrammar)
+                            ,       XMLPScanToken&  toFill)
 {
     //
     //  Avoid multiple entrance. We cannot enter here while a regular parse
@@ -584,7 +751,7 @@ bool SAXParser::parseFirst( const   InputSource&    source
     if (fParseInProgress)
         ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
 
-    return fScanner->scanFirst(source, toFill, reuseGrammar);
+    return fScanner->scanFirst(source, toFill);
 }
 
 bool SAXParser::parseNext(XMLPScanToken& token)
@@ -684,18 +851,35 @@ void SAXParser::endDocument()
 
 void SAXParser::endElement( const   XMLElementDecl& elemDecl
                             , const unsigned int    uriId
-                            , const bool            isRoot)
+                            , const bool            isRoot
+                            , const XMLCh* const    elemPrefix)
 {
     // Just map to the SAX document handler
-    if (fDocHandler)
-        fDocHandler->endElement(elemDecl.getFullName());
+    if (fDocHandler) {
+        if (fScanner->getDoNamespaces()) {
+
+            if (elemPrefix && *elemPrefix) {
+
+                fElemQNameBuf.set(elemPrefix);
+                fElemQNameBuf.append(chColon);
+                fElemQNameBuf.append(elemDecl.getBaseName());
+                fDocHandler->endElement(fElemQNameBuf.getRawBuffer());
+            }
+            else {
+                fDocHandler->endElement(elemDecl.getBaseName());
+            }
+        }
+        else
+            fDocHandler->endElement(elemDecl.getFullName());
+
+    }
 
     //
     //  If there are any installed advanced handlers, then lets call them
     //  with this info.
     //
     for (unsigned int index = 0; index < fAdvDHCount; index++)
-        fAdvDHList[index]->endElement(elemDecl, uriId, isRoot);
+        fAdvDHList[index]->endElement(elemDecl, uriId, isRoot, elemPrefix);
 
     //
     //  Dump the element depth down again. Don't let it underflow in case
@@ -790,11 +974,35 @@ startElement(   const   XMLElementDecl&         elemDecl
     if (fDocHandler)
     {
         fAttrList.setVector(&attrList, attrCount);
-        fDocHandler->startElement(elemDecl.getFullName(), fAttrList);
+        if (fScanner->getDoNamespaces()) {
 
-        // If its empty, send the end tag event now
-        if (isEmpty)
-            fDocHandler->endElement(elemDecl.getFullName());
+            if (elemPrefix && *elemPrefix) {
+
+                fElemQNameBuf.set(elemPrefix);
+                fElemQNameBuf.append(chColon);
+                fElemQNameBuf.append(elemDecl.getBaseName());
+                fDocHandler->startElement(fElemQNameBuf.getRawBuffer(), fAttrList);
+
+                // If its empty, send the end tag event now
+                if (isEmpty)
+                    fDocHandler->endElement(fElemQNameBuf.getRawBuffer());
+            }
+            else {
+
+                fDocHandler->startElement(elemDecl.getBaseName(), fAttrList);
+
+                // If its empty, send the end tag event now
+                if (isEmpty)
+                    fDocHandler->endElement(elemDecl.getBaseName());
+            }
+        }
+        else {
+            fDocHandler->startElement(elemDecl.getFullName(), fAttrList);
+
+            // If its empty, send the end tag event now
+            if (isEmpty)
+                fDocHandler->endElement(elemDecl.getFullName());
+        }
     }
 
     //
@@ -849,7 +1057,8 @@ void SAXParser::doctypeComment(const XMLCh* const)
 void SAXParser::doctypeDecl(const   DTDElementDecl& elemDecl
                             , const XMLCh* const    publicId
                             , const XMLCh* const    systemId
-                            , const bool            hasIntSubset)
+                            , const bool            hasIntSubset
+                            , const bool            hasExtSubset)
 {
     // Unused by SAX DTDHandler interface at this time
 }
@@ -981,8 +1190,8 @@ void SAXParser::error(  const   unsigned int                code
                         , const XMLCh* const                errorText
                         , const XMLCh* const                systemId
                         , const XMLCh* const                publicId
-                        , const unsigned int                lineNum
-                        , const unsigned int                colNum)
+                        , const XMLSSize_t                  lineNum
+                        , const XMLSSize_t                  colNum)
 {
     SAXParseException toThrow = SAXParseException
     (
@@ -1032,7 +1241,8 @@ void SAXParser::resetEntities()
 
 InputSource*
 SAXParser::resolveEntity(   const   XMLCh* const    publicId
-                            , const XMLCh* const    systemId)
+                            , const XMLCh* const    systemId
+                            , const XMLCh* const    baseURI)
 {
     // Just map to the SAX entity resolver handler
     if (fEntityResolver)
@@ -1072,3 +1282,90 @@ void SAXParser::setDoValidation(const bool newState)
         newState ? XMLScanner::Val_Always : XMLScanner::Val_Never
     );
 }
+
+
+// ---------------------------------------------------------------------------
+//  SAXParser: Grammar preparsing methods
+// ---------------------------------------------------------------------------
+Grammar* SAXParser::loadGrammar(const char* const systemId,
+                                const short grammarType,
+                                const bool toCache)
+{
+    // Avoid multiple entrance
+    if (fParseInProgress)
+        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+
+    Grammar* grammar = 0;
+    try
+    {
+        fParseInProgress = true;
+        grammar = fScanner->loadGrammar(systemId, grammarType, toCache);
+        fParseInProgress = false;
+    }
+
+    catch(...)
+    {
+        fParseInProgress = false;
+        throw;
+    }
+
+    return grammar;
+}
+
+Grammar* SAXParser::loadGrammar(const XMLCh* const systemId,
+                                const short grammarType,
+                                const bool toCache)
+{
+    // Avoid multiple entrance
+    if (fParseInProgress)
+        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+
+    Grammar* grammar = 0;
+    try
+    {
+        fParseInProgress = true;
+        grammar = fScanner->loadGrammar(systemId, grammarType, toCache);
+        fParseInProgress = false;
+    }
+
+    catch(...)
+    {
+        fParseInProgress = false;
+        throw;
+    }
+
+    return grammar;
+}
+
+Grammar* SAXParser::loadGrammar(const InputSource& source,
+                                const short grammarType,
+                                const bool toCache)
+{
+    // Avoid multiple entrance
+    if (fParseInProgress)
+        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+
+    Grammar* grammar = 0;
+    try
+    {
+        fParseInProgress = true;
+        grammar = fScanner->loadGrammar(source, grammarType, toCache);
+        fParseInProgress = false;
+    }
+
+    catch(...)
+    {
+        fParseInProgress = false;
+        throw;
+    }
+
+    return grammar;
+}
+
+void SAXParser::resetCachedGrammarPool()
+{
+    fGrammarResolver->resetCachedGrammar();
+}
+
+XERCES_CPP_NAMESPACE_END
+
