@@ -56,6 +56,16 @@
 
 /*
  * $Log: SAX2XMLReader.hpp,v $
+ * Revision 1.9  2003/01/09 19:07:27  tng
+ * [Bug 15802] Add "const" qualifier to getURIText.
+ *
+ * Revision 1.8  2002/12/23 15:23:18  knoaman
+ * Added a public api to various parsers to return the src offset within the input
+ * source.
+ *
+ * Revision 1.7  2002/11/04 14:55:45  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.6  2002/07/11 18:29:09  knoaman
  * Grammar caching/preparsing - initial implementation.
  *
@@ -156,6 +166,8 @@
 #include <xercesc/framework/XMLValidator.hpp>
 #include <xercesc/framework/XMLPScanToken.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 class ContentHandler ;
 class DTDHandler;
 class EntityResolver;
@@ -609,8 +621,15 @@ public:
       *
       * @param uriId id of the string in the URI string pool.
       * @return URI string corresponding to the URI id.
-      */    
-    virtual const XMLCh* getURIText(unsigned int uriId) = 0;
+      */
+    virtual const XMLCh* getURIText(unsigned int uriId) const = 0;
+
+    /**
+      * Returns the current src offset within the input source.
+      *
+      * @return offset within the input source
+      */
+    virtual unsigned int getSrcOffset() const = 0;
 
     //@}
 
@@ -983,4 +1002,6 @@ private :
 
 };
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
