@@ -56,6 +56,18 @@
 
 /*
  * $Log: PlatformUtils.cpp,v $
+ * Revision 1.6  2002/12/20 22:10:20  tng
+ * XML 1.1
+ *
+ * Revision 1.5  2002/11/08 21:06:16  peiyongz
+ * move comment to *hpp for API doc generation
+ *
+ * Revision 1.4  2002/11/04 22:24:21  peiyongz
+ * Locale setting for message loader
+ *
+ * Revision 1.3  2002/11/04 15:22:04  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.2  2002/07/18 20:05:31  knoaman
  * Add a new feature to control strict IANA encoding name.
  *
@@ -145,6 +157,7 @@
 #include <xercesc/util/XMLRegisterCleanup.hpp>
 #include <limits.h>
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  Local data members
@@ -160,7 +173,7 @@ static long                     gInitFlag = 0;
 //  Global data
 //
 //	gXMLCleanupList
-//		This is a list of cleanup functions to be called on 
+//		This is a list of cleanup functions to be called on
 //		XMLPlatformUtils::Terminate.  Their function is to reset static
 //		data in classes that use it.
 //
@@ -183,7 +196,7 @@ XMLTransService*    XMLPlatformUtils::fgTransService = 0;
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: Init/term methods
 // ---------------------------------------------------------------------------
-void XMLPlatformUtils::Initialize()
+void XMLPlatformUtils::Initialize(const char* const locale)
 {
     //
     //  Effects of overflow:
@@ -193,7 +206,7 @@ void XMLPlatformUtils::Initialize()
     //
     //  We got to prevent overflow from happening.
     //  no error or exception
-    //  
+    //
     if (gInitFlag == LONG_MAX)
         return;
 
@@ -257,6 +270,13 @@ void XMLPlatformUtils::Initialize()
     //  a zero pointer if this platform doesn't want to support this.
     //
     fgNetAccessor = makeNetAccessor();
+
+    /***
+     *  Locale setting for Message Loader
+     *
+     ***/
+    XMLMsgLoader::setLocale(locale);
+
 }
 
 
@@ -267,7 +287,7 @@ void XMLPlatformUtils::Terminate()
     // otherwise we come to delete non-existing resources.
     //
     //  no error or exception
-    //  
+    //
     if (gInitFlag == 0)
         return;
 
@@ -313,8 +333,16 @@ void XMLPlatformUtils::Terminate()
     //
     platformTerm();
 
+    /***
+     *  de-allocate resource
+     *
+     *  refer to discussion in the Initialize()
+     ***/
+    XMLMsgLoader::setLocale(0);
+
     // And say we are no longer initialized
     gInitFlag = 0;
+
 }
 
 
@@ -343,13 +371,13 @@ void XMLPlatformUtils::recognizeNEL(bool state) {
 
     if (state) {
 
-        if (!XMLReader::isNELRecognized()) {
-            XMLReader::enableNELWS();
+        if (!XMLChar1_0::isNELRecognized()) {
+            XMLChar1_0::enableNELWS();
         }
     }
     else {
 
-        if (XMLReader::isNELRecognized()) {
+        if (XMLChar1_0::isNELRecognized()) {
             ThrowXML(RuntimeException, XMLExcepts::NEL_RepeatedCalls);
         }
     }
@@ -358,7 +386,7 @@ void XMLPlatformUtils::recognizeNEL(bool state) {
 
 bool XMLPlatformUtils::isNELRecognized() {
 
-    return XMLReader::isNELRecognized();
+    return XMLChar1_0::isNELRecognized();
 }
 
 // ---------------------------------------------------------------------------
@@ -383,3 +411,4 @@ bool XMLPlatformUtils::isStrictIANAEncoding() {
     return false;
 }
 
+XERCES_CPP_NAMESPACE_END
