@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: AIXPlatformUtils.cpp,v 1.8 2002/07/15 21:52:32 peiyongz Exp $
+ * $Id: AIXPlatformUtils.cpp,v 1.10 2003/02/05 18:28:30 tng Exp $
  */
 
 
@@ -106,6 +106,8 @@
     #include <xercesc/util/NetAccessors/Socket/SocketNetAccessor.hpp>
 #endif
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: Platform init method
 // ---------------------------------------------------------------------------
@@ -283,7 +285,7 @@ XMLPlatformUtils::readFileBuffer(  FileHandle      theFile
 void
 XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
                                    , long                  toWrite
-                                   , const XMLByte* const  toFlush)                                   
+                                   , const XMLByte* const  toFlush)
 {
     if (!theFile        ||
         (toWrite <= 0 ) ||
@@ -501,7 +503,7 @@ void XMLPlatformUtils::closeMutex(void* const mtxHandle)
         ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotDestroy);
     }
     if ( (pthread_mutex_t*)mtxHandle)
-        delete mtxHandle;
+        delete (pthread_mutex_t*) mtxHandle;
 }
 void XMLPlatformUtils::lockMutex(void* const mtxHandle)
 {
@@ -651,3 +653,6 @@ void XMLPlatformUtils::platformInit()
 {
 }
 ********************* End of code attic *******************************/
+
+XERCES_CPP_NAMESPACE_END
+
