@@ -56,6 +56,15 @@
 
 /*
  * $Log: FreeBSDPlatformUtils.cpp,v $
+ * Revision 1.7  2002/12/12 16:28:48  peiyongz
+ * MsgCatalogLoader added.
+ *
+ * Revision 1.6  2002/12/02 19:16:26  tng
+ * [Bug 14723] Memory leak in atomicOpsMutex.  Patch from Adam Zell.
+ *
+ * Revision 1.5  2002/11/04 15:13:00  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.4  2002/07/05 21:29:58  peiyongz
  * Bug# 10250: patch from James Berry
  *
@@ -119,6 +128,8 @@
 
 #if defined(XML_USE_ICU_MESSAGELOADER)
     #include <xercesc/util/MsgLoaders/ICU/ICUMsgLoader.hpp>
+#elif defined (XML_USE_ICONV_MESSAGELOADER)
+    #include <xercesc/util/MsgLoaders/MsgCatalog/MsgCatalogLoader.hpp>
 #else
     // Same as -DXML_USE_INMEM_MESSAGELOADER
     #include <xercesc/util/MsgLoaders/InMemory/InMemMsgLoader.hpp>
@@ -129,6 +140,7 @@
     #include <xercesc/util/NetAccessors/Socket/SocketNetAccessor.hpp>
 #endif
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: Private Static Methods
@@ -157,6 +169,8 @@ XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
     {
 #if defined (XML_USE_ICU_MESSAGELOADER)
         retVal = new ICUMsgLoader(msgDomain);
+#elif defined (XML_USE_ICONV_MESSAGELOADER)
+        retVal = new MsgCatalogLoader(msgDomain);        
 #else
         // same as -DXML_USE_INMEM_MESSAGELOADER
         retVal = new InMemMsgLoader(msgDomain);
@@ -333,7 +347,7 @@ XMLPlatformUtils::readFileBuffer( FileHandle          theFile
     if (toRead == 0)
 	return 0;
     size_t noOfItemsRead = fread((void*) toFill, 1, toRead, (FILE*)theFile);
-    
+
     if(ferror((FILE*)theFile))
     {
         ThrowXML(XMLPlatformUtilsException,
@@ -578,8 +592,8 @@ void XMLPlatformUtils::platformInit()
     // Normally, mutexes are created on first use, but there is a
     // circular dependency between compareAndExchange() and
     // mutex creation that must be broken.
-
-    atomicOpsMutex.fHandle = XMLPlatformUtils::makeMutex();
+    if (atomicOpsMutex.fHandle == 0)
+        atomicOpsMutex.fHandle = XMLPlatformUtils::makeMutex();
 }
 
 void* XMLPlatformUtils::makeMutex()
@@ -723,3 +737,5 @@ void XMLPlatformUtils::platformTerm()
 	atomicOpsMutex.fHandle = 0;
 #endif
 }
+
+XERCES_CPP_NAMESPACE_END
