@@ -56,6 +56,22 @@
 
 /*
  * $Log: IRIXPlatformUtils.cpp,v $
+ * Revision 1.7  2003/02/05 18:28:21  tng
+ * [Bug 13437] Incorrect memory management in XXXPlatformUtils.cpp.
+ *
+ * Revision 1.6  2002/12/12 16:28:55  peiyongz
+ * MsgCatalogLoader added.
+ *
+ * Revision 1.5  2002/12/02 19:16:38  tng
+ * [Bug 14723] Memory leak in atomicOpsMutex.  Patch from Adam Zell.
+ *
+ * Revision 1.4  2002/11/15 21:05:45  tng
+ * [Bug 14598] IRIX 6.5 / g++ 3.0.4 compilation bugs.  Patch from Richard Balint
+ * .
+ *
+ * Revision 1.3  2002/11/04 15:13:00  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.2  2002/05/21 20:31:47  tng
  * Minor update: Remove obsolete code
  *
@@ -152,6 +168,8 @@
 
 #if defined(XML_USE_ICU_MESSAGELOADER)
     #include <xercesc/util/MsgLoaders/ICU/ICUMsgLoader.hpp>
+#elif defined (XML_USE_ICONV_MESSAGELOADER)
+    #include <xercesc/util/MsgLoaders/MsgCatalog/MsgCatalogLoader.hpp>
 #else
     // Same as -DXML_USE_INMEM_MESSAGELOADER
     #include <xercesc/util/MsgLoaders/InMemory/InMemMsgLoader.hpp>
@@ -162,6 +180,7 @@
     #include <xercesc/util/NetAccessors/Socket/SocketNetAccessor.hpp>
 #endif
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: Private Static Methods
@@ -188,6 +207,8 @@ XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
     {
 #if defined (XML_USE_ICU_MESSAGELOADER)
         retVal = new ICUMsgLoader(msgDomain);
+#elif defined (XML_USE_ICONV_MESSAGELOADER)
+        retVal = new MsgCatalogLoader(msgDomain);
 #else
         // same as -DXML_USE_INMEM_MESSAGELOADER
         retVal = new InMemMsgLoader(msgDomain);
@@ -315,10 +336,24 @@ FileHandle XMLPlatformUtils::openFile(const char* const fileName)
 
     if (retVal == NULL)
         return 0;
+
     return retVal;
 }
 
 
+FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
+{
+    const char* tmpFileName = XMLString::transcode(fileName);
+    ArrayJanitor<char> janText((char*)tmpFileName);
+    return fopen( tmpFileName , "wb" );
+}
+
+FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName)
+{
+    return fopen( fileName , "wb" );
+}
+
+
 FileHandle XMLPlatformUtils::openStdInHandle()
 {
         return (FileHandle)fdopen(dup(0), "rb");
@@ -339,10 +374,47 @@ XMLPlatformUtils::readFileBuffer( FileHandle          theFile
                  XMLExcepts::File_CouldNotReadFromFile);
     }
 
+
     return (unsigned int)noOfItemsRead;
 }
 
 
+void
+XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
+                                   , long                  toWrite
+                                   , const XMLByte* const  toFlush)
+{
+    if (!theFile        ||
+        (toWrite <= 0 ) ||
+        !toFlush         )
+        return;
+
+    const XMLByte* tmpFlush = (const XMLByte*) toFlush;
+    size_t bytesWritten = 0;
+
+    while (true)
+    {
+        bytesWritten=fwrite(tmpFlush, sizeof(XMLByte), toWrite, (FILE*)theFile);
+
+        if(ferror((FILE*)theFile))
+        {
+            ThrowXML(XMLPlatformUtilsException,XMLExcepts::File_CouldNotWriteToFile);
+        }
+
+        if (bytesWritten < toWrite) //incomplete write
+        {
+            tmpFlush+=bytesWritten;
+            toWrite-=bytesWritten;
+            bytesWritten=0;
+        }
+        else
+            return;
+    }
+
+    return;
+}
+
+
 void XMLPlatformUtils::resetFile(FileHandle theFile)
 {
     // Seek to the start of the file
@@ -530,8 +602,8 @@ static XMLMutex atomicOpsMutex;
 //  XMLPlatformUtils: Platform init method
 // ---------------------------------------------------------------------------
 
-static char* arenaName = 0;
-static usptr_t* arena = 0;
+static char* arenaName = NULL;
+static usptr_t* arena = NULL;
 
 void XMLPlatformUtils::platformInit()
 {
@@ -549,14 +621,21 @@ void XMLPlatformUtils::platformInit()
     arenaName = strdup ("/var/tmp/xerces-sharedmemXXXXXX");
     arena = usinit (mktemp (arenaName));
 
-    atomicOpsMutex.fHandle = XMLPlatformUtils::makeMutex();
+    if (atomicOpsMutex.fHandle == 0)
+        atomicOpsMutex.fHandle = XMLPlatformUtils::makeMutex();
 }
 
 void XMLPlatformUtils::platformTerm()
 {
+    // delete the mutex we created
+    closeMutex(atomicOpsMutex.fHandle);
+    atomicOpsMutex.fHandle = 0;
+
     usdetach (arena);
     unlink (arenaName);
     free (arenaName);
+
+    arena = NULL;
     // We don't have any termination requirements at this time
 }
 
@@ -585,8 +664,9 @@ void* XMLPlatformUtils::makeMutex()
 void XMLPlatformUtils::closeMutex(void* const mtxHandle)
 {
 
-    if (mtxHandle != NULL) {
-        usfreesema (mtxHandle, arena);
+    if ((mtxHandle != NULL) && (arena != NULL)) {
+
+        usfreesema ((usema_t *)mtxHandle, arena);
         // never returns anything testable for failure, so nothing
         // to throw an exception about.
     }
@@ -630,7 +710,8 @@ void XMLPlatformUtils::platformInit()
     // Normally, mutexes are created on first use, but there is a
     // circular dependency between compareAndExchange() and
     // mutex creation that must be broken.
-    atomicOpsMutex.fHandle = XMLPlatformUtils::makeMutex();
+    if (atomicOpsMutex.fHandle == 0)
+        atomicOpsMutex.fHandle = XMLPlatformUtils::makeMutex();
 }
 
 void XMLPlatformUtils::platformTerm()
@@ -666,7 +747,7 @@ void XMLPlatformUtils::closeMutex(void* const mtxHandle)
             ThrowXML(XMLPlatformUtilsException,
                      XMLExcepts::Mutex_CouldNotDestroy);
         }
-        delete mtxHandle;
+        delete (pthread_mutex_t*) mtxHandle;
     }
 }
 
@@ -784,4 +865,5 @@ void XMLPlatformUtils::platformTerm()
     // We don't have any termination requirements at this time
 //}
 
+XERCES_CPP_NAMESPACE_END
 
