@@ -56,6 +56,33 @@
 
 /*
  * $Log: LinuxPlatformUtils.cpp,v $
+ * Revision 1.11  2003/02/05 18:27:34  tng
+ * [Bug 13437] Incorrect memory management in LinuxPlatformUtils.cpp.  Fixed by Adam Zell.
+ *
+ * Revision 1.10  2002/12/02 19:16:46  tng
+ * [Bug 14723] Memory leak in atomicOpsMutex.  Patch from Adam Zell.
+ *
+ * Revision 1.9  2002/11/07 22:38:13  peiyongz
+ * build -miconv on hpux and linux
+ *
+ * Revision 1.8  2002/11/04 15:13:00  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.7  2002/08/19 19:38:17  tng
+ * [Bug 11771] Linux specific IconvGNU transcoder.  Patch from Vasily Tchekalkin.
+ *
+ * Revision 1.6  2002/07/15 21:53:04  peiyongz
+ * CouldNotWriteToFile
+ *
+ * Revision 1.5  2002/06/24 15:55:21  peiyongz
+ * Change from James Berry( jberry@criticalpath.com)
+ *
+ * Revision 1.4  2002/06/20 15:28:08  peiyongz
+ * Fix to compilation error
+ *
+ * Revision 1.2  2002/05/21 20:31:47  tng
+ * Minor update: Remove obsolete code
+ *
  * Revision 1.1.1.1  2002/02/01 22:22:24  peiyongz
  * sane_include
  *
@@ -177,6 +204,8 @@
 
 #if defined(XML_USE_ICU_TRANSCODER)
     #include <xercesc/util/Transcoders/ICU/ICUTransService.hpp>
+#elif defined (XML_USE_GNU_TRANSCODER)
+    #include <xercesc/util/Transcoders/IconvGNU/IconvGNUTransService.hpp>
 #else
     // Use native transcoder. Same as -DXML_USE_NATIVE_TRANSCODER
     #include <xercesc/util/Transcoders/Iconv/IconvTransService.hpp>
@@ -185,6 +214,8 @@
 
 #if defined(XML_USE_ICU_MESSAGELOADER)
     #include <xercesc/util/MsgLoaders/ICU/ICUMsgLoader.hpp>
+#elif defined (XML_USE_ICONV_MESSAGELOADER)
+    #include <xercesc/util/MsgLoaders/MsgCatalog/MsgCatalogLoader.hpp>
 #else
     // Same as -DXML_USE_INMEM_MESSAGELOADER
     #include <xercesc/util/MsgLoaders/InMemory/InMemMsgLoader.hpp>
@@ -196,42 +227,12 @@
 #endif
 
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
-//  Local Methods
+//  XMLPlatformUtils: Private Static Methods
 // ---------------------------------------------------------------------------
 
-static void WriteCharStr( FILE* stream, const char* const toWrite)
-{
-    if (fputs(toWrite, stream) == EOF)
-    {
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::Strm_StdErrWriteFailure);
-    }
-}
-
-static void WriteUStrStdErr( const XMLCh* const toWrite)
-{
-    char* tmpVal = XMLString::transcode(toWrite);
-    ArrayJanitor<char> janText(tmpVal);
-    if (fputs(tmpVal, stderr) == EOF)
-    {
-       ThrowXML(XMLPlatformUtilsException,
-                XMLExcepts::Strm_StdErrWriteFailure);
-   }
-}
-
-static void WriteUStrStdOut( const XMLCh* const toWrite)
- {
-    char* tmpVal = XMLString::transcode(toWrite);
-    ArrayJanitor<char> janText(tmpVal);
-    if (fputs(tmpVal, stdout) == EOF)
-    {
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::Strm_StdOutWriteFailure);
-    }
-}
-
 XMLNetAccessor* XMLPlatformUtils::makeNetAccessor()
 {
 #if defined (XML_USE_NETACCESSOR_SOCKET)
@@ -243,6 +244,275 @@ XMLNetAccessor* XMLPlatformUtils::makeNetAccessor()
 
 
 
+//
+//  This method is called by the platform independent part of this class
+//  when client code asks to have one of the supported message sets loaded.
+//
+
+XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
+{
+    XMLMsgLoader* retVal;
+    try
+    {
+#if defined (XML_USE_ICU_MESSAGELOADER)
+        retVal = new ICUMsgLoader(msgDomain);
+#elif defined (XML_USE_ICONV_MESSAGELOADER)
+        retVal = new MsgCatalogLoader(msgDomain);
+#else
+        // same as -DXML_USE_INMEM_MESSAGELOADER
+        retVal = new InMemMsgLoader(msgDomain);
+#endif
+    }
+
+    catch(...)
+    {
+        panic(XMLPlatformUtils::Panic_CantLoadMsgDomain);
+    }
+    return retVal;
+}
+
+
+//
+//  This method is called very early in the bootstrapping process. This guy
+//  must create a transcoding service and return it. It cannot use any string
+//  methods, any transcoding services, throw any exceptions, etc... It just
+//  makes a transcoding service and returns it, or returns zero on failure.
+//
+
+XMLTransService* XMLPlatformUtils::makeTransService()
+{
+#if defined (XML_USE_ICU_TRANSCODER)
+    // Use ICU transcoding services.
+    // same as -DXML_USE_ICU_MESSAGELOADER
+    return new ICUTransService;
+#elif defined (XML_USE_GNU_TRANSCODER)
+    return new IconvGNUTransService;
+#else
+    // Use native transcoding services.
+    // same as -DXML_USE_NATIVE_TRANSCODER
+    return new IconvTransService;
+
+#endif
+}
+
+// ---------------------------------------------------------------------------
+//  XMLPlatformUtils: The panic method
+// ---------------------------------------------------------------------------
+void XMLPlatformUtils::panic(const PanicReasons reason)
+{
+     const char* reasonStr = "Unknown reason";
+    if (reason == Panic_NoTransService)
+        reasonStr = "Could not load a transcoding service";
+    else if (reason == Panic_NoDefTranscoder)
+        reasonStr = "Could not load a local code page transcoder";
+    else if (reason == Panic_CantFindLib)
+        reasonStr = "Could not find the xerces-c DLL";
+    else if (reason == Panic_UnknownMsgDomain)
+        reasonStr = "Unknown message domain";
+    else if (reason == Panic_CantLoadMsgDomain)
+        reasonStr = "Cannot load message domain";
+    else if (reason == Panic_SynchronizationErr)
+        reasonStr = "Cannot synchronize system or mutex";
+    else if (reason == Panic_SystemInit)
+        reasonStr = "Cannot initialize the system or mutex";
+
+    fprintf(stderr, "%s\n", reasonStr);
+
+    exit(-1);
+}
+
+
+// ---------------------------------------------------------------------------
+//  XMLPlatformUtils: File Methods
+// ---------------------------------------------------------------------------
+
+unsigned int XMLPlatformUtils::curFilePos(FileHandle theFile)
+{
+    int curPos = ftell( (FILE*)theFile);
+    if (curPos == -1)
+        ThrowXML(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetSize);
+
+    return (unsigned int)curPos;
+}
+
+void XMLPlatformUtils::closeFile(FileHandle theFile)
+{
+    if (fclose((FILE*)theFile))
+        ThrowXML(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotCloseFile);
+}
+
+unsigned int XMLPlatformUtils::fileSize(FileHandle theFile)
+{
+    // Get the current position
+    long  int curPos = ftell((FILE*) theFile);
+    if (curPos == -1)
+        ThrowXML(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetCurPos);
+
+    // Seek to the end and save that value for return
+     if (fseek((FILE*) theFile, 0, SEEK_END))
+        ThrowXML(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotSeekToEnd);
+
+    long int retVal = ftell((FILE*)theFile);
+    if (retVal == -1)
+        ThrowXML(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotSeekToEnd);
+
+    // And put the pointer back
+
+    if (fseek( (FILE*)theFile, curPos, SEEK_SET) )
+        ThrowXML(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotSeekToPos);
+
+    return (unsigned int)retVal;
+}
+
+FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
+{
+    const char* tmpFileName = XMLString::transcode(fileName);
+    ArrayJanitor<char> janText((char*)tmpFileName);
+    FileHandle retVal = (FILE*)fopen( tmpFileName , "rb" );
+
+    if (retVal == NULL)
+        return 0;
+    return retVal;
+}
+
+FileHandle XMLPlatformUtils::openFile(const char* const fileName)
+{
+    FileHandle retVal = (FILE*)fopen( fileName , "rb" );
+
+    if (retVal == NULL)
+        return 0;
+    return retVal;
+}
+
+FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
+{
+    const char* tmpFileName = XMLString::transcode(fileName);
+    ArrayJanitor<char> janText((char*)tmpFileName);
+    return fopen( tmpFileName , "wb" );
+}
+
+FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName)
+{
+    return fopen( fileName , "wb" );
+}
+
+FileHandle XMLPlatformUtils::openStdInHandle()
+{
+    return (FileHandle)fdopen(dup(0), "rb");
+}
+
+unsigned int
+XMLPlatformUtils::readFileBuffer( FileHandle          theFile
+                                , const unsigned int  toRead
+                                , XMLByte* const      toFill)
+{
+    size_t noOfItemsRead = fread((void*) toFill, 1, toRead, (FILE*)theFile);
+
+    if(ferror((FILE*)theFile))
+    {
+        ThrowXML(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotReadFromFile);
+    }
+
+    return (unsigned int)noOfItemsRead;
+}
+
+void
+XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
+                                   , long                  toWrite
+                                   , const XMLByte* const  toFlush)
+{
+    if (!theFile        ||
+        (toWrite <= 0 ) ||
+        !toFlush         )
+        return;
+
+    const XMLByte* tmpFlush = (const XMLByte*) toFlush;
+    size_t bytesWritten = 0;
+
+    while (true)
+    {
+        bytesWritten=fwrite(tmpFlush, sizeof(XMLByte), toWrite, (FILE*)theFile);
+
+        if(ferror((FILE*)theFile))
+        {
+            ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile);
+        }
+
+        if (bytesWritten < toWrite) //incomplete write
+        {
+            tmpFlush+=bytesWritten;
+            toWrite-=bytesWritten;
+            bytesWritten=0;
+        }
+        else
+            return;
+    }
+
+    return;
+}
+
+void XMLPlatformUtils::resetFile(FileHandle theFile)
+{
+    // Seek to the start of the file
+    if (fseek((FILE*)theFile, 0, SEEK_SET))
+        ThrowXML(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotResetFile);
+}
+
+
+// ---------------------------------------------------------------------------
+//  XMLPlatformUtils: File system methods
+// ---------------------------------------------------------------------------
+XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
+{
+
+    //
+    //  NOTE: THe path provided has always already been opened successfully,
+    //  so we know that its not some pathological freaky path. It comes in
+    //  in native format, and goes out as Unicode always
+    //
+    char* newSrc = XMLString::transcode(srcPath);
+    ArrayJanitor<char> janText(newSrc);
+
+    // Use a local buffer that is big enough for the largest legal path
+    char *absPath = new char[PATH_MAX];
+    // get the absolute path
+    char* retPath = realpath(newSrc, absPath);
+    ArrayJanitor<char> janText2(retPath);
+
+    if (!retPath)
+    {
+        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetBasePathName);
+    }
+    return XMLString::transcode(absPath);
+}
+
+bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
+{
+    // Check for pathological case of empty path
+    if (!toCheck[0])
+        return false;
+
+    //
+    //  If it starts with a slash, then it cannot be relative. This covers
+    //  both something like "\Test\File.xml" and an NT Lan type remote path
+    //  that starts with a node like "\\MyNode\Test\File.xml".
+    //
+    if (toCheck[0] == XMLCh('/'))
+        return false;
+
+    // Else assume its a relative path
+    return true;
+}
+
+
 XMLCh* XMLPlatformUtils::weavePaths(const   XMLCh* const    basePath
                                     , const XMLCh* const    relativePath)
 
@@ -350,186 +620,6 @@ XMLCh* XMLPlatformUtils::weavePaths(const   XMLCh* const    basePath
 }
 
 
-// ---------------------------------------------------------------------------
-//  XMLPlatformUtils: Private Static Methods
-// ---------------------------------------------------------------------------
-
-//
-//  This method is called by the platform independent part of this class
-//  when client code asks to have one of the supported message sets loaded.
-//
-
-XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
-{
-    XMLMsgLoader* retVal;
-    try
-    {
-#if defined (XML_USE_ICU_MESSAGELOADER)
-        retVal = new ICUMsgLoader(msgDomain);
-#else
-        // same as -DXML_USE_INMEM_MESSAGELOADER
-        retVal = new InMemMsgLoader(msgDomain);
-#endif
-    }
-
-    catch(...)
-    {
-        panic(XMLPlatformUtils::Panic_CantLoadMsgDomain);
-    }
-    return retVal;
-}
-
-
-//
-//  This method is called very early in the bootstrapping process. This guy
-//  must create a transcoding service and return it. It cannot use any string
-//  methods, any transcoding services, throw any exceptions, etc... It just
-//  makes a transcoding service and returns it, or returns zero on failure.
-//
-
-XMLTransService* XMLPlatformUtils::makeTransService()
-{
-#if defined (XML_USE_ICU_TRANSCODER)
-    // Use ICU transcoding services.
-    // same as -DXML_USE_ICU_MESSAGELOADER
-    return new ICUTransService;
-#else
-    // Use native transcoding services.
-    // same as -DXML_USE_INMEM_MESSAGELOADER
-    return new IconvTransService;
-
-#endif
-}
-
-// ---------------------------------------------------------------------------
-//  XMLPlatformUtils: The panic method
-// ---------------------------------------------------------------------------
-void XMLPlatformUtils::panic(const PanicReasons reason)
-{
-     const char* reasonStr = "Unknown reason";
-    if (reason == Panic_NoTransService)
-        reasonStr = "Could not load a transcoding service";
-    else if (reason == Panic_NoDefTranscoder)
-        reasonStr = "Could not load a local code page transcoder";
-    else if (reason == Panic_CantFindLib)
-        reasonStr = "Could not find the xerces-c DLL";
-    else if (reason == Panic_UnknownMsgDomain)
-        reasonStr = "Unknown message domain";
-    else if (reason == Panic_CantLoadMsgDomain)
-        reasonStr = "Cannot load message domain";
-    else if (reason == Panic_SynchronizationErr)
-        reasonStr = "Cannot synchronize system or mutex";
-    else if (reason == Panic_SystemInit)
-        reasonStr = "Cannot initialize the system or mutex";
-
-    fprintf(stderr, "%s\n", reasonStr);
-
-    exit(-1);
-}
-
-
-// ---------------------------------------------------------------------------
-//  XMLPlatformUtils: File Methods
-// ---------------------------------------------------------------------------
-
-unsigned int XMLPlatformUtils::curFilePos(FileHandle theFile)
-{
-    int curPos = ftell( (FILE*)theFile);
-    if (curPos == -1)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotGetSize);
-
-    return (unsigned int)curPos;
-}
-
-void XMLPlatformUtils::closeFile(FileHandle theFile)
-{
-    if (fclose((FILE*)theFile))
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotCloseFile);
-}
-
-unsigned int XMLPlatformUtils::fileSize(FileHandle theFile)
-{
-    // Get the current position
-    long  int curPos = ftell((FILE*) theFile);
-    if (curPos == -1)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotGetCurPos);
-
-    // Seek to the end and save that value for return
-     if (fseek((FILE*) theFile, 0, SEEK_END))
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotSeekToEnd);
-
-    long int retVal = ftell((FILE*)theFile);
-    if (retVal == -1)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotSeekToEnd);
-
-    // And put the pointer back
-
-    if (fseek( (FILE*)theFile, curPos, SEEK_SET) )
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotSeekToPos);
-
-    return (unsigned int)retVal;
-}
-
-FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
-{
-    const char* tmpFileName = XMLString::transcode(fileName);
-    ArrayJanitor<char> janText((char*)tmpFileName);
-    FileHandle retVal = (FILE*)fopen( tmpFileName , "rb" );
-
-    if (retVal == NULL)
-        return 0;
-    return retVal;
-}
-
-FileHandle XMLPlatformUtils::openFile(const char* const fileName)
-{
-    FileHandle retVal = (FILE*)fopen( fileName , "rb" );
-
-    if (retVal == NULL)
-        return 0;
-    return retVal;
-}
-
-
-FileHandle XMLPlatformUtils::openStdInHandle()
-{
-        return (FileHandle)fdopen(dup(0), "rb");
-}
-
-
-
-unsigned int
-XMLPlatformUtils::readFileBuffer( FileHandle          theFile
-                                , const unsigned int  toRead
-                                , XMLByte* const      toFill)
-{
-    size_t noOfItemsRead = fread((void*) toFill, 1, toRead, (FILE*)theFile);
-
-    if(ferror((FILE*)theFile))
-    {
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotReadFromFile);
-    }
-
-    return (unsigned int)noOfItemsRead;
-}
-
-
-void XMLPlatformUtils::resetFile(FileHandle theFile)
-{
-    // Seek to the start of the file
-    if (fseek((FILE*)theFile, 0, SEEK_SET))
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotResetFile);
-}
-
-
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: Timing Methods
 // ---------------------------------------------------------------------------
@@ -542,49 +632,6 @@ unsigned long XMLPlatformUtils::getCurrentMillis()
 
 }
 
-XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
-{
-
-    //
-    //  NOTE: THe path provided has always already been opened successfully,
-    //  so we know that its not some pathological freaky path. It comes in
-    //  in native format, and goes out as Unicode always
-    //
-    char* newSrc = XMLString::transcode(srcPath);
-    ArrayJanitor<char> janText(newSrc);
-
-    // Use a local buffer that is big enough for the largest legal path
-    char *absPath = new char[PATH_MAX];
-    // get the absolute path
-    char* retPath = realpath(newSrc, absPath);
-    ArrayJanitor<char> janText2(retPath);
-
-    if (!retPath)
-    {
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetBasePathName);
-    }
-    return XMLString::transcode(absPath);
-}
-
-bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
-{
-    // Check for pathological case of empty path
-    if (!toCheck[0])
-        return false;
-
-    //
-    //  If it starts with a slash, then it cannot be relative. This covers
-    //  both something like "\Test\File.xml" and an NT Lan type remote path
-    //  that starts with a node like "\\MyNode\Test\File.xml".
-    //
-    if (toCheck[0] == XMLCh('/'))
-        return false;
-
-    // Else assume its a relative path
-    return true;
-}
-
-
 // -----------------------------------------------------------------------
 //  Mutex methods
 // -----------------------------------------------------------------------
@@ -604,8 +651,8 @@ void XMLPlatformUtils::platformInit()
     // Normally, mutexes are created on first use, but there is a
     // circular dependency between compareAndExchange() and
     // mutex creation that must be broken.
-
-    atomicOpsMutex.fHandle = XMLPlatformUtils::makeMutex();
+    if (atomicOpsMutex.fHandle == 0)
+        atomicOpsMutex.fHandle = XMLPlatformUtils::makeMutex();
 }
 
 void* XMLPlatformUtils::makeMutex()
@@ -634,7 +681,7 @@ void XMLPlatformUtils::closeMutex(void* const mtxHandle)
             ThrowXML(XMLPlatformUtilsException,
                      XMLExcepts::Mutex_CouldNotDestroy);
         }
-        delete mtxHandle;
+        delete (pthread_mutex_t*) mtxHandle;
     }
 }
 
@@ -750,3 +797,5 @@ void XMLPlatformUtils::platformTerm()
 #endif
 }
 
+XERCES_CPP_NAMESPACE_END
+
