@@ -55,29 +55,43 @@
  */
 
 /*
- * $Id: MacOSPlatformUtils.cpp,v 1.2 2002/02/18 06:32:51 jberry Exp $
+ * $Id: MacOSPlatformUtils.cpp,v 1.10 2002/12/06 16:41:10 tng Exp $
  */
 
 
-// ---------------------------------------------------------------------------
-// Note:
-//   Xerces is not officially supported on Macintosh. This file was sent
-//   in by one of the Macintosh users and is included in the distribution
-//   just for convenience. Please send any defects / modification
-//   reports to xml4c@us.ibm.com
-// ---------------------------------------------------------------------------
-
-
 // ---------------------------------------------------------------------------
 //  Includes
 // ---------------------------------------------------------------------------
+#include <cstring>
+#include <cstdlib>
+#include <cctype>
+#include <cstdio>
+#include <memory>
+#include <algorithm>
+
+#if defined(__APPLE__)
+    //	Include from Frameworks Headers under ProjectBuilder
+    #include <Carbon/Carbon.h>
+#else
+    //	Classic include styles
+    #include <Files.h>
+    #include <Gestalt.h>
+    #include <TextUtils.h>
+    #include <TextEncodingConverter.h>
+    #include <Multiprocessing.h>
+    #include <DriverSynchronization.h>
+    #include <DriverServices.h>
+    #include <CFString.h>
+    #include <URLAccess.h>
+#endif
+
+#include <xercesc/util/XercesDefs.hpp>
 #include <xercesc/util/Janitor.hpp>
 #include <xercesc/util/PlatformUtils.hpp>
 #include <xercesc/util/RuntimeException.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/util/XMLUni.hpp>
 #include <xercesc/util/XMLString.hpp>
-#include <xercesc/util/Platforms/MacOS/MacOSDefs.hpp>
 #include <xercesc/util/Platforms/MacOS/MacOSPlatformUtils.hpp>
 
 #if (defined(XML_USE_INMEMORY_MSGLOADER) || defined(XML_USE_INMEM_MESSAGELOADER))
@@ -100,31 +114,7 @@
    #include <xercesc/util/NetAccessors/MacOSURLAccess/MacOSURLAccess.hpp>
 #endif
 
-
-#include <cstring>
-#include <cstdlib>
-#include <cctype>
-#include <cstdio>
-#include <memory>
-#include <algorithm>
-
-#if defined(XML_MACOSX)
-    //	Include from Frameworks Headers under ProjectBuilder
-    #include <Carbon/Carbon.h>
-#else
-    //	Classic include styles
-    #include <Files.h>
-    #include <Gestalt.h>
-    #include <TextUtils.h>
-    #include <TextEncodingConverter.h>
-    #include <Multiprocessing.h>
-    #include <DriverSynchronization.h>
-    #include <DriverServices.h>
-    #include <CFString.h>
-    #include <URLAccess.h>
-#endif
-
-
+XERCES_CPP_NAMESPACE_BEGIN
 
 //----------------------------------------------------------------------------
 //	Local Constants
@@ -200,10 +190,10 @@ XMLMacFile::currPos()
 {
     OSErr err = noErr;
     unsigned int pos = 0;
-    
+
     if (!mFileValid)
         ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetCurPos);
-    
+
     if (gHasHFSPlusAPIs)
     {
         SInt64 bigPos = 0;
@@ -218,10 +208,10 @@ XMLMacFile::currPos()
         if (err == noErr)
             pos = longPos;
     }
-    
+
     if (err != noErr)
         ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetCurPos);
-    
+
     return pos;
 }
 
@@ -232,14 +222,14 @@ XMLMacFile::close()
     OSErr err = noErr;
     if (!mFileValid)
         ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotCloseFile);
-    
+
     if (gHasHFSPlusAPIs)
         err = FSCloseFork(mFileRefNum);
     else
         err = FSClose(mFileRefNum);
-    
+
     mFileValid = false;
-    
+
     if (err != noErr)
         ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotCloseFile);
 }
@@ -250,10 +240,10 @@ XMLMacFile::size()
 {
     OSErr err = noErr;
     unsigned int len = 0;
-    
+
     if (!mFileValid)
         ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetSize);
-    
+
     if (gHasHFSPlusAPIs)
     {
         SInt64 bigLen = 0;
@@ -268,61 +258,172 @@ XMLMacFile::size()
         if (err == noErr)
             len = longLen;
     }
-    
+
     if (err != noErr)
         ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetSize);
-    
+
     return len;
 }
 
 
 void
-XMLMacFile::open(const XMLCh* const fileName)
+XMLMacFile::openWithPermission(const XMLCh* const fileName, int macPermission)
 {
     OSErr err = noErr;
-    
+
     if (mFileValid)
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile);
-    
+        ThrowXML1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, fileName);
+
     if (gHasHFSPlusAPIs)
     {
         FSRef ref;
         if (!XMLParsePathToFSRef(fileName, ref))
             err = fnfErr;
-        
+
         HFSUniStr255 forkName;
         if (err == noErr)
             err = FSGetDataForkName(&forkName);
-        
+
         if (err == noErr)
-            err = FSOpenFork(&ref, forkName.length, forkName.unicode, fsRdPerm, &mFileRefNum);
+            err = FSOpenFork(&ref, forkName.length, forkName.unicode, macPermission, &mFileRefNum);
     }
     else
     {
         FSSpec spec;
         if (!XMLParsePathToFSSpec(fileName, spec))
             err = fnfErr;
-        
+
         if (err == noErr)
-            err = FSpOpenDF(&spec, fsRdPerm, &mFileRefNum);
+            err = FSpOpenDF(&spec, macPermission, &mFileRefNum);
     }
-    
+
     if (err != noErr)
         ThrowXML1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, fileName);
-    
+
     mFileValid = true;
 }
 
 
+void
+XMLMacFile::create(const XMLCh* const filePath)
+{
+    OSErr err = noErr;
+
+    //	Split path into directory and filename components
+    int posSlash = XMLString::lastIndexOf(filePath, '/', XMLString::stringLen(filePath) - 1);
+    int posName = (posSlash == -1) ? 0 : posSlash+1;
+
+    const XMLCh* namePtr = filePath + posName;
+    int nameLen = XMLString::stringLen(namePtr);
+
+    //	Make a temporary string of the directory
+    ArrayJanitor<XMLCh> dirPath(new XMLCh[namePtr - filePath + 1]);
+    XMLString::subString(dirPath.get(), filePath, 0, posName);
+
+    //	Create the file as appropriate for API set
+    if (gHasHFSPlusAPIs)
+    {
+    	//	HFS+
+        FSRef ref;
+
+        //	If we find an existing file, delete it
+        if (XMLParsePathToFSRef(filePath, ref))
+            FSDeleteObject(&ref);
+
+        //	Get a ref to the parent directory
+        if (!XMLParsePathToFSRef(dirPath.get(), ref))
+            err = fnfErr;
+
+        //	Create a new file using the unicode name
+        if (err == noErr)
+        {
+            UniChar uniName[256];
+            err = FSCreateFileUnicode(
+                    &ref,
+                    nameLen, CopyXMLChsToUniChars(namePtr, uniName, nameLen, sizeof(uniName)),
+                    0, NULL, NULL, NULL);
+        }
+    }
+    else
+    {
+    	//	HFS
+        FSSpec spec;
+
+        //	If we find an existing file, delete it
+        if (XMLParsePathToFSSpec(filePath, spec))
+            FSpDelete(&spec);
+
+        //	Get a spec to the parent directory
+        if (!XMLParsePathToFSSpec(dirPath.get(), spec))
+            err = fnfErr;
+
+        //	Check that the new name is not too long for HFS
+        if (err == noErr && nameLen > 31)
+            err = errFSNameTooLong;
+
+        if (err == noErr)
+        {
+            //	Transcode the unicode name to native encoding
+            ArrayJanitor<const char> nativeName(XMLString::transcode(namePtr));
+
+            // Make a partial pathname from our current spec (parent directory) to the new file
+            unsigned char name[31 * 2 + 1 * 2 + 1];
+            unsigned char* partial = &name[1];
+
+            *partial++ = ':';      			 // Partial leads with :
+            const unsigned char* specName = spec.name;	// Copy in spec name
+            for (int specCnt = *specName++; specCnt > 0; --specCnt)
+                *partial++ = *specName++;
+
+            *partial++ = ':';      			 // Path component separator
+            char c;
+            for (const char* p = nativeName.get(); (c = *p++) != 0; ) // Copy in new element
+                *partial++ = (c == ':') ? '/' : c;		// Convert : to /
+
+            name[0] = partial - &name[1];   // Set the pascal string name length
+
+            //	Update the spec: this will probably return fnfErr
+            //					 (since we just deleted any existing file)
+            err = FSMakeFSSpec(spec.vRefNum, spec.parID, name, &spec);
+
+            //	Create the file from the spec
+            err = FSpCreate(&spec, '??\??', 'TEXT', smSystemScript);
+        }
+    }
+
+    //	Fail if we didn't create the file
+    if (err != noErr)
+    {
+        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotReadFromFile);
+        //ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile);
+    }
+}
+
+
+void
+XMLMacFile::open(const XMLCh* const fileName)
+{
+    openWithPermission(fileName, fsRdPerm);
+}
+
+
+void
+XMLMacFile::openFileToWrite(const XMLCh* const fileName)
+{
+    create(fileName);
+    openWithPermission(fileName, fsRdWrPerm);
+}
+
+
 unsigned int
 XMLMacFile::read(const unsigned int toRead, XMLByte* const toFill)
 {
     unsigned int bytesRead = 0;
     OSErr err = noErr;
-    
+
     if (!mFileValid)
         ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotReadFromFile);
-    
+
     if (gHasHFSPlusAPIs)
     {
         ByteCount actualCount;
@@ -335,27 +436,61 @@ XMLMacFile::read(const unsigned int toRead, XMLByte* const toFill)
         err = FSRead(mFileRefNum, &byteCount, toFill);
         bytesRead = byteCount;
     }
-    
+
     if (err != noErr && err != eofErr)
         ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotReadFromFile);
-    
+
     return bytesRead;
 }
 
 
+void
+XMLMacFile::write(const long byteCount, const XMLByte* const buffer)
+{
+    long bytesWritten = 0;
+    OSErr err = noErr;
+
+    if (byteCount <= 0 || buffer == NULL)
+        return;
+
+    if (!mFileValid)
+    {
+        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile);
+    }
+
+    if (gHasHFSPlusAPIs)
+    {
+        ByteCount actualCount;
+        err = FSWriteFork(mFileRefNum, fsFromMark, 0, byteCount, buffer, &actualCount);
+        bytesWritten = actualCount;
+    }
+    else
+    {
+        long count = byteCount;
+        err = FSWrite(mFileRefNum, &count, buffer);
+        bytesWritten = count;
+    }
+
+    if ((err != noErr && err != eofErr) || (bytesWritten != byteCount))
+    {
+        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile);
+    }
+}
+
+
 void
 XMLMacFile::reset()
 {
     OSErr err = noErr;
-    
+
     if (!mFileValid)
         ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotResetFile);
-    
+
     if (gHasHFSPlusAPIs)
         err = FSSetForkPosition(mFileRefNum, fsFromStart, 0);
     else
         err = SetFPos(mFileRefNum, fsFromStart, 0);
-    
+
     if (err != noErr)
         ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotResetFile);
 }
@@ -391,10 +526,7 @@ XMLPlatformUtils::panic(const PanicReasons reason)
         reasonStr = "Failed to complete platform dependent initialization";
     else
         reasonStr = "Unknown error source";
-    
-    char text[256];
-    std::snprintf(text, sizeof(text), "Xerces Panic Error: %s", reasonStr);
-    
+
     //
     //  The default handling of panics is not very friendly.
     //	To replace it with something more friendly, you'll need to:
@@ -405,11 +537,19 @@ XMLPlatformUtils::panic(const PanicReasons reason)
 #if defined(XML_USE_CUSTOM_PANIC_PROC)
     XMLCustomPanicProc(reason, reasonStr);
 #else
+    char text[256];
+
+    #if defined(XML_METROWERKS) || defined(_GLIBCPP_USE_C99)
+    std::snprintf(text, sizeof(text), "Xerces Panic Error: %s", reasonStr);
+    #else
+    std::sprintf(text, "Xerces Panic Error: %s", reasonStr);
+    #endif
+
     Str255 pasText;
     CopyCStringToPascal(text, pasText);
     DebugStr(pasText);
 #endif
-    
+
     //	Life's got us down. Good-bye world.
     std::exit(-1);
 }
@@ -421,28 +561,27 @@ XMLPlatformUtils::panic(const PanicReasons reason)
 unsigned int
 XMLPlatformUtils::curFilePos(const FileHandle theFile)
 {
-    return theFile->currPos();
+	return reinterpret_cast<XMLMacAbstractFile*>(theFile)->currPos();
 }
 
 void
 XMLPlatformUtils::closeFile(const FileHandle theFile)
 {
-    theFile->close();
+    reinterpret_cast<XMLMacAbstractFile*>(theFile)->close();
 }
 
 unsigned int
 XMLPlatformUtils::fileSize(const FileHandle theFile)
 {
-    return theFile->size();
+    return reinterpret_cast<XMLMacAbstractFile*>(theFile)->size();
 }
 
 
 FileHandle
 XMLPlatformUtils::openFile(const char* const fileName)
 {
-    const XMLCh* xmlPath = XMLString::transcode(fileName);
-    ArrayJanitor<const XMLCh> jan(xmlPath);
-    return openFile(xmlPath);
+    ArrayJanitor<const XMLCh> xmlPath(XMLString::transcode(fileName));
+    return openFile(xmlPath.get());
 }
 
 
@@ -451,38 +590,68 @@ XMLPlatformUtils::openFile(const XMLCh* const fileName)
 {
     // Check to make sure the file system is in a state where we can use it
     if (!gFileSystemCompatible)
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile);
-    
-    XMLMacFile* file = new XMLMacFile();
-    Janitor<XMLMacAbstractFile> janFile(file);
+        ThrowXML1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, fileName);
+
+    Janitor<XMLMacAbstractFile> file(new XMLMacFile());
     file->open(fileName);
-    janFile.orphan();
-    
-    return file;
+
+    return file.release();
+}
+
+
+FileHandle
+XMLPlatformUtils::openFileToWrite(const char* const fileName)
+{
+    ArrayJanitor<const XMLCh> xmlPath(XMLString::transcode(fileName));
+    return openFileToWrite(xmlPath.get());
+}
+
+
+FileHandle
+XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
+{
+    // Check to make sure the file system is in a state where we can use it
+    if (!gFileSystemCompatible)
+        ThrowXML1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, fileName);
+
+    Janitor<XMLMacAbstractFile> file(new XMLMacFile());
+    file->openFileToWrite(fileName);
+
+    return file.release();
 }
 
 
 FileHandle
 XMLPlatformUtils::openStdInHandle()
 {
-    ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile);
+    XMLCh stdinStr[] = {chLatin_s, chLatin_t, chLatin_d, chLatin_i, chLatin_n, chNull};
+    ThrowXML1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, stdinStr);
     return NULL;
 }
 
 
 unsigned int
-XMLPlatformUtils::readFileBuffer(   const   FileHandle      theFile
-                                 , const unsigned int    toRead
-                                 ,       XMLByte* const  toFill)
+XMLPlatformUtils::readFileBuffer(   const FileHandle      theFile
+                                 ,  const unsigned int    toRead
+                                 ,        XMLByte* const  toFill)
 {
-    return theFile->read(toRead, toFill);
+    return reinterpret_cast<XMLMacAbstractFile*>(theFile)->read(toRead, toFill);
+}
+
+
+void
+XMLPlatformUtils::writeBufferToFile(   const   FileHandle   theFile
+                                    ,  const long		    toWrite
+                                    ,  const XMLByte* const toFlush)
+{
+    return reinterpret_cast<XMLMacAbstractFile*>(theFile)->write(toWrite, toFlush);
 }
 
 
 void
 XMLPlatformUtils::resetFile(FileHandle theFile)
 {
-    theFile->reset();
+    reinterpret_cast<XMLMacAbstractFile*>(theFile)->reset();
 }
 
 
@@ -493,7 +662,7 @@ XMLCh*
 XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
 {
     XMLCh* path = NULL;
-    
+
     if (gHasHFSPlusAPIs)
     {
         FSRef ref;
@@ -506,7 +675,7 @@ XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
         if (!XMLParsePathToFSSpec(srcPath, spec) || (path = XMLCreateFullPathFromFSSpec(spec)) == NULL)
             path = XMLString::replicate(srcPath);
     }
-    
+
     return path;
 }
 
@@ -521,7 +690,7 @@ XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
 XMLCh*
 XMLPlatformUtils::weavePaths(const   XMLCh* const    basePath
                                     , const XMLCh* const    relativePath)
-                                    
+
 {
     // Code from Windows largely unmodified for the Macintosh,
     // with the exception of removing support for '\' path
@@ -529,13 +698,13 @@ XMLPlatformUtils::weavePaths(const   XMLCh* const    basePath
     //
     // Note that there is no support currently for Macintosh
     // path separators ':'.
-    
+
     // Create a buffer as large as both parts and empty it
     ArrayJanitor<XMLCh> tmpBuf(new XMLCh[XMLString::stringLen(basePath)
         + XMLString::stringLen(relativePath)
         + 2]);
     tmpBuf[0] = 0;
-    
+
     //
     //  If we have no base path, then just take the relative path as
     //  is.
@@ -545,37 +714,37 @@ XMLPlatformUtils::weavePaths(const   XMLCh* const    basePath
         XMLString::copyString(tmpBuf.get(), relativePath);
         return tmpBuf.release();
     }
-    
+
     if (!*basePath)
     {
         XMLString::copyString(tmpBuf.get(), relativePath);
         return tmpBuf.release();
     }
-    
+
     const XMLCh* basePtr = basePath + (XMLString::stringLen(basePath) - 1);
     if (*basePtr != chForwardSlash)
     {
         while ((basePtr >= basePath) && (*basePtr != chForwardSlash))
             basePtr--;
     }
-    
+
     // There is no relevant base path, so just take the relative part
     if (basePtr < basePath)
     {
         XMLString::copyString(tmpBuf.get(), relativePath);
         return tmpBuf.release();
     }
-    
+
     //  We have some path part, so we need to check to see if we have to
     //  weave any of the parts together.
     const XMLCh* pathPtr = relativePath;
-    
+
     while (true)
     {
         // If it does not start with some period, then we are done
         if (*pathPtr != chPeriod)
             break;
-        
+
         unsigned int periodCount = 1;
         pathPtr++;
         if (*pathPtr == chPeriod)
@@ -583,36 +752,36 @@ XMLPlatformUtils::weavePaths(const   XMLCh* const    basePath
             pathPtr++;
             periodCount++;
         }
-        
+
         // Has to be followed by a / or the null to mean anything
         if ((*pathPtr != chForwardSlash) &&  *pathPtr)
             break;
-        
+
         if (*pathPtr)
             pathPtr++;
-        
+
         // If it's one period, just eat it, else move backwards in the base
         if (periodCount == 2)
         {
             basePtr--;
             while ((basePtr >= basePath) && (*basePtr != chForwardSlash))
                 basePtr--;
-            
+
             // The base cannot provide enough levels, so it's in error/
             if (basePtr < basePath)
                 ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_BasePathUnderflow);
         }
     }
-    
+
     // Copy the base part up to the base pointer
     XMLCh* bufPtr = tmpBuf.get();
     const XMLCh* tmpPtr = basePath;
     while (tmpPtr <= basePtr)
         *bufPtr++ = *tmpPtr++;
-    
+
     // And then copy on the rest of our path
     XMLString::copyString(bufPtr, pathPtr);
-    
+
     // Orphan the buffer and return it
     return tmpBuf.release();
 }
@@ -735,12 +904,12 @@ XMLPlatformUtils::compareAndSwap(       void**      toFill
 {
     // Replace *toFill with newValue iff *toFill == toCompare,
     // returning previous value of *toFill
-    
+
     Boolean success = CompareAndSwap(
         reinterpret_cast<UInt32>(toCompare),
         reinterpret_cast<UInt32>(newValue),
         reinterpret_cast<UInt32*>(toFill));
-    
+
     return (success) ? const_cast<void*>(toCompare) : *toFill;
 }
 
@@ -787,19 +956,19 @@ XMLPlatformUtils::platformInit()
         gHasFSSpecAPIs		= (value & (1 << gestaltHasFSSpecCalls)) != 0;
         gHasFS2TBAPIs		= (value & (1 << gestaltFSSupports2TBVols)) != 0;
         gHasHFSPlusAPIs		= (value & (1 << gestaltHasHFSPlusAPIs)) != 0;
-    
+
         #if TARGET_API_MAC_CARBON
         gHasFSPathAPIs		= ((void*)kUnresolvedCFragSymbolAddress != FSPathMakeRef);
         #else
         gHasFSPathAPIs		= false;
         #endif
-        
+
         gPathAPIsUsePosixPaths = gHasFSPathAPIs && (value & (1 << gestaltFSUsesPOSIXPathsForConversion));
     }
-    
+
     //	Look for MP
 	gHasMPAPIs				= MPLibraryIsLoaded();
-    
+
 	//	We require FSSpecs at a minimum
     gFileSystemCompatible	= gHasFSSpecAPIs;
 }
@@ -1032,7 +1201,7 @@ XMLParsePathToFSRef_X(const XMLCh* const pathName, FSRef& ref)
 	
 	//	If it's a relative path, pre-pend the current directory to the path.
 	//	FSPathMakeRef doesn't deal with relativity on the front of the path
-	if (*p == '.')
+	if (*p != '/')
 	{
 		//	Right justify the user path to make room for the pre-pended path
 		std::memmove(p + kMaxStaticPathChars - pathLen, p, pathLen);
@@ -1085,23 +1254,23 @@ XMLParsePathToFSRef_Classic(const XMLCh* const pathName, FSRef& ref)
 	
 	const std::size_t kXMLBufCount = 256;
 	XMLCh xmlBuf[kXMLBufCount];
-    
+
     OSErr err = noErr;
-    
+
     if (*p == L'/')
     {
         // Absolute name: grab the first component as volume name
-        
+
         // Find the end of the path segment
         for (pEnd = ++p; *pEnd && *pEnd != L'/'; ++pEnd) ;
         segLen = pEnd - p;
-        
+
         // Try to find a volume that matches this name
         for (ItemCount volIndex = 1; err == noErr; ++volIndex)
         {
             HFSUniStr255 hfsStr;
             hfsStr.length = 0;
-            
+
             // Get the volume name
             err = FSGetVolumeInfo(
                 0,
@@ -1112,7 +1281,7 @@ XMLParsePathToFSRef_Classic(const XMLCh* const pathName, FSRef& ref)
                 &hfsStr,
                 &ref
                 );
-            
+
             // Compare against our path segment
             if (err == noErr && segLen == hfsStr.length)
             {
@@ -1125,7 +1294,7 @@ XMLParsePathToFSRef_Classic(const XMLCh* const pathName, FSRef& ref)
                     break;  // we found our volume
             }
         }
-        
+
         p = pEnd;
     }
     else
@@ -1136,7 +1305,7 @@ XMLParsePathToFSRef_Classic(const XMLCh* const pathName, FSRef& ref)
         if (err == noErr)
             err = FSpMakeFSRef(&spec, &ref);
     }
-    
+
     // ref now refers to the a parent directory: parse the rest of the path
     while (err == noErr && *p)
     {
@@ -1145,7 +1314,7 @@ XMLParsePathToFSRef_Classic(const XMLCh* const pathName, FSRef& ref)
         case L'/':   // Just skip any number of path separators
             ++p;
             break;
-            
+
         case L'.':   // Potentially "current directory" or "parent directory"
             if (p[1] == L'/' || p[1] == 0)       // "current directory"
             {
@@ -1155,7 +1324,7 @@ XMLParsePathToFSRef_Classic(const XMLCh* const pathName, FSRef& ref)
             else if (p[1] == L'.' && (p[2] == L'/' || p[2] == 0)) // "parent directory"
             {
                 p += 2;  // Get the parent of our parent
-                
+
                 FSCatalogInfo catalogInfo;
                 err = FSGetCatalogInfo(
                     &ref,
@@ -1165,16 +1334,16 @@ XMLParsePathToFSRef_Classic(const XMLCh* const pathName, FSRef& ref)
                     static_cast<FSSpec*>(NULL),
                     &ref
                     );
-                
+
                 // Check that we didn't go too far
                 if (err != noErr || catalogInfo.parentDirID == fsRtParID)
                     return false;
-                
+
                 break;
             }
             else // some other sequence of periods...fall through and treat as segment
                 ;
-            
+
         default:
             // Find the end of the path segment
             for (pEnd = p; *pEnd && *pEnd != L'/'; ++pEnd) ;
@@ -1191,12 +1360,12 @@ XMLParsePathToFSRef_Classic(const XMLCh* const pathName, FSRef& ref)
                 kTextEncodingUnknown,
                 &ref
                 );
-            
+
             p = pEnd;
             break;
         }
     }
-    
+
     return err == noErr;
 }
 
@@ -1207,7 +1376,7 @@ XMLParsePathToFSSpec(const XMLCh* const pathName, FSSpec& spec)
 	//	Parse Path to an FSSpec
 
 	//	If we've got HFS+ APIs, do this in terms of refs so that
-	//	we can grandfather in the use of FSPathMakeRef under Mac OS X 
+	//	we can grandfather in the use of FSPathMakeRef under Mac OS X
 	//	and CarbonLib 1.1. Otherwise, do it the hard way.
 	
 	bool result = false;
@@ -1246,23 +1415,23 @@ XMLParsePathToFSSpec_Classic(const XMLCh* const pathName, FSSpec& spec)
     ArrayJanitor<const char> janPath(p);
     const char* pEnd;
     std::size_t segLen;
-    
+
     OSStatus err = noErr;
     Str255 name;  // Must be long enough for a partial pathname consisting of two segments (64 bytes)
-    
+
     if (*p == '/')
     {
         // Absolute name: grab the first component as volume name
-        
+
         // Find the end of the path segment
         for (pEnd = ++p; *pEnd && *pEnd != '/'; ++pEnd) ;
         segLen = pEnd - p;
-        
+
         // Try to find a volume that matches this name
         for (ItemCount volIndex = 1; err == noErr; ++volIndex)
         {
             FSVolumeRefNum volRefNum;
-            
+
             if (gHasFS2TBAPIs)
             {
                 XVolumeParam xVolParam;
@@ -1288,7 +1457,7 @@ XMLParsePathToFSSpec_Classic(const XMLCh* const pathName, FSSpec& spec)
                 err = nsvErr;
 #endif
             }
-            
+
             // Compare against our path segment
             if (err == noErr && segLen == StrLength(name))
             {
@@ -1303,7 +1472,7 @@ XMLParsePathToFSSpec_Classic(const XMLCh* const pathName, FSSpec& spec)
                 }
             }
         }
-        
+
         p = pEnd;
     }
     else
@@ -1311,7 +1480,7 @@ XMLParsePathToFSSpec_Classic(const XMLCh* const pathName, FSSpec& spec)
         // Relative name, so get the default directory as parent spec
         err = FSMakeFSSpec(0, 0, NULL, &spec);
     }
-    
+
     // We now have a parent directory in the spec.
     while (err == noErr && *p)
     {
@@ -1320,7 +1489,7 @@ XMLParsePathToFSSpec_Classic(const XMLCh* const pathName, FSSpec& spec)
         case '/': 	// Just skip any number of path separators
             ++p;
             break;
-            
+
         case L'.': 	// Potentially "current directory" or "parent directory"
             if (p[1] == '/' || p[1] == 0)      // "current directory"
             {
@@ -1330,45 +1499,45 @@ XMLParsePathToFSSpec_Classic(const XMLCh* const pathName, FSSpec& spec)
             else if (p[1] == '.' && (p[2] == '/' || p[2] == 0)) // "parent directory"
             {
                 p += 2;  // Get the parent of our parent
-                
+
                 CInfoPBRec catInfo;
                 catInfo.dirInfo.ioNamePtr = NULL;
                 catInfo.dirInfo.ioVRefNum = spec.vRefNum;
                 catInfo.dirInfo.ioFDirIndex = -1;
                 catInfo.dirInfo.ioDrDirID = spec.parID;
                 err = PBGetCatInfoSync(&catInfo);
-                
+
                 // Check that we didn't go too far
                 if (err != noErr || catInfo.dirInfo.ioDrParID == fsRtParID)
                     return false;
-                
+
                 // Update our spec
                 if (err == noErr)
                     err = FSMakeFSSpec(spec.vRefNum, catInfo.dirInfo.ioDrDirID, NULL, &spec);
-                
+
                 break;
             }
             else // some other sequence of periods...fall through and treat as segment
                 ;
-            
+
         default:
             {
                 // Find the end of the path segment
                 for (pEnd = p; *pEnd && *pEnd != '/'; ++pEnd) ;
                 segLen = pEnd - p;
-                
+
                 // Check for name length overflow
                 if (segLen > 31)
                     return false;
-                
+
                 // Make a partial pathname from our current spec to the new object
                 unsigned char* partial = &name[1];
-                
+
                 *partial++ = ':';       // Partial leads with :
                 const unsigned char* specName = spec.name; // Copy in spec name
                 for (int specCnt = *specName++; specCnt > 0; --specCnt)
                     *partial++ = *specName++;
-                
+
                 *partial++ = ':';       // Separator
                 while (p != pEnd)       // Copy in new element
                	{
@@ -1380,16 +1549,16 @@ XMLParsePathToFSSpec_Classic(const XMLCh* const pathName, FSSpec& spec)
                 	else
                 		*partial++ = *p++;
 				}
-				                
+				
                 name[0] = partial - &name[1];   // Set the name length
-                
+
                 // Update the spec
                 err = FSMakeFSSpec(spec.vRefNum, spec.parID, name, &spec);
             }
             break;
         }
     }
-    
+
     return err == noErr;
 }
 
@@ -1461,17 +1630,17 @@ XMLCreateFullPathFromFSRef_Classic(const FSRef& startingRef)
     FSCatalogInfo catalogInfo;
     HFSUniStr255 name;
     FSRef ref = startingRef;
-    
+
     XMLCh buf[kMaxStaticPathChars];
     std::size_t bufPos   = kMaxStaticPathChars;
     std::size_t bufCnt   = 0;
-    
+
     ArrayJanitor<XMLCh> result(NULL);
     std::size_t resultLen = 0;
-    
+
     buf[--bufPos] = L'\0';
     ++bufCnt;
-    
+
 	do
 	{
 		err = FSGetCatalogInfo(
@@ -1565,17 +1734,17 @@ XMLCreateFullPathFromFSSpec_Classic(const FSSpec& startingSpec)
 	//	Manually create the path using FSSpec APIs.
     OSStatus err = noErr;
     FSSpec spec = startingSpec;
-    
+
     char buf[kMaxStaticPathChars];
     std::size_t bufPos   = kMaxStaticPathChars;
     std::size_t bufCnt   = 0;
-    
+
     ArrayJanitor<char> result(NULL);
     std::size_t resultLen = 0;
-    
+
     buf[--bufPos] = '\0';
     ++bufCnt;
-    
+
 	short index = 0;
 	do
 	{
@@ -1634,7 +1803,7 @@ XMLCreateFullPathFromFSSpec_Classic(const FSSpec& startingSpec)
 	// Copy in the old buffer
 	if (resultLen > 0)
 		std::memcpy(final.get() + bufCnt, result.get(), resultLen);
-    
+
     // Cleanup and transcode to unicode
     return XMLString::transcode(final.get());
 }
@@ -1648,23 +1817,23 @@ TranscodeUniCharsToUTF8(UniChar* src, char* dst, std::size_t srcCnt, std::size_t
     //	Use the text encoding converter to perform the format conversion.
     //	Note that this is slightly heavyweight, but we're not in a performance
     //	sensitive code-path.
-    
+
     OSStatus err = noErr;
     TECObjectRef tec = 0;
     ByteCount bytesConsumed = 0;
     ByteCount bytesProduced = 0;
-    
+
     TextEncoding inputEncoding	= CreateTextEncoding(kTextEncodingUnicodeDefault,
                                         kTextEncodingDefaultVariant,
                                         kUnicode16BitFormat);
-                                        
+
     TextEncoding outputEncoding = CreateTextEncoding(kTextEncodingUnicodeDefault,
                                         kTextEncodingDefaultVariant,
                                         kUnicodeUTF8Format);
-    
+
     if (err == noErr)
         err = TECCreateConverter(&tec, inputEncoding, outputEncoding);
-        
+
     if (err == noErr)
         err = TECConvertText(tec,
                     (ConstTextPtr) src,
@@ -1673,9 +1842,9 @@ TranscodeUniCharsToUTF8(UniChar* src, char* dst, std::size_t srcCnt, std::size_t
                     (TextPtr) dst,				// outputBuffer
                     maxChars * sizeof(char),	// outputBufferLength
                     &bytesProduced);			// actualOutputLength
-                    
+
     TECDisposeConverter(tec);
-    
+
     result = bytesProduced;
 	
     //	Return number of chars in dst
@@ -1691,24 +1860,23 @@ TranscodeUTF8ToUniChars(char* src, UniChar* dst, std::size_t maxChars)
     //	Use the text encoding converter to perform the format conversion.
     //	Note that this is slightly heavyweight, but we're not in a performance
     //	sensitive code-path.
-    
+
     OSStatus err = noErr;
     TECObjectRef tec = 0;
     ByteCount bytesConsumed = 0;
     ByteCount bytesProduced = 0;
-    
-    
+
     TextEncoding inputEncoding	= CreateTextEncoding(kTextEncodingUnicodeDefault,
                                         kTextEncodingDefaultVariant,
                                         kUnicodeUTF8Format);
-                                        
+
     TextEncoding outputEncoding = CreateTextEncoding(kTextEncodingUnicodeDefault,
                                         kTextEncodingDefaultVariant,
                                         kUnicode16BitFormat);
-    
+
     if (err == noErr)
         err = TECCreateConverter(&tec, inputEncoding, outputEncoding);
-        
+
     if (err == noErr)
         err = TECConvertText(tec,
                     (ConstTextPtr) src,
@@ -1717,11 +1885,13 @@ TranscodeUTF8ToUniChars(char* src, UniChar* dst, std::size_t maxChars)
                     (TextPtr) dst,				// outputBuffer
                     maxChars * sizeof(UniChar),	// outputBufferLength
                     &bytesProduced);			// actualOutputLength
-                    
+
     TECDisposeConverter(tec);
-    
+
     result = bytesProduced / sizeof(UniChar);
 	
     //	Return number of unicode characters in dst
 	return result;
 }
+
+XERCES_CPP_NAMESPACE_END
