@@ -1,7 +1,7 @@
 /*
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
+ * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: OS390PlatformUtils.cpp,v 1.3 2002/08/08 16:40:35 tng Exp $
+ * $Id: OS390PlatformUtils.cpp,v 1.7 2003/02/06 14:44:14 tng Exp $
  */
 
 
@@ -88,9 +88,13 @@
 #include    <xercesc/util/XMLString.hpp>
 #include    <xercesc/util/XMLUniDefs.hpp>
 #include    <xercesc/util/XMLUni.hpp>
+#include    "Path390.hpp"
+#include    "FileHandleImpl.hpp"
 
 #if defined (XML_USE_ICU_TRANSCODER)
     #include <xercesc/util/Transcoders/ICU/ICUTransService.hpp>
+#elif defined (XML_USE_UNICONV390_TRANSCODER)
+    #include <xercesc/util/Transcoders/Uniconv390/Uniconv390TransService.hpp>
 #else   // use native transcoder
     #include <xercesc/util/Transcoders/Iconv390/Iconv390TransService.hpp>
 #endif
@@ -106,6 +110,12 @@
     #include <xercesc/util/NetAccessors/Socket/SocketNetAccessor.hpp>
 #endif
 
+XERCES_CPP_NAMESPACE_BEGIN
+
+// ---------------------------------------------------------------------------
+//  Static data
+// ---------------------------------------------------------------------------
+static bool isPosixEnabled = false;
 
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: Platform init method
@@ -113,12 +123,44 @@
 
 void XMLPlatformUtils::platformInit()
 {
+
 // The next conditional section is to turn on support for allowing
 // the NEL character as a valid whitespace character. Once this is
 // enabled the parser is non-compliant with the XML standard.
+// Assumption: Once this is enabled, it cannot be reset and
+// should remain the same during the same process.
 #if defined XML_ALLOW_NELWS
+   try
+   {
      XMLPlatformUtils::recognizeNEL(true);
+   }
+   catch (...)
+   {
+      panic(XMLPlatformUtils::Panic_SystemInit);
+   }
 #endif
+
+// The next section checks if Posix is enabled on the OS/390 system.
+// It stores the result in the static isPosixEnabled flag.
+// The __isPosixOn() native routine uses too much
+// overhead to call repeatedly at runtime so this will be the only
+// remaining call of it.
+// Assumption: Once this is enabled, it cannot be reset and
+// should remain the same during the same process.
+
+   try
+   {
+      if (__isPosixOn())
+      {
+        if (!isPosixEnabled)
+            isPosixEnabled = true;
+      }
+   }
+   catch (...)
+   {
+      panic(XMLPlatformUtils::Panic_SystemInit);
+   }
+
 }
 
 // ---------------------------------------------------------------------------
@@ -155,7 +197,7 @@ XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
 
     catch(...)
     {
-	panic(XMLPlatformUtils::Panic_CantLoadMsgDomain);
+   panic(XMLPlatformUtils::Panic_CantLoadMsgDomain);
     }
     return retVal;
 }
@@ -172,6 +214,11 @@ XMLTransService* XMLPlatformUtils::makeTransService()
 
     return new ICUTransService;
 }
+#elif defined (XML_USE_UNICONV390_TRANSCODER)
+{
+
+    return new Uniconv390TransService;
+}
 #else
 {
     return new Iconv390TransService;
@@ -207,55 +254,119 @@ void XMLPlatformUtils::panic(const PanicReasons reason)
 
 
 // ---------------------------------------------------------------------------
-//  XMLPlatformUtils: File Methods
+//  Local Functions
 // ---------------------------------------------------------------------------
-unsigned int XMLPlatformUtils::curFilePos(FileHandle theFile)
+static void emptyBuffer(XMLByte* writeBuffer,
+                        size_t   bytesToWrite,
+                        FileHandleImpl* fhiPtr)
 {
-    // Get the current position
-    int curPos = ftell( (FILE*)theFile);
-    if (curPos == -1)
-		ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetSize);
+     FILE* fileHandle = (FILE*)fhiPtr->getHandle();
+     size_t bytesThisWrite = 0;
+     size_t bytesThisPass = 0;
+     int typeRecordLrecl = 0;
+     bool isTypeRecord = fhiPtr->isRecordType();
 
-    return (unsigned int)curPos;
+     if (isTypeRecord)
+        typeRecordLrecl = fhiPtr->getLrecl();
+
+     // Try to write as many bytes as possible at a time to the file.
+     // If less than the total bytes were written then loop as many times
+     // as necessary until all bytes are written. For the case of
+     // an MVS dataset with "type=record" specified, we must not
+     // attempt to write more than one logical record at a time
+     // or it will fail and not return the number of bytes remaining.
+
+     while (bytesToWrite > 0)
+     {
+        if ((isTypeRecord) &&
+            (bytesToWrite > typeRecordLrecl))
+        {
+           bytesThisPass = typeRecordLrecl;
+
+        }
+        else
+           bytesThisPass = bytesToWrite;
+
+        bytesThisWrite = fwrite(writeBuffer, sizeof(XMLByte), bytesThisPass, fileHandle);
+
+        if (ferror(fileHandle))
+           ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile);
+
+        bytesToWrite -= bytesThisWrite;
+        writeBuffer += bytesThisWrite;
+     }  // while (bytesToWrite > 0)
+
+  return;
 }
 
-void XMLPlatformUtils::closeFile(FileHandle theFile)
+
+static FileHandleImpl* openRead(char* tmpFileName)
 {
-    if (fclose((FILE*)theFile))
-		ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotCloseFile);
-}
+     FileHandleImpl* retVal;
+     FILE* fileHandle = 0;
+     Path390 pathobj;
+     pathobj.setPath(tmpFileName);
+     int optionBufferSize = strlen("rb,");
 
-unsigned int XMLPlatformUtils::fileSize(FileHandle theFile)
-{
-    // Get the current position
-    long  int curPos = ftell((FILE*)theFile);
-    if (curPos == -1)
-		ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetCurPos);
+     // Check if we have fopen options specified in addition to a
+     // filename. If we do then we need to build a buffer that has
+     // "rb," followed by the options. Otherwise we just pass "rb"
+     // in as the options.
 
-    // Seek to the end and save that value for return
-     if (fseek( (FILE*)theFile, 0, SEEK_END) )
-		ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotSeekToEnd);
+     if (pathobj.getfopenParms())
+     {
+        optionBufferSize += (strlen(pathobj.getfopenParms()) + 1);
+     }
 
-    long int retVal = ftell( (FILE*)theFile);
-    if (retVal == -1)
-		ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotSeekToEnd);
+     char* optionBuffer = new char[optionBufferSize];
+     ArrayJanitor<char> janText((char*)optionBuffer);
+     strcpy(optionBuffer,"rb");
 
+     // Build the options buffer
 
-    // And put the pointer back
-    if (fseek( (FILE*)theFile, curPos, SEEK_SET) )
-		ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotSeekToPos);
+     if (pathobj.getfopenParms())
+     {
+        strcpy(optionBuffer + 2, ",");
+        strcpy(optionBuffer + 3, pathobj.getfopenParms());
+     }
 
-    return (unsigned int)retVal;
-}
+     // If we don't have a valid path specified just exit now
 
-FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
-{
-    char* tmpFileName = XMLString::transcode(fileName);
-    ArrayJanitor<char> janText((char*)tmpFileName);
-    FileHandle retVal = (FILE*)fopen( tmpFileName , "rb" );
+     if (pathobj.getfopenPath() == NULL)
+        return 0;
 
-#ifdef OS390BATCH
-    if (retVal == NULL) {
+     // Take the file handle returned by fopen and create an instance
+     // of the FileHandleImpl class.
+
+     fileHandle = fopen(pathobj.getfopenPath(), optionBuffer);
+     retVal = new FileHandleImpl(fileHandle, _FHI_READ, pathobj.isRecordType());
+
+     // temp fix for HFS files with "type=record" specified
+     // close the file and re-open it without the fopen options
+     // since they don't work too well with HFS files.
+
+     if ((pathobj.isRecordType()) && (fileHandle != NULL))
+     {
+        int fldata_rc = 0;
+        fldata_t fileinfo;
+        fldata_rc = fldata(fileHandle, pathobj.getfopenPath(), &fileinfo);
+
+        if (fldata_rc)
+           ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotReadFromFile);
+
+        if (fileinfo.__dsorgHFS)
+        {
+           if (fclose(fileHandle))
+              ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotCloseFile);
+
+           fileHandle=fopen(pathobj.getfopenPath(), "rb");
+           retVal->setHandle(fileHandle);
+           retVal->setRecordType(_FHI_NOT_TYPE_RECORD);
+        }
+     }
+     // End temp fix ......
+
+    if ((fileHandle == NULL) && (pathobj.getPathType() == PATH390_OTHER)) {
     //
     // We failed to open the file using its native format (OE or MVS)
     // Try to go an extra step to map the path into a MVS dataset under BATCH:
@@ -267,7 +378,7 @@ FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
     //     path/path2/filename      => //path.path2.filename
 
     char* datasetName = new char[ strlen(tmpFileName) + 5 ];
-    ArrayJanitor<char> janText((char*)datasetName);
+    ArrayJanitor<char> janText1((char*)datasetName);
     char *datasetPos = datasetName, *tmpPos = tmpFileName;
 
     // We are in EBCDIC mode here
@@ -275,7 +386,7 @@ FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
     // or a MVS data set
 
     strcpy(datasetName, "//");
-    *datasetPos += 2;
+    datasetPos += 2;
 
     // If we have a leading '/' then the path is absolute
 
@@ -323,283 +434,405 @@ FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
     if( *tmpFileName == '/' ) *datasetPos++ = '\'';
     *datasetPos = '\0';
 
-    retVal = (FILE*)fopen( datasetName , "rb" );
+    fileHandle = fopen( datasetName , optionBuffer);
+    retVal->setHandle(fileHandle);
 
     }
-#endif
 
     // fix for file:// protocol
     // the tmpFileName has a prefix of //absolute path
     // for example, file:////u/.... instead of file:///u/....
     // the fopen() on OS/390 cannot open a //u/... POSIX file
-    if (retVal == NULL) {
-	if ((tmpFileName[0] == '/') && (tmpFileName[1] == '/')) {
-	    char *srcName = tmpFileName + 1; // points past the first '/'
-	    retVal = (FILE*)fopen( srcName , "rb" );
-	}
+
+    if ((fileHandle == NULL) && (pathobj.getPathType() == PATH390_OTHER))
+    {
+       if ((tmpFileName[0] == '/') && (tmpFileName[1] == '/'))
+        {
+            char *srcName = tmpFileName + 1; // points past the first '/'
+            fileHandle = fopen(srcName , optionBuffer);
+
+            retVal->setHandle(fileHandle);
+
+            // temp fix for HFS files with type=record specified
+            // close the file and re-open it without the fopen options
+            // since they don't work too well with HFS files.
+
+            if (pathobj.isRecordType())
+            {
+               int fldata_rc = 0;
+               fldata_t fileinfo;
+               fldata_rc = fldata(fileHandle, srcName, &fileinfo);
+
+               if (fldata_rc)
+                 ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotReadFromFile);
+
+               if (fileinfo.__dsorgHFS)
+               {
+                  if (fclose(fileHandle))
+                     ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotCloseFile);
+
+                  fileHandle=fopen(srcName, "rb");
+                  retVal->setHandle(fileHandle);
+                  retVal->setRecordType(_FHI_NOT_TYPE_RECORD);
+               }
+            }
+            // End temp fix ......
+   }
     }
 
-    if( retVal == NULL )
-	return 0;
+    // If the fopen failed we have a pointer to the FileHandleImpl class but
+    // inside that class a file handle of NULL. In this case we need to delete
+    // the class and return a NULL to the caller since they should be checking
+    // for a NULL returned as a file open failed indicator.
+
+    if (fileHandle == NULL)
+    {
+       delete retVal;
+       retVal = 0;
+    }
+
+    return retVal;
+
+}
+
+
+static FileHandleImpl* openWrite(char* tmpFileName)
+{
+    FileHandleImpl* retVal;
+    FILE* fileHandle = 0;
+    Path390 pathobj;
+    pathobj.setPath(tmpFileName);
+    fldata_t fileinfo;
+    int fldata_rc = 0;
+    int optionBufferSize = strlen("wb,");
+    int fileLrecl = 0;
+    bool isTypeRecord = _FHI_NOT_TYPE_RECORD;
+
+    // Check if we have fopen options specified in addition to a
+    // filename. If we do then we need to build a buffer that has
+    // "wb," followed by the options. Otherwise we just pass "wb"
+    // in as the options.
+
+    if (pathobj.getfopenParms())
+       optionBufferSize += (strlen(pathobj.getfopenParms()) + 1);
+
+    char* optionBuffer = new char[optionBufferSize];
+    ArrayJanitor<char> janText((char*)optionBuffer);
+    strcpy(optionBuffer,"wb");
+
+    // Build the options buffer
+
+    if (pathobj.getfopenParms())
+    {
+       strcpy(optionBuffer + 2, ",");
+       strcpy(optionBuffer + 3, pathobj.getfopenParms());
+    }
+
+    // FileHandleImpl class. The constructor will allocate our staging
+    // buffer if it is a "type=record" open.
+
+    fileHandle = fopen(pathobj.getfopenPath(),optionBuffer);
+
+    // If this is a "type=record" open then we need to determine the
+    // LRECL of the file. The fldata info will have this in it.
+    // We save this in the FileHandleImpl class for use on subsequent
+    // writes to the file.
+
+    if (pathobj.isRecordType())
+    {
+       isTypeRecord = _FHI_TYPE_RECORD;
+       fldata_rc = fldata(fileHandle, pathobj.getfopenPath(), &fileinfo);
+
+       if (fldata_rc)
+          ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile);
+
+       fileLrecl = fileinfo.__maxreclen;
+
+       // temp fix for HFS files
+       // close the file and re-open it without the fopen options
+       // since they don't work too well with HFS files.
+
+       if (fileinfo.__dsorgHFS)
+       {
+          if (fclose(fileHandle))
+            ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotCloseFile);
+
+          fileHandle=fopen(pathobj.getfopenPath(), "wb");
+
+       }
+      // end temp fix
+
+    }
+
+    // If the fopen failed we won't need to create an instance of the
+    // FileHandleImpl class .... just return a NULL wich the caller recognizes
+    // as a failure. If we have a valid file handle then we'll build create
+    // an instance of the class and return that to the caller.
+
+    if (fileHandle == NULL)
+       retVal = 0;
+    else
+       retVal = new FileHandleImpl(fileHandle, _FHI_WRITE, isTypeRecord,fileLrecl);
+
     return retVal;
 }
 
+
+// ---------------------------------------------------------------------------
+//  XMLPlatformUtils: File Methods
+// ---------------------------------------------------------------------------
+unsigned int XMLPlatformUtils::curFilePos(FileHandle theFile)
+{
+    // Get the current position
+    FileHandleImpl* fhiPtr = (FileHandleImpl*) theFile;
+    FILE* fileHandle = (FILE*)fhiPtr->getHandle();
+    int curPos = ftell(fileHandle);
+
+    if (curPos == -1)
+       ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetSize);
+
+    return (unsigned int)curPos;
+}
+
+void XMLPlatformUtils::closeFile(FileHandle theFile)
+{
+    FileHandleImpl* fhiPtr = (FileHandleImpl*) theFile;
+    FILE* fileHandle = (FILE*)fhiPtr->getHandle();
+
+    // We need to handle a close differently if this was a
+    // "type=record" open. In that case we may have data sitting in
+    // the staging buffer that is waiting to be written to the file.
+    // In this case write that data to the file before we actually
+    // close it.
+
+    if ( (fhiPtr->getOpenType() == _FHI_WRITE) &&
+         (fhiPtr->isRecordType())  &&
+         (fhiPtr->getNextByte()) )
+    {
+        XMLByte* tmpFlush = fhiPtr->getStgBufferPtr();
+        size_t bytesToWrite = fhiPtr->getNextByte();
+        emptyBuffer(tmpFlush, bytesToWrite,fhiPtr);
+    }
+
+    // Do the regular close stuff ....
+
+    if (fclose(fileHandle))
+       ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotCloseFile);
+
+    // Delete the instance of the FileHandleImpl class (this will free the
+    // staging buffer for us)
+
+    delete fhiPtr;
+
+}
+
+unsigned int XMLPlatformUtils::fileSize(FileHandle theFile)
+{
+    // Get the current position
+    FileHandleImpl* fhiPtr = (FileHandleImpl*) theFile;
+    FILE* fileHandle = (FILE*)fhiPtr->getHandle();
+    long  int curPos = ftell(fileHandle);
+
+    if (curPos == -1)
+       ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetCurPos);
+
+    // Seek to the end and save that value for return
+     if (fseek(fileHandle, 0, SEEK_END))
+        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotSeekToEnd);
+
+    long int retVal = ftell(fileHandle);
+
+    if (retVal == -1)
+       ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotSeekToEnd);
+
+
+    // And put the pointer back
+    if (fseek(fileHandle, curPos, SEEK_SET) )
+       ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotSeekToPos);
+
+    return (unsigned int)retVal;
+}
+
+FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
+{
+    char* tmpFileName = XMLString::transcode(fileName);
+    ArrayJanitor<char> janText((char*)tmpFileName);
+
+    return openRead(tmpFileName);
+}
+
+
 FileHandle XMLPlatformUtils::openFile(const char* const fileName)
 {
-    FileHandle retVal = (FILE*)fopen( fileName , "rb" );
+    char* tmpFileName = new char[strlen(fileName) + 1];
+    ArrayJanitor<char> janText((char*)tmpFileName);
+    strcpy(tmpFileName,fileName);
 
-#ifdef OS390BATCH
-    if (retVal == NULL) {
-    //
-    // We failed to open the file using its native format (OE or MVS)
-    // Try to go an extra step to map the path into a MVS dataset under BATCH:
-    //
-    //     /path/path2/filename.ext => //'path.path2.ext(filename)'
-    //     /path/path2/filename     => //'path.path2.filename'
-    // and
-    //     path/path2/filename.ext  => //path.path2.ext(filename)
-    //     path/path2/filename      => //path.path2.filename
-
-    char* datasetName = new char[ strlen(fileName) + 5 ];
-    ArrayJanitor<char> janText((char*)datasetName);
-    char *datasetPos = datasetName, *tmpPos = (char *)fileName;
-
-    // We are in EBCDIC mode here
-    // Specify "//" to indicate that the filename refers to a non-POSIX file
-    // or a MVS data set
-
-    strcpy(datasetName, "//");
-    *datasetPos += 2;
-
-    // If we have a leading '/' then the path is absolute
-
-    if( *tmpPos == '/' ) {
-        *datasetPos++ = '\'';
-        tmpPos++;
-    }
-
-    // Find the last '/' in the path - this seperates the path from the
-    // filename.  Then copy the pathname.
-
-    char* pathEnd = strrchr((char*) tmpPos, '/' );
-    if( pathEnd == NULL ) pathEnd = tmpPos - 1;
-    while( tmpPos <= pathEnd ) {
-        switch( *tmpPos ) {
-                case '/':
-                        *datasetPos = '.';
-                        break;
-                default:
-                        *datasetPos = *tmpPos;
-        }
-        datasetPos++; tmpPos++;
-    }
-
-    // Now we try to locate the extension, and copy that.
-
-    char* extStart = strrchr((char*) fileName, '.' );
-    if ( extStart != NULL ) {
-        tmpPos = extStart + 1;
-        while( *tmpPos != '\0' ) {
-                *datasetPos++ = *tmpPos++;
-        }
-        *datasetPos++ = '(';
-    }
-
-    // Now we copy in the filename.
-
-    tmpPos = pathEnd + 1;
-    while( *tmpPos != '\0' && ((extStart == NULL) || (tmpPos < extStart)) ) {
-        *datasetPos++ = *tmpPos++;
-    }
-
-    // Finally cap off the filename with optional ")" and "'", plus a null
-    if( extStart != NULL ) *datasetPos++ = ')';
-    if( *fileName == '/' ) *datasetPos++ = '\'';
-    *datasetPos = '\0';
-
-    retVal = (FILE*)fopen( datasetName , "rb" );
-
-    }
-#endif
-
-    // fix for file:// protocol
-    // the fileName has a prefix of //absolute path
-    // for example, file:////u/.... instead of file:///u/....
-    // the fopen() on OS/390 cannot open a //u/... POSIX file
-    if (retVal == NULL) {
-	if ((fileName[0] == '/') && (fileName[1] == '/')) {
-	    const char *srcName = fileName + 1; // points past the first '/'
-	    retVal = (FILE*)fopen( srcName , "rb" );
-	}          	
-    }
-
-    if( retVal == NULL )
-        return 0;
-    return retVal;
+    return openRead(tmpFileName);
 }
 
+
 FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
 {
-    const char* tmpFileName = XMLString::transcode(fileName);
+    char* tmpFileName = XMLString::transcode(fileName);
     ArrayJanitor<char> janText((char*)tmpFileName);
-    char* semicolon;
-    char* optStart;
-    int   optOffset = 0;
-    int   dsnSize = 0;
-    int   optSize = 0;
-
-    // Check if this is an MVS dataset. If it is we need to allow
-    // DCB options to be passed in the URI we get. The options
-    // will be after the ";" in the URI in this case.
-    if ((tmpFileName[0] == '/') && (tmpFileName[1] == '/'))
-    {
-       semicolon = strrchr((char*)tmpFileName,';');
-       if (semicolon == NULL)
-          dsnSize = strlen(tmpFileName);
-       else
-       {
-          dsnSize = semicolon - tmpFileName;
-          optStart = semicolon;
-          optStart++;
-          optOffset = optStart - tmpFileName;
-          optSize = (strlen(tmpFileName) - optOffset);
-        }
-
-        // Make a copy of the dataset name to pass to fopen
-        char* dsnbuf = new char[dsnSize + 1];
-        ArrayJanitor<char> janText1((char*)dsnbuf);
-        strncpy(dsnbuf, tmpFileName, dsnSize);
-
-        // If no options specified, then we just put a "wb" in there.
-        // Otherwise we put a "wb" followed by a comma followed by
-        // the options string.
-        if (optSize == 0)
-           optSize += 3;
-        else
-           optSize += 4;
-
-        // Save the options for fopen
-        char* optbuf = new char[optSize];
-        ArrayJanitor<char> janText2((char*)optbuf);
-        strcpy(optbuf, "wb");
-
-        if (optSize != 3)
-        {
-           strcpy(optbuf + 2, ",");
-           strncpy(optbuf +3, optStart, optSize);
-        }
-
-        return fopen(dsnbuf, optbuf);
-
-    }
-    else
-       return fopen(tmpFileName, "wb");
 
+    return openWrite(tmpFileName);
 }
 
+
 FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName)
 {
-    char* semicolon;
-    char* optStart;
-    int   optOffset = 0;
-    int   dsnSize = 0;
-    int   optSize = 0;
-
-    // Check if this is an MVS dataset. If it is we need to allow
-    // DCB options to be passed in the URI we get. The options
-    // will be after the ";" in the URI in this case.
-    if ((fileName[0] == '/') && (fileName[1] == '/'))
-    {
-       semicolon = strrchr((char*)fileName,';');
-       if (semicolon == NULL)
-          dsnSize = strlen(fileName);
-       else
-       {
-          dsnSize = semicolon - fileName;
-          optStart = semicolon;
-          optStart++;
-          optOffset = optStart - fileName;
-          optSize = (strlen(fileName) - optOffset);
-        }
-
-        // Make a copy of the dataset name to pass to fopen
-        char* dsnbuf = new char[dsnSize + 1];
-        ArrayJanitor<char> janText1((char*)dsnbuf);
-        strncpy(dsnbuf, fileName, dsnSize);
-
-        // If no options specified, then we just put a "wb" in there.
-        // Otherwise we put a "wb" followed by a comma followed by
-        // the options string.
-        if (optSize == 0)
-           optSize += 3;
-        else
-           optSize += 4;
-
-        // Save the options for fopen
-        char* optbuf = new char[optSize];
-        ArrayJanitor<char> janText2((char*)optbuf);
-        strcpy(optbuf, "wb");
-
-        if (optSize != 3)
-        {
-           strcpy(optbuf + 2, ",");
-           strncpy(optbuf +3, optStart, optSize);
-        }
-
-        return fopen(dsnbuf, optbuf);
-
-    }
-    else
-       return fopen(fileName, "wb");
+    char* tmpFileName = new char[strlen(fileName) + 1];
+    ArrayJanitor<char> janText((char*)tmpFileName);
+    strcpy(tmpFileName,fileName);
 
+    return openWrite(tmpFileName);
 }
 
+
 void
 XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
                                    , long                  toWrite
                                    , const XMLByte* const  toFlush)
 {
-    if (!theFile        ||
+    FileHandleImpl* fhiPtr = (FileHandleImpl*) theFile;
+    FILE* fileHandle = (FILE*)fhiPtr->getHandle();
+
+    if (!fileHandle     ||
         (toWrite <= 0 ) ||
-        !toFlush         )
+        !toFlush )
         return;
 
-    const XMLByte* tmpFlush = (const XMLByte*) toFlush;
-    size_t bytesWritten = 0;
+    XMLByte* tmpFlush = (XMLByte*) toFlush;
 
-    while (true)
+    // For writes where the file has been opened with the "type=record"
+    // option we must do a lot of extra work. In this case we have
+    // a staging buffer that we copy data to and only write it out
+    // when we find a newline or line feed in the buffer.
+
+    if (fhiPtr->isRecordType())
     {
-        bytesWritten=fwrite(tmpFlush, sizeof(XMLByte), toWrite, (FILE*)theFile);
+       XMLByte* inputBufferPtr =  tmpFlush;
+       XMLByte* stageBufferStartPtr =  fhiPtr->getStgBufferPtr();
+       int stageBufferNextByte = fhiPtr->getNextByte();
+       XMLByte* stageBufferAddPtr =  stageBufferStartPtr + stageBufferNextByte;
+       int stageThisAdd = 0;
+       int stageTotalAdded = 0;
+       int stageToWrite = 0;
+       int fileLrecl = fhiPtr->getLrecl();
 
-        if(ferror((FILE*)theFile))
-        {
-            ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile);
-        }
+       // Check each byte passed into us. If it is a new line or
+       // line feed we copy up to and including that byte to the
+       // staging buffer starting at the next available byte. We
+       // then write the valid contents of the staging buffer to
+       // the file and then continue going thru the input data
+       // since a newline is not the end of the input.
+       // Also we want to copy the data to the staging buffer and
+       // write it to disk as soon as the staging buffer contains
+       // a logical record's worth of data.
 
-        if (bytesWritten < toWrite) //incomplete write
-        {
-            tmpFlush+=bytesWritten;
-            toWrite-=bytesWritten;
-            bytesWritten=0;
-        }
-        else
-            return;
+       for(int i = 0; i < toWrite; i++)
+       {
+          if ( (tmpFlush[i] == '\n') ||
+               (tmpFlush[i] == 0x25) ||
+               ((stageBufferNextByte + (i + 1)) - stageTotalAdded == fileLrecl) )
+          {
+             stageThisAdd = (i + 1) - stageTotalAdded;
+             memcpy((void*) stageBufferAddPtr, (void*) inputBufferPtr, stageThisAdd);
+             stageTotalAdded += stageThisAdd;
+             inputBufferPtr += stageThisAdd;
+             stageBufferAddPtr += stageThisAdd;
+             stageToWrite = stageBufferAddPtr - stageBufferStartPtr;
+             emptyBuffer(stageBufferStartPtr, stageToWrite, fhiPtr);
+             stageBufferAddPtr = stageBufferStartPtr;
+             stageBufferNextByte = 0;
+          }
+       }  // for loop
+
+
+      // When we finish looping thru the input buffer we may have data
+      // left in it that was not copied to the staging buffer yet. Copy
+      // whatever is left in the input buffer to the staging buffer now
+      // and update the staging buffer next byte offset.
+
+      if (stageTotalAdded < toWrite)
+      {
+         stageThisAdd = toWrite - stageTotalAdded;
+         memcpy((void*) stageBufferAddPtr, (void*) inputBufferPtr, stageThisAdd);
+         stageBufferNextByte += stageThisAdd;
+      }
+
+      fhiPtr->setNextByte(stageBufferNextByte);
     }
 
-    return;
+    // Normal type of write, just write data to disk ...
+
+    else
+       emptyBuffer(tmpFlush, toWrite, fhiPtr);
+
+  return;
 }
 
+
 unsigned int
 XMLPlatformUtils::readFileBuffer(  FileHandle      theFile
                                 , const unsigned int    toRead
                                 , XMLByte* const  toFill)
 {
-    size_t noOfItemsRead = fread( (void*) toFill, 1, toRead, (FILE*)theFile);
+    FileHandleImpl* fhiPtr = (FileHandleImpl*) theFile;
+    FILE* fileHandle = (FILE*)fhiPtr->getHandle();
+    size_t noOfItemsRead = 0;
+    bool isTypeRecord = fhiPtr->isRecordType();
 
-    if(ferror((FILE*)theFile))
+    // On a read, if this was a "type=record" open then we do not want
+    // to return any terminating NULLS that fill out a logical
+    // record. We also will discard a record that is all NULLS.
+
+    while (true)
     {
-		ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotReadFromFile);
-    }
+       noOfItemsRead = fread( (void*) toFill, 1, toRead, fileHandle);
+
+       if(ferror(fileHandle))
+          ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotReadFromFile);
+
+       if (isTypeRecord)
+       {
+          if (noOfItemsRead == 0)
+              break;
+
+          // Go thru the data we read and strip off trailing NULLS
+          // until a non-NULL is encountered. If a record is all
+          // NULLS then we discard it.
+
+          bool recordIsValid = false;
+
+          for (int i = noOfItemsRead; i > 0; i--)
+          {
+             if (toFill[i - 1])
+             {
+                recordIsValid = true;
+                break;
+             }
+             else
+                noOfItemsRead--;
+          }
+
+          // If we didn't hit a non-NULL we need will go read another
+          // record in hopes of returning some valid data.
+
+          if (recordIsValid)
+             break;
+          else
+          {
+             noOfItemsRead = 0;
+             break;
+          }
+       } // isTypeRecord
+       // for a "non-type=record" read we do only one read
+       else
+          break;
+    }  // while (true)
 
     return (unsigned int)noOfItemsRead;
 }
@@ -608,11 +841,12 @@ XMLPlatformUtils::readFileBuffer(  FileHandle      theFile
 void XMLPlatformUtils::resetFile(FileHandle theFile)
 {
     // Seek to the start of the file
-    if (fseek((FILE*)theFile, 0, SEEK_SET) )
-	ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotResetFile);
+    FileHandleImpl* fhiPtr = (FileHandleImpl*) theFile;
+    FILE* fileHandle = (FILE*)fhiPtr->getHandle();
+    if (fseek(fileHandle, 0, SEEK_SET) )
+   ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotResetFile);
 }
 
-
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: File system methods
 // ---------------------------------------------------------------------------
@@ -627,22 +861,23 @@ XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
     char* newSrc = XMLString::transcode(srcPath);
     ArrayJanitor<char> janText(newSrc);
 
-    if ( (newSrc != NULL) && (newSrc[0] != NULL) && (newSrc[1] != NULL) && //@DEM
-         (newSrc[0] == '/') && (newSrc[1] == '/') )       //@DEM
-        return XMLString::transcode(newSrc);              //@DEM
+    Path390 pathobj;
+    pathobj.setPath(newSrc);
 
+    char* retPath = 0;
     // Use a local buffer that is big enough for the largest legal path
     char *absPath = new char[_POSIX_PATH_MAX];
     ArrayJanitor<char> janText2(absPath);
-	//get the absolute path
-	char* retPath = realpath(newSrc, absPath);
 
-
-    if (!retPath)
-    {
-                ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetBasePathName);
+    if ( (pathobj.getPathType() == PATH390_HFS) || (pathobj.getPathType() == PATH390_OTHER) ) {
+       //get the absolute path
+       char* retPath = realpath(pathobj.getfopenPath(), absPath);
+       if (!retPath) {
+          ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetBasePathName);
+          }
+       return XMLString::transcode(absPath);
     }
-    return XMLString::transcode(absPath);
+    return XMLString::transcode(newSrc);
 }
 
 bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
@@ -662,7 +897,6 @@ bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
     if (toCheck[0] == 0x2F)
         return false;
 
-#ifdef OS390BATCH
     //
     //  Determine whether we are using the DD convention
     //  'D': 0x44 'd': 0x64 ':': 0x3A
@@ -672,13 +906,22 @@ bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
           ((toCheck[0] == (0x44)) || (toCheck[0] == (0x64))) &&
           ((toCheck[1] == (0x44)) || (toCheck[1] == (0x64))) &&
           (toCheck[2] == XMLCh(0x3A))
-	 )
-	return false;
+         )
+         return false;
     }
-#endif
 
-    // Else assume its a relative path
-    return true;
+    char* tmpFileName = XMLString::transcode(toCheck);
+    ArrayJanitor<char> janText((char*)tmpFileName);
+    Path390 pathobj;
+    pathobj.setPath(tmpFileName);
+
+    // let Path390 do the final determination
+    return pathobj.isRelative();
+
+
+
+
+
 }
 
 XMLCh* XMLPlatformUtils::weavePaths
@@ -769,7 +1012,7 @@ XMLCh* XMLPlatformUtils::weavePaths
 
             // The base cannot provide enough levels, so its in error/
             if (basePtr < basePath)
-				ThrowXML(XMLPlatformUtilsException,	XMLExcepts::File_BasePathUnderflow);
+            ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_BasePathUnderflow);
 
         }
     }
@@ -799,6 +1042,7 @@ unsigned long XMLPlatformUtils::getCurrentMillis()
 
 }
 
+
 // -----------------------------------------------------------------------
 //  Mutex methods
 //  OS390BATCH: Unless POSIX(ON) is specified, pthread functions are
@@ -810,57 +1054,47 @@ void XMLPlatformUtils::closeMutex(void* const mtxHandle)
 {
     if (mtxHandle == NULL)
         return;
-#ifdef OS390BATCH
-    if (__isPosixOn()) {
-#endif
+    if (isPosixEnabled) {
     if (pthread_mutex_destroy( (pthread_mutex_t*)mtxHandle))
     {
-	ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotDestroy);
+   ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotDestroy);
     }
     if ( (pthread_mutex_t*)mtxHandle)
-        delete mtxHandle;
-#ifdef OS390BATCH
+        delete (pthread_mutex_t*) mtxHandle;
     } // __isPosixOn
     else {
     if ( (int*)mtxHandle)
-        delete mtxHandle;
+        delete (int*) mtxHandle;
     }
-#endif
 }
 void XMLPlatformUtils::lockMutex(void* const mtxHandle)
 {
     if (mtxHandle == NULL)
         return;
-#ifdef OS390BATCH
-    if (__isPosixOn()) {
-#endif
+    if (isPosixEnabled) {
     if (pthread_mutex_lock( (pthread_mutex_t*)mtxHandle))
     {
-	ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotLock);
+   ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotLock);
     }
-#ifdef OS390BATCH
     } // __isPosixOn
     else {
-	int locked = 1, unlocked;
+   int locked = 1, unlocked;
 
     do {
         unlocked = 0;
         compareAndSwap( (void**) &mtxHandle, (void*) locked, (void*) unlocked );
     } while( unlocked != 0 );
     }
-#endif
     return;
 
 }
 void* XMLPlatformUtils::makeMutex()
 {
-#ifdef OS390BATCH
-    if (__isPosixOn()) {
-#endif
+    if (isPosixEnabled) {
     pthread_mutex_t* mutex = new pthread_mutex_t;
     if (mutex == NULL)
     {
-	ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotCreate);
+   ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotCreate);
     }
 
     pthread_mutexattr_t*  attr = new pthread_mutexattr_t;
@@ -876,33 +1110,27 @@ void* XMLPlatformUtils::makeMutex()
 
     return (void*)(mutex);
 
-#ifdef OS390BATCH
     } // __isPosixOn
     else {
-	int* mutex = new int;
-	*mutex = 0;
-	return (void*)(mutex);
+   int* mutex = new int;
+   *mutex = 0;
+   return (void*)(mutex);
     }
-#endif
 }
 void XMLPlatformUtils::unlockMutex(void* const mtxHandle)
 {
     if (mtxHandle == NULL)
         return;
-#ifdef OS390BATCH
-    if (__isPosixOn()) {
-#endif
+    if (isPosixEnabled) {
     if (pthread_mutex_unlock( (pthread_mutex_t*)mtxHandle))
     {
-	ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotUnlock);
+   ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotUnlock);
     }
-#ifdef OS390BATCH
     } // __isPosixOn
     else {
-	if (*(int*) mtxHandle == 0 )
-	*(int*) mtxHandle = 0;
+   if (*(int*) mtxHandle == 0 )
+   *(int*) mtxHandle = 0;
     }
-#endif
 }
 
 #else // #ifndef APP_NO_THREADS
@@ -926,6 +1154,7 @@ void XMLPlatformUtils::unlockMutex(void* const mtxHandle)
 
 #endif // APP_NO_THREADS
 
+
 // -----------------------------------------------------------------------
 //  Miscellaneous synchronization methods
 // -----------------------------------------------------------------------
@@ -963,7 +1192,16 @@ int XMLPlatformUtils::atomicDecrement(int &location)
 
 FileHandle XMLPlatformUtils::openStdInHandle()
 {
-    return (FileHandle)fdopen(dup(0), "rb");
+     FileHandleImpl* retVal;
+     FILE* fileHandle = 0;
+     fileHandle = fdopen(dup(0), "rb");
+     if (fileHandle)
+        retVal = new FileHandleImpl(fileHandle, _FHI_READ, _FHI_NOT_TYPE_RECORD);
+     else
+        retVal = 0;
+
+     return retVal;
+
 }
 
 void XMLPlatformUtils::platformTerm()
@@ -971,3 +1209,4 @@ void XMLPlatformUtils::platformTerm()
     // We don't have any termination requirements at this time
 }
 
+XERCES_CPP_NAMESPACE_END
