@@ -55,7 +55,7 @@
  */
 
 /**
- * $Id: OS400PlatformUtils.cpp,v 1.3 2002/07/16 15:42:47 peiyongz Exp $
+ * $Id: OS400PlatformUtils.cpp,v 1.5 2002/11/04 15:13:01 tng Exp $
  */
 
 // ---------------------------------------------------------------------------
@@ -97,7 +97,7 @@
 	 #include <xercesc/util/MsgLoaders/ICU/ICUMsgLoader.hpp>
 #endif
 
-
+XERCES_CPP_NAMESPACE_BEGIN
 
 char* PackingRepText(const char * const repText1,
 		     const char * const repText2,
@@ -159,7 +159,7 @@ XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
 
     catch(...)
     {
-        panic( XMLPlatformUtils::Panic_NoDefTranscoder );
+        panic( XMLPlatformUtils::Panic_CantLoadMsgDomain );
     }
     return retVal;
 }
@@ -304,7 +304,7 @@ XMLPlatformUtils::readFileBuffer(  FileHandle      theFile
 void
 XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
                                    , long                  toWrite
-                                   , const XMLByte* const  toFlush)                                   
+                                   , const XMLByte* const  toFlush)
 {
     if (!theFile        ||
         (toWrite <= 0 ) ||
@@ -864,3 +864,5 @@ void XMLPlatformUtils::platformTerm()
 	cleanupDefaultConverter();
 #endif
 }
+
+XERCES_CPP_NAMESPACE_END
