@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: SolarisPlatformUtils.cpp,v 1.6 2002/07/15 21:53:45 peiyongz Exp $
+ * $Id: SolarisPlatformUtils.cpp,v 1.9 2003/01/03 16:02:12 tng Exp $
  */
 
 
@@ -110,6 +110,11 @@
 #endif
 
 
+#if defined (SOLARIS)
+extern "C" int ftime(struct timeb *); // Solaris headers missing this decl
+#endif
+
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: Private Static Methods
@@ -146,7 +151,7 @@ XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
     }
     catch(...)
     {
-        panic(XMLPlatformUtils::Panic_NoDefTranscoder);
+        panic(XMLPlatformUtils::Panic_CantLoadMsgDomain);
     }
     return retVal;
 }
@@ -302,7 +307,7 @@ XMLPlatformUtils::readFileBuffer(FileHandle              theFile
 void
 XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
                                    , long                  toWrite
-                                   , const XMLByte* const  toFlush)                                   
+                                   , const XMLByte* const  toFlush)
 {
     if (!theFile        ||
         (toWrite <= 0 ) ||
@@ -501,11 +506,6 @@ XMLCh* XMLPlatformUtils::weavePaths
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: Timing Methods
 // ---------------------------------------------------------------------------
-
-#if defined (SOLARIS)
-extern "C" int ftime(struct timeb *); // Solaris headers missing this decl
-#endif
-
 unsigned long XMLPlatformUtils::getCurrentMillis()
 {
     timeb aTime;
@@ -781,3 +781,4 @@ void XMLPlatformUtils::platformTerm()
 #endif
 }
 
+XERCES_CPP_NAMESPACE_END
