@@ -56,6 +56,18 @@
 
 /*
  * $Log: TandemPlatformUtils.cpp,v $
+ * Revision 1.6  2003/02/05 18:29:27  tng
+ * [Bug 13437] Incorrect memory management in XXXPlatformUtils.cpp.
+ *
+ * Revision 1.5  2003/01/09 15:30:39  tng
+ * Missing panic function in Tandem
+ *
+ * Revision 1.4  2002/12/12 16:29:30  peiyongz
+ * loadAMsgSet() added
+ *
+ * Revision 1.3  2002/11/04 15:13:01  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.2  2002/05/21 20:31:48  tng
  * Minor update: Remove obsolete code
  *
@@ -98,6 +110,15 @@
 #include    <sys/timeb.h>
 #include    <string.h>
 
+#if defined (XML_USE_ICU_MESSAGELOADER)
+    #include <xercesc/util/MsgLoaders/ICU/ICUMsgLoader.hpp>
+#elif defined (XML_USE_ICONV_MESSAGELOADER)
+    #include <xercesc/util/MsgLoaders/MsgCatalog/MsgCatalogLoader.hpp>
+#else   // use In-memory message loader
+    #include <xercesc/util/MsgLoaders/InMemory/InMemMsgLoader.hpp>
+#endif
+
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: Platform init method
@@ -106,6 +127,63 @@ void XMLPlatformUtils::platformInit()
 {
 }
 
+//
+//  This method is called by the platform independent part of this class
+//  when client code asks to have one of the supported message sets loaded.
+//  In our case, we use the ICU based message loader mechanism.
+//
+XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
+{
+    XMLMsgLoader* retVal;
+    try
+    {
+#if defined (XML_USE_ICU_MESSAGELOADER)
+        retVal = new ICUMsgLoader(msgDomain);
+#elif defined (XML_USE_ICONV_MESSAGELOADER)
+        retVal = new MsgCatalogLoader(msgDomain);
+#else
+        retVal = new InMemMsgLoader(msgDomain);
+#endif
+    }
+    catch(...)
+    {
+        panic(XMLPlatformUtils::Panic_CantLoadMsgDomain);
+    }
+    return retVal;
+}
+
+void XMLPlatformUtils::panic(const PanicReasons reason)
+{
+    const char* reasonStr = "Unknown reason";
+    switch (reason)
+    {
+    case Panic_NoTransService:
+        reasonStr = "Could not load a transcoding service";
+        break;
+    case Panic_NoDefTranscoder:
+        reasonStr = "Could not load a local code page transcoder";
+        break;
+    case Panic_CantFindLib:
+        reasonStr = "Could not find the xerces-c DLL";
+        break;
+    case Panic_UnknownMsgDomain:
+        reasonStr = "Unknown message domain";
+        break;
+    case Panic_CantLoadMsgDomain:
+        reasonStr = "Cannot load message domain";
+        break;
+    case Panic_SynchronizationErr:
+        reasonStr = "Cannot synchronize system or mutex";
+        break;
+    case Panic_SystemInit:
+        reasonStr = "Cannot initialize the system or mutex";
+        break;
+    }
+
+    fprintf(stderr, "Xerces Panic Error: %s\n", reasonStr);
+
+    exit(-1);
+}
 
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: File Methods
@@ -253,7 +331,7 @@ void XMLPlatformUtils::closeMutex(void* const mtxHandle)
         throw XMLPlatformUtilsException("Could not destroy a mutex");
     }
     if ( (pthread_mutex_t*)mtxHandle)
-        delete mtxHandle;
+        delete (pthread_mutex_t*) mtxHandle;
 }
 void XMLPlatformUtils::lockMutex(void* const mtxHandle)
 {
@@ -325,3 +403,5 @@ void XMLPlatformUtils::platformTerm()
     // We don't have any termination requirements at this time
 }
 
+
+XERCES_CPP_NAMESPACE_END
