@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: Tru64PlatformUtils.cpp,v 1.1.1.1 2002/02/01 22:22:26 peiyongz Exp $
+ * $Id: Tru64PlatformUtils.cpp,v 1.5 2002/11/04 15:13:01 tng Exp $
  */
 
 // ---------------------------------------------------------------------------
@@ -112,39 +112,11 @@
 #include <xercesc/util/NetAccessors/Socket/SocketNetAccessor.hpp>
 #endif
 
-// ---------------------------------------------------------------------------
-//  Local Methods
-// ---------------------------------------------------------------------------
-static void WriteCharStr( FILE* stream, const char* const toWrite)
-{
-  if (fputs(toWrite, stream) == EOF)
-  {
-    ThrowXML(XMLPlatformUtilsException,
-	     XMLExcepts::Strm_StdErrWriteFailure);
-  }
-}
+XERCES_CPP_NAMESPACE_BEGIN
 
-static void WriteUStrStdErr( const XMLCh* const toWrite)
-{
-  char* tmpVal = XMLString::transcode(toWrite);
-  ArrayJanitor<char> janText(tmpVal);
-  if (fputs(tmpVal, stderr) == EOF)
-  {
-    ThrowXML(XMLPlatformUtilsException,
-	     XMLExcepts::Strm_StdErrWriteFailure);
-  }
-}
-
-static void WriteUStrStdOut( const XMLCh* const toWrite)
-{
-  char* tmpVal = XMLString::transcode(toWrite);
-  ArrayJanitor<char> janText(tmpVal);
-  if (fputs(tmpVal, stdout) == EOF)
-  {
-    ThrowXML(XMLPlatformUtilsException,
-	     XMLExcepts::Strm_StdOutWriteFailure);
-  }
-}
+// ---------------------------------------------------------------------------
+//  XMLPlatformUtils: Private Static Methods
+// ---------------------------------------------------------------------------
 
 XMLNetAccessor* XMLPlatformUtils::makeNetAccessor()
 #if defined (XML_USE_NETACCESSOR_LIBWWW)
@@ -161,10 +133,6 @@ XMLNetAccessor* XMLPlatformUtils::makeNetAccessor()
 }
 #endif
 
-// ---------------------------------------------------------------------------
-//  XMLPlatformUtils: Private Static Methods
-// ---------------------------------------------------------------------------
-
 //
 //  This method is called by the platform independent part of this class
 //  when client code asks to have one of the supported message sets loaded.
@@ -185,7 +153,7 @@ XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
   }
   catch(...)
   {
-    panic(XMLPlatformUtils::Panic_NoDefTranscoder);
+    panic(XMLPlatformUtils::Panic_CantLoadMsgDomain);
   }
   return retVal;
 }
@@ -306,6 +274,18 @@ FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
   return retVal;
 }
 
+FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
+{
+    const char* tmpFileName = XMLString::transcode(fileName);
+    ArrayJanitor<char> janText((char*)tmpFileName);
+    return fopen( tmpFileName , "wb" );
+}
+
+FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName)
+{
+    return fopen( fileName , "wb" );
+}
+
 unsigned int XMLPlatformUtils::readFileBuffer(FileHandle theFile,
 					      const unsigned int toRead,
 					      XMLByte* const toFill)
@@ -322,6 +302,41 @@ unsigned int XMLPlatformUtils::readFileBuffer(FileHandle theFile,
   return (unsigned int) noOfItemsRead;
 }
 
+void
+XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
+                                   , long                  toWrite
+                                   , const XMLByte* const  toFlush)
+{
+    if (!theFile        ||
+        (toWrite <= 0 ) ||
+        !toFlush         )
+        return;
+
+    const XMLByte* tmpFlush = (const XMLByte*) toFlush;
+    size_t bytesWritten = 0;
+
+    while (true)
+    {
+        bytesWritten=fwrite(tmpFlush, sizeof(XMLByte), toWrite, (FILE*)theFile);
+
+        if(ferror((FILE*)theFile))
+        {
+            ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile);
+        }
+
+        if (bytesWritten < toWrite) //incomplete write
+        {
+            tmpFlush+=bytesWritten;
+            toWrite-=bytesWritten;
+            bytesWritten=0;
+        }
+        else
+            return;
+    }
+
+    return;
+}
+
 void XMLPlatformUtils::resetFile(FileHandle theFile)
 {
   // Seek to the start of the file
@@ -331,18 +346,9 @@ void XMLPlatformUtils::resetFile(FileHandle theFile)
 }
 
 
-
 // ---------------------------------------------------------------------------
-//  XMLPlatformUtils: Timing Methods
+//  XMLPlatformUtils: File system methods
 // ---------------------------------------------------------------------------
-
-unsigned long XMLPlatformUtils::getCurrentMillis()
-{
-  timespec ts;
-  clock_gettime (CLOCK_REALTIME, &ts);
-  return (unsigned long) ((ts.tv_sec * 1000) + (ts.tv_nsec / 1000000));
-}
-
 XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
 {
   //
@@ -493,6 +499,18 @@ XMLCh* XMLPlatformUtils::weavePaths (
 }
 
 
+// ---------------------------------------------------------------------------
+//  XMLPlatformUtils: Timing Methods
+// ---------------------------------------------------------------------------
+
+unsigned long XMLPlatformUtils::getCurrentMillis()
+{
+  timespec ts;
+  clock_gettime (CLOCK_REALTIME, &ts);
+  return (unsigned long) ((ts.tv_sec * 1000) + (ts.tv_nsec / 1000000));
+}
+
+
 
 #if !defined (APP_NO_THREADS)
 // ---------------------------------------------------------------------------
@@ -721,3 +739,5 @@ void XMLPlatformUtils::platformTerm()
 	gAtomicOpMutex = 0;
 #endif
 }
+
+XERCES_CPP_NAMESPACE_END
