@@ -56,6 +56,9 @@
 
 /*
  * $Log: UnixWarePlatformUtils.cpp,v $
+ * Revision 1.4  2002/11/04 15:13:01  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.3  2002/05/21 20:31:48  tng
  * Minor update: Remove obsolete code
  *
@@ -155,6 +158,7 @@
     #include <xercesc/util/NetAccessors/Socket/SocketNetAccessor.hpp>
 #endif
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: Private Static Methods
@@ -719,3 +723,4 @@ void XMLPlatformUtils::platformTerm()
 #endif
 }
 
+XERCES_CPP_NAMESPACE_END
