@@ -1,37 +1,37 @@
 /*
  * The Apache Software License, Version 1.1
- * 
- * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
+ *
+ * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
  * reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
- * 
+ *
  * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
- * 
+ *    notice, this list of conditions and the following disclaimer.
+ *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in
  *    the documentation and/or other materials provided with the
  *    distribution.
- * 
+ *
  * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:  
+ *    if any, must include the following acknowledgment:
  *       "This product includes software developed by the
  *        Apache Software Foundation (http://www.apache.org/)."
  *    Alternately, this acknowledgment may appear in the software itself,
  *    if and wherever such third-party acknowledgments normally appear.
- * 
+ *
  * 4. The names "Xerces" and "Apache Software Foundation" must
  *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written 
+ *    software without prior written permission. For written
  *    permission, please contact apache\@apache.org.
- * 
+ *
  * 5. Products derived from this software may not be called "Apache",
  *    nor may "Apache" appear in their name, without prior written
  *    permission of the Apache Software Foundation.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
  * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
  * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -45,7 +45,7 @@
  * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  * ====================================================================
- * 
+ *
  * This software consists of voluntary contributions made by many
  * individuals on behalf of the Apache Software Foundation, and was
  * originally based on software copyright (c) 1999, International
@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: Win32PlatformUtils.cpp,v 1.1.1.1 2002/02/01 22:22:28 peiyongz Exp $
+ * $Id: Win32PlatformUtils.cpp,v 1.13 2003/01/09 15:34:31 tng Exp $
  */
 
 // ---------------------------------------------------------------------------
@@ -69,6 +69,7 @@
 #include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/util/XMLUni.hpp>
 #include <windows.h>
+#include <stdio.h>
 #include <stdlib.h>
 
 #ifdef _DEBUG
@@ -88,6 +89,8 @@
     #include <xercesc/util/Transcoders/ICU/ICUTransService.hpp>
 #elif defined (XML_USE_WIN32_TRANSCODER)
     #include <xercesc/util/Transcoders/Win32/Win32TransService.hpp>
+#elif defined (XML_USE_CYGWIN_TRANSCODER)
+    #include <xercesc/util/Transcoders/Cygwin/CygwinTransService.hpp>
 #else
     #error A transcoding service must be chosen
 #endif
@@ -97,10 +100,12 @@
 //  They allow this to be controlled from the build process by just defining
 //  one of these values.
 //
-#if defined (XML_USE_INMEMORY_MSGLOADER)
+#if defined (XML_USE_INMEM_MESSAGELOADER)
     #include <xercesc/util/MsgLoaders/InMemory/InMemMsgLoader.hpp>
 #elif defined (XML_USE_WIN32_MSGLOADER)
     #include <xercesc/util/MsgLoaders/Win32/Win32MsgLoader.hpp>
+#elif defined(XML_USE_ICU_MESSAGELOADER)
+    #include <xercesc/util/MsgLoaders/ICU/ICUMsgLoader.hpp>
 #else
     #error A message loading service must be chosen
 #endif
@@ -116,7 +121,7 @@
     #include <xercesc/util/NetAccessors/WinSock/WinSockNetAccessor.hpp>
 #endif
 
-
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  Local data
@@ -134,32 +139,33 @@ static bool     gOnNT;
 void XMLPlatformUtils::panic(const PanicReasons reason)
 {
     const char* reasonStr = "Unknown reason";
-    if (reason == Panic_NoTransService)
+    switch (reason)
+    {
+    case Panic_NoTransService:
         reasonStr = "Could not load a transcoding service";
-    else if (reason == Panic_NoDefTranscoder)
+        break;
+    case Panic_NoDefTranscoder:
         reasonStr = "Could not load a local code page transcoder";
-    else if (reason == Panic_CantFindLib)
+        break;
+    case Panic_CantFindLib:
         reasonStr = "Could not find the xerces-c DLL";
-    else if (reason == Panic_UnknownMsgDomain)
+        break;
+    case Panic_UnknownMsgDomain:
         reasonStr = "Unknown message domain";
-    else if (reason == Panic_CantLoadMsgDomain)
+        break;
+    case Panic_CantLoadMsgDomain:
         reasonStr = "Cannot load message domain";
-    else if (reason == Panic_SynchronizationErr)
-        reasonStr = "A system synchronization error occurred";
-    else if (reason == Panic_SystemInit)
-        reasonStr = "Failed to complete platfrom dependent initialization";
+        break;
+    case Panic_SynchronizationErr:
+        reasonStr = "Cannot synchronize system or mutex";
+        break;
+    case Panic_SystemInit:
+        reasonStr = "Cannot initialize the system or mutex";
+        break;
+    }
+
+    fprintf(stderr, "Xerces Panic Error: %s\n", reasonStr);
 
-    //
-    //  We just do a popup and exit. Replace this code to do whatever
-    //  you need to do.
-    //
-    MessageBoxA
-    (
-        0
-        , "Xerces Panic Error"
-        , reasonStr
-        , MB_OK | MB_ICONSTOP
-    );
     exit(-1);
 }
 
@@ -256,18 +262,48 @@ FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
     if (!fileName)
         return 0;
 
+    //
+    //  We have to play a little trick here. If its /x:.....
+    //  style fully qualified path, we have to toss the leading /
+    //  character.
+    //
+    const XMLCh* nameToOpen = fileName;
+    if (*fileName == chForwardSlash)
+    {
+        if (XMLString::stringLen(fileName) > 3)
+        {
+            if (*(fileName + 2) == chColon)
+            {
+                const XMLCh chDrive = *(fileName + 1);
+                if (((chDrive >= chLatin_A) && (chDrive <= chLatin_Z))
+                ||  ((chDrive >= chLatin_a) && (chDrive <= chLatin_z)))
+                {
+                    nameToOpen = fileName + 1;
+                }
+            }
+
+            // Similarly for UNC paths
+            if ( *(fileName + 1) == *(fileName + 2) &&
+                 (*(fileName + 1) == chForwardSlash ||
+                  *(fileName + 1) == chBackSlash) )
+            {
+                nameToOpen = fileName + 1;
+            }
+        }
+    }
+
     //  Ok, this might look stupid but its a semi-expedient way to deal
     //  with a thorny problem. Shift-JIS and some other Asian encodings
     //  are fundamentally broken and map both the backslash and the Yen
     //  sign to the same code point. Transcoders have to pick one or the
-    //  other to map '\' to Unicode and tend to choose the Yen sign. 
+    //  other to map '\' to Unicode and tend to choose the Yen sign.
     //
     //  Unicode Yen or Won signs as directory separators will fail.
     //
     //  So, we will check this path name for Yen or won signs and, if they are
-    //  there, we'll replace them with slashes.  
+    //  there, we'll replace them with slashes.
     //
-    //  A further twist:  we replace Yen and Won with forward slashes rather 
+    //  A further twist:  we replace Yen and Won with forward slashes rather
     //   than back slashes.  Either form of slash will work as a directory
     //   separator.  On Win 95 and 98, though, Unicode back-slashes may
     //   fail to transode back to 8-bit 0x5C with some Unicode converters
@@ -275,9 +311,8 @@ FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
     //   transcode correctly back to 8 bit char * form.
     //
     XMLCh *tmpUName = 0;
-    const XMLCh *nameToOpen = fileName;
-    
-    const XMLCh* srcPtr = fileName;
+
+    const XMLCh* srcPtr = nameToOpen;
     while (*srcPtr)
     {
         if (*srcPtr == chYenSign ||
@@ -285,15 +320,15 @@ FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
             break;
         srcPtr++;
     }
-    
+
     //
     //  If we found a yen, then we have to create a temp file name. Else
     //  go with the file name as is and save the overhead.
     //
     if (*srcPtr)
     {
-        tmpUName = XMLString::replicate(fileName);
-        
+        tmpUName = XMLString::replicate(nameToOpen);
+
         XMLCh* tmpPtr = tmpUName;
         while (*tmpPtr)
         {
@@ -309,7 +344,7 @@ FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
     {
         retVal = ::CreateFileW
             (
-            nameToOpen
+            (LPCWSTR) nameToOpen
             , GENERIC_READ
             , FILE_SHARE_READ
             , 0
@@ -338,15 +373,130 @@ FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
         delete [] tmpName;
     }
 
-    if (tmpUName)  
+    if (tmpUName)
         delete [] tmpUName;
-    
+
     if (retVal == INVALID_HANDLE_VALUE)
         return 0;
-    
+
     return retVal;
 }
 
+FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName)
+{
+    FileHandle retVal = ::CreateFileA
+    (
+        fileName
+        , GENERIC_WRITE
+        , 0              // no shared write
+        , 0
+        , CREATE_ALWAYS
+        , FILE_FLAG_WRITE_THROUGH
+        , 0
+    );
+
+    if (retVal == INVALID_HANDLE_VALUE)
+        return 0;
+
+    return retVal;
+}
+
+FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
+{
+    // Watch for obvious wierdness
+    if (!fileName)
+        return 0;
+
+    //  Ok, this might look stupid but its a semi-expedient way to deal
+    //  with a thorny problem. Shift-JIS and some other Asian encodings
+    //  are fundamentally broken and map both the backslash and the Yen
+    //  sign to the same code point. Transcoders have to pick one or the
+    //  other to map '\' to Unicode and tend to choose the Yen sign.
+    //
+    //  Unicode Yen or Won signs as directory separators will fail.
+    //
+    //  So, we will check this path name for Yen or won signs and, if they are
+    //  there, we'll replace them with slashes.
+    //
+    //  A further twist:  we replace Yen and Won with forward slashes rather
+    //   than back slashes.  Either form of slash will work as a directory
+    //   separator.  On Win 95 and 98, though, Unicode back-slashes may
+    //   fail to transode back to 8-bit 0x5C with some Unicode converters
+    //   to  some of the problematic code pages.  Forward slashes always
+    //   transcode correctly back to 8 bit char * form.
+    //
+    XMLCh *tmpUName = 0;
+    const XMLCh *nameToOpen = fileName;
+
+    const XMLCh* srcPtr = fileName;
+    while (*srcPtr)
+    {
+        if (*srcPtr == chYenSign ||
+            *srcPtr == chWonSign)
+            break;
+        srcPtr++;
+    }
+
+    //
+    //  If we found a yen, then we have to create a temp file name. Else
+    //  go with the file name as is and save the overhead.
+    //
+    if (*srcPtr)
+    {
+        tmpUName = XMLString::replicate(fileName);
+
+        XMLCh* tmpPtr = tmpUName;
+        while (*tmpPtr)
+        {
+            if (*tmpPtr == chYenSign ||
+                *tmpPtr == chWonSign)
+                *tmpPtr = chForwardSlash;
+            tmpPtr++;
+        }
+        nameToOpen = tmpUName;
+    }
+    FileHandle retVal = 0;
+    if (gOnNT)
+    {
+        retVal = ::CreateFileW
+            (
+            (LPCWSTR) nameToOpen
+            , GENERIC_WRITE
+            , 0              // no shared write
+            , 0
+            , CREATE_ALWAYS
+            , FILE_FLAG_WRITE_THROUGH
+            , 0
+            );
+    }
+    else
+    {
+        //
+        //  We are Win 95 / 98.  Take the Unicode file name back to (char *)
+        //    so that we can open it.
+        //
+        char* tmpName = XMLString::transcode(nameToOpen);
+        retVal = ::CreateFileA
+            (
+            tmpName
+            , GENERIC_WRITE
+            , 0              // no shared write
+            , 0
+            , CREATE_ALWAYS
+            , FILE_FLAG_WRITE_THROUGH
+            , 0
+            );
+        delete [] tmpName;
+    }
+
+    if (tmpUName)
+        delete [] tmpUName;
+
+    if (retVal == INVALID_HANDLE_VALUE)
+        return 0;
+
+    return retVal;
+}
 
 FileHandle XMLPlatformUtils::openStdInHandle()
 {
@@ -357,8 +507,10 @@ FileHandle XMLPlatformUtils::openStdInHandle()
     //  shutting it would prevent any further output.
     //
     HANDLE stdInOrg = ::GetStdHandle(STD_INPUT_HANDLE);
-    if (stdInOrg == INVALID_HANDLE_VALUE)
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile);
+    if (stdInOrg == INVALID_HANDLE_VALUE) {
+        XMLCh stdinStr[] = {chLatin_s, chLatin_t, chLatin_d, chLatin_i, chLatin_n, chNull};
+        ThrowXML1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, stdinStr);
+    }
 
     HANDLE retHandle;
     if (!::DuplicateHandle
@@ -395,6 +547,36 @@ XMLPlatformUtils::readFileBuffer(       FileHandle      theFile
     return (unsigned int)bytesRead;
 }
 
+void
+XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
+                                   , long                  toWrite
+                                   , const XMLByte* const  toFlush)
+{
+    if (!theFile        ||
+        (toWrite <= 0 ) ||
+        !toFlush         )
+        return;
+
+    const XMLByte* tmpFlush = (const XMLByte*) toFlush;
+    unsigned long  bytesWritten = 0;
+
+    while (true)
+    {
+        if (!::WriteFile(theFile, tmpFlush, toWrite, &bytesWritten, 0))
+            ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile);
+
+        if (bytesWritten < (unsigned long) toWrite) //incomplete write
+        {
+            tmpFlush+=bytesWritten;
+            toWrite-=bytesWritten;
+            bytesWritten=0;
+        }
+        else
+            return;
+    }
+
+    return;
+}
 
 void XMLPlatformUtils::resetFile(FileHandle theFile)
 {
@@ -421,7 +603,7 @@ XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
         XMLCh tmpPath[bufSize + 1];
 
         XMLCh* namePart = 0;
-        if (!::GetFullPathNameW(srcPath, bufSize, tmpPath, &namePart))
+        if (!::GetFullPathNameW((LPCWSTR)srcPath, bufSize, (LPWSTR)tmpPath, (LPWSTR*)&namePart))
             return 0;
 
         // Return a copy of the path
@@ -595,7 +777,7 @@ unsigned long XMLPlatformUtils::getCurrentMillis()
 void XMLPlatformUtils::closeMutex(void* const mtxHandle)
 {
     ::DeleteCriticalSection((LPCRITICAL_SECTION)mtxHandle);
-    delete mtxHandle;
+    delete (CRITICAL_SECTION*)mtxHandle;
 }
 
 
@@ -627,6 +809,10 @@ XMLPlatformUtils::compareAndSwap(       void**      toFill
                                 , const void* const newValue
                                 , const void* const toCompare)
 {
+#if defined WIN64
+    return ::InterlockedCompareExchangePointer(toFill, (void*)newValue, (void*)toCompare);
+#else
+
     //
     //  InterlockedCompareExchange is only supported on Windows 98,
     //  Windows NT 4.0, and newer -- not on Windows 95...
@@ -649,18 +835,17 @@ XMLPlatformUtils::compareAndSwap(       void**      toFill
 
     #else
 
-    //    
+    //
     //  Note we have to cast off the constness of some of these because
     //  the system APIs are not C++ aware in all cases.
     //
-    return (void *) ::InterlockedCompareExchange
-    (
-        (long *)toFill
-        , (long)newValue
-        , (long)toCompare
-    );
+
+    return (void*) ::InterlockedCompareExchange((LPLONG)toFill, (LONG)newValue, (LONG)toCompare);
 
     #endif
+
+#endif
+
 }
 
 
@@ -708,10 +893,12 @@ XMLNetAccessor* XMLPlatformUtils::makeNetAccessor()
 //
 XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
 {
-#if defined (XML_USE_INMEMORY_MSGLOADER)
+#if defined (XML_USE_INMEM_MESSAGELOADER)
     return new InMemMsgLoader(msgDomain);
 #elif defined (XML_USE_WIN32_MSGLOADER)
     return new Win32MsgLoader(msgDomain);
+#elif defined (XML_USE_ICU_MESSAGELOADER)
+    return new ICUMsgLoader(msgDomain);
 #else
     #error You must provide a message loader
 #endif
@@ -736,6 +923,8 @@ XMLTransService* XMLPlatformUtils::makeTransService()
     return new ICUTransService;
 #elif defined (XML_USE_WIN32_TRANSCODER)
     return new Win32TransService;
+#elif defined (XML_USE_CYGWIN_TRANSCODER)
+    return new CygwinTransService;
 #else
     #error You must provide a transcoding service implementation
 #endif
@@ -752,7 +941,7 @@ void XMLPlatformUtils::platformInit()
 
 #if 0 && defined(_DEBUG)
     //  Enable this code for memeory leak testing
-    
+
    // Send all reports to STDOUT
    _CrtSetReportMode( _CRT_WARN, _CRTDBG_MODE_FILE );
    _CrtSetReportFile( _CRT_WARN, _CRTDBG_FILE_STDOUT );
@@ -779,3 +968,5 @@ void XMLPlatformUtils::platformTerm()
 {
     // We don't have any temrination requirements for win32 at this time
 }
+
+XERCES_CPP_NAMESPACE_END
