@@ -56,6 +56,18 @@
 
 /*
  * $Log: QName.cpp,v $
+ * Revision 1.6  2002/11/04 15:22:04  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.5  2002/10/30 21:52:00  tng
+ * [Bug 13641] compiler-generated copy-constructor for QName doesn't do the right thing.
+ *
+ * Revision 1.4  2002/09/24 19:51:24  tng
+ * Performance: use XMLString::equals instead of XMLString::compareString
+ *
+ * Revision 1.3  2002/09/05 16:06:41  tng
+ * [Bug 12232] Make operator to be constant.
+ *
  * Revision 1.2  2002/04/02 15:31:48  knoaman
  * Modiy QName comparison (operator=).
  *
@@ -94,6 +106,8 @@
 #include <xercesc/util/QName.hpp>
 #include <xercesc/util/Janitor.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 // ---------------------------------------------------------------------------
 //  QName: Constructors and Destructor
 // ---------------------------------------------------------------------------
@@ -169,7 +183,7 @@ QName::~QName()
 // ---------------------------------------------------------------------------
 //  QName: Copy Constructors
 // ---------------------------------------------------------------------------
-QName::QName(const QName* const qname) :
+QName::QName(const QName& qname) :
       fPrefix(0)
     , fPrefixBufSz(0)
     , fLocalPart(0)
@@ -180,17 +194,17 @@ QName::QName(const QName* const qname) :
 {
     unsigned int newLen;
 
-    newLen = XMLString::stringLen(qname->getLocalPart());
+    newLen = XMLString::stringLen(qname.getLocalPart());
     fLocalPartBufSz = newLen + 8;
     fLocalPart = new XMLCh[fLocalPartBufSz + 1];
-    XMLString::moveChars(fLocalPart, qname->getLocalPart(), newLen + 1);
+    XMLString::moveChars(fLocalPart, qname.getLocalPart(), newLen + 1);
 
-    newLen = XMLString::stringLen(qname->getPrefix());
+    newLen = XMLString::stringLen(qname.getPrefix());
     fPrefixBufSz = newLen + 8;
     fPrefix = new XMLCh[fPrefixBufSz + 1];
-    XMLString::moveChars(fPrefix, qname->getPrefix(), newLen + 1);
+    XMLString::moveChars(fPrefix, qname.getPrefix(), newLen + 1);
 
-    fURIId = qname->getURI();
+    fURIId = qname.getURI();
 }
 
 // ---------------------------------------------------------------------------
@@ -415,13 +429,13 @@ void QName::setValues(const QName& qname)
 // -----------------------------------------------------------------------
 //  comparison
 // -----------------------------------------------------------------------
-bool QName::operator==(const QName& qname)
+bool QName::operator==(const QName& qname) const
 {
     if (fURIId == 0) // null URI
-        return (XMLString::compareString(getRawName(),qname.getRawName())==0);
+        return (XMLString::equals(getRawName(),qname.getRawName()));
 
     return ((fURIId == qname.getURI()) &&
-           (XMLString::compareString(fLocalPart, qname.getLocalPart())==0));
+           (XMLString::equals(fLocalPart, qname.getLocalPart())));
 }
 
 // ---------------------------------------------------------------------------
@@ -434,3 +448,4 @@ void QName::cleanUp()
     delete [] fRawName;
 }
 
+XERCES_CPP_NAMESPACE_END
