@@ -55,13 +55,14 @@
  */
 
 /*
- * $Id: TransService.cpp,v 1.6 2002/07/24 19:10:57 knoaman Exp $
+ * $Id: TransService.cpp,v 1.11 2003/01/24 22:05:35 tng Exp $
  */
 // ---------------------------------------------------------------------------
 //  Includes
 // ---------------------------------------------------------------------------
 #include <xercesc/util/Janitor.hpp>
 #include <xercesc/util/RefHashTableOf.hpp>
+#include <xercesc/util/RefVectorOf.hpp>
 #include <xercesc/util/XML88591Transcoder.hpp>
 #include <xercesc/util/XMLASCIITranscoder.hpp>
 #include <xercesc/util/XMLChTranscoder.hpp>
@@ -75,8 +76,10 @@
 #include <xercesc/util/XMLUni.hpp>
 #include <xercesc/util/TransENameMap.hpp>
 #include <xercesc/util/EncodingValidator.hpp>
+#include <xercesc/util/XMLRegisterCleanup.hpp>
+#include <xercesc/util/PlatformUtils.hpp>
 
-
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  Local, static data
@@ -85,34 +88,69 @@
 //      This is a hash table of ENameMap objects. It is created and filled
 //      in when the platform init calls our initTransService() method.
 //
-//  gDisallow1
-//  gDisallowX
-//      These area small set of encoding names that, for temporary reasons,
-//      we disallow at this time.
+//  gMappingsRecognizer
+//      This is an array of ENameMap objects, predefined for those
+//      already recognized by XMLRecognizer::Encodings.
 //
-//  gDisallowList
-//  gDisallowListSize
-//      An array of the disallow strings, for easier searching below.
+//  gStrictIANAEncoding
+//      A flag to control whether strict IANA encoding names checking should
+//      be done
 //
-//  gDisallowPre
-//      All of the disallowed encodings start with 'IBM', so we have a prefix
-//      we can check for and quickly decide if we need to search the list.
 // ---------------------------------------------------------------------------
 static RefHashTableOf<ENameMap>*    gMappings = 0;
+static RefVectorOf<ENameMap>*       gMappingsRecognizer = 0;
 static bool gStrictIANAEncoding = false;
 
+// -----------------------------------------------------------------------
+//  Notification that lazy data has been deleted
+// -----------------------------------------------------------------------
+static void reinitMappings() {
+    delete gMappings;    // The contents of the gMappings hash table are owned by
+    gMappings = 0;       //   the it, and so will be deleted by gMapping's destructor.
+}
+
+static void reinitMappingsRecognizer() {
+    delete gMappingsRecognizer;
+    gMappingsRecognizer = 0;
+}
 
 // ---------------------------------------------------------------------------
-//  XLMTransService: Constructors and destructor
+//  XMLTransService: Constructors and destructor
 // ---------------------------------------------------------------------------
 XMLTransService::XMLTransService()
 {
+    static XMLRegisterCleanup mappingsCleanup;
+    static XMLRegisterCleanup mappingsRecognizerCleanup;
+
+    if (!gMappings) {
+        RefHashTableOf<ENameMap>* t = new RefHashTableOf<ENameMap>(103);
+
+        if (XMLPlatformUtils::compareAndSwap((void **)&gMappings, t, 0) != 0)
+        {
+            delete t;
+        }
+        else
+        {
+            mappingsCleanup.registerCleanup(reinitMappings);
+        }
+    }
+
+    if (!gMappingsRecognizer) {
+        RefVectorOf<ENameMap>* t = new RefVectorOf<ENameMap>(XMLRecognizer::Encodings_Count);
+
+        if (XMLPlatformUtils::compareAndSwap((void **)&gMappingsRecognizer, t, 0) != 0)
+        {
+            delete t;
+        }
+        else
+        {
+            mappingsRecognizerCleanup.registerCleanup(reinitMappingsRecognizer);
+        }
+    }
 }
 
 XMLTransService::~XMLTransService()
 {
-    delete gMappings;    // The contents of the gMappings hash table are owned by
-    gMappings = 0;       //   the it, and so will be deleted by gMapping's destructor.
 }
 
 // ---------------------------------------------------------------------------
@@ -120,16 +158,13 @@ XMLTransService::~XMLTransService()
 //	Should be called after platform init
 // ---------------------------------------------------------------------------
 void XMLTransService::addEncoding(const XMLCh* const encoding,
-								  ENameMap* const ownMapping) {
-
-	if (gMappings) {
-
-		gMappings->put((void *) encoding, ownMapping);
-	}
+								  ENameMap* const ownMapping)
+{
+    gMappings->put((void *) encoding, ownMapping);
 }
 
 // ---------------------------------------------------------------------------
-//  XLMTranscoder: Non-virtual API
+//  XMLTransService: Non-virtual API
 // ---------------------------------------------------------------------------
 XMLTranscoder*
 XMLTransService::makeNewTranscoderFor(  const   char* const             encodingName
@@ -154,7 +189,7 @@ XMLTransService::makeNewTranscoderFor(  const   XMLCh* const            encoding
     {
         if (!EncodingValidator::instance()->isValidEncoding(encodingName))
         {
-            resValue = XMLTransService::UnsupportedEncoding;               
+            resValue = XMLTransService::UnsupportedEncoding;
             return 0;
         }
     }
@@ -190,84 +225,73 @@ XMLTransService::makeNewTranscoderFor(  const   XMLCh* const            encoding
 
     XMLTranscoder* temp =  makeNewXMLTranscoder(encodingName, resValue, blockSize);
 
-    // if successful, set resValue to OK  
+    // if successful, set resValue to OK
     // if failed, the makeNewXMLTranscoder has already set the proper failing resValue
-    if (temp) resValue =  XMLTransService::Ok;    
+    if (temp) resValue =  XMLTransService::Ok;
 
     return temp;
 
 }
 
 
-// ---------------------------------------------------------------------------
-//  XLMTranscoder: Public Destructor
-// ---------------------------------------------------------------------------
-XMLTranscoder::~XMLTranscoder()
+XMLTranscoder*
+XMLTransService::makeNewTranscoderFor(  XMLRecognizer::Encodings        encodingEnum
+                                        ,       XMLTransService::Codes& resValue
+                                        , const unsigned int            blockSize)
 {
-    delete [] fEncodingName;
+    //
+    // We can only make transcoder if the passed encodingEnum is under this range
+    //
+    if (encodingEnum < XMLRecognizer::Encodings_Min || encodingEnum > XMLRecognizer::Encodings_Max) {
+        resValue = XMLTransService::InternalFailure;
+        return 0;
+    }
+
+    ENameMap* ourMapping = gMappingsRecognizer->elementAt(encodingEnum);
+
+    // If we found it, then call the factory method for it
+    if (ourMapping)	{		
+       XMLTranscoder* temp = ourMapping->makeNew(blockSize);
+       resValue = temp ? XMLTransService::Ok : XMLTransService::InternalFailure;
+       return temp;
+    }
+    else {
+        XMLTranscoder* temp =  makeNewXMLTranscoder(XMLRecognizer::nameForEncoding(encodingEnum), resValue, blockSize);
+
+        // if successful, set resValue to OK
+        // if failed, the makeNewXMLTranscoder has already set the proper failing resValue
+        if (temp) resValue =  XMLTransService::Ok;
+
+        return temp;
+    }
+
 }
 
 
 // ---------------------------------------------------------------------------
-//  XLMTranscoder: Hidden Constructors
-// ---------------------------------------------------------------------------
-XMLTranscoder::XMLTranscoder(const  XMLCh* const    encodingName
-                            , const unsigned int    blockSize) :
-    fEncodingName(0)
-    , fBlockSize(blockSize)
-{
-    fEncodingName = XMLString::replicate(encodingName);
-}
-
-
-// ---------------------------------------------------------------------------
-//  XLMTranscoder: Protected helpers
-// ---------------------------------------------------------------------------
-void XMLTranscoder::checkBlockSize(const unsigned int toCheck)
-{
-//    if (toCheck > fBlockSize)
-//        ThrowXML(TranscodingException, XMLExcepts::Trans_BadBlockSize);
-}
-
-
-// ---------------------------------------------------------------------------
-//  XLMLCPTranscoder: Public Destructor
-// ---------------------------------------------------------------------------
-XMLLCPTranscoder::XMLLCPTranscoder()
-{
-}
-
-
-// ---------------------------------------------------------------------------
-//  XLMTranscoder: Hidden Constructors
-// ---------------------------------------------------------------------------
-XMLLCPTranscoder::~XMLLCPTranscoder()
-{
-}
-
-// ---------------------------------------------------------------------------
-//  XLMTranscoder: Hidden Init Method
+//  XMLTransTransService: Hidden Init Method
 //
 //  This is called by platform utils during startup.
 // ---------------------------------------------------------------------------
 void XMLTransService::initTransService()
 {
-
     //
-    //  Create our hash table that we will fill with mappings. The default
-    //  is to adopt the elements, which is fine with us.
+    //  A stupid way to increment the fCurCount inside the RefVectorOf
     //
-    gMappings = new RefHashTableOf<ENameMap>(103);
+    for (unsigned int i = 0; i < XMLRecognizer::Encodings_Count; i++)
+        gMappingsRecognizer->addElement(0);
 
     //
     //  Add in the magical mapping for the native XMLCh transcoder. This
     //  is used for internal entities.
     //
+    gMappingsRecognizer->setElementAt(new ENameMapFor<XMLChTranscoder>(XMLUni::fgXMLChEncodingString), XMLRecognizer::XERCES_XMLCH);
     gMappings->put((void*)XMLUni::fgXMLChEncodingString, new ENameMapFor<XMLChTranscoder>(XMLUni::fgXMLChEncodingString));
 
     //
     //  Add in our mappings for ASCII.
     //
+    gMappingsRecognizer->setElementAt(new ENameMapFor<XMLASCIITranscoder>(XMLUni::fgUSASCIIEncodingString), XMLRecognizer::US_ASCII);
     gMappings->put((void*)XMLUni::fgUSASCIIEncodingString, new ENameMapFor<XMLASCIITranscoder>(XMLUni::fgUSASCIIEncodingString));
     gMappings->put((void*)XMLUni::fgUSASCIIEncodingString2, new ENameMapFor<XMLASCIITranscoder>(XMLUni::fgUSASCIIEncodingString2));
     gMappings->put((void*)XMLUni::fgUSASCIIEncodingString3, new ENameMapFor<XMLASCIITranscoder>(XMLUni::fgUSASCIIEncodingString3));
@@ -277,6 +301,7 @@ void XMLTransService::initTransService()
     //
     //  Add in our mappings for UTF-8
     //
+    gMappingsRecognizer->setElementAt(new ENameMapFor<XMLUTF8Transcoder>(XMLUni::fgUTF8EncodingString), XMLRecognizer::UTF_8);
     gMappings->put((void*)XMLUni::fgUTF8EncodingString, new ENameMapFor<XMLUTF8Transcoder>(XMLUni::fgUTF8EncodingString));
     gMappings->put((void*)XMLUni::fgUTF8EncodingString2, new ENameMapFor<XMLUTF8Transcoder>(XMLUni::fgUTF8EncodingString2));
 
@@ -291,6 +316,10 @@ void XMLTransService::initTransService()
     gMappings->put((void*)XMLUni::fgISO88591EncodingString6, new ENameMapFor<XML88591Transcoder>(XMLUni::fgISO88591EncodingString6));
     gMappings->put((void*)XMLUni::fgISO88591EncodingString7, new ENameMapFor<XML88591Transcoder>(XMLUni::fgISO88591EncodingString7));
     gMappings->put((void*)XMLUni::fgISO88591EncodingString8, new ENameMapFor<XML88591Transcoder>(XMLUni::fgISO88591EncodingString8));
+    gMappings->put((void*)XMLUni::fgISO88591EncodingString9, new ENameMapFor<XML88591Transcoder>(XMLUni::fgISO88591EncodingString9));
+    gMappings->put((void*)XMLUni::fgISO88591EncodingString10, new ENameMapFor<XML88591Transcoder>(XMLUni::fgISO88591EncodingString10));
+    gMappings->put((void*)XMLUni::fgISO88591EncodingString11, new ENameMapFor<XML88591Transcoder>(XMLUni::fgISO88591EncodingString11));
+    gMappings->put((void*)XMLUni::fgISO88591EncodingString12, new ENameMapFor<XML88591Transcoder>(XMLUni::fgISO88591EncodingString12));
 
     //
     //  Add in our mappings for UTF-16 and UCS-4, little endian
@@ -300,6 +329,7 @@ void XMLTransService::initTransService()
     #if defined(ENDIANMODE_BIG)
     swapped = true;
     #endif
+    gMappingsRecognizer->setElementAt(new EEndianNameMapFor<XMLUTF16Transcoder>(XMLUni::fgUTF16LEncodingString, swapped), XMLRecognizer::UTF_16L);
     gMappings->put
     (
 		(void*)XMLUni::fgUTF16LEncodingString,
@@ -320,6 +350,7 @@ void XMLTransService::initTransService()
         )
     );
 
+    gMappingsRecognizer->setElementAt(new EEndianNameMapFor<XMLUCS4Transcoder>(XMLUni::fgUCS4LEncodingString, swapped), XMLRecognizer::UCS_4L);
     gMappings->put
     (
 		(void*)XMLUni::fgUCS4LEncodingString,
@@ -347,6 +378,7 @@ void XMLTransService::initTransService()
     #if defined(ENDIANMODE_LITTLE)
     swapped = true;
     #endif
+    gMappingsRecognizer->setElementAt(new EEndianNameMapFor<XMLUTF16Transcoder>(XMLUni::fgUTF16BEncodingString, swapped), XMLRecognizer::UTF_16B);
     gMappings->put
     (
 		(void*)XMLUni::fgUTF16BEncodingString,
@@ -367,6 +399,7 @@ void XMLTransService::initTransService()
         )
     );
 
+    gMappingsRecognizer->setElementAt(new EEndianNameMapFor<XMLUCS4Transcoder>(XMLUni::fgUCS4BEncodingString, swapped), XMLRecognizer::UCS_4B);
     gMappings->put
     (
 		(void*)XMLUni::fgUCS4BEncodingString,
@@ -460,6 +493,7 @@ void XMLTransService::initTransService()
     //  Add in our mappings for IBM037, and the one alias we support for
     //  it, which is EBCDIC-CP-US.
     //
+    gMappingsRecognizer->setElementAt(new ENameMapFor<XMLEBCDICTranscoder>(XMLUni::fgEBCDICEncodingString), XMLRecognizer::EBCDIC);
     gMappings->put((void*)XMLUni::fgIBM037EncodingString, new ENameMapFor<XMLEBCDICTranscoder>(XMLUni::fgIBM037EncodingString));
     gMappings->put((void*)XMLUni::fgIBM037EncodingString2, new ENameMapFor<XMLEBCDICTranscoder>(XMLUni::fgIBM037EncodingString2));
 
@@ -469,6 +503,8 @@ void XMLTransService::initTransService()
     //
     gMappings->put((void*)XMLUni::fgIBM1140EncodingString, new ENameMapFor<XMLIBM1140Transcoder>(XMLUni::fgIBM1140EncodingString));
     gMappings->put((void*)XMLUni::fgIBM1140EncodingString2, new ENameMapFor<XMLIBM1140Transcoder>(XMLUni::fgIBM1140EncodingString2));
+    gMappings->put((void*)XMLUni::fgIBM1140EncodingString3, new ENameMapFor<XMLIBM1140Transcoder>(XMLUni::fgIBM1140EncodingString3));
+    gMappings->put((void*)XMLUni::fgIBM1140EncodingString4, new ENameMapFor<XMLIBM1140Transcoder>(XMLUni::fgIBM1140EncodingString4));
 
     //
     //  Add in our mappings for Windows-1252. We don't have any aliases for
@@ -479,7 +515,7 @@ void XMLTransService::initTransService()
 }
 
 // ---------------------------------------------------------------------------
-//  XLMTransService: IANA encoding setting
+//  XMLTransService: IANA encoding setting
 // ---------------------------------------------------------------------------
 void XMLTransService::strictIANAEncoding(const bool newState)
 {
@@ -490,3 +526,51 @@ bool XMLTransService::isStrictIANAEncoding()
 {
     return gStrictIANAEncoding;
 }
+
+// ---------------------------------------------------------------------------
+//  XMLTranscoder: Public Destructor
+// ---------------------------------------------------------------------------
+XMLTranscoder::~XMLTranscoder()
+{
+    delete [] fEncodingName;
+}
+
+
+// ---------------------------------------------------------------------------
+//  XMLTranscoder: Hidden Constructors
+// ---------------------------------------------------------------------------
+XMLTranscoder::XMLTranscoder(const  XMLCh* const    encodingName
+                            , const unsigned int    blockSize) :
+    fEncodingName(0)
+    , fBlockSize(blockSize)
+{
+    fEncodingName = XMLString::replicate(encodingName);
+}
+
+
+// ---------------------------------------------------------------------------
+//  XMLTranscoder: Protected helpers
+// ---------------------------------------------------------------------------
+void XMLTranscoder::checkBlockSize(const unsigned int toCheck)
+{
+//    if (toCheck > fBlockSize)
+//        ThrowXML(TranscodingException, XMLExcepts::Trans_BadBlockSize);
+}
+
+
+// ---------------------------------------------------------------------------
+//  XMLLCPTranscoder: Public Destructor
+// ---------------------------------------------------------------------------
+XMLLCPTranscoder::XMLLCPTranscoder()
+{
+}
+
+
+// ---------------------------------------------------------------------------
+//  XMLLCPTranscoder: Hidden Constructors
+// ---------------------------------------------------------------------------
+XMLLCPTranscoder::~XMLLCPTranscoder()
+{
+}
+
+XERCES_CPP_NAMESPACE_END
