@@ -56,6 +56,16 @@
 
 /*
  * $Log: TransService.hpp,v $
+ * Revision 1.6  2003/02/04 22:11:52  peiyongz
+ * bug#16784: Obsolete documentation on XMLTranscoder -- reported by
+ * Colin Paul Adams, Preston Lancashire
+ *
+ * Revision 1.5  2002/11/25 21:27:52  tng
+ * Performance: use XMLRecognizer::Encodings enum to make new transcode, faster than comparing the encoding string every time.
+ *
+ * Revision 1.4  2002/11/04 15:22:04  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.3  2002/07/18 20:05:31  knoaman
  * Add a new feature to control strict IANA encoding name.
  *
@@ -127,7 +137,9 @@
 #define TRANSSERVICE_HPP
 
 #include <xercesc/util/XercesDefs.hpp>
+#include <xercesc/framework/XMLRecognizer.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 // Forward references
 class XMLPlatformUtils;
@@ -191,6 +203,13 @@ public :
         , const unsigned int            blockSize
     );
 
+    XMLTranscoder* makeNewTranscoderFor
+    (
+        XMLRecognizer::Encodings        encodingEnum
+        ,       XMLTransService::Codes& resValue
+        , const unsigned int            blockSize
+    );
+
 
     // -----------------------------------------------------------------------
     //  The virtual transcoding service API
@@ -280,7 +299,7 @@ private :
   * strings, it is not meant to be a comphrehensive string class.
   */
 
-/** 
+/**
   *   <code>XMLTranscoder</code> is for transcoding non-local code
   *   page encodings, i.e.  named encodings. These are used internally
   *   by the scanner to internalize raw XML into the internal Unicode
@@ -292,7 +311,7 @@ class XMLUTIL_EXPORT XMLTranscoder
 {
 public :
 
-	/** 
+	/**
 	 * This enum is used by the <code>transcodeTo()</code> method
 	 * to indicate how to react to unrepresentable characters. The
 	 * <code>transcodeFrom()</code> method always works the
@@ -358,7 +377,7 @@ public :
       * @param maxBytes   the max number of bytes in the destination buffer
       * @param charsEaten after transcoding, this will hold the number of chars
       *    that were processed from the source buffer
-      * @param options    options to pass to the transcoder that explain how to 
+      * @param options    options to pass to the transcoder that explain how to
       *    respond to an unrepresentable character
       * @return Returns the number of chars put into the target buffer
       */
@@ -390,11 +409,7 @@ public :
 
     /** Get the internal block size
      *
-       * @return The block size indicated in the constructor. This
-       *   lets the derived class preallocate appopriately sized
-       *   buffers. This sets the maximum number of characters which can
-       *   be internalized per call to <code>transcodeFrom()</code> and
-       *   <code>transcodeTo()</code>.
+       * @return The block size indicated in the constructor.
        */
     unsigned int getBlockSize() const;
 
@@ -435,10 +450,7 @@ private :
     //  Private data members
     //
     //  fBlockSize
-    //      This is the block size indicated in the constructor. This lets
-    //      the derived class preallocate appopriately sized buffers. This
-    //      sets the maximum number of characters which can be internalized
-    //      per call to transcodeFrom() and transcodeTo().
+    //      This is the block size indicated in the constructor.
     //
     //  fEncodingName
     //      This is the name of the encoding this encoder is for. All basic
@@ -526,4 +538,6 @@ inline const XMLCh* XMLTranscoder::getEncodingName() const
     return fEncodingName;
 }
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
