@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: IconvTransService.cpp,v 1.4 2002/06/18 14:43:32 tng Exp $
+ * $Id: IconvTransService.cpp,v 1.10 2003/01/10 18:43:31 tng Exp $
  */
 
 
@@ -67,7 +67,17 @@
 #include "IconvTransService.hpp"
 #include <wchar.h>
 #if defined (XML_GCC) || defined (XML_PTX) || defined (XML_IBMVAOS2)
-    #if !defined(XML_OPENSERVER)
+    #if defined(XML_BEOS)
+        wint_t towlower(wint_t wc) {
+          return ((wc>'A')&&(wc<'Z') ? wc+'a'-'A' : wc);
+        }
+        wint_t towupper(wint_t wc) {
+          return ((wc>'a')&&(wc<'z') ? wc-'a'+'A' : wc);
+        }
+        wint_t iswspace(wint_t wc) {
+          return (wc==(wint_t)' ');
+        }
+    #elif !defined(XML_OPENSERVER)
         #include <wctype.h>
     #endif
 #endif
@@ -75,6 +85,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  Local, const data
@@ -246,11 +257,15 @@ unsigned int IconvLCPTranscoder::calcRequiredSize(const char* const srcText)
     if (!srcText)
         return 0;
 
-    const unsigned int retVal = ::mbstowcs(NULL, srcText, 0);
-
-    if (retVal == ~0)
+    unsigned charLen = ::mblen(srcText, MB_CUR_MAX);
+    if (charLen == -1)
         return 0;
-    return retVal;
+    else if (charLen != 0)
+        charLen = strlen(srcText)/charLen;
+
+    if (charLen == -1)
+        return 0;
+    return charLen;
 }
 
 
@@ -371,14 +386,15 @@ bool IconvLCPTranscoder::transcode( const   XMLCh* const    toTranscode
     wideCharBuf[wLent] = 0x00;
 
     // Ok, go ahead and try the transcoding. If it fails, then ...
-    if (::wcstombs(toFill, wideCharBuf, maxBytes) == -1)
+    size_t mblen = ::wcstombs(toFill, wideCharBuf, maxBytes);
+    if (mblen == -1)
     {
         delete [] allocatedArray;
         return false;
     }
 
     // Cap it off just in case
-    toFill[wLent] = 0;
+    toFill[mblen] = 0;
     delete [] allocatedArray;
     return true;
 }
@@ -487,71 +503,4 @@ IconvLCPTranscoder::~IconvLCPTranscoder()
 {
 }
 
-// ---------------------------------------------------------------------------
-//  IconvTranscoder: Constructors and Destructor
-// ---------------------------------------------------------------------------
-IconvTranscoder::IconvTranscoder(const  XMLCh* const    encodingName
-                                , const unsigned int    blockSize) :
-
-    XMLTranscoder(encodingName, blockSize)
-{
-}
-
-IconvTranscoder::~IconvTranscoder()
-{
-}
-
-
-// ---------------------------------------------------------------------------
-//  IconvTranscoder: Implementation of the virtual transcoder API
-// ---------------------------------------------------------------------------
-XMLCh IconvTranscoder::transcodeOne(const   XMLByte* const  srcData
-                                    , const unsigned int    srcBytes
-                                    ,       unsigned int&   bytesEaten)
-{
-    wchar_t  toFill;
-    int eaten = ::mbtowc(&toFill, (const char*)srcData, srcBytes);
-    if (eaten == -1)
-    {
-        bytesEaten = 0;
-        return 0;
-    }
-
-    // Return the bytes we ate and the resulting char.
-    bytesEaten = eaten;
-    return toFill;
-}
-
-
-unsigned int
-IconvTranscoder::transcodeXML(  const   XMLByte* const          srcData
-                                , const unsigned int            srcCount
-                                ,       XMLCh* const            toFill
-                                , const unsigned int            maxChars
-                                ,       unsigned int&           bytesEaten
-                                ,       unsigned char* const    charSizes)
-{
-    //
-    //  For this one, because we have to maintain the offset table, we have
-    //  to do them one char at a time until we run out of source data.
-    //
-    unsigned int countIn = 0;
-    unsigned int countOut = 0;
-    while (countOut < maxChars)
-    {
-        wchar_t   oneWideChar;
-        const int bytesEaten =
-            ::mbtowc(&oneWideChar, (const char*)&srcData[countIn], srcCount - countIn);
-
-        // We are done, so break out
-        if (bytesEaten == -1)
-            break;
-        toFill[countOut] = (XMLCh) oneWideChar;
-        countIn += (unsigned int) bytesEaten;
-        countOut++;
-    }
-
-    // Give back the counts of eaten and transcoded
-    bytesEaten = countIn;
-    return countOut;
-}
+XERCES_CPP_NAMESPACE_END
