@@ -56,6 +56,12 @@
 
 /*
  * $Log: Iconv390TransService.hpp,v $
+ * Revision 1.4  2003/01/06 21:46:08  tng
+ * Since 390 uses ICU or Uniconv390 TransService, change Iconv390 to use intrinsic transcoders only so as to bypass Bug 2393.
+ *
+ * Revision 1.3  2002/11/04 15:14:33  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.2  2002/04/09 15:44:00  knoaman
  * Add lower case string support.
  *
@@ -83,6 +89,8 @@
 #include <xercesc/util/Mutexes.hpp>
 #include <iconv.h>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 /* Max encoding name in characters. */
 #define UCNV_MAX_CONVERTER_NAME_LENGTH 60
 typedef struct iconvconverter {
@@ -150,50 +158,6 @@ private :
     void operator=(const Iconv390TransService&);
 };
 
-class XMLUTIL_EXPORT Iconv390Transcoder : public XMLTranscoder
-{
-public :
-    // -----------------------------------------------------------------------
-    //  Constructors and Destructor
-    // -----------------------------------------------------------------------
-    Iconv390Transcoder(const XMLCh* const encodingName, const unsigned int blockSize);
-    Iconv390Transcoder(iconvconverter_t* const toAdopt,
-         const XMLCh* const encodingName, const unsigned int blockSize);
-    ~Iconv390Transcoder();
-
-
-    // -----------------------------------------------------------------------
-    //  Implementation of the virtual transcoder interface
-    // -----------------------------------------------------------------------
-    virtual XMLCh transcodeOne
-    (
-        const   XMLByte* const     srcData
-        , const unsigned int    srcBytes
-        ,       unsigned int&   bytesEaten
-    );
-
-    virtual unsigned int transcodeXML
-    (
-        const   XMLByte* const          srcData
-        , const unsigned int            srcCount
-        ,       XMLCh* const            toFill
-        , const unsigned int            maxChars
-        ,       unsigned int&           bytesEaten
-	,       unsigned char* const    charSizes
-    );
-
-
-private :
-    // -----------------------------------------------------------------------
-    //  Unimplemented constructors and operators
-    // -----------------------------------------------------------------------
-    Iconv390Transcoder(const Iconv390Transcoder&);
-    void operator=(const Iconv390Transcoder&);
-    iconvconverter_t *converter;
-};
-
-
-
 class XMLUTIL_EXPORT Iconv390LCPTranscoder : public XMLLCPTranscoder
 {
 public :
@@ -239,4 +203,6 @@ private :
     iconvconverter_t *converter;
 };
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
