@@ -1,37 +1,37 @@
 /*
  * The Apache Software License, Version 1.1
- * 
- * Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
+ *
+ * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
  * reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
- * 
+ *
  * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
- * 
+ *    notice, this list of conditions and the following disclaimer.
+ *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in
  *    the documentation and/or other materials provided with the
  *    distribution.
- * 
+ *
  * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:  
+ *    if any, must include the following acknowledgment:
  *       "This product includes software developed by the
  *        Apache Software Foundation (http://www.apache.org/)."
  *    Alternately, this acknowledgment may appear in the software itself,
  *    if and wherever such third-party acknowledgments normally appear.
- * 
+ *
  * 4. The names "Xerces" and "Apache Software Foundation" must
  *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written 
+ *    software without prior written permission. For written
  *    permission, please contact apache\@apache.org.
- * 
+ *
  * 5. Products derived from this software may not be called "Apache",
  *    nor may "Apache" appear in their name, without prior written
  *    permission of the Apache Software Foundation.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
  * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
  * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -45,7 +45,7 @@
  * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  * ====================================================================
- * 
+ *
  * This software consists of voluntary contributions made by many
  * individuals on behalf of the Apache Software Foundation, and was
  * originally based on software copyright (c) 1999, International
@@ -56,7 +56,8 @@
 
 /**
  * @01A D998714.1 V5R2M0    100301   Swan    :Fix error return flags
- * $Id: Iconv400TransService.cpp,v 1.1.1.1 2002/02/01 22:22:36 peiyongz Exp $
+ * @02A           V5R2M0    200419   jrhansen : support lowercase function
+ * $Id: Iconv400TransService.cpp,v 1.7 2003/01/06 21:47:48 tng Exp $
  */
 
 // ---------------------------------------------------------------------------
@@ -73,6 +74,8 @@
 #include <xercesc/util/XMLString.hpp>
 #include <xercesc/util/Janitor.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 // ---------------------------------------------------------------------------
 //  Local functions
 // ---------------------------------------------------------------------------
@@ -110,10 +113,15 @@ static const XMLCh gMyServiceId[] =
 // ---------------------------------------------------------------------------
 Iconv400TransService::Iconv400TransService()
 {
-    memset((char*)&convertCtlblk,'\0',sizeof(convertCtlblk));
-    convertCtlblk.Type_of_Request = 1;
-    convertCtlblk.Case_Request = 0;
-    convertCtlblk.CCSID_of_Input_Data = 13488;
+    memset((char*)&convertCtlblkUpper,'\0',sizeof(convertCtlblkUpper));
+    convertCtlblkUpper.Type_of_Request = 1;
+    convertCtlblkUpper.Case_Request = 0;   // upper case
+    convertCtlblkUpper.CCSID_of_Input_Data = 13488;
+
+    memset((char*)&convertCtlblkLower,'\0',sizeof(convertCtlblkLower));
+    convertCtlblkLower.Type_of_Request = 1;
+    convertCtlblkLower.Case_Request = 1;
+    convertCtlblkLower.CCSID_of_Input_Data = 13488;
 }
 
 Iconv400TransService::~Iconv400TransService()
@@ -128,11 +136,11 @@ int Iconv400TransService::compareIString(const   XMLCh* const    comp1
                                          , const XMLCh* const    comp2)
 {
     const XMLCh* psz1 = comp1;
-    const XMLCh* psz2 = comp2;    
+    const XMLCh* psz2 = comp2;
 
     while (true)
     {
-                       
+
         if (toUnicodeUpper(*psz1) != toUnicodeUpper(*psz2))
             return int(*psz1) - int(*psz2);
 
@@ -154,13 +162,13 @@ int Iconv400TransService::compareNIString(const  XMLCh* const    comp1
 {
     const XMLCh* psz1 = comp1;
     const XMLCh* psz2 = comp2;
-  
+
 
     unsigned int curCount = 0;
     while (true)
     {
         // If an inequality, then return the difference
-  
+
 
         // If an inequality, then return difference
         if (toUnicodeUpper(*psz1) != toUnicodeUpper(*psz2))
@@ -175,7 +183,7 @@ int Iconv400TransService::compareNIString(const  XMLCh* const    comp1
         psz2++;
 
         //
-        //  Bump the count of chars done. If it equals the count then we 
+        //  Bump the count of chars done. If it equals the count then we
         //  are equal for the requested count, so break out and return
         //  equal.
         //
@@ -245,6 +253,16 @@ void Iconv400TransService::upperCase(XMLCh* const toUpperCase) const
     }
 }
 
+void Iconv400TransService::lowerCase(XMLCh* const toLowerCase) const
+{
+    XMLCh* outPtr = toLowerCase;
+    while (*outPtr)
+    {
+        *outPtr = toUnicodeLower(*outPtr);
+        outPtr++;
+    }
+}
+
 // ---------------------------------------------------------------------------
 //  Iconv400TransService: The virtual transcoding service API
 // ---------------------------------------------------------------------------
@@ -259,9 +277,32 @@ XMLCh Iconv400TransService::toUnicodeUpper(XMLCh comp1) const
              char exception_data[15];
             } error_code;
      error_code.bytes_available = sizeof(error_code);
- 
+
     long charlen =2;
-    QlgConvertCase((char*)&convertCtlblk,
+
+    QlgConvertCase((char*)&convertCtlblkUpper,
+                       (char*)&comp1,
+                       (char*)&chRet,
+                       (long*)&charlen,
+                       (char*)&error_code);
+    return chRet;
+}
+
+XMLCh Iconv400TransService::toUnicodeLower(XMLCh comp1) const
+{
+    XMLCh chRet;
+    struct {
+             int bytes_available;
+             int bytes_used;
+             char exception_id[7];
+             char reserved;
+             char exception_data[15];
+            } error_code;
+     error_code.bytes_available = sizeof(error_code);
+
+    long charlen =2;
+
+    QlgConvertCase((char*)&convertCtlblkLower,
                        (char*)&comp1,
                        (char*)&chRet,
                        (long*)&charlen,
@@ -324,124 +365,6 @@ Iconv400Transcoder::~Iconv400Transcoder()
 }
 
 
-
-XMLCh Iconv400Transcoder::transcodeOne(  const   XMLByte* const  srcData
-                                    , const unsigned int    srcBytes
-                                    ,       unsigned int&   bytesEaten)
-{
-    // Check for stupid stuff
-    if (!srcBytes)
-        return 0;
-
-    UErrorCode err = U_ZERO_ERROR;
-    const XMLByte* startSrc = srcData;
-    const XMLCh chRet = ucnv_getNextUChar
-    (
-        fConverter
-        , (const char**)&startSrc
-        , (const char*)((srcData + srcBytes) - 1)
-        , &err
-    );
-
-    // Bail out if an error
-    if (U_FAILURE(err))
-        return 0;
-
-    // Calculate the bytes eaten and return the char
-    bytesEaten = startSrc - srcData;
-    return chRet;
-}
-
-
-unsigned int
-Iconv400Transcoder::transcodeXML(const   XMLByte* const          srcData
-                            , const unsigned int            srcCount
-                            ,       XMLCh* const            toFill
-                            , const unsigned int            maxChars
-                            ,       unsigned int&           bytesEaten
-                            ,       unsigned char* const    charSizes)
-{
-    // If debugging, insure the block size is legal
-    #if defined(XML4C_DEBUG)
-    checkBlockSize(maxChars);
-    #endif
-
-    // Set up pointers to the source and destination buffers.
-    UChar*          startTarget = toFill;
-    const XMLByte*  startSrc = srcData;
-    const XMLByte*  endSrc = srcData + srcCount;
-
-    //
-    //  Transoode the buffer.  Buffer overflow errors are normal, occuring
-    //  when the raw input buffer holds more characters than will fit in
-    //  the Unicode output buffer.
-    //
-    UErrorCode  err = U_ZERO_ERROR;
-    ucnv_toUnicode
-    (
-        fConverter
-        , &startTarget
-        , toFill + maxChars
-        , (const char**)&startSrc
-        , (const char*)endSrc
-        , (fFixed ? 0 : fSrcOffsets)
-        , false
-        , &err
-    );
-
-    if ((err != U_ZERO_ERROR) && (err != U_BUFFER_OVERFLOW_ERROR))
-        ThrowXML(TranscodingException, XMLExcepts::Trans_Unrepresentable);
-
-    // Calculate the bytes eaten and store in caller's param
-    bytesEaten = startSrc - srcData;
-
-    // And the characters decoded
-    const unsigned int charsDecoded = startTarget - toFill;
-
-    //
-    //  Translate the array of char offsets into an array of character
-    //  sizes, which is what the transcoder interface semantics requires.
-    //  If its fixed, then we can optimize it.
-    //
-    if (fFixed)
-    {
-        const unsigned char fillSize = (unsigned char)ucnv_getMaxCharSize(fConverter);;
-        memset(charSizes, fillSize, maxChars);
-    }
-     else
-    {
-        //
-        //  We have to convert the series of offsets into a series of
-        //  sizes. If just one char was decoded, then its the total bytes
-        //  eaten. Otherwise, do a loop and subtract out each element from
-        //  its previous element.
-        //
-        if (charsDecoded == 1)
-        {
-            charSizes[0] = (unsigned char)bytesEaten;
-        }
-         else
-        {
-            //  ICU does not return an extra element to allow us to figure
-            //  out the last char size, so we have to compute it from the
-            //  total bytes used.
-            unsigned int index;
-            for (index = 0; index < charsDecoded - 1; index++)
-            {
-                charSizes[index] = (unsigned char)(fSrcOffsets[index + 1]
-                                                    - fSrcOffsets[index]);
-            }
-            if( charsDecoded > 0 ) {
-                charSizes[charsDecoded - 1] = (unsigned char)(bytesEaten
-                                              - fSrcOffsets[charsDecoded - 1]);
-            }
-        }
-    }
-
-    // Return the chars we put into the target buffer
-    return charsDecoded;
-}
-
 // ---------------------------------------------------------------------------
 //  Iconv400Transcoder: The virtual transcoder API
 // ---------------------------------------------------------------------------
@@ -678,7 +601,7 @@ bool Iconv400Transcoder::canTranscodeTo(const unsigned int toCheck) const
     char*           startTarget = tmpBuf;
     const UChar*    startSrc = srcBuf;
     UErrorCode  err = U_ZERO_ERROR;
-   
+
     ucnv_fromUnicode
     (
         fConverter
@@ -691,7 +614,7 @@ bool Iconv400Transcoder::canTranscodeTo(const unsigned int toCheck) const
         , &err
     );
 
- 
+
     return (err==U_ZERO_ERROR);  /*@01A*/
 }
 
@@ -997,3 +920,5 @@ bool Iconv400LCPTranscoder::transcode(   const   XMLCh* const    toTranscode
     return true;
 }
 
+XERCES_CPP_NAMESPACE_END
+
