@@ -1,37 +1,37 @@
 /*
  * The Apache Software License, Version 1.1
- * 
+ *
  * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
  * reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
- * 
+ *
  * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
- * 
+ *    notice, this list of conditions and the following disclaimer.
+ *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in
  *    the documentation and/or other materials provided with the
  *    distribution.
- * 
+ *
  * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:  
+ *    if any, must include the following acknowledgment:
  *       "This product includes software developed by the
  *        Apache Software Foundation (http://www.apache.org/)."
  *    Alternately, this acknowledgment may appear in the software itself,
  *    if and wherever such third-party acknowledgments normally appear.
- * 
+ *
  * 4. The names "Xerces" and "Apache Software Foundation" must
  *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written 
+ *    software without prior written permission. For written
  *    permission, please contact apache\@apache.org.
- * 
+ *
  * 5. Products derived from this software may not be called "Apache",
  *    nor may "Apache" appear in their name, without prior written
  *    permission of the Apache Software Foundation.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
  * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
  * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -45,7 +45,7 @@
  * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  * ====================================================================
- * 
+ *
  * This software consists of voluntary contributions made by many
  * individuals on behalf of the Apache Software Foundation, and was
  * originally based on software copyright (c) 1999, International
@@ -55,31 +55,24 @@
  */
 
 /*
- *	$Id: MacOSUnicodeConverter.cpp,v 1.5 2002/08/21 16:55:05 jberry Exp $
+ *	$Id: MacOSUnicodeConverter.cpp,v 1.9 2002/12/16 19:40:43 jberry Exp $
  */
 
 
 // ---------------------------------------------------------------------------
 //  Includes
 // ---------------------------------------------------------------------------
-#include <xercesc/util/Transcoders/MacOSUnicodeConverter/MacOSUnicodeConverter.hpp>
-#include <xercesc/util/XMLUniDefs.hpp>
-#include <xercesc/util/XMLUni.hpp>
-#include <xercesc/util/XMLString.hpp>
-#include <xercesc/util/TranscodingException.hpp>
-#include <xercesc/util/PlatformUtils.hpp>
-#include <xercesc/util/Janitor.hpp>
-#include <xercesc/util/Platforms/MacOS/MacOSPlatformUtils.hpp>
+#include <xercesc/util/XercesDefs.hpp>
+
+#include <algorithm>
+#include <cstddef>
+#include <cstring>
 
 #if defined(XML_METROWERKS) || (__GNUC__ >= 3 && _GLIBCPP_USE_WCHAR_T)
 	// Only used under metrowerks.
 	#include <cwctype>
 #endif
 
-#include <algorithm>
-#include <cstddef>
-#include <cstring>
-
 #if defined(__APPLE__)
     //	Framework includes from ProjectBuilder
     #include <CoreServices/CoreServices.h>
@@ -97,6 +90,16 @@
     #include <CFString.h>
 #endif
 
+#include <xercesc/util/Transcoders/MacOSUnicodeConverter/MacOSUnicodeConverter.hpp>
+#include <xercesc/util/XMLUniDefs.hpp>
+#include <xercesc/util/XMLUni.hpp>
+#include <xercesc/util/XMLString.hpp>
+#include <xercesc/util/TranscodingException.hpp>
+#include <xercesc/util/PlatformUtils.hpp>
+#include <xercesc/util/Janitor.hpp>
+#include <xercesc/util/Platforms/MacOS/MacOSPlatformUtils.hpp>
+
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //	Typedefs
@@ -104,7 +107,7 @@
 
 //	TempUniBuf is used for cases where we need a temporary conversion due to
 //	a mismatch between UniChar (the 16-bit type that the Unicode converter uses)
-//	and XMLCH (the type that Xerces uses to represent a Unicode character).
+//	and XMLCh (the type that Xerces uses to represent a Unicode character).
 //	In the case of Metrowerks, these are the same size. For ProjectBuilder, they
 //	used to differ, but they are now the same since XMLCh is now always fixed
 //	as a 16 bit character, rather than floating with wchar_t as it used to.
@@ -166,7 +169,7 @@ int MacOSUnicodeConverter::compareIString(  const XMLCh* const    comp1
 		//	to allow for the case where the strings being compared
 		//	are larger than the static buffer.
 
-		UCCollateOptions collateOptions = 
+		UCCollateOptions collateOptions =
 								kUCCollateComposeInsensitiveMask
 								| kUCCollateWidthInsensitiveMask
 								| kUCCollateCaseInsensitiveMask
@@ -282,7 +285,7 @@ int MacOSUnicodeConverter::compareNIString( const   XMLCh* const    comp1
 		//	to allow for the case where the strings being compared
 		//	are larger than the static buffer.
 
-		UCCollateOptions collateOptions = 
+		UCCollateOptions collateOptions =
 								kUCCollateComposeInsensitiveMask
 								| kUCCollateWidthInsensitiveMask
 								| kUCCollateCaseInsensitiveMask
@@ -462,15 +465,16 @@ void MacOSUnicodeConverter::upperCase(XMLCh* const toUpperCase) const
 #if TARGET_API_MAC_CARBON
 
    // If we're targeting carbon, use the CFString conversion to uppercase
+   int len = XMLString::stringLen(toUpperCase);
    CFMutableStringRef cfString = CFStringCreateMutableWithExternalCharactersNoCopy(
-        NULL,
+        kCFAllocatorDefault,
         (UniChar*)toUpperCase,
-        XMLString::stringLen(toUpperCase),
-        0,
+        len,		// length
+        len,		// capacity
         kCFAllocatorNull);
    CFStringUppercase(cfString, NULL);
    CFRelease(cfString);
-   
+
 #elif defined(XML_METROWERKS) || (__GNUC__ >= 3 && _GLIBCPP_USE_WCHAR_T)
 
 	// Use this if there's a reasonable c library available.
@@ -490,15 +494,16 @@ void MacOSUnicodeConverter::lowerCase(XMLCh* const toLowerCase) const
 #if TARGET_API_MAC_CARBON
 
    // If we're targeting carbon, use the CFString conversion to uppercase
+   int len = XMLString::stringLen(toLowerCase);
    CFMutableStringRef cfString = CFStringCreateMutableWithExternalCharactersNoCopy(
-        NULL,
+        kCFAllocatorDefault,
         (UniChar*)toLowerCase,
-        XMLString::stringLen(toLowerCase),
-        0,
+        len,		// length
+        len,		// capacity
         kCFAllocatorNull);
    CFStringLowercase(cfString, NULL);
    CFRelease(cfString);
-   
+
 #elif defined(XML_METROWERKS) || (__GNUC__ >= 3 && _GLIBCPP_USE_WCHAR_T)
 
 	// Use this if there's a reasonable c library available.
@@ -627,7 +632,7 @@ MacOSTranscoder::transcodeFrom(  const  XMLByte* const			srcData
 {
     ByteCount	bytesConsumed = 0;
     ByteCount	bytesProduced = 0;
-    
+
     OSStatus status = ConvertFromTextToUnicode(
     	mTextToUnicodeInfo,
     	srcCount,
@@ -681,7 +686,7 @@ MacOSTranscoder::transcodeTo(const  XMLCh* const    srcData
 	
 	const XMLCh* src	= srcData;
 	std::size_t srcCnt	= srcCount;
-    
+
 	OptionBits controlFlags =
 		  0
 		// | kUnicodeUseFallbacksMask
@@ -1083,6 +1088,13 @@ MacOSLCPTranscoder::transcode(const XMLCh* const srcText)
 	
 	if (status != noErr && status != kTECPartialCharErr)
 		result.reset();
+	else if (!result.get())
+	{
+		//	No error, and no result: we probably processed a zero length
+		//	input, in which case we want a valid zero length output.
+		result.reset(new char[1]);
+		result[0] = '\0';
+	}
 	
 	return result.release();
 }
@@ -1165,6 +1177,13 @@ MacOSLCPTranscoder::transcode(const char* const srcText)
 	
 	if (status != noErr && status != kTECPartialCharErr)
 		result.reset();
+	else if (!result.get())
+	{
+		//	No error, and no result: we probably processed a zero length
+		//	input, in which case we want a valid zero length output.
+		result.reset(new XMLCh[1]);
+		result[0] = '\0';
+	}
 	
 	return result.release();
 }
@@ -1222,7 +1241,7 @@ MacOSLCPTranscoder::transcode( 		 const   char* const	toTranscode
 	//	XMLCh in place within the output buffer.
 	if (UNICODE_SIZE_MISMATCH)
 		CopyUniCharsToXMLChs(reinterpret_cast<UniChar* const>(toFill), toFill, charsProduced, maxChars);
-		    
+		
     //	Zero terminate the output string
     toFill[charsProduced] = L'\0';
 
@@ -1309,3 +1328,5 @@ MacOSLCPTranscoder::transcode( 		const   XMLCh* const    toTranscode
 
 	return (status == noErr);
 }
+
+XERCES_CPP_NAMESPACE_END
