@@ -1,37 +1,37 @@
 /*
  * The Apache Software License, Version 1.1
- * 
+ *
  * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
  * reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
- * 
+ *
  * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
- * 
+ *    notice, this list of conditions and the following disclaimer.
+ *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in
  *    the documentation and/or other materials provided with the
  *    distribution.
- * 
+ *
  * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:  
+ *    if any, must include the following acknowledgment:
  *       "This product includes software developed by the
  *        Apache Software Foundation (http://www.apache.org/)."
  *    Alternately, this acknowledgment may appear in the software itself,
  *    if and wherever such third-party acknowledgments normally appear.
- * 
+ *
  * 4. The names "Xerces" and "Apache Software Foundation" must
  *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written 
+ *    software without prior written permission. For written
  *    permission, please contact apache\@apache.org.
- * 
+ *
  * 5. Products derived from this software may not be called "Apache",
  *    nor may "Apache" appear in their name, without prior written
  *    permission of the Apache Software Foundation.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
  * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
  * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -45,7 +45,7 @@
  * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  * ====================================================================
- * 
+ *
  * This software consists of voluntary contributions made by many
  * individuals on behalf of the Apache Software Foundation, and was
  * originally based on software copyright (c) 1999, International
@@ -55,14 +55,14 @@
  */
 
 /*
- * $Id: MacOSUnicodeConverter.hpp,v 1.3 2002/07/12 16:48:50 jberry Exp $
+ * $Id: MacOSUnicodeConverter.hpp,v 1.5 2002/11/13 17:37:36 jberry Exp $
  */
 
 #ifndef MACOSUNICODECONVERTER_HPP
 #define MACOSUNICODECONVERTER_HPP
 
-#include <xercesc/util/TransService.hpp>
 #include <cstddef>
+#include <xercesc/util/TransService.hpp>
 
 #if defined(__APPLE__)
     //	Framework includes from ProjectBuilder
@@ -72,6 +72,8 @@
     #include <UnicodeConverter.h>
 #endif
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 //
 //  The transcoding service has to provide a couple of required string
 //  and character operations, but its most important service is the creation
@@ -128,7 +130,7 @@ protected :
         ,       XMLTransService::Codes& resValue
         , const unsigned int            blockSize
     );
-    
+
     //	Sniff for available functionality
     static bool IsMacOSUnicodeConverterSupported(void);
 
@@ -143,7 +145,7 @@ private :
     // -----------------------------------------------------------------------
     MacOSUnicodeConverter(const MacOSUnicodeConverter&);
     void operator=(const MacOSUnicodeConverter&);
-    
+
     // -----------------------------------------------------------------------
     //  Private methods
     // -----------------------------------------------------------------------
@@ -210,7 +212,7 @@ private :
     // -----------------------------------------------------------------------
     MacOSTranscoder(const MacOSTranscoder&);
     void operator=(const MacOSTranscoder&);
-    
+
     // -----------------------------------------------------------------------
     //  Private members
     // -----------------------------------------------------------------------
@@ -275,7 +277,7 @@ private :
     // -----------------------------------------------------------------------
     MacOSLCPTranscoder(const MacOSLCPTranscoder&);
     void operator=(const MacOSLCPTranscoder&);
-    
+
     // -----------------------------------------------------------------------
     //  Private data members
     // -----------------------------------------------------------------------
@@ -283,4 +285,6 @@ private :
     UnicodeToTextInfo	mUnicodeToTextInfo;
  };
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
