@@ -56,6 +56,9 @@
 
 /*
  * $Log: ValueArrayOf.hpp,v $
+ * Revision 1.3  2002/11/04 15:22:05  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.2  2002/08/21 17:45:00  tng
  * [Bug 7087] compiler warnings when using gcc.
  *
@@ -90,6 +93,8 @@
 #include <xercesc/util/ArrayIndexOutOfBoundsException.hpp>
 #include <xercesc/util/IllegalArgumentException.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 template <class TElem> class ValueArrayOf
 {
 public :
@@ -185,6 +190,7 @@ private :
     ValueArrayOf<TElem>*    fToEnum;
 };
 
+XERCES_CPP_NAMESPACE_END
 
 #if !defined(XERCES_TMPLSINC)
 #include <xercesc/util/ValueArrayOf.c>
