@@ -55,8 +55,23 @@
  */
 
 /*
- * $Id: XMLAbstractDoubleFloat.cpp,v 1.4 2002/05/03 16:05:45 peiyongz Exp $
+ * $Id: XMLAbstractDoubleFloat.cpp,v 1.9 2003/02/02 23:54:43 peiyongz Exp $
  * $Log: XMLAbstractDoubleFloat.cpp,v $
+ * Revision 1.9  2003/02/02 23:54:43  peiyongz
+ * getFormattedString() added to return original and converted value.
+ *
+ * Revision 1.8  2003/01/30 21:55:22  tng
+ * Performance: create getRawData which is similar to toString but return the internal data directly, user is not required to delete the returned memory.
+ *
+ * Revision 1.7  2002/12/11 00:20:02  peiyongz
+ * Doing businesss in value space. Converting out-of-bound value into special values.
+ *
+ * Revision 1.6  2002/11/04 15:22:05  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.5  2002/09/24 19:51:24  tng
+ * Performance: use XMLString::equals instead of XMLString::compareString
+ *
  * Revision 1.4  2002/05/03 16:05:45  peiyongz
  * Bug 7341: Missing newline at end of util and DOM source files,
  * patch from Martin Kalen.
@@ -85,8 +100,11 @@
 #include <xercesc/util/XMLAbstractDoubleFloat.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/util/NumberFormatException.hpp>
+#include <xercesc/util/XMLString.hpp>
 #include <xercesc/util/Janitor.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 // ---------------------------------------------------------------------------
 //  local data member
 // ---------------------------------------------------------------------------
@@ -97,19 +115,19 @@ static XMLCh value1[BUF_LEN+1];
 //  ctor/dtor
 // ---------------------------------------------------------------------------
 XMLAbstractDoubleFloat::XMLAbstractDoubleFloat()
-:fMantissa(0)
-,fExponent(0)
+:fValue(0)
 ,fType(Normal)
+,fDataConverted(false)
+,fSign(0)
+,fRawData(0)
+,fFormattedString(0)
 {
 }
 
 XMLAbstractDoubleFloat::~XMLAbstractDoubleFloat()
 {
-    if (fMantissa)
-        delete fMantissa;
-
-    if (fExponent)
-        delete fExponent;
+     delete [] fRawData;
+     delete [] fFormattedString;
 }
 
 void XMLAbstractDoubleFloat::init(const XMLCh* const strValue)
@@ -117,142 +135,120 @@ void XMLAbstractDoubleFloat::init(const XMLCh* const strValue)
     if ((!strValue) || (!*strValue))
         ThrowXML(NumberFormatException, XMLExcepts::XMLNUM_emptyString);
 
+    fRawData = XMLString::replicate(strValue);   // preserve the raw data form
+
     XMLCh* tmpStrValue = XMLString::replicate(strValue);
     ArrayJanitor<XMLCh> janTmpName(tmpStrValue);
     XMLString::trim(tmpStrValue);
 
     normalizeZero(tmpStrValue);
 
-    if (XMLString::compareString(tmpStrValue, XMLUni::fgNegINFString) == 0)
+    if (XMLString::equals(tmpStrValue, XMLUni::fgNegINFString) )
     {
         fType = NegINF;
-        return;
+        fSign = -1;
     }
-    else if (XMLString::compareString(tmpStrValue, XMLUni::fgNegZeroString) == 0)
+    else if (XMLString::equals(tmpStrValue, XMLUni::fgNegZeroString) )
     {
         fType = NegZero;
-        return;
+        fSign = -1;
     }
-    else if (XMLString::compareString(tmpStrValue, XMLUni::fgPosZeroString) == 0)
+    else if (XMLString::equals(tmpStrValue, XMLUni::fgPosZeroString) )
     {
         fType = PosZero;
-        return;
+        fSign = 1;
     }
-    else if (XMLString::compareString(tmpStrValue, XMLUni::fgPosINFString) == 0)
+    else if (XMLString::equals(tmpStrValue, XMLUni::fgPosINFString) )
     {
         fType = PosINF;
-        return;
+        fSign = 1;
     }
-    else if (XMLString::compareString(tmpStrValue, XMLUni::fgNaNString) == 0)
+    else if (XMLString::equals(tmpStrValue, XMLUni::fgNaNString) )
     {
         fType = NaN;
-        return;
-    }
-
-    //
-    // Normal case
-    //
-    int ePos = 0;
-    int tmpStrLen = XMLString::stringLen(tmpStrValue);
-
-    if ((( ePos = XMLString::indexOf(tmpStrValue, chLatin_E)) != -1 ) ||
-        (( ePos = XMLString::indexOf(tmpStrValue, chLatin_e)) != -1 )  )
-    {
-        XMLCh* tmpMantissa = new XMLCh [ePos+1];
-        XMLString::subString(tmpMantissa, tmpStrValue, 0, ePos);
-        ArrayJanitor<XMLCh> janMantissa(tmpMantissa);
-        fMantissa = new XMLBigDecimal(tmpMantissa);
-
-        //
-        // 1234.56E78
-        //
-        if ( ePos < tmpStrLen - 1)
-        {
-            XMLCh* tmpExponent = new XMLCh [tmpStrLen - ePos];
-            XMLString::subString(tmpExponent, tmpStrValue, ePos + 1, tmpStrLen);
-            ArrayJanitor<XMLCh> janExponent(tmpExponent);
-            fExponent = new XMLBigInteger(tmpExponent);
-        }
-        //
-        // 1234.56E
-        //
-        else
-        {
-            ThrowXML1(NumberFormatException
-                    , XMLExcepts::XMLNUM_DBL_FLT_No_Exponent
-                    , strValue);
-        }
-
+        fSign = 1;
     }
     else
+        //
+        // Normal case
+        //
     {
-        fMantissa = new XMLBigDecimal(tmpStrValue);
-        fExponent = new XMLBigInteger(XMLUni::fgZeroString);
+        checkBoundary(tmpStrValue);
     }
 
-    checkBoundary(tmpStrValue);
 }
 
 //
-// Add the 'E' as necessary
-// The caller needs to de-allocate the memory allocated by this function
-// Deallocate the memory allocated by XMLBigInteger
+// 
 //
 XMLCh*  XMLAbstractDoubleFloat::toString() const
 {
+    return XMLString::replicate(fRawData);
+}
+
+XMLCh*  XMLAbstractDoubleFloat::getRawData() const
+{
+    return fRawData;
+}
+
+const XMLCh*  XMLAbstractDoubleFloat::getFormattedString() const
+{
+    if (!fDataConverted)
+    {
+        return fRawData;
+    }
+    else 
+    {
+        if (!fFormattedString)    	
+        {
+            XMLAbstractDoubleFloat *temp = (XMLAbstractDoubleFloat *) this;
+            temp->formatString();
+        }
+
+        return fFormattedString;           
+    }
+
+}
+
+void XMLAbstractDoubleFloat::formatString()
+{
+
+    unsigned int rawDataLen = XMLString::stringLen(fRawData);
+    fFormattedString = new XMLCh [ rawDataLen + 8];
+    for (unsigned int i = 0; i < rawDataLen + 8; i++)
+        fFormattedString[i] = chNull;
+
+    XMLString::copyString(fFormattedString, fRawData);
+
+    fFormattedString[rawDataLen] = chSpace;
+    fFormattedString[rawDataLen + 1] = chOpenParen;
+
     switch (fType)
     {
-    case NegINF:
-        return XMLString::replicate(XMLUni::fgNegINFString);
-
-    case NegZero:
-        return XMLString::replicate(XMLUni::fgNegZeroString);
-
-    case PosZero:
-        return XMLString::replicate(XMLUni::fgPosZeroString);
-
-    case PosINF:
-        return XMLString::replicate(XMLUni::fgPosINFString);
-
-    case NaN:
-        return XMLString::replicate(XMLUni::fgNaNString);
-
-    case Normal:
-
-        XMLCh *ret_mantissa;
-        ret_mantissa = fMantissa->toString();
-
-        if (fExponent)
-        {
-            XMLCh *ret_exponent = fExponent->toString();
-            XMLCh *ret_val = new XMLCh [ XMLString::stringLen(ret_mantissa) +
-                                         XMLString::stringLen(ret_exponent) + 2];
-            *ret_val = 0;
-            XMLString::catString(ret_val, ret_mantissa);
-            XMLString::catString(ret_val, XMLUni::fgEString);
-            XMLString::catString(ret_val, ret_exponent);
-
-            delete [] ret_mantissa;
-            delete [] ret_exponent;
-            return ret_val;
-        }
-        else
-            return ret_mantissa;
-
+    case NegINF:       
+        XMLString::catString(fFormattedString, XMLUni::fgNegINFString);
         break;
-    default:
-        XMLString::binToText(fType, value1, 16, 10);
-        ThrowXML1(NumberFormatException
-                , XMLExcepts::XMLNUM_DBL_FLT_InvalidType
-                , value1);
-    };
+    case NegZero:
+        XMLString::catString(fFormattedString, XMLUni::fgNegZeroString);
+        break;
+    case PosZero:
+        XMLString::catString(fFormattedString, XMLUni::fgPosZeroString);
+        break;
+    case PosINF:
+        XMLString::catString(fFormattedString, XMLUni::fgPosINFString);
+        break;
+    case NaN:
+        XMLString::catString(fFormattedString, XMLUni::fgNaNString);
+        break;
+    }
+
+    fFormattedString[XMLString::stringLen(fFormattedString)] = chCloseParen;
 
-    return (XMLCh*) 0;
 }
 
 int XMLAbstractDoubleFloat::getSign() const
 {
-    return fMantissa->getSign();
+    return fSign;
 }
 
 //
@@ -268,16 +264,11 @@ int XMLAbstractDoubleFloat::compareValues(const XMLAbstractDoubleFloat* const lV
     if ((!lValue->isSpecialValue()) &&
         (!rValue->isSpecialValue())  )
     {
-        //
-        // if we use fValue to compare two
-        // sequences "12.3456E4" and "1234.56E2",
-        // they are _NOT_ the same. so we ask
-        // BigDecimal to compare.
-        //
-        XMLBigDecimal ldv = XMLBigDecimal(*(lValue->fMantissa), lValue->fExponent->intValue());
-        XMLBigDecimal rdv = XMLBigDecimal(*(rValue->fMantissa), rValue->fExponent->intValue());
+        if (lValue->fValue == rValue->fValue)
+            return 0;
+        else
+            return (lValue->fValue > rValue->fValue) ? 1: -1;
 
-        return XMLBigDecimal::compareValues(&ldv, &rdv);
     }
     //
     // case#2: lValue special
@@ -290,7 +281,7 @@ int XMLAbstractDoubleFloat::compareValues(const XMLAbstractDoubleFloat* const lV
         if (lValue->fType == rValue->fType)
             return 0;
         else
-            return (lValue->fType > rValue->fType ? 1 : -1);
+            return (lValue->fType > rValue->fType) ? 1 : -1;
     }
     //
     // case#3: lValue special
@@ -351,9 +342,9 @@ int XMLAbstractDoubleFloat::compareSpecial(const XMLAbstractDoubleFloat* const s
 //
 //  Return:
 //
-//  for input comforming to [+]? [0]* '.'? [0]*, 
-//            normalize the input to positive zero string 
-//  for input comforming to '-' [0]* '.'? [0]*, 
+//  for input comforming to [+]? [0]* '.'? [0]*,
+//            normalize the input to positive zero string
+//  for input comforming to '-' [0]* '.'? [0]*,
 //            normalize the input to negative zero string
 //  otherwise, do nothing
 //
@@ -361,10 +352,10 @@ void XMLAbstractDoubleFloat::normalizeZero(XMLCh* const inData)
 {
 
 	// do a quick check
-	if (!inData  || 
+	if (!inData  ||
 		!*inData ||
-        (XMLString::compareString(inData, XMLUni::fgNegZeroString) == 0) ||
-        (XMLString::compareString(inData, XMLUni::fgPosZeroString) == 0)  )
+        (XMLString::equals(inData, XMLUni::fgNegZeroString) ) ||
+        (XMLString::equals(inData, XMLUni::fgPosZeroString) )  )
         return;
 
     XMLCh*   srcStr = inData;
@@ -404,6 +395,15 @@ void XMLAbstractDoubleFloat::normalizeZero(XMLCh* const inData)
 		else
 			XMLString::copyString(inData, XMLUni::fgPosZeroString);
 	}
+    else
+    {
+        // we got to set the sign first, since this string may
+        // eventaully turn out to be beyond the minimum representable 
+        // number and reduced to -0 or +0.
+        fSign = minusSeen ? -1 : 1;
+    }
 
-	return;
-}
+    return;
+} 
+
+XERCES_CPP_NAMESPACE_END
