@@ -56,6 +56,15 @@
 
 /*
  * $Log: XMLBigDecimal.cpp,v $
+ * Revision 1.5  2003/02/02 23:54:43  peiyongz
+ * getFormattedString() added to return original and converted value.
+ *
+ * Revision 1.4  2003/01/30 21:55:22  tng
+ * Performance: create getRawData which is similar to toString but return the internal data directly, user is not required to delete the returned memory.
+ *
+ * Revision 1.3  2002/11/04 15:22:05  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.2  2002/08/13 22:11:23  peiyongz
  * Fix to Bug#9442
  *
@@ -99,6 +108,8 @@
 #include <xercesc/util/TransService.hpp>
 #include <xercesc/util/Janitor.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 /**
  * Constructs a BigDecimal from a string containing an optional (plus | minus)
  * sign followed by a sequence of zero or more decimal digits, optionally
@@ -110,7 +121,7 @@
  * Any extraneous characters (including whitespace) will result in
  * a NumberFormatException.
 
- * since parseBigDecimal and XMLBigInteger() may throw exception, 
+ * since parseBigDecimal and XMLBigInteger() may throw exception,
  * caller of XMLBigDecimal need to catch it.
 //
 **/
@@ -148,7 +159,7 @@ XMLBigDecimal::XMLBigDecimal(const XMLBigDecimal& toCopy, const int addExponent)
 {
     //invoke XMLBigInteger' copy ctor
     fIntVal = new XMLBigInteger(*(toCopy.getValue()));
- 
+
     if ( addExponent > 0 )
     {
         if (fScale >= (unsigned int)addExponent)
@@ -239,7 +250,7 @@ void XMLBigDecimal::parseBigDecimal(const XMLCh* const toConvert
     // '+' or '-' is allowed only at the first position
     //
     if (*startPtr == chDash)
-    {       
+    {
         *retPtr = chDash;  // copy the '-'
         startPtr++;
         retPtr++;
@@ -351,6 +362,16 @@ void XMLBigDecimal::reScale(unsigned int newScale)
     return;
 }
 
+XMLCh*  XMLBigDecimal::getRawData() const
+{
+    return fRawData;
+}
+
+const XMLCh*  XMLBigDecimal::getFormattedString() const
+{
+    return fRawData;
+}
+
 //
 // The caller needs to de-allocate the memory allocated by this function
 //
@@ -359,3 +380,5 @@ XMLCh*  XMLBigDecimal::toString() const
     return XMLString::replicate(fRawData);
 }
 
+XERCES_CPP_NAMESPACE_END
+
