@@ -1,37 +1,37 @@
 /*
  * The Apache Software License, Version 1.1
- * 
+ *
  * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
  * reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
- * 
+ *
  * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
- * 
+ *    notice, this list of conditions and the following disclaimer.
+ *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in
  *    the documentation and/or other materials provided with the
  *    distribution.
- * 
+ *
  * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:  
+ *    if any, must include the following acknowledgment:
  *       "This product includes software developed by the
  *        Apache Software Foundation (http://www.apache.org/)."
  *    Alternately, this acknowledgment may appear in the software itself,
  *    if and wherever such third-party acknowledgments normally appear.
- * 
+ *
  * 4. The names "Xerces" and "Apache Software Foundation" must
  *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written 
+ *    software without prior written permission. For written
  *    permission, please contact apache\@apache.org.
- * 
+ *
  * 5. Products derived from this software may not be called "Apache",
  *    nor may "Apache" appear in their name, without prior written
  *    permission of the Apache Software Foundation.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
  * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
  * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -45,7 +45,7 @@
  * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  * ====================================================================
- * 
+ *
  * This software consists of voluntary contributions made by many
  * individuals on behalf of the Apache Software Foundation, and was
  * originally based on software copyright (c) 1999, International
@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: XMLException.cpp,v 1.2 2002/05/03 16:05:45 peiyongz Exp $
+ * $Id: XMLException.cpp,v 1.3 2002/11/04 15:22:05 tng Exp $
  */
 
 
@@ -71,6 +71,7 @@
 #include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/util/XMLUni.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  Local data
@@ -79,7 +80,7 @@
 //      For now, if we cannot load an exception message, we load up the
 //      default.
 // ---------------------------------------------------------------------------
-static const XMLCh  gDefErrMsg[] = 
+static const XMLCh  gDefErrMsg[] =
 {
         chLatin_C, chLatin_o, chLatin_u, chLatin_l, chLatin_d
     ,   chSpace, chLatin_n, chLatin_o, chLatin_t, chSpace
@@ -122,7 +123,7 @@ static XMLMutex& gMsgMutex()
             // This is the real mutex.  Register it for cleanup at Termination.
 			msgMutexCleanup.registerCleanup(XMLException::reinitMsgMutex);
         }
-        
+
     }
 
     return *sMsgMutex;
@@ -150,7 +151,7 @@ static XMLMsgLoader& gGetMsgLoader()
         //
         msgLoaderCleanup.registerCleanup(XMLException::reinitMsgLoader);
     }
-    
+
     // We got it, so return it
     return *sMsgLoader;
 }
@@ -327,3 +328,5 @@ void XMLException::reinitMsgLoader()
 	delete sMsgLoader;
 	sMsgLoader = 0;
 }
+
+XERCES_CPP_NAMESPACE_END
