@@ -1,7 +1,7 @@
 /*
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
+ * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: XMLString.cpp,v 1.6 2002/08/23 20:55:10 tng Exp $
+ * $Id: XMLString.cpp,v 1.21 2003/02/05 18:50:56 tng Exp $
  */
 
 
@@ -71,6 +71,7 @@
 #include <xercesc/util/NumberFormatException.hpp>
 #include <xercesc/util/Janitor.hpp>
 #include <xercesc/util/PlatformUtils.hpp>
+#include <xercesc/util/RefArrayVectorOf.hpp>
 #include <xercesc/util/RuntimeException.hpp>
 #include <xercesc/util/TransService.hpp>
 #include <xercesc/util/TranscodingException.hpp>
@@ -80,6 +81,7 @@
 #include <xercesc/util/XMLUni.hpp>
 #include <xercesc/internal/XMLReader.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  Local static data
@@ -591,85 +593,39 @@ void XMLString::trim(char* const toTrim)
 void XMLString::subString(char* const targetStr, const char* const srcStr
                           , const int startIndex, const int endIndex)
 {
-	//if (startIndex < 0 || endIndex < 0)
-    //    ThrowXML(ArrayIndexOutOfBoundsException, XMLExcepts::Str_NegativeIndex);
-
-	if (targetStr == 0)
+    if (targetStr == 0)
         ThrowXML(IllegalArgumentException, XMLExcepts::Str_ZeroSizedTargetBuf);
 
     const int srcLen = strlen(srcStr);
-	const int copySize = endIndex - startIndex;
+    const int copySize = endIndex - startIndex;
 
     // Make sure the start index is within the XMLString bounds
-	if (startIndex > srcLen-1 || endIndex > srcLen )
+    if ( startIndex < 0 || startIndex > endIndex || endIndex > srcLen)
         ThrowXML(ArrayIndexOutOfBoundsException, XMLExcepts::Str_StartIndexPastEnd);
 
-	for (int i= startIndex; i < endIndex; i++) {
+    for (int i= startIndex; i < endIndex; i++) {
+        targetStr[i-startIndex] = srcStr[i];
+    }
 
-		targetStr[i-startIndex] = srcStr[i];
-	}
-
-	targetStr[copySize] = 0;
+    targetStr[copySize] = 0;
 }
 
 /**
-  * isValidNCName
-  *
-  *    NCName::= (Letter | '_') (NCNameChar)*
-  *    NCNameChar ::= Letter | Digit | '.' | '-' | '_'
-  *                   | CombiningChar | Extender
+  * Deprecated: isValidNCName
+  *    Check first char, and then the rest of the name char.
+  *    But colon should be excluded
   */
 bool XMLString::isValidNCName(const XMLCh* const name) {
-
-    if (XMLString::stringLen(name) == 0
-        || XMLString::indexOf(name, chColon) != -1) {
-        return false;
-    }
-
-    const XMLCh* tempName = name;
-    XMLCh firstChar = *tempName++;
-
-    if (!XMLReader::isXMLLetter(firstChar) && firstChar != chUnderscore) {
-
-        return false;
-    }
-
-    while(*tempName) {
-
-        if (*tempName == chColon || !XMLReader::isNameChar(*tempName++)) {
-            return false;
-        }
-    }
-
-    return true;
+    return XMLChar1_0::isValidNCName(name, XMLString::stringLen(name));
 }
 
 /**
-  * isValidName
+  * Deprecated: isValidName
+  *    Check first char, and then the rest of the name char
   *
-  *    Name::= (Letter | '_' | ':') (NameChar)*
-  *    NameChar ::= Letter | Digit | '.' | '-' | '_' | ':'
-  *                 | CombiningChar | Extender
   */
 bool XMLString::isValidName(const XMLCh* const name) {
-
-    if (!name ||
-        (XMLString::stringLen(name) == 0))
-        return false;
-
-    const XMLCh* tempName = name;
-    XMLCh firstChar = *tempName++;
-
-    if (!XMLReader::isXMLLetter(firstChar) &&
-        (firstChar != chUnderscore)        &&
-        (firstChar != chColon)              )
-        return false;
-
-    while(*tempName)
-        if (!XMLReader::isNameChar(*tempName++))
-            return false;
-
-    return true;
+    return XMLChar1_0::isValidName(name, XMLString::stringLen(name));
 }
 
 /**
@@ -681,8 +637,7 @@ bool XMLString::isValidName(const XMLCh* const name) {
 bool XMLString::isValidEncName(const XMLCh* const name)
 {
 
-    if ( ( !name) ||
-         ( XMLString::stringLen(name) == 0 ))
+    if ( XMLString::stringLen(name) == 0 )
         return false;
 
     const XMLCh* tempName = name;
@@ -707,43 +662,12 @@ bool XMLString::isValidEncName(const XMLCh* const name)
 }
 
 /**
-  * isValidQName
-  *
-  * [6]  QName ::=  (Prefix ':')? LocalPart
-  * [7]  Prefix ::=  NCName
-  * [8]  LocalPart ::=  NCName
+  * Deprecated: isValidQName
   *
   */
 bool XMLString::isValidQName(const XMLCh* const name)
 {
-    if (!name)
-        return false;
-
-    int strLen = XMLString::stringLen(name);
-    if (strLen == 0)
-        return false;
-
-    int colonPos = XMLString::indexOf(name, chColon);
-    if ((colonPos == 0) ||         // ":abcd"
-        (colonPos == strLen-1))    // "abcd:"
-        return false;
-
-    //
-    // prefix
-    //
-    if (colonPos != -1)
-    {
-        XMLCh *prefix = new XMLCh[colonPos+1];
-        XMLString::subString(prefix, name, 0, colonPos);
-        ArrayJanitor<XMLCh> janName(prefix);
-        if (XMLString::isValidNCName(prefix)==false)
-            return false;
-    }
-
-    //
-    // LocalPart
-    //
-    return XMLString::isValidNCName(name+colonPos+1);
+    return XMLChar1_0::isValidQName(name, XMLString::stringLen(name));
 }
 
 bool XMLString::isAlpha(XMLCh const theChar)
@@ -775,22 +699,10 @@ bool XMLString::isHex(XMLCh const theChar)
 			(theChar >= chLatin_A && theChar <= chLatin_F));
 }
 
+// Deprecated
 bool XMLString::isAllWhiteSpace(const XMLCh* const toCheck)
 {
-    if ( !toCheck )
-        return true;
-
-    const XMLCh* startPtr = toCheck;
-
-    while (*startPtr)
-    {
-        if (!XMLPlatformUtils::fgTransService->isSpace(*startPtr))
-            return false;
-
-        startPtr++;
-    }
-
-    return true;
+    return XMLChar1_0::isAllSpaces(toCheck, XMLString::stringLen(toCheck));
 }
 
 // ---------------------------------------------------------------------------
@@ -984,7 +896,7 @@ int XMLString::compareNString(  const   XMLCh* const    str1
             return int(*psz1) - int(*psz2);
 
         // If either ended, then both ended, so equal
-        if (!*psz1 || !*psz2)
+        if (!*psz1)
             break;
 
         // Move upwards to next chars
@@ -1035,7 +947,7 @@ int XMLString::compareString(   const   XMLCh* const    str1
             return int(*psz1) - int(*psz2);
 
         // If either has ended, then they both ended, so equal
-        if (!*psz1 || !*psz2)
+        if (!*psz1)
             break;
 
         // Move upwards for the next round
@@ -1173,28 +1085,28 @@ XMLCh* XMLString::findAny(          XMLCh* const    toSearch
     return 0;
 }
 
-int XMLString::patternMatch(        XMLCh* const    toSearch
+int XMLString::patternMatch(  const XMLCh* const    toSearch
                             , const XMLCh* const    pattern)
 {
-    if (!toSearch || !pattern )
+    if (!toSearch || !*toSearch )
         return -1;
 
-    XMLCh* srcPtr = toSearch;
     const int patnLen = XMLString::stringLen(pattern);
+	if ( !patnLen )
+		return -1;
+
+    const XMLCh* srcPtr    = toSearch;
+    const XMLCh* patnStart = toSearch;
     int  patnIndex = 0;
-    bool patnMatch;
 
     while (*srcPtr)
     {
-        patnMatch = (*srcPtr++ == pattern[patnIndex++]);
-
-        if (!patnMatch)
+        if ( !(*srcPtr++ == pattern[patnIndex++]))
         {
             patnIndex = 0;
-            patnMatch = (*srcPtr == pattern[patnIndex++]);
+            srcPtr = ++patnStart;
         }
-
-        if (patnMatch)
+        else
         {
             if (patnIndex == patnLen)
                 // full pattern match found
@@ -1205,6 +1117,7 @@ int XMLString::patternMatch(        XMLCh* const    toSearch
     return -1;
 }
 
+
 unsigned int XMLString::hashN(  const   XMLCh* const    tohash
                                 , const unsigned int    n
                                 , const unsigned int    hashModulus)
@@ -1231,11 +1144,16 @@ unsigned int XMLString::hashN(  const   XMLCh* const    tohash
 
 int XMLString::indexOf(const XMLCh* const toSearch, const XMLCh ch)
 {
-    const unsigned int len = stringLen(toSearch);
-    for (unsigned int i = 0; i < len; i++)
+    if (toSearch)
     {
-        if (toSearch[i] == ch)
-            return i;
+        const XMLCh* srcPtr = toSearch;
+        while (*srcPtr)
+        {
+            if (ch == *srcPtr)
+                return (int)(srcPtr - toSearch);
+
+            srcPtr++;
+        }
     }
     return -1;
 }
@@ -1448,34 +1366,33 @@ void XMLString::lowerCase(XMLCh* const toLowerCase)
 void XMLString::subString(XMLCh* const targetStr, const XMLCh* const srcStr
                           , const int startIndex, const int endIndex)
 {
-	//if (startIndex < 0 || endIndex < 0)
+    //if (startIndex < 0 || endIndex < 0)
     //    ThrowXML(ArrayIndexOutOfBoundsException, XMLExcepts::Str_NegativeIndex);
 
-	if (targetStr == 0)
+    if (targetStr == 0)
         ThrowXML(IllegalArgumentException, XMLExcepts::Str_ZeroSizedTargetBuf);
 
     const int srcLen = stringLen(srcStr);
-	const int copySize = endIndex - startIndex;
+    const int copySize = endIndex - startIndex;
 
     // Make sure the start index is within the XMLString bounds
-	if (startIndex > srcLen-1 || endIndex > srcLen )
+    if ( startIndex < 0 || startIndex > endIndex || endIndex > srcLen)
         ThrowXML(ArrayIndexOutOfBoundsException, XMLExcepts::Str_StartIndexPastEnd);
 
-	for (int i= startIndex; i < endIndex; i++) {
+    for (int i= startIndex; i < endIndex; i++) {
+        targetStr[i-startIndex] = srcStr[i];
+    }
 
-		targetStr[i-startIndex] = srcStr[i];
-	}
-
-	targetStr[copySize] = 0;
+    targetStr[copySize] = 0;
 }
 
-RefVectorOf<XMLCh>* XMLString::tokenizeString(const XMLCh* const tokenizeSrc)
+BaseRefVectorOf<XMLCh>* XMLString::tokenizeString(const XMLCh* const tokenizeSrc)
 {
     XMLCh* orgText = replicate(tokenizeSrc);
     ArrayJanitor<XMLCh> janText(orgText);
     XMLCh* tokenizeStr = orgText;
 
-    RefVectorOf<XMLCh>* tokenStack = new RefVectorOf<XMLCh>(16, true);
+    RefArrayVectorOf<XMLCh>* tokenStack = new RefArrayVectorOf<XMLCh>(16, true);
 
     unsigned int len = stringLen(tokenizeStr);
     unsigned int skip;
@@ -1600,11 +1517,10 @@ bool XMLString::isWSReplaced(const XMLCh* const toCheck)
 //
 void XMLString::replaceWS(XMLCh* const toConvert)
 {
-    // If no string, then its a failure
-    if (( !toConvert ) || ( !*toConvert ))
+    int strLen = XMLString::stringLen(toConvert);
+    if (strLen == 0)
         return;
 
-    int strLen = XMLString::stringLen(toConvert);
     XMLCh* retBuf = new XMLCh[strLen+1];
     XMLCh* retPtr = &(retBuf[0]);
     XMLCh* startPtr = toConvert;
@@ -1735,6 +1651,123 @@ void XMLString::collapseWS(XMLCh* const toConvert)
     return;
 }
 
+//
+// remove whitespace
+//
+void XMLString::removeWS(XMLCh* const toConvert)
+{
+    // If no string, then its a failure
+    if (( !toConvert ) || ( !*toConvert ))
+        return;
+
+    XMLCh* retBuf = new XMLCh[ XMLString::stringLen(toConvert) + 1];
+    XMLCh* retPtr = &(retBuf[0]);
+    XMLCh* startPtr = toConvert;
+
+    while (*startPtr)
+    {
+        if ( ( *startPtr != chCR)    &&
+             ( *startPtr != chLF)    &&
+             ( *startPtr != chHTab)  &&
+             ( *startPtr != chSpace)  )
+        {
+            *retPtr++ = *startPtr;
+        }
+
+        startPtr++;
+
+    }
+
+    *retPtr = chNull;
+    XMLString::moveChars(toConvert, retBuf, stringLen(retBuf)+1); //copy the last chNull as well
+    delete[] retBuf;
+    return;
+}
+
+/**
+ * Fixes a platform dependent absolute path filename to standard URI form.
+ * 1. Windows: fix 'x:' to 'file:///x:' and convert any backslash to forward slash
+ * 2. UNIX: fix '/blah/blahblah' to 'file:///blah/blahblah'
+ */
+void XMLString::fixURI(const XMLCh* const str, XMLCh* const target)
+{
+    if (!str || !*str)
+        return;
+
+    int colonIdx = XMLString::indexOf(str, chColon);
+    int slashIdx = XMLString::indexOf(str, chForwardSlash);
+
+    // If starts with a '/' we assume
+    // this is an absolute (UNIX) file path and prefix it with file://
+    if (colonIdx == -1 && XMLString::indexOf(str, chForwardSlash) == 0) {
+        unsigned index = 0;
+        target[index++] = chLatin_f;
+        target[index++] = chLatin_i;
+        target[index++] = chLatin_l;
+        target[index++] = chLatin_e;
+        target[index++] = chColon;
+        target[index++] = chForwardSlash;
+        target[index++] = chForwardSlash;
+
+        // copy the string
+        const XMLCh* inPtr = str;
+        while (*inPtr)
+            target[index++] = *inPtr++;
+
+        target[index] = chNull;
+    }
+    else if (colonIdx == 1 && XMLString::isAlpha(*str)) {
+        // If starts with a driver letter 'x:' we assume
+        // this is an absolute (Windows) file path and prefix it with file:///
+        unsigned index = 0;
+        target[index++] = chLatin_f;
+        target[index++] = chLatin_i;
+        target[index++] = chLatin_l;
+        target[index++] = chLatin_e;
+        target[index++] = chColon;
+        target[index++] = chForwardSlash;
+        target[index++] = chForwardSlash;
+        target[index++] = chForwardSlash;
+
+        // copy the string and fix any backward slash
+        const XMLCh* inPtr = str;
+        while (*inPtr) {
+            if (*inPtr == chYenSign ||
+                *inPtr == chWonSign ||
+                *inPtr == chBackSlash)
+                target[index++] = chForwardSlash;
+            else
+                target[index++] = *inPtr;
+            inPtr++;
+        }
+
+        // cap it with null
+        target[index] = chNull;
+    }
+    else {
+        // not specific case, so just copy the string over
+        copyString(target, str);
+    }
+}
+
+void XMLString::release(char** buf)
+{
+    delete [] *buf;
+    *buf = 0;
+}
+
+void XMLString::release(XMLCh** buf)
+{
+    delete [] *buf;
+    *buf = 0;
+}
+
+void XMLString::release(XMLByte** buf)
+{
+    delete [] *buf;
+    *buf = 0;
+}
+
 // ---------------------------------------------------------------------------
 //  XMLString: Private static methods
 // ---------------------------------------------------------------------------
@@ -1750,3 +1783,5 @@ void XMLString::termString()
     delete gTranscoder;
     gTranscoder = 0;
 }
+
+XERCES_CPP_NAMESPACE_END
