@@ -56,6 +56,39 @@
 
 /*
  * $Log: XMLString.hpp,v $
+ * Revision 1.13  2003/02/05 18:50:56  tng
+ * [Bug 11915] Utility for freeing memory.
+ *
+ * Revision 1.12  2003/01/24 23:16:33  peiyongz
+ * removeWS() added;
+ *
+ * Revision 1.11  2002/12/20 22:10:21  tng
+ * XML 1.1
+ *
+ * Revision 1.10  2002/12/18 14:17:54  gareth
+ * Fix to bug #13438. When you eant a vector that calls delete[] on its members you should use RefArrayVectorOf.
+ *
+ * Revision 1.9  2002/12/04 02:32:43  knoaman
+ * #include cleanup.
+ *
+ * Revision 1.8  2002/11/05 17:42:39  peiyongz
+ * equals( const char* const, const char* const)
+ *
+ * Revision 1.7  2002/11/04 15:22:05  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.6  2002/10/01 19:45:22  tng
+ * Performance in XMLString::equals, only need to check one string for null as they are equal already.
+ *
+ * Revision 1.5  2002/09/24 19:41:21  tng
+ * New inline function equals that is modified from compareString but simply return true or false.
+ *
+ * Revision 1.4  2002/09/23 18:42:18  tng
+ * DOM L3: Support baseURI.   Add utility fixURI to transform an absolute path filename to standard URI form.
+ *
+ * Revision 1.3  2002/08/27 19:24:43  peiyongz
+ * Bug#12087: patch from Thomas Ford (tom@decisionsoft.com)
+ *
  * Revision 1.2  2002/02/20 18:17:02  tng
  * [Bug 5977] Warnings on generating apiDocs.
  *
@@ -189,11 +222,11 @@
 #if !defined(XMLSTRING_HPP)
 #define XMLSTRING_HPP
 
-#include <xercesc/util/XercesDefs.hpp>
-#include <xercesc/util/RefVectorOf.hpp>
+#include <xercesc/util/BaseRefVectorOf.hpp>
+
+XERCES_CPP_NAMESPACE_BEGIN
 
 class XMLLCPTranscoder;
-
 /**
   * Class for representing native character strings and handling common string
   * operations
@@ -209,187 +242,7 @@ class XMLUTIL_EXPORT XMLString
 {
 public:
     /* Static methods for native character mode string manipulation */
-    /** @name Conversion functions */
-    //@{
 
-    /** Converts binary data to a text string based a given radix
-      *
-      * @param toFormat The beginning of the input string to convert
-      * @param toFill The buffer that will hold the output on return. The
-      *        size of this buffer should at least be 'maxChars + 1'.
-      * @param maxChars The maximum number of output characters that can be
-      *         accepted. If the result will not fit, it is an error.
-      * @param radix The radix of the input data, based on which the conversion
-      * will be done
-      */
-    static void binToText
-    (
-        const   unsigned int    toFormat
-        ,       char* const     toFill
-        , const unsigned int    maxChars
-        , const unsigned int    radix
-    );
-
-    /** Converts binary data to a text string based a given radix
-      *
-      * @param toFormat The beginning of the input string to convert
-      * @param toFill The buffer that will hold the output on return. The
-      *        size of this buffer should at least be 'maxChars + 1'.
-      * @param maxChars The maximum number of output characters that can be
-      *         accepted. If the result will not fit, it is an error.
-      * @param radix The radix of the input data, based on which the conversion
-      * will be done
-      */
-    static void binToText
-    (
-        const   unsigned int    toFormat
-        ,       XMLCh* const    toFill
-        , const unsigned int    maxChars
-        , const unsigned int    radix
-    );
-
-    /** Converts binary data to a text string based a given radix
-      *
-      * @param toFormat The beginning of the input string to convert
-      * @param toFill The buffer that will hold the output on return. The
-      *        size of this buffer should at least be 'maxChars + 1'.
-      * @param maxChars The maximum number of output characters that can be
-      *         accepted. If the result will not fit, it is an error.
-      * @param radix The radix of the input data, based on which the conversion
-      * will be done
-      */
-    static void binToText
-    (
-        const   unsigned long   toFormat
-        ,       char* const     toFill
-        , const unsigned int    maxChars
-        , const unsigned int    radix
-    );
-
-    /** Converts binary data to a text string based a given radix
-      *
-      * @param toFormat The beginning of the input string to convert
-      * @param toFill The buffer that will hold the output on return. The
-      *        size of this buffer should at least be 'maxChars + 1'.
-      * @param maxChars The maximum number of output characters that can be
-      *         accepted. If the result will not fit, it is an error.
-      * @param radix The radix of the input data, based on which the conversion
-      * will be done
-      */
-    static void binToText
-    (
-        const   unsigned long   toFormat
-        ,       XMLCh* const    toFill
-        , const unsigned int    maxChars
-        , const unsigned int    radix
-    );
-
-    /** Converts binary data to a text string based a given radix
-      *
-      * @param toFormat The beginning of the input string to convert
-      * @param toFill The buffer that will hold the output on return. The
-      *        size of this buffer should at least be 'maxChars + 1'.
-      * @param maxChars The maximum number of output characters that can be
-      *         accepted. If the result will not fit, it is an error.
-      * @param radix The radix of the input data, based on which the conversion
-      * will be done
-      */
-    static void binToText
-    (
-        const   long            toFormat
-        ,       char* const     toFill
-        , const unsigned int    maxChars
-        , const unsigned int    radix
-    );
-
-    /** Converts binary data to a text string based a given radix
-      *
-      * @param toFormat The beginning of the input string to convert
-      * @param toFill The buffer that will hold the output on return. The
-      *        size of this buffer should at least be 'maxChars + 1'.
-      * @param maxChars The maximum number of output characters that can be
-      *         accepted. If the result will not fit, it is an error.
-      * @param radix The radix of the input data, based on which the conversion
-      * will be done
-      */
-    static void binToText
-    (
-        const   long            toFormat
-        ,       XMLCh* const    toFill
-        , const unsigned int    maxChars
-        , const unsigned int    radix
-    );
-
-    /** Converts binary data to a text string based a given radix
-      *
-      * @param toFormat The beginning of the input string to convert
-      * @param toFill The buffer that will hold the output on return. The
-      *        size of this buffer should at least be 'maxChars + 1'.
-      * @param maxChars The maximum number of output characters that can be
-      *         accepted. If the result will not fit, it is an error.
-      * @param radix The radix of the input data, based on which the conversion
-      * will be done
-      */
-    static void binToText
-    (
-        const   int             toFormat
-        ,       char* const     toFill
-        , const unsigned int    maxChars
-        , const unsigned int    radix
-    );
-
-    /** Converts binary data to a text string based a given radix
-      *
-      * @param toFormat The beginning of the input string to convert
-      * @param toFill The buffer that will hold the output on return. The
-      *        size of this buffer should at least be 'maxChars + 1'.
-      * @param maxChars The maximum number of output characters that can be
-      *         accepted. If the result will not fit, it is an error.
-      * @param radix The radix of the input data, based on which the conversion
-      * will be done
-      */
-    static void binToText
-    (
-        const   int             toFormat
-        ,       XMLCh* const    toFill
-        , const unsigned int    maxChars
-        , const unsigned int    radix
-    );
-
-    /**
-      * Converts a string of decimal chars to a binary value
-      *
-      * Note that leading and trailng whitespace is legal and will be ignored
-      * but the remainder must be all decimal digits.
-      *
-      * @param toConvert The string of digits to convert
-      * @param toFill    The unsigned int value to fill with the converted
-      *                  value.
-      */
-    static bool textToBin
-    (
-        const   XMLCh* const    toConvert
-        ,       unsigned int&   toFill
-    );
-
-    /**
-      * Converts a string of decimal chars to a binary value
-      *
-      * Note that leading and trailng whitespace is legal and will be ignored,
-      *
-      * Only one and either of (+,-) after the leading whitespace, before
-      * any other characters are allowed.
-      *
-      * but the remainder must be all decimal digits.
-      *
-      * @param toConvert The string of digits to convert
-      */
-    static int parseInt
-    (
-        const   XMLCh* const    toConvert
-    );
-
-    //@}
 
     /** @name String concatenation functions */
     //@{
@@ -585,6 +438,26 @@ public:
         , const XMLCh* const    str2
     );
 
+    /** compares <code>str1</code> and <code>str2</code>
+      *
+      * @param str1 Null-terminated string to compare
+      * @param str2 Null-terminated string to compare
+      * @return true if two strings are equal, false if not
+      *  If one string is null, while the other is zero-length string,
+      *  it is considered as equal.
+      */
+    static bool equals
+    (
+          const XMLCh* const    str1
+        , const XMLCh* const    str2
+    );
+
+    static bool equals
+    (
+          const char* const    str1
+        , const char* const    str2
+    );
+
 	/** Lexicographically compares <code>str1</code> and <code>str2</code>
 	  * regions and returns true if they are equal, otherwise false.
 	  *
@@ -882,7 +755,7 @@ public:
 
     //@}
 
-	    /** @name Substring function */
+    /** @name Substring function */
     //@{
     /** Create a substring of a givend string. The substring begins at the
       * specified beginIndex and extends to the character at index
@@ -921,14 +794,24 @@ public:
     /** @name Replication function */
     //@{
     /** Replicates a string
+      * NOTE: The returned buffer is dynamically allocated and is the
+      * responsibility of the caller to delete it when not longer needed.
+      * You can call XMLString::release to release this returned buffer.
+      *
       * @param toRep The string to replicate
       * @return Returns a pointer to the replicated string
+      * @see   XMLString::release(char**)
       */
     static char* replicate(const char* const toRep);
 
     /** Replicates a string
+      * NOTE: The returned buffer is dynamically allocated and is the
+      * responsibility of the caller to delete it when not longer needed.
+      * You can call XMLString::release to release this returned buffer.
+
       * @param toRep The string to replicate
       * @return Returns a pointer to the replicated string
+      * @see   XMLString::release(XMLCh**)
       */
     static XMLCh* replicate(const XMLCh* const toRep);
 
@@ -1034,7 +917,7 @@ public:
       */
     static int patternMatch
     (
-                XMLCh* const    toSearch
+          const XMLCh* const    toSearch
         , const XMLCh* const    pattern
     );
 
@@ -1051,14 +934,18 @@ public:
     static unsigned int stringLen(const XMLCh* const src);
 
     /**
-      * Checks whether an name is a valid NCName.
+      * Deprecated: please use XMLChar1_0::isValidNCName
+      *
+      * Checks whether an name is a valid NCName according to XML 1.0
       * @param name The string to check its NCName validity
       * @return Returns true if name is NCName valid, otherwise false
       */
     static bool isValidNCName(const XMLCh* const name);
 
     /**
-      * Checks whether an name is a valid Name.
+      * Deprecated: please use XMLChar1_0::isValidName
+      *
+      * Checks whether an name is a valid Name according to XML 1.0
       * @param name The string to check its Name validity
       * @return Returns true if name is Name valid, otherwise false
       */
@@ -1072,7 +959,9 @@ public:
     static bool isValidEncName(const XMLCh* const name);
 
     /**
-      * Checks whether an name is a valid QName.
+      * Deprecated: please use XMLChar1_0::isValidQName
+      *
+      * Checks whether an name is a valid QName according to XML 1.0
       * @param name The string to check its QName validity
       * @return Returns true if name is QName valid, otherwise false
       */
@@ -1108,17 +997,203 @@ public:
     static bool isHex(XMLCh const theChar);
 
     /**
-      * Checks whether a string contains only whitespace.
+      * Deprecated: please use XMLChar1_0::isAllWhiteSpace
+      *
+      * Checks whether aa string contains only whitespace according to XML 1.0
       * @param toCheck the string to check
       * @return Returns true if it is, otherwise false
       */
     static bool isAllWhiteSpace(const XMLCh* const toCheck);
 
+    /** Find is the string appears in the enum list
+      * @param toFind the string to be found
+      * @param enumList the list
+      * return true if found
+      */
+    static bool isInList(const XMLCh* const toFind, const XMLCh* const enumList);
+
     //@}
 
     /** @name Conversion functions */
     //@{
 
+    /** Converts binary data to a text string based a given radix
+      *
+      * @param toFormat The beginning of the input string to convert
+      * @param toFill The buffer that will hold the output on return. The
+      *        size of this buffer should at least be 'maxChars + 1'.
+      * @param maxChars The maximum number of output characters that can be
+      *         accepted. If the result will not fit, it is an error.
+      * @param radix The radix of the input data, based on which the conversion
+      * will be done
+      */
+    static void binToText
+    (
+        const   unsigned int    toFormat
+        ,       char* const     toFill
+        , const unsigned int    maxChars
+        , const unsigned int    radix
+    );
+
+    /** Converts binary data to a text string based a given radix
+      *
+      * @param toFormat The beginning of the input string to convert
+      * @param toFill The buffer that will hold the output on return. The
+      *        size of this buffer should at least be 'maxChars + 1'.
+      * @param maxChars The maximum number of output characters that can be
+      *         accepted. If the result will not fit, it is an error.
+      * @param radix The radix of the input data, based on which the conversion
+      * will be done
+      */
+    static void binToText
+    (
+        const   unsigned int    toFormat
+        ,       XMLCh* const    toFill
+        , const unsigned int    maxChars
+        , const unsigned int    radix
+    );
+
+    /** Converts binary data to a text string based a given radix
+      *
+      * @param toFormat The beginning of the input string to convert
+      * @param toFill The buffer that will hold the output on return. The
+      *        size of this buffer should at least be 'maxChars + 1'.
+      * @param maxChars The maximum number of output characters that can be
+      *         accepted. If the result will not fit, it is an error.
+      * @param radix The radix of the input data, based on which the conversion
+      * will be done
+      */
+    static void binToText
+    (
+        const   unsigned long   toFormat
+        ,       char* const     toFill
+        , const unsigned int    maxChars
+        , const unsigned int    radix
+    );
+
+    /** Converts binary data to a text string based a given radix
+      *
+      * @param toFormat The beginning of the input string to convert
+      * @param toFill The buffer that will hold the output on return. The
+      *        size of this buffer should at least be 'maxChars + 1'.
+      * @param maxChars The maximum number of output characters that can be
+      *         accepted. If the result will not fit, it is an error.
+      * @param radix The radix of the input data, based on which the conversion
+      * will be done
+      */
+    static void binToText
+    (
+        const   unsigned long   toFormat
+        ,       XMLCh* const    toFill
+        , const unsigned int    maxChars
+        , const unsigned int    radix
+    );
+
+    /** Converts binary data to a text string based a given radix
+      *
+      * @param toFormat The beginning of the input string to convert
+      * @param toFill The buffer that will hold the output on return. The
+      *        size of this buffer should at least be 'maxChars + 1'.
+      * @param maxChars The maximum number of output characters that can be
+      *         accepted. If the result will not fit, it is an error.
+      * @param radix The radix of the input data, based on which the conversion
+      * will be done
+      */
+    static void binToText
+    (
+        const   long            toFormat
+        ,       char* const     toFill
+        , const unsigned int    maxChars
+        , const unsigned int    radix
+    );
+
+    /** Converts binary data to a text string based a given radix
+      *
+      * @param toFormat The beginning of the input string to convert
+      * @param toFill The buffer that will hold the output on return. The
+      *        size of this buffer should at least be 'maxChars + 1'.
+      * @param maxChars The maximum number of output characters that can be
+      *         accepted. If the result will not fit, it is an error.
+      * @param radix The radix of the input data, based on which the conversion
+      * will be done
+      */
+    static void binToText
+    (
+        const   long            toFormat
+        ,       XMLCh* const    toFill
+        , const unsigned int    maxChars
+        , const unsigned int    radix
+    );
+
+    /** Converts binary data to a text string based a given radix
+      *
+      * @param toFormat The beginning of the input string to convert
+      * @param toFill The buffer that will hold the output on return. The
+      *        size of this buffer should at least be 'maxChars + 1'.
+      * @param maxChars The maximum number of output characters that can be
+      *         accepted. If the result will not fit, it is an error.
+      * @param radix The radix of the input data, based on which the conversion
+      * will be done
+      */
+    static void binToText
+    (
+        const   int             toFormat
+        ,       char* const     toFill
+        , const unsigned int    maxChars
+        , const unsigned int    radix
+    );
+
+    /** Converts binary data to a text string based a given radix
+      *
+      * @param toFormat The beginning of the input string to convert
+      * @param toFill The buffer that will hold the output on return. The
+      *        size of this buffer should at least be 'maxChars + 1'.
+      * @param maxChars The maximum number of output characters that can be
+      *         accepted. If the result will not fit, it is an error.
+      * @param radix The radix of the input data, based on which the conversion
+      * will be done
+      */
+    static void binToText
+    (
+        const   int             toFormat
+        ,       XMLCh* const    toFill
+        , const unsigned int    maxChars
+        , const unsigned int    radix
+    );
+
+    /**
+      * Converts a string of decimal chars to a binary value
+      *
+      * Note that leading and trailng whitespace is legal and will be ignored
+      * but the remainder must be all decimal digits.
+      *
+      * @param toConvert The string of digits to convert
+      * @param toFill    The unsigned int value to fill with the converted
+      *                  value.
+      */
+    static bool textToBin
+    (
+        const   XMLCh* const    toConvert
+        ,       unsigned int&   toFill
+    );
+
+    /**
+      * Converts a string of decimal chars to a binary value
+      *
+      * Note that leading and trailng whitespace is legal and will be ignored,
+      *
+      * Only one and either of (+,-) after the leading whitespace, before
+      * any other characters are allowed.
+      *
+      * but the remainder must be all decimal digits.
+      *
+      * @param toConvert The string of digits to convert
+      */
+    static int parseInt
+    (
+        const   XMLCh* const    toConvert
+    );
+
     /** Cut leading chars from a string
       *
       * @param toCutFrom The string to cut chars from
@@ -1134,9 +1209,11 @@ public:
       *
       * NOTE: The returned buffer is dynamically allocated and is the
       * responsibility of the caller to delete it when not longer needed.
+      * You can call XMLString::release to release this returned buffer.
       *
       * @param toTranscode The string to be transcoded
       * @return Returns the transcoded string
+      * @see   XMLString::release(XMLCh**)
       */
     static char* transcode
     (
@@ -1170,9 +1247,11 @@ public:
       *
       * NOTE: The returned buffer is dynamically allocated and is the
       * responsibility of the caller to delete it when not longer needed.
+      * You can call XMLString::release to release this returned buffer.
       *
       * @param toTranscode The string to be transcoded
       * @return Returns the transcoded string
+      * @see   XMLString::release(char**)
       */
     static XMLCh* transcode
     (
@@ -1217,14 +1296,7 @@ public:
       * @param tokenizeSrc String to be tokenized
       * @return a vector of all the tokenized string
       */
-    static RefVectorOf<XMLCh>* tokenizeString(const XMLCh* const tokenizeSrc);
-
-    /** Find is the string appears in the enum list
-      * @param toFind the string to be found
-      * @param enumList the list
-      * return true if found
-      */
-    static bool isInList(const XMLCh* const toFind, const XMLCh* const enumList);
+    static BaseRefVectorOf<XMLCh>* tokenizeString(const XMLCh* const tokenizeSrc);
 
     //@}
 
@@ -1302,6 +1374,49 @@ public:
       *        On return , this buffer also holds the converted string
       */
     static void collapseWS(XMLCh* const toConvert);
+
+    /** Remove whitespace
+      * @param toConvert The string which needs to be whitespace removed.
+      *        On return , this buffer also holds the converted string
+      */
+    static void removeWS(XMLCh* const toConvert);
+
+    /**
+     * Fixes a platform dependent absolute path filename to standard URI form.
+     * 1. Windows: fix 'x:' to 'file:///x:' and convert any backslash to forward slash
+     * 2. UNIX: fix '/blah/blahblah' to 'file:///blah/blahblah'
+     * @param str    The string that has the absolute path filename
+     * @param target The target string pre-allocated to store the fixed uri
+     */
+    static void fixURI(const XMLCh* const str, XMLCh* const target);
+
+    //@}
+    /** @name String Memory Management functions */
+    //@{
+    /**
+     * Release the parameter char string that was allocated by the implementation (i.e.the parser).
+     *   The implementation will call operator delete[] and then turn the string to a null pointer.
+     *
+     * @param buf  The string to be deleted and become a null pointer.
+     */
+    static void release(char** buf);
+
+    /**
+     * Release the parameter XMLCh string that was allocated by the implementation (i.e.the parser).
+     *   The implementation will call operator delete[] and then turn the string to a null pointer.
+     *
+     * @param buf  The string to be deleted and become a null pointer.
+     */
+    static void release(XMLCh** buf);
+
+    /**
+     * Release the parameter XMLByte string that was allocated by the implementation (i.e.the parser).
+     *   The implementation will call operator delete[] and then turn the string to a null pointer.
+     *
+     * @param buf  The string to be deleted and become a null pointer.
+     */
+    static void release(XMLByte** buf);
+
     //@}
 
 
@@ -1419,4 +1534,58 @@ inline bool XMLString::validateRegion(const XMLCh* const str1,
 	return true;
 }
 
+inline bool XMLString::equals(   const XMLCh* const    str1
+                               , const XMLCh* const    str2)
+{
+    const XMLCh* psz1 = str1;
+    const XMLCh* psz2 = str2;
+
+    if (psz1 == 0 || psz2 == 0) {
+        if ((psz1 != 0 && *psz1) || (psz2 != 0 && *psz2))
+            return false;
+        else
+            return true;
+    }
+
+    while (*psz1 == *psz2)
+    {
+        // If either has ended, then they both ended, so equal
+        if (!*psz1)
+            return true;
+
+        // Move upwards for the next round
+        psz1++;
+        psz2++;
+    }
+    return false;
+}
+
+inline bool XMLString::equals(   const char* const    str1
+                               , const char* const    str2)
+{
+    const char* psz1 = str1;
+    const char* psz2 = str2;
+
+    if (psz1 == 0 || psz2 == 0) {
+        if ((psz1 != 0 && *psz1) || (psz2 != 0 && *psz2))
+            return false;
+        else
+            return true;
+    }
+
+    while (*psz1 == *psz2)
+    {
+        // If either has ended, then they both ended, so equal
+        if (!*psz1)
+            return true;
+
+        // Move upwards for the next round
+        psz1++;
+        psz2++;
+    }
+    return false;
+}
+
+XERCES_CPP_NAMESPACE_END
+
 #endif
