@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: XMLURL.cpp,v 1.1.1.1 2002/02/01 22:22:17 peiyongz Exp $
+ * $Id: XMLURL.cpp,v 1.5 2003/01/06 19:43:18 tng Exp $
  */
 
 
@@ -72,6 +72,9 @@
 #include <xercesc/util/XMLString.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/util/XMLUni.hpp>
+#include <xercesc/util/XMLUri.hpp>
+
+XERCES_CPP_NAMESPACE_BEGIN
 
 
 
@@ -189,6 +192,7 @@ XMLURL::XMLURL() :
     , fQuery(0)
     , fUser(0)
     , fURLText(0)
+    , fHasInvalidChar(false)
 {
 }
 
@@ -204,6 +208,7 @@ XMLURL::XMLURL(const XMLCh* const    baseURL
     , fQuery(0)
     , fUser(0)
     , fURLText(0)
+    , fHasInvalidChar(false)
 {
 	try
 	{
@@ -228,6 +233,7 @@ XMLURL::XMLURL(const XMLCh* const    baseURL
     , fQuery(0)
     , fUser(0)
     , fURLText(0)
+    , fHasInvalidChar(false)
 {
     XMLCh* tmpRel = XMLString::transcode(relativeURL);
     ArrayJanitor<XMLCh> janRel(tmpRel);
@@ -254,6 +260,7 @@ XMLURL::XMLURL(const XMLURL&         baseURL
     , fQuery(0)
     , fUser(0)
     , fURLText(0)
+    , fHasInvalidChar(false)
 {
 	try
 	{
@@ -278,6 +285,7 @@ XMLURL::XMLURL(const  XMLURL&        baseURL
     , fQuery(0)
     , fUser(0)
     , fURLText(0)
+    , fHasInvalidChar(false)
 {
     XMLCh* tmpRel = XMLString::transcode(relativeURL);
     ArrayJanitor<XMLCh> janRel(tmpRel);
@@ -304,6 +312,7 @@ XMLURL::XMLURL(const XMLCh* const urlText) :
     , fQuery(0)
     , fUser(0)
     , fURLText(0)
+    , fHasInvalidChar(false)
 {
 	try
 	{
@@ -327,6 +336,7 @@ XMLURL::XMLURL(const char* const urlText) :
     , fQuery(0)
     , fUser(0)
     , fURLText(0)
+    , fHasInvalidChar(false)
 {
     XMLCh* tmpText = XMLString::transcode(urlText);
     ArrayJanitor<XMLCh> janRel(tmpText);
@@ -352,6 +362,7 @@ XMLURL::XMLURL(const XMLURL& toCopy) :
     , fQuery(XMLString::replicate(toCopy.fQuery))
     , fUser(XMLString::replicate(toCopy.fUser))
     , fURLText(XMLString::replicate(toCopy.fURLText))
+    , fHasInvalidChar(toCopy.fHasInvalidChar)
 {
 }
 
@@ -392,7 +403,7 @@ bool XMLURL::operator==(const XMLURL& toCompare) const
     //  way so they should now be the same even if they came in via different
     //  relative parts.
     //
-    if (XMLString::compareString(getURLText(), toCompare.getURLText()))
+    if (!XMLString::equals(getURLText(), toCompare.getURLText()))
         return false;
 
     return true;
@@ -501,6 +512,11 @@ bool XMLURL::isRelative() const
 }
 
 
+bool XMLURL::hasInvalidChar() const {
+    return fHasInvalidChar;
+}
+
+
 BinInputStream* XMLURL::makeNewStream() const
 {
     //
@@ -512,36 +528,46 @@ BinInputStream* XMLURL::makeNewStream() const
     {
         if (!fHost || !XMLString::compareIString(fHost, XMLUni::fgLocalHostString))
         {
-            //
-            //  We have to play a little trick here. If its really a Windows
-            //  style fully qualified path, we have to toss the leading /
-            //  character.
-            //
-            const XMLCh* realPath = fPath;
-            if (*fPath == chForwardSlash)
-            {
-                if (XMLString::stringLen(fPath) > 3)
-                {
-                    if (*(fPath + 2) == chColon)
-                    {
-                        const XMLCh chDrive = *(fPath + 1);
-                        if (((chDrive >= chLatin_A) && (chDrive <= chLatin_Z))
-                        ||  ((chDrive >= chLatin_a) && (chDrive <= chLatin_z)))
-                        {
-                            realPath = fPath + 1;
-                        }
-                    }
 
-                    // Similarly for UNC paths
-                    if ( *(fPath + 1) == *(fPath + 2) &&
-                         (*(fPath + 1) == chForwardSlash ||
-                          *(fPath + 1) == chBackSlash) )
-                    {
-                        realPath = fPath + 1;
-                    }
+            XMLCh* realPath = XMLString::replicate(fPath);
+            ArrayJanitor<XMLCh> basePathName(realPath);
+
+            //
+            // Need to manually replace any character reference %xx first
+            // HTTP protocol will be done automatically by the netaccessor
+            //
+            int end = XMLString::stringLen(realPath);
+            int percentIndex = XMLString::indexOf(realPath, chPercent, 0);
+
+            while (percentIndex != -1) {
+
+                if (percentIndex+2 >= end ||
+                    !isHexDigit(realPath[percentIndex+1]) ||
+                    !isHexDigit(realPath[percentIndex+2]))
+                {
+                    XMLCh value1[4];
+                    XMLString::moveChars(value1, &(realPath[percentIndex]), 3);
+                    value1[3] = chNull;
+                    ThrowXML2(MalformedURLException
+                            , XMLExcepts::XMLNUM_URI_Component_Invalid_EscapeSequence
+                            , realPath
+                            , value1);
                 }
+
+                unsigned int value = (xlatHexDigit(realPath[percentIndex+1]) * 16) + xlatHexDigit(realPath[percentIndex+2]);
+
+                realPath[percentIndex] = XMLCh(value);
+
+                int i =0;
+                for (i = percentIndex + 1; i < end - 2 ; i++)
+                    realPath[i] = realPath[i+2];
+                realPath[i] = chNull;
+                end = i;
+
+                percentIndex = XMLString::indexOf(realPath, chPercent, percentIndex);
             }
 
+
             BinFileInputStream* retStrm = new BinFileInputStream(realPath);
             if (!retStrm->getIsOpen())
             {
@@ -795,8 +821,11 @@ bool XMLURL::conglomerateWithBase(const XMLURL& baseURL, bool useExceptions)
     }
 
     // Its a relative path, so weave them together.
-    if (baseURL.fPath)
-        weavePaths(baseURL.fPath);
+    if (baseURL.fPath) {
+        XMLCh* temp = XMLPlatformUtils::weavePaths(baseURL.fPath, fPath);
+        delete [] fPath;
+        fPath = temp;
+    }
 
     // If we had any original path, then we are done
     if (hadPath)
@@ -820,6 +849,12 @@ void XMLURL::parse(const XMLCh* const urlText)
     if (!*urlText)
         ThrowXML(MalformedURLException, XMLExcepts::URL_NoProtocolPresent);
 
+    // Before we start, check if this urlText contains valid uri characters
+    if (!XMLUri::isURIString(urlText))
+        fHasInvalidChar = true;
+    else
+        fHasInvalidChar = false;
+
     //
     //  The first thing we will do is to check for a file name, so that
     //  we don't waste time thinking its a URL. If its in the form x:\
@@ -945,21 +980,20 @@ void XMLURL::parse(const XMLCh* const urlText)
                 srcPtr += XMLString::stringLen(fHost);
             }
         }
-    } 
-    else 
+    }
+    else
     {
-	    //
-	    // http protocol requires two forward slashes
-	    // we didn't get them, so throw an exception
-	    //
-	if (fProtocol == HTTP) {
-                ThrowXML1
+        //
+        // http protocol requires two forward slashes
+        // we didn't get them, so throw an exception
+        //
+        if (fProtocol == HTTP) {
+            ThrowXML
                 (
                     MalformedURLException
                     , XMLExcepts::URL_ExpectingTwoSlashes
-                    , "Found 'http' protocol"
                 );
-	}
+        }
     }
 
     //
@@ -1097,114 +1131,5 @@ void XMLURL::parse(const XMLCh* const urlText)
 }
 
 
-void XMLURL::weavePaths(const XMLCh* const basePart)
-{
-    // Watch for stupid stuff
-    if (!basePart)
-        return;
-    if (!*basePart)
-        return;
+XERCES_CPP_NAMESPACE_END
 
-    //
-    //  Ok, lets start at the end of the base path and work backwards and
-    //  our path part and work forwards. For each leading . we see, we just
-    //  eat it. For each leading .. we see, we eat it and throw away one
-    //  level in the source URL.
-    //
-    //  If the last character in the base part is a forward slash, back
-    //  up one first before we look for the last slash.
-    //
-    const XMLCh* basePtr = basePart + (XMLString::stringLen(basePart) - 1);
-    if (*basePtr == chForwardSlash)
-        basePtr--;
-
-    while ((basePtr >= basePart)
-    &&     ((*basePtr != chForwardSlash) && (*basePtr != chBackSlash)))
-    {
-        basePtr--;
-    }
-
-    if (basePtr < basePart)
-        return;
-
-    // Create a buffer as large as both parts
-    XMLCh* tmpBuf = new XMLCh[XMLString::stringLen(fPath)
-                              + XMLString::stringLen(basePart)
-                              + 2];
-    //
-    //  If we have no path part, then copy the base part up to the
-    //  base pointer
-    //
-    if (!fPath)
-    {
-        XMLCh* bufPtr = tmpBuf;
-        const XMLCh* tmpPtr = basePart;
-        while (tmpPtr <= basePtr)
-            *bufPtr++ = *tmpPtr++;
-        *bufPtr = 0;
-
-        fPath = tmpBuf;
-        return;
-    }
-
-    // After this, make sure the buffer gets handled if we exit early
-    ArrayJanitor<XMLCh> janBuf(tmpBuf);
-
-    //
-    //  We have some path part, so we need to check to see if we ahve to
-    //  weave any of the parts together.
-    //
-    XMLCh* pathPtr = fPath;
-    while (true)
-    {
-        // If it does not start with some period, then we are done
-        if (*pathPtr != chPeriod)
-            break;
-
-        unsigned int periodCount = 1;
-        pathPtr++;
-        if (*pathPtr == chPeriod)
-        {
-            pathPtr++;
-            periodCount++;
-        }
-
-        // Has to be followed by a / or \ or the null to mean anything
-        if ((*pathPtr != chForwardSlash) && (*pathPtr != chBackSlash)
-        &&  *pathPtr)
-        {
-            break;
-        }
-        if (*pathPtr)
-            pathPtr++;
-
-        // If its one period, just eat it, else move backwards in the base
-        if (periodCount == 2)
-        {
-            basePtr--;
-            while ((basePtr >= basePart)
-            &&     ((*basePtr != chForwardSlash) && (*basePtr != chBackSlash)))
-            {
-                basePtr--;
-            }
-
-            // There are not enough levels to handle all the .. parts
-            if (basePtr < basePart)
-                ThrowXML(MalformedURLException, XMLExcepts::URL_BaseUnderflow);
-        }
-    }
-
-    // Copy the base part up to the base pointer
-    XMLCh* bufPtr = tmpBuf;
-    const XMLCh* tmpPtr = basePart;
-    while (tmpPtr <= basePtr)
-        *bufPtr++ = *tmpPtr++;
-
-    // And then copy on the rest of our path
-    XMLString::copyString(bufPtr, pathPtr);
-
-    // Now delete our path and make the new buffer our path
-    delete [] fPath;
-    janBuf.orphan();
-    fPath = tmpBuf;
-}
