@@ -55,17 +55,17 @@
  */
 
 /*
- * $Id: XMLUni.cpp,v 1.5 2002/03/06 19:13:12 peiyongz Exp $
+ * $Id: XMLUni.cpp,v 1.29 2003/01/28 18:33:23 peiyongz Exp $
  */
 
 
 // ---------------------------------------------------------------------------
 //  Includes
 // ---------------------------------------------------------------------------
-#include <xercesc/util/XercesDefs.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/util/XMLUni.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  XMLUni: Static data
@@ -178,6 +178,16 @@ const XMLCh XMLUni::fgIBM1140EncodingString2[] =
     chLatin_I, chLatin_B, chLatin_M, chDigit_0, chDigit_1, chDigit_1, chDigit_4, chDigit_0, chNull
 };
 
+const XMLCh XMLUni::fgIBM1140EncodingString3[] =
+{
+    chLatin_C, chLatin_C, chLatin_S, chLatin_I, chLatin_D, chDigit_0, chDigit_1, chDigit_1, chDigit_4, chDigit_0, chNull
+};
+
+const XMLCh XMLUni::fgIBM1140EncodingString4[] =
+{
+    chLatin_C, chLatin_P, chDigit_0, chDigit_1, chDigit_1, chDigit_4, chDigit_0, chNull
+};
+
 const XMLCh XMLUni::fgIESString[] =
 {
     chLatin_I, chLatin_E, chLatin_S, chNull
@@ -253,6 +263,26 @@ const XMLCh XMLUni::fgISO88591EncodingString8[] =
     chLatin_L, chLatin_A, chLatin_T, chLatin_I, chLatin_N, chUnderscore, chDigit_1, chNull
 };
 
+const XMLCh XMLUni::fgISO88591EncodingString9[] =
+{
+    chLatin_C, chLatin_P, chDigit_8, chDigit_1, chDigit_9, chNull
+};
+
+const XMLCh XMLUni::fgISO88591EncodingString10[] =
+{
+    chLatin_C, chLatin_S, chLatin_I, chLatin_S, chLatin_O, chLatin_L, chLatin_A, chLatin_T, chLatin_I, chLatin_N, chDigit_1, chNull
+};
+
+const XMLCh XMLUni::fgISO88591EncodingString11[] =
+{
+    chLatin_I, chLatin_S, chLatin_O, chDash, chLatin_I, chLatin_R, chDash, chDigit_1, chDigit_0, chDigit_0, chNull
+};
+
+const XMLCh XMLUni::fgISO88591EncodingString12[] =
+{
+    chLatin_L, chDigit_1, chNull
+};
+
 const XMLCh XMLUni::fgLocalHostString[] =
 {
     chLatin_l, chLatin_o, chLatin_c, chLatin_a, chLatin_l, chLatin_h, chLatin_o, chLatin_s, chLatin_t, chNull
@@ -313,11 +343,16 @@ const XMLCh XMLUni::fgStandaloneString[] =
     chLatin_s, chLatin_t, chLatin_a, chLatin_n, chLatin_d, chLatin_a, chLatin_l, chLatin_o, chLatin_n, chLatin_e, chNull
 };
 
-const XMLCh XMLUni::fgSupportedVersion[] =
+const XMLCh XMLUni::fgVersion1_0[] =
 {
     chDigit_1, chPeriod, chDigit_0, chNull
 };
 
+const XMLCh XMLUni::fgVersion1_1[] =
+{
+    chDigit_1, chPeriod, chDigit_1, chNull
+};
+
 const XMLCh XMLUni::fgSysIDString[] =
 {
     chLatin_S, chLatin_Y, chLatin_S, chLatin_T, chLatin_E, chLatin_M, chNull
@@ -418,6 +453,11 @@ const XMLCh XMLUni::fgUTF16EncodingString4[] =
     chLatin_I, chLatin_B, chLatin_M, chDash, chDigit_1, chDigit_2, chDigit_0, chDigit_0, chNull
 };
 
+const XMLCh XMLUni::fgUTF16EncodingString5[] =
+{
+    chLatin_U, chLatin_T, chLatin_F, chDigit_1, chDigit_6, chNull
+};
+
 const XMLCh XMLUni::fgUTF16BEncodingString[] =
 {
     chLatin_U, chLatin_T, chLatin_F, chDash, chDigit_1, chDigit_6, chSpace, chOpenParen, chLatin_B, chLatin_E, chCloseParen, chNull
@@ -603,11 +643,65 @@ const XMLCh XMLUni::fgXMLURIName[] =
 
 const XMLCh XMLUni::fgYesString[] =
 {
-    chLatin_y, chLatin_e, chLatin_s, chNull
+        chLatin_y, chLatin_e, chLatin_s, chNull
 };
 
 const XMLCh XMLUni::fgZeroLenString[] = { chNull };
 
+const XMLCh XMLUni::fgDTDEntityString[] =
+{
+        chOpenSquare, chLatin_d, chLatin_t, chLatin_d, chCloseSquare, chNull
+};
+
+const XMLCh XMLUni::fgAmp[] =
+{
+        chLatin_a, chLatin_m, chLatin_p, chNull
+};
+
+const XMLCh XMLUni::fgLT[] =
+{
+        chLatin_l, chLatin_t, chNull
+};
+
+const XMLCh XMLUni::fgGT[] =
+{
+        chLatin_g, chLatin_t, chNull
+};
+
+const XMLCh XMLUni::fgQuot[] =
+{
+        chLatin_q, chLatin_u, chLatin_o, chLatin_t, chNull
+};
+
+const XMLCh XMLUni::fgApos[] =
+{
+        chLatin_a, chLatin_p, chLatin_o, chLatin_s, chNull
+};
+
+const XMLCh XMLUni::fgWFXMLScanner[] =
+{
+        chLatin_W, chLatin_F, chLatin_X, chLatin_M, chLatin_L, chLatin_S
+    ,   chLatin_c, chLatin_a, chLatin_n, chLatin_n, chLatin_e, chLatin_r, chNull
+};
+
+const XMLCh XMLUni::fgIGXMLScanner[] =
+{
+        chLatin_I, chLatin_G, chLatin_X, chLatin_M, chLatin_L, chLatin_S
+    ,   chLatin_c, chLatin_a, chLatin_n, chLatin_n, chLatin_e, chLatin_r, chNull
+};
+
+const XMLCh XMLUni::fgSGXMLScanner[] =
+{
+        chLatin_S, chLatin_G, chLatin_X, chLatin_M, chLatin_L, chLatin_S
+    ,   chLatin_c, chLatin_a, chLatin_n, chLatin_n, chLatin_e, chLatin_r, chNull
+};
+
+const XMLCh XMLUni::fgDGXMLScanner[] =
+{
+        chLatin_D, chLatin_G, chLatin_X, chLatin_M, chLatin_L, chLatin_S
+    ,   chLatin_c, chLatin_a, chLatin_n, chLatin_n, chLatin_e, chLatin_r, chNull
+};
+
 //Exception strings
 const XMLCh XMLUni::fgArrayIndexOutOfBoundsException_Name[] =
 {
@@ -860,7 +954,7 @@ const XMLCh XMLUni::fgSAX2CoreNameSpacePrefixes[] =
 };
 
 //Xerces: http://apache.org/xml/features/validation/dynamic
-const XMLCh XMLUni::fgSAX2XercesDynamic[] =
+const XMLCh XMLUni::fgXercesDynamic[] =
 {
         chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
     ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h
@@ -872,22 +966,8 @@ const XMLCh XMLUni::fgSAX2XercesDynamic[] =
     ,   chLatin_d, chLatin_y, chLatin_n, chLatin_a, chLatin_m, chLatin_i, chLatin_c, chNull
 };
 
-//Xerces: http://apache.org/xml/features/validation/reuse-grammar
-const XMLCh XMLUni::fgSAX2XercesReuseGrammar[] =
-{
-        chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
-    ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h
-    ,   chLatin_e, chPeriod, chLatin_o, chLatin_r, chLatin_g, chForwardSlash
-    ,   chLatin_x, chLatin_m, chLatin_l, chForwardSlash, chLatin_f, chLatin_e
-    ,   chLatin_a, chLatin_t, chLatin_u, chLatin_r, chLatin_e, chLatin_s
-    ,   chForwardSlash, chLatin_v, chLatin_a, chLatin_l, chLatin_i, chLatin_d
-    ,   chLatin_a, chLatin_t, chLatin_i, chLatin_o, chLatin_n, chForwardSlash
-    ,   chLatin_r, chLatin_e, chLatin_u, chLatin_s, chLatin_e, chDash, chLatin_g
-    ,   chLatin_r, chLatin_a,	chLatin_m, chLatin_m, chLatin_a, chLatin_r, chNull
-};
-
 //Xerces: http://apache.org/xml/features/validation/schema
-const XMLCh XMLUni::fgSAX2XercesSchema[] =
+const XMLCh XMLUni::fgXercesSchema[] =
 {
         chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
     ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h
@@ -900,7 +980,7 @@ const XMLCh XMLUni::fgSAX2XercesSchema[] =
 };
 
 //Xerces: http://apache.org/xml/features/validation/schema-full-checking
-const XMLCh XMLUni::fgSAX2XercesSchemaFullChecking[] =
+const XMLCh XMLUni::fgXercesSchemaFullChecking[] =
 {
         chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
     ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h
@@ -914,26 +994,81 @@ const XMLCh XMLUni::fgSAX2XercesSchemaFullChecking[] =
     ,   chLatin_e, chLatin_c, chLatin_k, chLatin_i, chLatin_n, chLatin_g, chNull
 };
 
-//deprecated
-//Xerces: http://apache.org/xml/features/validation/reuse-validator
-const XMLCh XMLUni::fgSAX2XercesReuseValidator[] =
+//Xerces: http://apache.org/xml/features/nonvalidating/load-external-dtd
+const XMLCh XMLUni::fgXercesLoadExternalDTD[] =
 {
         chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
     ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h
     ,   chLatin_e, chPeriod, chLatin_o, chLatin_r, chLatin_g, chForwardSlash
     ,   chLatin_x, chLatin_m, chLatin_l, chForwardSlash, chLatin_f, chLatin_e
     ,   chLatin_a, chLatin_t, chLatin_u, chLatin_r, chLatin_e, chLatin_s
-    ,   chForwardSlash, chLatin_v, chLatin_a, chLatin_l, chLatin_i, chLatin_d
-    ,   chLatin_a, chLatin_t, chLatin_i, chLatin_o, chLatin_n, chForwardSlash
-    ,   chLatin_r, chLatin_e, chLatin_u, chLatin_s, chLatin_e, chDash, chLatin_v
-    ,   chLatin_a, chLatin_l, chLatin_i, chLatin_d, chLatin_a, chLatin_t
-    ,   chLatin_o, chLatin_r, chNull
+    ,   chForwardSlash, chLatin_n, chLatin_o, chLatin_n
+    ,   chLatin_v, chLatin_a, chLatin_l, chLatin_i, chLatin_d
+    ,   chLatin_a, chLatin_t, chLatin_i, chLatin_n, chLatin_g, chForwardSlash
+    ,   chLatin_l, chLatin_o, chLatin_a, chLatin_d, chDash
+    ,   chLatin_e, chLatin_x, chLatin_t, chLatin_e, chLatin_r, chLatin_n, chLatin_a, chLatin_l, chDash
+    ,   chLatin_d, chLatin_t, chLatin_d, chNull
+};
+
+//Xerces: http://apache.org/xml/features/continue-after-fatal-error
+const XMLCh XMLUni::fgXercesContinueAfterFatalError[] =
+{
+        chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
+    ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h
+    ,   chLatin_e, chPeriod, chLatin_o, chLatin_r, chLatin_g, chForwardSlash
+    ,   chLatin_x, chLatin_m, chLatin_l, chForwardSlash, chLatin_f, chLatin_e
+    ,   chLatin_a, chLatin_t, chLatin_u, chLatin_r, chLatin_e, chLatin_s
+    ,   chForwardSlash, chLatin_c, chLatin_o, chLatin_n,  chLatin_t, chLatin_i, chLatin_n, chLatin_u, chLatin_e, chDash
+    ,   chLatin_a, chLatin_f, chLatin_t, chLatin_e, chLatin_r, chDash
+    ,   chLatin_f, chLatin_a, chLatin_t, chLatin_a, chLatin_l, chDash
+    ,   chLatin_e, chLatin_r, chLatin_r, chLatin_o, chLatin_r, chNull
+};
+
+//Xerces: http://apache.org/xml/features/validation-error-as-fatal
+const XMLCh XMLUni::fgXercesValidationErrorAsFatal[] =
+{
+        chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
+    ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h
+    ,   chLatin_e, chPeriod, chLatin_o, chLatin_r, chLatin_g, chForwardSlash
+    ,   chLatin_x, chLatin_m, chLatin_l, chForwardSlash, chLatin_f, chLatin_e
+    ,   chLatin_a, chLatin_t, chLatin_u, chLatin_r, chLatin_e, chLatin_s
+    ,   chForwardSlash, chLatin_v, chLatin_a, chLatin_l, chLatin_i, chLatin_d, chLatin_a, chLatin_t, chLatin_i, chLatin_o, chLatin_n, chDash
+    ,   chLatin_e, chLatin_r, chLatin_r, chLatin_o, chLatin_r, chDash
+    ,   chLatin_a, chLatin_s, chDash
+    ,   chLatin_f, chLatin_a, chLatin_t, chLatin_a, chLatin_l, chNull
+};
+
+//Xerces: http://apache.org/xml/features/calculate-src-ofs
+const XMLCh XMLUni::fgXercesCalculateSrcOfs[] =
+{
+        chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
+    ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h
+    ,   chLatin_e, chPeriod, chLatin_o, chLatin_r, chLatin_g, chForwardSlash
+    ,   chLatin_x, chLatin_m, chLatin_l, chForwardSlash, chLatin_f, chLatin_e
+    ,   chLatin_a, chLatin_t, chLatin_u, chLatin_r, chLatin_e, chLatin_s
+    ,   chForwardSlash, chLatin_c, chLatin_a, chLatin_l, chLatin_c, chLatin_u
+    ,   chLatin_l, chLatin_a, chLatin_t, chLatin_e, chDash, chLatin_s, chLatin_r
+    ,   chLatin_c, chDash, chLatin_o, chLatin_f, chLatin_s, chNull
+};
+
+//Xerces: http://apache.org/xml/features/standard-uri-conformant
+const XMLCh XMLUni::fgXercesStandardUriConformant[] =
+{
+        chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
+    ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h
+    ,   chLatin_e, chPeriod, chLatin_o, chLatin_r, chLatin_g, chForwardSlash
+    ,   chLatin_x, chLatin_m, chLatin_l, chForwardSlash, chLatin_f, chLatin_e
+    ,   chLatin_a, chLatin_t, chLatin_u, chLatin_r, chLatin_e, chLatin_s
+    ,   chForwardSlash, chLatin_s, chLatin_t, chLatin_a, chLatin_n, chLatin_d
+    ,   chLatin_a, chLatin_r, chLatin_d, chDash, chLatin_u, chLatin_r, chLatin_i
+    ,   chDash, chLatin_c, chLatin_o, chLatin_n, chLatin_f, chLatin_o, chLatin_r
+    ,   chLatin_m, chLatin_a, chLatin_n, chLatin_t, chNull
 };
 
 
 //Property
 //Xerces: http://apache.org/xml/properties/schema/external-schemaLocation
-const XMLCh XMLUni::fgSAX2XercesSchemaExternalSchemaLocation[] =
+const XMLCh XMLUni::fgXercesSchemaExternalSchemaLocation[] =
 {
         chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
     ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h
@@ -949,7 +1084,7 @@ const XMLCh XMLUni::fgSAX2XercesSchemaExternalSchemaLocation[] =
 
 //Property
 //Xerces: http://apache.org/xml/properties/schema/external-noNamespaceSchemaLocation
-const XMLCh XMLUni::fgSAX2XercesSchemaExternalNoNameSpaceSchemaLocation[] =
+const XMLCh XMLUni::fgXercesSchemaExternalNoNameSpaceSchemaLocation[] =
 {
         chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
     ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h
@@ -965,3 +1100,260 @@ const XMLCh XMLUni::fgSAX2XercesSchemaExternalNoNameSpaceSchemaLocation[] =
     ,   chLatin_t, chLatin_i, chLatin_o, chLatin_n, chNull
 };
 
+//Property
+//Xerces: http://apache.org/xml/properties/scannerName
+const XMLCh XMLUni::fgXercesScannerName[] =
+{
+        chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
+    ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h
+    ,   chLatin_e, chPeriod, chLatin_o, chLatin_r, chLatin_g, chForwardSlash
+    ,   chLatin_x, chLatin_m, chLatin_l, chForwardSlash, chLatin_p, chLatin_r
+    ,   chLatin_o, chLatin_p, chLatin_e, chLatin_r, chLatin_t, chLatin_i
+    ,   chLatin_e, chLatin_s, chForwardSlash, chLatin_s, chLatin_c, chLatin_a
+    ,   chLatin_n, chLatin_n, chLatin_e, chLatin_r, chLatin_N, chLatin_a
+    ,   chLatin_m, chLatin_e, chNull
+};
+
+//Xerces: http://apache.org/xml/features/dom/user-adopts-DOMDocument
+const XMLCh XMLUni::fgXercesUserAdoptsDOMDocument[] =
+{
+        chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
+    ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h
+    ,   chLatin_e, chPeriod, chLatin_o, chLatin_r, chLatin_g, chForwardSlash
+    ,   chLatin_x, chLatin_m, chLatin_l, chForwardSlash, chLatin_f, chLatin_e
+    ,   chLatin_a, chLatin_t, chLatin_u, chLatin_r, chLatin_e, chLatin_s
+    ,   chForwardSlash, chLatin_d, chLatin_o, chLatin_m, chForwardSlash
+    ,   chLatin_u, chLatin_s, chLatin_e, chLatin_r, chDash
+    ,   chLatin_a, chLatin_d, chLatin_o, chLatin_p, chLatin_t, chLatin_s, chDash
+    ,   chLatin_D, chLatin_O, chLatin_M
+    ,   chLatin_D, chLatin_o, chLatin_c, chLatin_u, chLatin_m, chLatin_e, chLatin_n, chLatin_t, chNull
+};
+
+//Xerces: http://apache.org/xml/features/validation/cache-grammarFromParse
+const XMLCh XMLUni::fgXercesCacheGrammarFromParse[] =
+{
+        chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
+    ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h
+    ,   chLatin_e, chPeriod, chLatin_o, chLatin_r, chLatin_g, chForwardSlash
+    ,   chLatin_x, chLatin_m, chLatin_l, chForwardSlash, chLatin_f, chLatin_e
+    ,   chLatin_a, chLatin_t, chLatin_u, chLatin_r, chLatin_e, chLatin_s
+    ,   chForwardSlash, chLatin_v, chLatin_a, chLatin_l, chLatin_i, chLatin_d
+    ,   chLatin_a, chLatin_t, chLatin_i, chLatin_o, chLatin_n, chForwardSlash
+    ,   chLatin_c, chLatin_a, chLatin_c, chLatin_h, chLatin_e, chDash, chLatin_g
+    ,   chLatin_r, chLatin_a, chLatin_m, chLatin_m, chLatin_a, chLatin_r
+    ,   chLatin_F, chLatin_r, chLatin_o, chLatin_m, chLatin_P, chLatin_a, chLatin_r
+    ,   chLatin_s, chLatin_e, chNull
+};
+
+//Xerces: http://apache.org/xml/features/validation/use-cachedGrammarInParse
+const XMLCh XMLUni::fgXercesUseCachedGrammarInParse[] =
+{
+        chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
+    ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h
+    ,   chLatin_e, chPeriod, chLatin_o, chLatin_r, chLatin_g, chForwardSlash
+    ,   chLatin_x, chLatin_m, chLatin_l, chForwardSlash, chLatin_f, chLatin_e
+    ,   chLatin_a, chLatin_t, chLatin_u, chLatin_r, chLatin_e, chLatin_s
+    ,   chForwardSlash, chLatin_v, chLatin_a, chLatin_l, chLatin_i, chLatin_d
+    ,   chLatin_a, chLatin_t, chLatin_i, chLatin_o, chLatin_n, chForwardSlash
+    ,   chLatin_u, chLatin_s, chLatin_e, chDash, chLatin_c, chLatin_a, chLatin_c
+    ,   chLatin_h, chLatin_e, chLatin_d, chLatin_G, chLatin_r, chLatin_a, chLatin_m
+    ,   chLatin_m, chLatin_a, chLatin_r, chLatin_I, chLatin_n, chLatin_P, chLatin_r
+    ,   chLatin_s, chLatin_e, chNull
+};
+
+
+//Introduced in DOM Level 3
+const XMLCh XMLUni::fgDOMCanonicalForm[] =
+{
+        chLatin_c, chLatin_a, chLatin_n, chLatin_o, chLatin_n, chLatin_i, chLatin_c
+    ,   chLatin_a, chLatin_l, chDash, chLatin_f, chLatin_o, chLatin_r, chLatin_m, chNull
+};
+
+const XMLCh XMLUni::fgDOMCDATASections[] =
+{
+        chLatin_c, chLatin_d, chLatin_a, chLatin_t, chLatin_a, chDash, chLatin_s
+    ,   chLatin_e, chLatin_c, chLatin_t, chLatin_i, chLatin_o, chLatin_n, chLatin_s, chNull
+};
+
+const XMLCh XMLUni::fgDOMComments[] =
+{
+        chLatin_c, chLatin_o, chLatin_m, chLatin_m, chLatin_e, chLatin_n, chLatin_t
+    ,   chLatin_s, chNull
+};
+
+const XMLCh XMLUni::fgDOMCharsetOverridesXMLEncoding[] =
+{
+        chLatin_c, chLatin_h, chLatin_a, chLatin_r, chLatin_s, chLatin_e, chLatin_t
+    ,   chDash, chLatin_o, chLatin_v, chLatin_e, chLatin_r, chLatin_r, chLatin_i
+    ,   chLatin_d, chLatin_e, chLatin_s, chDash, chLatin_x, chLatin_m, chLatin_l
+    ,   chDash, chLatin_e, chLatin_n, chLatin_c, chLatin_o, chLatin_d, chLatin_i
+    ,   chLatin_n, chLatin_g, chNull
+};
+
+const XMLCh XMLUni::fgDOMDatatypeNormalization[] =
+{
+        chLatin_d, chLatin_a, chLatin_t, chLatin_a, chLatin_t, chLatin_y, chLatin_p
+    ,   chLatin_e, chDash, chLatin_n, chLatin_o, chLatin_r, chLatin_m, chLatin_a
+    ,   chLatin_l, chLatin_i, chLatin_z, chLatin_a, chLatin_t, chLatin_i, chLatin_o
+    ,   chLatin_n, chNull
+};
+
+const XMLCh XMLUni::fgDOMEntities[] =
+{
+        chLatin_e, chLatin_n, chLatin_t, chLatin_i, chLatin_t, chLatin_i, chLatin_e
+    ,   chLatin_s, chNull
+};
+
+const XMLCh XMLUni::fgDOMInfoset[] =
+{
+        chLatin_i, chLatin_n, chLatin_f, chLatin_o, chLatin_s, chLatin_e, chLatin_t, chNull
+};
+
+const XMLCh XMLUni::fgDOMNamespaces[] =
+{
+        chLatin_n, chLatin_a, chLatin_m, chLatin_e, chLatin_s, chLatin_p, chLatin_a
+    ,   chLatin_c, chLatin_e, chLatin_s, chNull
+};
+
+const XMLCh XMLUni::fgDOMNamespaceDeclarations[] =
+{
+        chLatin_n, chLatin_a, chLatin_m, chLatin_e, chLatin_s, chLatin_p, chLatin_a
+    ,   chLatin_c, chLatin_e, chDash, chLatin_d, chLatin_e, chLatin_c, chLatin_l
+    ,   chLatin_a, chLatin_r, chLatin_a, chLatin_t, chLatin_i, chLatin_o, chLatin_n
+    ,   chLatin_s, chNull
+};
+
+const XMLCh XMLUni::fgDOMSupportedMediatypesOnly[] =
+{
+        chLatin_s, chLatin_u, chLatin_p, chLatin_p, chLatin_o, chLatin_r, chLatin_t
+    ,   chLatin_e, chLatin_d, chDash, chLatin_m, chLatin_e, chLatin_d, chLatin_i
+    ,   chLatin_a, chLatin_t, chLatin_y, chLatin_p, chLatin_e, chLatin_s, chDash
+    ,   chLatin_o, chLatin_n, chLatin_l, chLatin_y, chNull
+};
+
+const XMLCh XMLUni::fgDOMValidateIfSchema[] =
+{
+        chLatin_v, chLatin_a, chLatin_l, chLatin_i, chLatin_d, chLatin_a, chLatin_t
+    ,   chLatin_e, chDash, chLatin_i, chLatin_f, chDash, chLatin_s, chLatin_c
+    ,   chLatin_h, chLatin_e, chLatin_m, chLatin_a, chNull
+};
+
+const XMLCh XMLUni::fgDOMValidation[] =
+{
+        chLatin_v, chLatin_a, chLatin_l, chLatin_i, chLatin_d, chLatin_a, chLatin_t
+    ,   chLatin_i, chLatin_o, chLatin_n, chNull
+};
+
+const XMLCh XMLUni::fgDOMWhitespaceInElementContent[] =
+{
+        chLatin_w, chLatin_h, chLatin_i, chLatin_t, chLatin_e, chLatin_s, chLatin_p
+    ,   chLatin_a, chLatin_c, chLatin_e, chDash, chLatin_i, chLatin_n, chDash
+    ,   chLatin_e, chLatin_l, chLatin_e, chLatin_m, chLatin_e, chLatin_n, chLatin_t
+    ,   chDash, chLatin_c, chLatin_o, chLatin_n, chLatin_t, chLatin_e, chLatin_n
+    ,   chLatin_t, chNull
+};
+
+//Introduced in DOM Level 3
+//canonical-form
+const XMLCh XMLUni::fgDOMWRTCanonicalForm[] =
+{
+	chLatin_c, chLatin_a, chLatin_n, chLatin_o, chLatin_n, chLatin_i,
+    chLatin_c, chLatin_a, chLatin_l, chDash,    chLatin_f, chLatin_o,
+	chLatin_r, chLatin_m, chNull
+};
+
+//discard-default-content
+const XMLCh XMLUni::fgDOMWRTDiscardDefaultContent[] =
+{
+	chLatin_d, chLatin_i, chLatin_s, chLatin_c, chLatin_a, chLatin_r,
+    chLatin_d, chDash,    chLatin_d, chLatin_e, chLatin_f, chLatin_a,
+	chLatin_u, chLatin_l, chLatin_l, chLatin_t, chDash,    chLatin_c,
+	chLatin_o, chLatin_n, chLatin_t, chLatin_e, chLatin_n, chLatin_t,
+	chNull
+};
+
+//entities
+const XMLCh XMLUni::fgDOMWRTEntities[] =
+{
+	chLatin_e, chLatin_n, chLatin_t, chLatin_i, chLatin_t, chLatin_i,
+	chLatin_e, chLatin_s, chNull
+};
+
+//format-pretty-print
+const XMLCh XMLUni::fgDOMWRTFormatPrettyPrint[] =
+{
+	chLatin_f, chLatin_o, chLatin_r, chLatin_m, chLatin_a, chLatin_t,
+    chDash,    chLatin_p, chLatin_r, chLatin_e, chLatin_t, chLatin_t,
+	chLatin_y, chDash,    chLatin_p, chLatin_r, chLatin_i, chLatin_n,
+	chLatin_t, chNull
+};
+
+//normalize-characters
+const XMLCh XMLUni::fgDOMWRTNormalizeCharacters[] =
+{
+	chLatin_n, chLatin_o, chLatin_r, chLatin_m, chLatin_a, chLatin_l,
+    chLatin_i, chLatin_z, chLatin_e, chDash,    chLatin_c, chLatin_h,
+	chLatin_a, chLatin_r, chLatin_a, chLatin_c, chLatin_t, chLatin_e,
+	chLatin_r, chLatin_s, chNull
+};
+
+//split-cdata-sections
+const XMLCh XMLUni::fgDOMWRTSplitCdataSections[] =
+{
+	chLatin_s, chLatin_p, chLatin_l, chLatin_i, chLatin_t, chDash,
+    chLatin_c, chLatin_d, chLatin_a, chLatin_t, chLatin_a, chDash,
+	chLatin_s, chLatin_e, chLatin_c, chLatin_t, chLatin_i, chLatin_o,
+	chLatin_n, chLatin_s, chNull
+};
+
+//validation
+const XMLCh XMLUni::fgDOMWRTValidation[] =
+{
+	chLatin_v, chLatin_a, chLatin_l, chLatin_i, chLatin_d, chLatin_a,
+	chLatin_t, chLatin_i, chLatin_o, chLatin_n, chNull
+};
+
+//whitespace-in-element-content
+const XMLCh XMLUni::fgDOMWRTWhitespaceInElementContent[] =
+{
+	chLatin_w, chLatin_h, chLatin_i, chLatin_t, chLatin_e, chLatin_s,
+    chLatin_p, chLatin_a, chLatin_c, chLatin_e, chDash,    chLatin_i,
+	chLatin_n, chDash,    chLatin_e, chLatin_l, chLatin_e, chLatin_m,
+	chLatin_e, chLatin_n, chLatin_t, chDash,    chLatin_c, chLatin_o,
+	chLatin_n, chLatin_t, chLatin_e, chLatin_n, chLatin_t, chNull
+};
+
+//Xerces: http://apache.org/xml/features/dom/byte-order-mark
+const XMLCh XMLUni::fgDOMWRTBOM[] =
+{
+    chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash,
+    chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h,
+    chLatin_e, chPeriod, chLatin_o, chLatin_r, chLatin_g, chForwardSlash,
+    chLatin_x, chLatin_m, chLatin_l, chForwardSlash, chLatin_f, chLatin_e,
+    chLatin_a, chLatin_t, chLatin_u, chLatin_r, chLatin_e, chLatin_s,
+    chForwardSlash, chLatin_d, chLatin_o, chLatin_m, chForwardSlash,
+    chLatin_b, chLatin_y, chLatin_t, chLatin_e, chDash,
+    chLatin_o, chLatin_r, chLatin_d, chLatin_e, chLatin_r, chDash,
+    chLatin_m, chLatin_a, chLatin_r, chLatin_k, chNull
+};
+
+// en_US
+const char XMLUni::fgXercescDefaultLocale[] = "en_US";
+
+// DOM Message Domain
+const XMLCh XMLUni::fgXMLDOMMsgDomain[] =
+{
+        chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
+    ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c
+    ,   chLatin_h, chLatin_e, chPeriod, chLatin_o, chLatin_r, chLatin_g
+    ,   chForwardSlash, chLatin_x, chLatin_m, chLatin_l, chForwardSlash
+    ,   chLatin_m, chLatin_e, chLatin_s, chLatin_s, chLatin_a, chLatin_g
+    ,   chLatin_e, chLatin_s, chForwardSlash, chLatin_X, chLatin_M, chLatin_L
+    ,   chLatin_D, chLatin_O, chLatin_M, chLatin_M, chLatin_s, chLatin_g
+    ,   chNull
+};
+
+
+
+XERCES_CPP_NAMESPACE_END
+
