@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: XMLUri.cpp,v 1.3 2002/08/23 20:45:24 tng Exp $
+ * $Id: XMLUri.cpp,v 1.9 2002/12/06 16:54:58 tng Exp $
  */
 
 // ---------------------------------------------------------------------------
@@ -63,12 +63,14 @@
 // ---------------------------------------------------------------------------
 #include <xercesc/util/PlatformUtils.hpp>
 #include <xercesc/util/Janitor.hpp>
-#include <xercesc/util/NumberFormatException.hpp>
+#include <xercesc/util/XMLURL.hpp>
 #include <xercesc/util/XMLUri.hpp>
 #include <xercesc/util/XMLString.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/util/XMLUni.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 // ---------------------------------------------------------------------------
 //  XMLUri: static data
 // ---------------------------------------------------------------------------
@@ -247,6 +249,7 @@ XMLUri::XMLUri(const XMLCh* const uriSpec)
 ,fPath(0)
 ,fQueryString(0)
 ,fFragment(0)
+,fURIText(0)
 {
     try {
         initialize((XMLUri *)0, uriSpec);
@@ -268,6 +271,7 @@ XMLUri::XMLUri(const XMLUri* const      baseURI
 ,fPath(0)
 ,fQueryString(0)
 ,fFragment(0)
+,fURIText(0)
 {
     try {
         initialize(baseURI, uriSpec);
@@ -279,6 +283,33 @@ XMLUri::XMLUri(const XMLUri* const      baseURI
     }
 }
 
+//Copy constructor
+XMLUri::XMLUri(const XMLUri& toCopy)
+{
+    try {
+        initialize(toCopy);
+    }
+    catch (...)
+    {
+        cleanUp();
+        throw;
+    }
+
+}
+
+XMLUri& XMLUri::operator=(const XMLUri& toAssign)
+{
+    try {
+        initialize(toAssign);
+    }
+    catch (...)
+    {
+        cleanUp();
+        throw;
+    }
+    return *this;
+}
+
 XMLUri::~XMLUri()
 {
     cleanUp();
@@ -303,6 +334,8 @@ void XMLUri::cleanUp()
 
     if (getFragment())
         delete[] fFragment;
+
+    delete[] fURIText;
 }
 
 void XMLUri::initialize(const XMLUri& toCopy)
@@ -327,7 +360,7 @@ void XMLUri::initialize(const XMLUri* const baseURI
     // get a trimmed version of uriSpec
     // uriSpec will NO LONGER be used in this function.
     //
-    XMLCh* const trimedUriSpec = XMLString::replicate(uriSpec);
+    XMLCh* trimedUriSpec = XMLString::replicate(uriSpec);
     XMLString::trim(trimedUriSpec);
     ArrayJanitor<XMLCh> janName(trimedUriSpec);
     int trimedUriSpecLen = XMLString::stringLen(trimedUriSpec);
@@ -335,7 +368,7 @@ void XMLUri::initialize(const XMLUri* const baseURI
     if ( !baseURI &&
         (!trimedUriSpec || trimedUriSpecLen == 0))
     {
-        ThrowXML1(NumberFormatException
+        ThrowXML1(MalformedURLException
                , XMLExcepts::XMLNUM_URI_Component_Empty
                , errMsg_PARAMS);
     }
@@ -353,6 +386,7 @@ void XMLUri::initialize(const XMLUri* const baseURI
 	// DOS drive letters ('D:'), so 1-character schemes are not allowed.
     int colonIdx = XMLString::indexOf(trimedUriSpec, chColon);
     int slashIdx = XMLString::indexOf(trimedUriSpec, chForwardSlash);
+
 	if ((colonIdx < 2)                         ||
         (colonIdx > slashIdx && slashIdx != -1) )
     {
@@ -360,7 +394,7 @@ void XMLUri::initialize(const XMLUri* const baseURI
         // A standalone base is a valid URI according to spec
         if ( !baseURI && fragmentIdx != 0 )
         {
-            ThrowXML(NumberFormatException, XMLExcepts::XMLNUM_URI_No_Scheme);
+            ThrowXML(MalformedURLException, XMLExcepts::XMLNUM_URI_No_Scheme);
         }
     }
 	else
@@ -369,6 +403,14 @@ void XMLUri::initialize(const XMLUri* const baseURI
         index = XMLString::stringLen(fScheme)+1;
     }
 
+    // It's an error if we stop here
+    if (index == trimedUriSpecLen)
+    {
+        ThrowXML1(MalformedURLException
+                , XMLExcepts::XMLNUM_URI_Component_Empty
+                , errMsg_PATH);
+	}
+
 	// two slashes means generic URI syntax, so we get the authority
     XMLCh* authUriSpec = new XMLCh[trimedUriSpecLen+1];
     ArrayJanitor<XMLCh> authName(authUriSpec);
@@ -433,7 +475,7 @@ void XMLUri::initialize(const XMLUri* const baseURI
         // string or a fragment (e.g. "?y" or "#s") -
         // see <http://www.ics.uci.edu/~fielding/url/test1.html> which
         // identified this as a bug in the RFC
-        if ((!fPath || XMLString::stringLen(fPath) == 0) &&
+        if ((!fPath || !*fPath) &&
             fScheme == 0 &&
             fHost == 0)
         {
@@ -478,7 +520,7 @@ void XMLUri::initialize(const XMLUri* const baseURI
         }
 
         // check for absolute path - RFC 2396 5.2 #5
-        if (XMLString::stringLen(fPath) > 0 &&
+        if ((fPath && *fPath) &&
             XMLString::startsWith(fPath, SINGLE_SLASH))
         {
             return;
@@ -486,6 +528,7 @@ void XMLUri::initialize(const XMLUri* const baseURI
 
         // if we get to this point, we need to resolve relative path
         // RFC 2396 5.2 #6
+
         XMLCh* basePath = XMLString::replicate(baseURI->getPath());
         ArrayJanitor<XMLCh> basePathName(basePath);
 
@@ -576,6 +619,7 @@ void XMLUri::initialize(const XMLUri* const baseURI
             delete [] fPath;
 
         fPath = XMLString::replicate(path);
+
     }
 }
 
@@ -647,13 +691,13 @@ void XMLUri::initializeAuthority(const XMLCh* const uriSpec)
     ArrayJanitor<XMLCh> portName(portStr);
     int port = -1;
 
-    if ((XMLString::stringLen(host) > 0) &&   // non empty host
+    if ((host && *host) &&   // non empty host
         (index != -1)                    &&   // ":" found
         (start < end)                     )   // ":" is not the last
     {
         XMLString::subString(portStr, &(uriSpec[start]), 0, end-start);
 
-        if (XMLString::stringLen(portStr) > 0)
+        if (portStr && *portStr)
         {
             try
             {
@@ -680,7 +724,7 @@ void XMLUri::initializeScheme(const XMLCh* const uriSpec)
 
     if ( !tmpPtr )
     {
-        ThrowXML(NumberFormatException, XMLExcepts::XMLNUM_URI_No_Scheme);
+        ThrowXML(MalformedURLException, XMLExcepts::XMLNUM_URI_No_Scheme);
     }
 	else
     {
@@ -696,7 +740,7 @@ void XMLUri::initializePath(const XMLCh* const uriSpec)
 {
 	if ( !uriSpec )
     {
-        ThrowXML1(NumberFormatException
+        ThrowXML1(MalformedURLException
                 , XMLExcepts::XMLNUM_URI_Component_Empty
                 , errMsg_PATH);
 	}
@@ -724,7 +768,7 @@ void XMLUri::initializePath(const XMLCh* const uriSpec)
             {
                 XMLString::moveChars(value1, &(uriSpec[index]), 3);
                 value1[3] = chNull;
-                ThrowXML2(NumberFormatException
+                ThrowXML2(MalformedURLException
                         , XMLExcepts::XMLNUM_URI_Component_Invalid_EscapeSequence
                         , errMsg_PATH
                         , value1);
@@ -735,7 +779,7 @@ void XMLUri::initializePath(const XMLCh* const uriSpec)
         {
             value1[0] = testChar;
             value1[1] = chNull;
-            ThrowXML2(NumberFormatException
+            ThrowXML2(MalformedURLException
                     , XMLExcepts::XMLNUM_URI_Component_Invalid_Char
                     , errMsg_PATH
                     , value1);
@@ -773,7 +817,7 @@ void XMLUri::initializePath(const XMLCh* const uriSpec)
                 {
                     XMLString::moveChars(value1, &(uriSpec[index]), 3);
                     value1[3] = chNull;
-                    ThrowXML2(NumberFormatException
+                    ThrowXML2(MalformedURLException
                             , XMLExcepts::XMLNUM_URI_Component_Invalid_EscapeSequence
                             , errMsg_QUERY
                             , value1);
@@ -784,7 +828,7 @@ void XMLUri::initializePath(const XMLCh* const uriSpec)
             {
                 value1[0] = testChar;
                 value1[1] = chNull;
-                ThrowXML2(NumberFormatException
+                ThrowXML2(MalformedURLException
                         , XMLExcepts::XMLNUM_URI_Component_Invalid_Char
                         , errMsg_QUERY
                         , value1);
@@ -818,7 +862,7 @@ void XMLUri::initializePath(const XMLCh* const uriSpec)
                 {
                     XMLString::moveChars(value1, &(uriSpec[index]), 3);
                     value1[3] = chNull;
-                    ThrowXML2(NumberFormatException
+                    ThrowXML2(MalformedURLException
                             , XMLExcepts::XMLNUM_URI_Component_Invalid_EscapeSequence
                             , errMsg_FRAGMENT
                             , value1);
@@ -829,7 +873,7 @@ void XMLUri::initializePath(const XMLCh* const uriSpec)
             {
                 value1[0] = testChar;
                 value1[1] = chNull;
-                ThrowXML2(NumberFormatException
+                ThrowXML2(MalformedURLException
                         , XMLExcepts::XMLNUM_URI_Component_Invalid_Char
                         , errMsg_FRAGMENT
                         , value1);
@@ -870,14 +914,14 @@ void XMLUri::setScheme(const XMLCh* const newScheme)
 {
     if ( !newScheme )
     {
-        ThrowXML1(NumberFormatException
+        ThrowXML1(MalformedURLException
                 , XMLExcepts::XMLNUM_URI_Component_Set_Null
                 , errMsg_SCHEME);
     }
 
     if (!isConformantSchemeName(newScheme))
     {
-        ThrowXML2(NumberFormatException
+        ThrowXML2(MalformedURLException
                 , XMLExcepts::XMLNUM_URI_Component_Not_Conformant
                 , errMsg_SCHEME
                 , newScheme);
@@ -890,7 +934,6 @@ void XMLUri::setScheme(const XMLCh* const newScheme)
 
     fScheme = XMLString::replicate(newScheme);
     XMLString::lowerCase(fScheme);
-
 }
 
 //
@@ -908,7 +951,7 @@ void XMLUri::setUserInfo(const XMLCh* const newUserInfo)
     if ( newUserInfo &&
          !getHost()    )
     {
-        ThrowXML2(NumberFormatException
+        ThrowXML2(MalformedURLException
                 , XMLExcepts::XMLNUM_URI_NullHost
                 , errMsg_USERINFO
                 , newUserInfo);
@@ -928,7 +971,11 @@ void XMLUri::setUserInfo(const XMLCh* const newUserInfo)
         delete [] fUserInfo;
     }
 
-    fUserInfo = XMLString::replicate(newUserInfo);
+    //sometimes we get passed a empty string rather than a null.
+    //Other procedures rely on it being null
+    if(newUserInfo && *newUserInfo) {
+        fUserInfo = XMLString::replicate(newUserInfo);
+    }
 
 }
 
@@ -949,7 +996,7 @@ void XMLUri::setHost(const XMLCh* const newHost)
 
     if (!isWellFormedAddress(newHost))
     {
-        ThrowXML2(NumberFormatException
+        ThrowXML2(MalformedURLException
                 , XMLExcepts::XMLNUM_URI_Component_Not_Conformant
                 , errMsg_HOST
                 , newHost);
@@ -970,7 +1017,7 @@ void XMLUri::setPort(int newPort)
         if (!getHost())
         {
             XMLString::binToText(newPort, value1, BUF_LEN, 10);
-            ThrowXML2(NumberFormatException
+            ThrowXML2(MalformedURLException
                     , XMLExcepts::XMLNUM_URI_NullHost
                     , errMsg_PORT
                     , value1);
@@ -979,7 +1026,7 @@ void XMLUri::setPort(int newPort)
     else if (newPort != -1)
     {
         XMLString::binToText(newPort, value1, BUF_LEN, 10);
-        ThrowXML1(NumberFormatException
+        ThrowXML1(MalformedURLException
                 , XMLExcepts::XMLNUM_URI_PortNo_Invalid
                 , value1);
     }
@@ -1023,21 +1070,21 @@ void XMLUri::setFragment(const XMLCh* const newFragment)
 	}
 	else if (!isGenericURI())
     {
-        ThrowXML2(NumberFormatException
+        ThrowXML2(MalformedURLException
                 , XMLExcepts::XMLNUM_URI_Component_for_GenURI_Only
                 , errMsg_FRAGMENT
                 , newFragment);
 	}
 	else if ( !getPath() )
     {
-        ThrowXML2(NumberFormatException
+        ThrowXML2(MalformedURLException
                , XMLExcepts::XMLNUM_URI_NullPath
                , errMsg_FRAGMENT
                , newFragment);
 	}
 	else if (!isURIString(newFragment))
     {
-        ThrowXML1(NumberFormatException
+        ThrowXML1(MalformedURLException
                 , XMLExcepts::XMLNUM_URI_Component_Invalid_Char
                 , errMsg_FRAGMENT);
 	}
@@ -1066,21 +1113,21 @@ void XMLUri::setQueryString(const XMLCh* const newQueryString)
 	}
 	else if (!isGenericURI())
     {
-        ThrowXML2(NumberFormatException
+        ThrowXML2(MalformedURLException
                 , XMLExcepts::XMLNUM_URI_Component_for_GenURI_Only
                 , errMsg_QUERY
                 , newQueryString);
 	}
 	else if ( !getPath() )
     {
-        ThrowXML2(NumberFormatException
+        ThrowXML2(MalformedURLException
                 , XMLExcepts::XMLNUM_URI_NullPath
                 , errMsg_QUERY
                 , newQueryString);
 	}
 	else if (!isURIString(newQueryString))
     {
-        ThrowXML2(NumberFormatException
+        ThrowXML2(MalformedURLException
                , XMLExcepts::XMLNUM_URI_Component_Invalid_Char
                , errMsg_QUERY
                , newQueryString);
@@ -1158,7 +1205,7 @@ void XMLUri::isConformantUserInfo(const XMLCh* const userInfo)
                 value1[2] = *(tmpStr+2);
                 value1[3] = chNull;
 
-                ThrowXML2(NumberFormatException
+                ThrowXML2(MalformedURLException
                         , XMLExcepts::XMLNUM_URI_Component_Invalid_EscapeSequence
                         , errMsg_USERINFO
                         , value1);
@@ -1166,7 +1213,7 @@ void XMLUri::isConformantUserInfo(const XMLCh* const userInfo)
         }
         else
         {	
-            ThrowXML2(NumberFormatException
+            ThrowXML2(MalformedURLException
                     , XMLExcepts::XMLNUM_URI_Component_Invalid_Char
                     , errMsg_USERINFO
                     , userInfo);
@@ -1236,7 +1283,7 @@ bool XMLUri::isWellFormedAddress(const XMLCh* const addrString)
     XMLCh* tmpAddr = XMLString::replicate(addrString);
     ArrayJanitor<XMLCh>  janName(tmpAddr);
     XMLString::trim(tmpAddr);
-    if ((XMLString::stringLen(tmpAddr) == 0) ||
+    if ((!tmpAddr || !*tmpAddr) ||
         (XMLString::stringLen(tmpAddr) > 255) )
         return false;
 
@@ -1339,3 +1386,97 @@ bool XMLUri::isGenericURI()
 }
 
 
+//
+//  This method will take the broken out parts of the URI and build up the
+//  full text. We don't do this unless someone asks us to, since its often
+//  never required.
+//
+void XMLUri::buildFullText()
+{
+    // Calculate the worst case size of the buffer required
+    unsigned int bufSize = XMLString::stringLen(fScheme) + 1
+                           + XMLString::stringLen(fFragment) + 1
+                           + XMLString::stringLen(fHost) + 2
+                           + XMLString::stringLen(fPath)
+                           + XMLString::stringLen(fQueryString) + 1
+                           + XMLString::stringLen(fUserInfo) + 1
+                           + 32;
+
+    // Clean up the existing buffer and allocate another
+    delete [] fURIText;
+    fURIText = new XMLCh[bufSize];
+    *fURIText = 0;
+
+    XMLCh* outPtr = fURIText;
+    if (fScheme != 0)
+    {
+        XMLString::catString(fURIText, getScheme());
+        outPtr += XMLString::stringLen(fURIText);
+        *outPtr++ = chColon;
+        *outPtr++ = chForwardSlash;
+        *outPtr++ = chForwardSlash;
+    }
+
+    if (fUserInfo)
+    {
+        XMLString::copyString(outPtr, fUserInfo);
+        outPtr += XMLString::stringLen(fUserInfo);
+
+
+        /*REVISIT dont have password field in uri - is this right??
+        if (fPassword)
+        {
+            *outPtr++ = chColon;
+            XMLString::copyString(outPtr, fPassword);
+            outPtr += XMLString::stringLen(fPassword);
+        }
+        */
+        *outPtr++ = chAt;
+    }
+
+    if (fHost)
+    {
+        XMLString::copyString(outPtr, fHost);
+        outPtr += XMLString::stringLen(fHost);
+
+        //
+        //  If the port is -1, then we don't put it in. Else we need
+        //  to because it was explicitly provided.
+        //
+        if (fPort != -1)
+        {
+            *outPtr++ = chColon;
+
+            XMLCh tmpBuf[16];
+            XMLString::binToText(fPort, tmpBuf, 16, 10);
+            XMLString::copyString(outPtr, tmpBuf);
+            outPtr += XMLString::stringLen(tmpBuf);
+        }
+    }
+
+    if (fPath)
+    {
+        XMLString::copyString(outPtr, fPath);
+        outPtr += XMLString::stringLen(fPath);
+    }
+
+    if (fQueryString)
+    {
+        *outPtr++ = chQuestion;
+        XMLString::copyString(outPtr, fQueryString);
+        outPtr += XMLString::stringLen(fQueryString);
+    }
+
+    if (fFragment)
+    {
+        *outPtr++ = chPound;
+        XMLString::copyString(outPtr, fFragment);
+        outPtr += XMLString::stringLen(fFragment);
+    }
+
+    // Cap it off in case the last op was not a string copy
+    *outPtr = 0;
+}
+
+XERCES_CPP_NAMESPACE_END
+
