@@ -56,6 +56,18 @@
 
 /*
  * $Log: XercesDefs.hpp,v $
+ * Revision 1.11  2002/12/31 19:31:07  tng
+ * [Bug 15590] BeOSDefs.hpp has wrong case in CVS.
+ *
+ * Revision 1.10  2002/12/02 20:40:49  tng
+ * [Bug 12490] Patches required to build Xerces-C++ on BeOS R5.  Patch from Andrew Bachmann.
+ *
+ * Revision 1.9  2002/11/05 21:44:21  tng
+ * Do not code using namespace in a global header.
+ *
+ * Revision 1.8  2002/11/04 14:40:12  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.7  2002/07/12 16:48:49  jberry
  * Remove reliance on XML_MACOSX. XML_MACOS is used solely. Where qualification
  * by compiler is required, look for the compiler explicitly such as with
@@ -247,6 +259,10 @@
 #include    <xercesc/util/Platforms/Tandem/TandemDefs.hpp>
 #endif
 
+#if defined(XML_BEOS)
+#include    <xercesc/util/Platforms/BeOS/BeOSDef.hpp>
+#endif
+
 #if defined(XML_LINUX)
 #include    <xercesc/util/Platforms/Linux/LinuxDefs.hpp>
 #endif
@@ -400,6 +416,23 @@ typedef XMLUInt32           UCS4Ch;
 #endif
 
 
+// ---------------------------------------------------------------------------
+// Define namespace symbols if the compiler supports it.
+// ---------------------------------------------------------------------------
+#if defined(XERCES_HAS_CPP_NAMESPACE)
+    #define XERCES_CPP_NAMESPACE_BEGIN namespace XERCES_CPP_NAMESPACE {
+    #define XERCES_CPP_NAMESPACE_END  }
+    #define XERCES_CPP_NAMESPACE_USE using namespace XERCES_CPP_NAMESPACE;
+    #define XERCES_CPP_NAMESPACE_QUALIFIER XERCES_CPP_NAMESPACE::
+
+    namespace XERCES_CPP_NAMESPACE { }
+    namespace xercesc = XERCES_CPP_NAMESPACE;
+#else
+    #define XERCES_CPP_NAMESPACE_BEGIN
+    #define XERCES_CPP_NAMESPACE_END
+    #define XERCES_CPP_NAMESPACE_USE
+    #define XERCES_CPP_NAMESPACE_QUALIFIER
+#endif
 
 // ---------------------------------------------------------------------------
 //  Set up the import/export keyword  for our core projects. The
