@@ -1,7 +1,7 @@
 /*
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 2001 The Apache Software Foundation.  All rights
+ * Copyright (c) 2001-2002 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -56,6 +56,21 @@
 
 /*
  * $Log: RangeToken.cpp,v $
+ * Revision 1.6  2002/11/04 15:17:00  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.5  2002/10/15 18:15:02  knoaman
+ * [Bug 13490]: - new[]/delete mismatch in RangeToken.cpp
+ *
+ * Revision 1.4  2002/05/27 11:46:53  tng
+ * Fix compilation error.  The definition should match declaration.
+ *
+ * Revision 1.3  2002/05/24 16:42:20  knoaman
+ * Performance fixes: eliminate mulitple calls to addRange and sort.
+ *
+ * Revision 1.2  2002/03/18 19:29:53  knoaman
+ * Change constant names to eliminate possible conflict with user defined ones.
+ *
  * Revision 1.1.1.1  2002/02/01 22:22:30  peiyongz
  * sane_include
  *
@@ -92,6 +107,8 @@
 #include <xercesc/util/regx/TokenFactory.hpp>
 #include <xercesc/util/XMLExceptMsgs.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 // ---------------------------------------------------------------------------
 //  Static member data initialization
 // ---------------------------------------------------------------------------
@@ -106,7 +123,7 @@ RangeToken::RangeToken(const unsigned short tokType) : Token(tokType)
     , fCompacted(false)
     , fNonMapIndex(0)
     , fElemCount(0)
-    , fMaxCount(0)
+    , fMaxCount(INITIALSIZE)
     , fMap(0)
     , fRanges(0)
     , fCaseIToken(0)
@@ -116,7 +133,7 @@ RangeToken::RangeToken(const unsigned short tokType) : Token(tokType)
 
 RangeToken::~RangeToken() {
 
-    delete fMap;
+    delete [] fMap;
     delete[] fRanges;
 }
 
@@ -131,7 +148,7 @@ RangeToken* RangeToken::getCaseInsensitiveToken(TokenFactory* const tokFactory)
     // For now we will return a copy of ourselves.
     if (fCaseIToken == 0 && tokFactory) {
 
-        bool isNRange = (getTokenType() == NRANGE) ? true : false;
+        bool isNRange = (getTokenType() == T_NRANGE) ? true : false;
         RangeToken* lwrToken = tokFactory->createRange(isNRange);
 
         lwrToken->mergeRanges(this);
@@ -141,6 +158,27 @@ RangeToken* RangeToken::getCaseInsensitiveToken(TokenFactory* const tokFactory)
     return fCaseIToken;
 }
 
+// ---------------------------------------------------------------------------
+//  RangeToken: Setter methods
+// ---------------------------------------------------------------------------
+void RangeToken::setRangeValues(XMLInt32* const rangeValues, const unsigned int count)
+{
+    if (fRanges) {
+
+        if (fMap) {
+            delete [] fMap;
+            fMap = 0;
+        }
+
+        fElemCount = 0;
+        delete [] fRanges;
+        fRanges = 0;
+    }
+
+    fElemCount = fMaxCount = count;
+    fRanges = rangeValues;
+}
+
 // ---------------------------------------------------------------------------
 //  RangeToken: Range manipulation methods
 // ---------------------------------------------------------------------------
@@ -163,7 +201,6 @@ void RangeToken::addRange(const XMLInt32 start, const XMLInt32 end) {
 
     if (fRanges == 0) {
 
-        fMaxCount = INITIALSIZE;
         fRanges = new XMLInt32[fMaxCount];
         fRanges[0] = val1;
         fRanges[1] = val2;
@@ -265,13 +302,7 @@ void RangeToken::compactRanges() {
         base += 2;
     }
 
-    if (base != fElemCount) {
-
-        while (fElemCount > base) {
-            fRanges[fElemCount--] = 0;
-        }
-    }
-
+    fElemCount = base;
     fCompacted = true;
 }
 
@@ -348,7 +379,7 @@ void RangeToken::subtractRanges(RangeToken* const tok) {
     if (fRanges == 0 || tok->fRanges == 0)
         return;
 
-    if (tok->getTokenType() == NRANGE) {
+    if (tok->getTokenType() == T_NRANGE) {
 
         intersectRanges(tok);
         return;
@@ -518,7 +549,7 @@ void RangeToken::intersectRanges(RangeToken* const tok) {
 Token* RangeToken::complementRanges(RangeToken* const tok,
                                     TokenFactory* const tokFactory) {
 
-    if (tok->getTokenType() != RANGE && tok->getTokenType() != NRANGE)
+    if (tok->getTokenType() != T_RANGE && tok->getTokenType() != T_NRANGE)
         ThrowXML(IllegalArgumentException, XMLExcepts::Regex_ComplementRangesInvalidArg);
 
     tok->sortRanges();
@@ -555,7 +586,7 @@ bool RangeToken::match(const XMLInt32 ch) {
 
     bool ret;
 
-    if (getTokenType() == RANGE) {
+    if (getTokenType() == T_RANGE) {
 
         if (ch < MAPSIZE)
             return ((fMap[ch/32] & (1<<(ch&0x1f))) != 0);
@@ -640,6 +671,9 @@ void RangeToken::createMap() {
         }
     }
 }
+
+XERCES_CPP_NAMESPACE_END
+
 /**
   * End of file RangeToken.cpp
   */
