@@ -56,6 +56,18 @@
 
 /*
  * $Log: RegularExpression.cpp,v $
+ * Revision 1.5  2002/12/18 13:01:02  gareth
+ * New functionality - tokenize and replace. Fixed REVISIT for case insensitive match. Patch by Jennifer Schachter.
+ *
+ * Revision 1.4  2002/11/04 15:17:00  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.3  2002/10/15 18:56:02  knoaman
+ * [Bug 13604] while loop never terminates.
+ *
+ * Revision 1.2  2002/03/18 19:29:53  knoaman
+ * Change constant names to eliminate possible conflict with user defined ones.
+ *
  * Revision 1.1.1.1  2002/02/01 22:22:30  peiyongz
  * sane_include
  *
@@ -95,7 +107,6 @@
 //  Includes
 // ---------------------------------------------------------------------------
 #include <xercesc/util/regx/RegularExpression.hpp>
-#include <xercesc/util/XMLString.hpp>
 #include <xercesc/util/PlatformUtils.hpp>
 #include <xercesc/util/regx/RegxUtil.hpp>
 #include <xercesc/util/regx/Match.hpp>
@@ -105,7 +116,9 @@
 #include <xercesc/util/regx/ParserForXMLSchema.hpp>
 #include <xercesc/util/Janitor.hpp>
 #include <xercesc/util/ParseException.hpp>
+#include <xercesc/framework/XMLBuffer.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  Static member data initialization
@@ -519,8 +532,8 @@ bool RegularExpression::matches(const XMLCh* const expression, const int start,
 	/*
 	 *	Check whether the expression start with ".*"
 	 */
-	if (fOperations != 0 && fOperations->getOpType() == Op::CLOSURE
-        && fOperations->getChild()->getOpType() == Op::DOT) {
+	if (fOperations != 0 && fOperations->getOpType() == Op::O_CLOSURE
+        && fOperations->getChild()->getOpType() == Op::O_DOT) {
 
 		if (isSet(fOptions, SINGLE_LINE)) {
 			matchStart = context->fStart;
@@ -609,6 +622,253 @@ bool RegularExpression::matches(const XMLCh* const expression, const int start,
 	return false;
 }
 
+// ---------------------------------------------------------------------------
+//  RegularExpression: Tokenize methods
+// ---------------------------------------------------------------------------
+RefArrayVectorOf<XMLCh>* RegularExpression::tokenize(const char* const expression) {
+
+  XMLCh* tmpBuf = XMLString::transcode(expression);
+  ArrayJanitor<XMLCh> janBuf(tmpBuf);
+  return tokenize(tmpBuf, 0, XMLString::stringLen(tmpBuf));
+}
+
+RefArrayVectorOf<XMLCh>* RegularExpression::tokenize(const char* const expression,
+								const int start, const int end) {
+
+  XMLCh* tmpBuf = XMLString::transcode(expression);
+  ArrayJanitor<XMLCh> janBuf(tmpBuf);
+  return tokenize(tmpBuf, start, end);
+}
+
+
+
+// ---------------------------------------------------------------------------
+//  RegularExpression: Tokenize methods - Wide char version
+// ---------------------------------------------------------------------------
+RefArrayVectorOf<XMLCh>* RegularExpression::tokenize(const XMLCh* const expression) {
+  return tokenize(expression, 0, XMLString::stringLen(expression), 0);
+}
+
+RefArrayVectorOf<XMLCh>* RegularExpression::tokenize(const XMLCh* const expression,
+								                                     const int start, const int end)
+{
+  return tokenize(expression, start, end, 0);
+}
+
+RefArrayVectorOf<XMLCh>* RegularExpression::tokenize(const XMLCh* const expression, 
+                                                     const int start, const int end,
+                                                     RefVectorOf<Match> *subEx){
+  
+  if (fOperations == 0)
+	  prepare();
+
+  Match* pMatch = 0;
+
+  if (subEx){
+    pMatch = new Match();
+  }
+
+  RefArrayVectorOf<XMLCh>* tokenStack = new RefArrayVectorOf<XMLCh>(16, true);
+
+  Context* context = 0;
+  Context* tmpContext = 0;
+
+  int		 strLength = XMLString::stringLen(expression);
+ 
+  {
+ 	   XMLMutexLock lockInit(&fMutex);
+
+ 	   if (fContext == 0)
+ 	     fContext = new Context();
+
+ 	   if (fContext->fInUse) {
+ 	     context = new Context();
+ 	     tmpContext = context;
+ 	   }
+ 	   else {
+ 	     context = fContext;
+ 	   }
+
+ 	   context->reset(expression, start, end, fNoClosures);
+  }
+
+  Janitor<Context> janContext(tmpContext);
+
+  bool adoptMatch = false;
+  Match* lMatch = pMatch;
+
+  if (lMatch != 0) {
+ 	  lMatch->setNoGroups(fNoGroups);
+  }
+  else if (fHasBackReferences) {
+
+ 	  lMatch = new Match();
+ 	  lMatch->setNoGroups(fNoGroups);
+ 	  adoptMatch = true;
+  }
+
+  if (context->fAdoptMatch)
+ 	  delete context->fMatch;
+  
+  context->fMatch = lMatch;
+  context->fAdoptMatch = adoptMatch;
+
+  int tokStart = start;
+  int matchStart = start;
+
+  for (; matchStart <= end; matchStart++) { 
+  
+ 	  int matchEnd = match(context, fOperations, matchStart, 1);
+  
+ 	  if (matchEnd != -1) {
+
+ 	    if (context->fMatch != 0) {
+ 	      context->fMatch->setStartPos(0, context->fStart);
+ 	      context->fMatch->setEndPos(0, matchEnd);
+ 	    }
+
+      if (subEx){
+        subEx->addElement(lMatch);
+        lMatch = new Match(*(context->fMatch));
+        context->fMatch = lMatch;
+      }
+  
+ 	    context->fInUse = false;
+
+      XMLCh* token;
+      if (tokStart == matchStart){
+  
+        if (tokStart == strLength){
+          tokStart--;
+          break;  
+        }
+
+        token = new XMLCh[1];
+        token[0] = chNull;
+
+        // When you tokenize using zero string, will return each
+        // token in the string. Since the zero string will also 
+        // match the start/end characters, resulting in empty 
+        // tokens, we ignore them and do not add them to the stack. 
+        if (!XMLString::equals(fPattern, &chNull)) 
+          tokenStack->addElement(token); 
+  
+      } else {
+        token = new XMLCh[matchStart + 1 - tokStart];
+        XMLString::subString(token, expression, tokStart, matchStart);
+        tokenStack->addElement(token);
+      } 
+
+      tokStart = matchEnd;
+
+      //decrement matchStart as will increment it at the top of the loop
+      if (matchStart < matchEnd - 1) 
+        matchStart = matchEnd - 1; 	    
+    }
+  }
+ 
+  XMLCh* token;
+ 
+  if (matchStart == tokStart + 1){
+    token = new XMLCh[1];
+    token[0] = chNull;
+  
+  } else {
+    token = new XMLCh[strLength + 1 - tokStart];
+    XMLString::subString(token, expression, tokStart, strLength);
+  }  
+
+  if (!XMLString::equals(fPattern, &chNull)) 
+    tokenStack->addElement(token);
+
+  return tokenStack;
+
+}
+
+
+// -----------------------------------------------------------------------
+//  RegularExpression: Replace methods
+// -----------------------------------------------------------------------
+XMLCh* RegularExpression::replace(const char* const matchString, 
+                                  const char* const replaceString){
+
+	XMLCh* tmpBuf = XMLString::transcode(matchString);
+    ArrayJanitor<XMLCh> janBuf(tmpBuf);
+	XMLCh* tmpBuf2 = XMLString::transcode(replaceString);
+    ArrayJanitor<XMLCh> janBuf2(tmpBuf2);
+
+	return replace(tmpBuf, tmpBuf2, 0, XMLString::stringLen(tmpBuf));
+}
+
+XMLCh* RegularExpression::replace(const char* const matchString, 
+                                  const char* const replaceString,
+                                  const int start, const int end){
+
+ 	XMLCh* tmpBuf = XMLString::transcode(matchString);
+    ArrayJanitor<XMLCh> janBuf(tmpBuf);
+ 	XMLCh* tmpBuf2 = XMLString::transcode(replaceString);
+    ArrayJanitor<XMLCh> janBuf2(tmpBuf2);
+  
+  return replace(tmpBuf, tmpBuf2, start, end);
+}
+
+
+// ---------------------------------------------------------------------------
+//  RegularExpression: Replace methods - Wide char version
+// ---------------------------------------------------------------------------
+XMLCh* RegularExpression::replace(const XMLCh* const matchString, 
+                                  const XMLCh* const replaceString){
+
+  return replace(matchString, replaceString, 0, 
+                 XMLString::stringLen(matchString));
+}
+
+XMLCh* RegularExpression::replace(const XMLCh* const matchString,  
+                                  const XMLCh* const replaceString,
+                                  const int start, const int end)
+{
+  //check if matches zero length string - throw error if so
+  if (matches(XMLUni::fgZeroLenString)){
+		ThrowXML(RuntimeException, XMLExcepts::Regex_RepPatMatchesZeroString);
+  }
+      
+  RefVectorOf<Match> *subEx = new RefVectorOf<Match>(10, true);
+	  Janitor<RefVectorOf<Match> > janSubEx(subEx);
+
+  //Call to tokenize with Match vector so that we keep track of the locations
+  //of the subExpression within each of the matches
+  RefArrayVectorOf<XMLCh>* tokenStack = tokenize(matchString, start, end, subEx);
+	  Janitor<RefArrayVectorOf<XMLCh> > janTokStack(tokenStack);
+    
+  XMLBuffer result;
+  
+  int numSubEx = 0;
+  
+  if (subEx && subEx->size() > 0)
+    numSubEx = subEx->elementAt(0)->getNoGroups() - 1;
+  
+  int tokStackSize = tokenStack->size();
+  const XMLCh* curRepString = XMLString::replicate(replaceString);
+    
+  for (int i = 0; i < tokStackSize; i++){
+      
+    result.append(tokenStack->elementAt(i));
+  
+    if (i != tokStackSize - 1) {
+       
+      //if there are subExpressions, then determine the string we want to 
+      //substitute in.
+      if (numSubEx != 0)
+        curRepString = subInExp(replaceString, matchString, subEx->elementAt(i)); 
+    
+      result.append(curRepString);
+    }
+  }  
+    
+  return XMLString::replicate(result.getRawBuffer()); 
+    
+}
+
 // ---------------------------------------------------------------------------
 //  RegularExpression: Helpers methods
 // ---------------------------------------------------------------------------
@@ -655,6 +915,7 @@ int RegularExpression::getOptionValue(const XMLCh ch) {
 	return ret;
 }
 
+
 int RegularExpression::match(Context* const context, const Op* const operations
 							 , int offset, const short direction)
 {
@@ -670,41 +931,41 @@ int RegularExpression::match(Context* const context, const Op* const operations
 			return -1;
 
 		switch(tmpOp->getOpType()) {
-		case Op::CHAR:
+		case Op::O_CHAR:
 			if (!matchChar(context, tmpOp->getData(), offset, direction,
 						   ignoreCase))
 				return -1;
 			tmpOp = tmpOp->getNextOp();
 			break;
-		case Op::DOT:
+		case Op::O_DOT:
 			if (!matchDot(context, offset, direction))
 				return -1;
 			tmpOp = tmpOp->getNextOp();
 			break;
-		case Op::RANGE:
-		case Op::NRANGE:
+		case Op::O_RANGE:
+		case Op::O_NRANGE:
 			if (!matchRange(context, tmpOp, offset, direction, ignoreCase))
 				return -1;
 			tmpOp = tmpOp->getNextOp();
 			break;
-		case Op::ANCHOR:
+		case Op::O_ANCHOR:
 			if (!matchAnchor(context, tmpOp->getData(), offset))
 				return -1;
-			tmpOp->getNextOp();
+			tmpOp = tmpOp->getNextOp();
 			break;
-		case Op::BACKREFERENCE:
+		case Op::O_BACKREFERENCE:
 			if (!matchBackReference(context, tmpOp->getData(), offset,
 									direction, ignoreCase))
 				return -1;
-			tmpOp->getNextOp();
+			tmpOp = tmpOp->getNextOp();
 			break;
-		case Op::STRING:
+		case Op::O_STRING:
 			if (!matchString(context, tmpOp->getLiteral(), offset, direction,
 							 ignoreCase))
 				return -1;
 			tmpOp = tmpOp->getNextOp();
 			break;
-		case Op::CLOSURE:
+		case Op::O_CLOSURE:
 			{
 				XMLInt32 id = tmpOp->getData();
 				if (id >= 0) {
@@ -731,7 +992,7 @@ int RegularExpression::match(Context* const context, const Op* const operations
 				tmpOp = tmpOp->getNextOp();
 			}
 			break;
-		case Op::QUESTION:
+		case Op::O_QUESTION:
 			{
 				int ret = match(context, tmpOp->getChild(), offset, direction);
 				if (ret >= 0)
@@ -739,8 +1000,8 @@ int RegularExpression::match(Context* const context, const Op* const operations
 				tmpOp = tmpOp->getNextOp();
 			}
 			break;
-		case Op::NONGREEDYCLOSURE:
-		case Op::NONGREEDYQUESTION:
+		case Op::O_NONGREEDYCLOSURE:
+		case Op::O_NONGREEDYQUESTION:
 			{
 				int ret = match(context,tmpOp->getNextOp(),offset,direction);
 				if (ret >= 0)
@@ -748,39 +1009,39 @@ int RegularExpression::match(Context* const context, const Op* const operations
 				tmpOp = tmpOp->getChild();
 			}
 			break;
-		case Op::UNION:
+		case Op::O_UNION:
 			{
 				return matchUnion(context, tmpOp, offset, direction);
 			}
-		case Op::CAPTURE:
+		case Op::O_CAPTURE:
 			if (context->fMatch != 0 && tmpOp->getData() != 0)
 				return matchCapture(context, tmpOp, offset, direction);
 			tmpOp = tmpOp->getNextOp();
 			break;
-		case Op::LOOKAHEAD:
+		case Op::O_LOOKAHEAD:
 			if (0 > match(context, tmpOp->getChild(), offset, 1))
 				return -1;
 			tmpOp = tmpOp->getNextOp();
 			break;
-		case Op::NEGATIVELOOKAHEAD:
+		case Op::O_NEGATIVELOOKAHEAD:
 			if (0 <= match(context, tmpOp->getChild(), offset, 1))
 				return -1;
 			tmpOp = tmpOp->getNextOp();
 			break;
-		case Op::LOOKBEHIND:
+		case Op::O_LOOKBEHIND:
 			if (0 > match(context, tmpOp->getChild(), offset, -1))
 				return - 1;
 			tmpOp = tmpOp->getNextOp();
 			break;
-		case Op::NEGATIVELOOKBEHIND:
+		case Op::O_NEGATIVELOOKBEHIND:
 			if (0 <= match(context, tmpOp->getChild(), offset, -1))
 				return -1;
 			tmpOp = tmpOp->getNextOp();
 			break;
-		case Op::INDEPENDENT:
-        case Op::MODIFIER:
+		case Op::O_INDEPENDENT:
+        case Op::O_MODIFIER:
 			{
-				int ret = (tmpOp->getOpType() == Op::INDEPENDENT)
+				int ret = (tmpOp->getOpType() == Op::O_INDEPENDENT)
 					   ? match(context, tmpOp->getChild(), offset, direction)
                        : matchModifier(context, tmpOp, offset, direction);
                 if (ret < 0)
@@ -789,7 +1050,7 @@ int RegularExpression::match(Context* const context, const Op* const operations
 				tmpOp = tmpOp->getNextOp();
 			}
 			break;
-		case Op::CONDITION:
+		case Op::O_CONDITION:
 			if (tmpOp->getRefNo() >= fNoGroups)
 				return -1;
 			if (matchCondition(context, tmpOp, offset, direction))
@@ -805,7 +1066,6 @@ int RegularExpression::match(Context* const context, const Op* const operations
 	
 	return offset;
 }
-
 bool RegularExpression::matchChar(Context* const context,
 								  const XMLInt32 ch, int& offset,
 								  const short direction, const bool ignoreCase)
@@ -822,7 +1082,6 @@ bool RegularExpression::matchChar(Context* const context,
 
 	bool match = ignoreCase ? matchIgnoreCase(ch, strCh)
 		                    : (ch == strCh);
-	
 	if (!match)
 		return false;
 
@@ -1126,42 +1385,42 @@ Op* RegularExpression::compile(const Token* const token, Op* const next,
 	const unsigned short tokenType = token->getTokenType();
 
 	switch(tokenType) {
-	case Token::DOT:
-	case Token::CHAR:
-	case Token::ANCHOR:
-	case Token::RANGE:
-	case Token::NRANGE:
-	case Token::STRING:
-	case Token::BACKREFERENCE:
-	case Token::EMPTY:
+	case Token::T_DOT:
+	case Token::T_CHAR:
+	case Token::T_ANCHOR:
+	case Token::T_RANGE:
+	case Token::T_NRANGE:
+	case Token::T_STRING:
+	case Token::T_BACKREFERENCE:
+	case Token::T_EMPTY:
 		ret = compileSingle(token, next, tokenType);
 		break;
-	case Token::CONCAT:
+	case Token::T_CONCAT:
 		ret = compileConcat(token, next, reverse);
 		break;
-	case Token::UNION:
+	case Token::T_UNION:
 		ret = compileUnion(token, next, reverse);
 		break;
-	case Token::CLOSURE:
-	case Token::NONGREEDYCLOSURE:
+	case Token::T_CLOSURE:
+	case Token::T_NONGREEDYCLOSURE:
 		ret = compileClosure(token, next, reverse, tokenType);
 		break;
-	case Token::PAREN:
+	case Token::T_PAREN:
 		ret = compileParenthesis(token, next, reverse);
 		break;
-	case Token::LOOKAHEAD:
-	case Token::NEGATIVELOOKAHEAD:
+	case Token::T_LOOKAHEAD:
+	case Token::T_NEGATIVELOOKAHEAD:
 		ret = compileLook(token, next, false, tokenType);
 		break;
-	case Token::LOOKBEHIND:
-	case Token::NEGATIVELOOKBEHIND:
+	case Token::T_LOOKBEHIND:
+	case Token::T_NEGATIVELOOKBEHIND:
 		ret = compileLook(token, next, true, tokenType);
 		break;
-	case Token::INDEPENDENT:
-	case Token::MODIFIERGROUP:
+	case Token::T_INDEPENDENT:
+	case Token::T_MODIFIERGROUP:
 		ret = compileLook(token, next, reverse, tokenType);
 		break;
-	case Token::CONDITION:
+	case Token::T_CONDITION:
 		ret = compileCondition(token, next, reverse);
 		break;
 	default:
@@ -1172,6 +1431,87 @@ Op* RegularExpression::compile(const Token* const token, Op* const next,
 	return ret;
 }
 
+/*
+ * Helper for Replace. This method prepares the replacement string by substituting
+ * in actual values for parenthesized sub expressions. 
+ *
+ * An error will be thrown if:
+ *  1) repString references an undefined subExpression
+ *  2) there is an unescaped chDollar which is not followed by a digit
+ *
+ */
+const XMLCh* RegularExpression::subInExp(const XMLCh* const repString, 
+                                         const XMLCh* const origString, 
+                                         const Match* subEx){
+
+  int numSubExp = subEx->getNoGroups() - 1;
+
+  if (numSubExp == 0)
+    return XMLString::replicate(repString);
+  
+  bool notEscaped = true;                 
+  
+  XMLBuffer newString;                   
+  
+  XMLCh *indexStr = new XMLCh[2];                   //holds the string rep of a 
+        ArrayJanitor<XMLCh> indexJan(indexStr);     //digit
+
+  indexStr[1] = chNull;
+  int index = -1;
+
+  for (const XMLCh* ptr = repString; *ptr != chNull; ptr++){
+
+    if ((*ptr == chDollarSign) && notEscaped) {
+      
+      ptr++;
+      
+      //check that after the $ is a digit 
+      if (!XMLString::isDigit(*ptr)){
+       
+        //invalid replace string - $ must be followed by a digit
+				ThrowXML(RuntimeException, XMLExcepts::Regex_InvalidRepPattern);
+      }
+        
+      indexStr[0] = *ptr;                     //get the digit 
+      index = XMLString::parseInt(indexStr);  //convert it to an int
+
+      //now check that the index is legal
+      if (index > numSubExp){
+				ThrowXML(RuntimeException, XMLExcepts::Regex_InvalidRepPattern);
+      }
+        
+      int start = subEx->getStartPos(index);
+      int end = subEx->getEndPos(index);
+
+      //now copy the substring into the new string
+      for (int i=start; i<end; i++){
+        newString.append(origString[i]);
+      }
+          
+    } else {
+ 
+      //if you have a slash and then a character that's not a $ or /, 
+      //then it's an invalid replace string  
+      if (!notEscaped && (*ptr != chDollarSign && *ptr != chBackSlash)){
+				ThrowXML(RuntimeException, XMLExcepts::Regex_InvalidRepPattern);
+      }
+      
+      if (*ptr == chBackSlash){
+        notEscaped = false;
+        continue;
+        
+      }else   
+        notEscaped = true;  
+
+      newString.append(*ptr);
+    }
+  }
+
+  return XMLString::replicate(newString.getRawBuffer());
+       
+}
+
+
 /*
  * Prepares for matching. This method is called just before starting matching
  */
@@ -1198,12 +1538,12 @@ void RegularExpression::prepare() {
 	}
 
 	if (fOperations != 0 && fOperations->getNextOp() == 0 &&
-		(fOperations->getOpType() == Op::STRING ||
-		 fOperations->getOpType() == Op::CHAR) )			 {
+		(fOperations->getOpType() == Op::O_STRING ||
+		 fOperations->getOpType() == Op::O_CHAR) )			 {
 
 		fFixedStringOnly = true;
 
-		if (fOperations->getOpType() == Op::STRING) {
+		if (fOperations->getOpType() == Op::O_STRING) {
 			delete [] fFixedString;
 			fFixedString = XMLString::replicate(fOperations->getLiteral());
 		}
@@ -1303,6 +1643,9 @@ unsigned short RegularExpression::getCharType(const XMLCh ch) {
     return WT_OTHER;
 }
 
+
+XERCES_CPP_NAMESPACE_END
+
 /**
   *	End of file RegularExpression.cpp
   */
