@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: Token.hpp,v 1.3 2002/08/22 15:41:18 tng Exp $
+ * $Id: Token.hpp,v 1.4 2002/11/04 15:17:00 tng Exp $
  */
 
 #if !defined(TOKEN_HPP)
@@ -67,6 +67,8 @@
 #include <xercesc/util/XercesDefs.hpp>
 #include <xercesc/util/RuntimeException.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 // ---------------------------------------------------------------------------
 //  Forward Declaration
 // ---------------------------------------------------------------------------
@@ -288,6 +290,8 @@ inline void Token::intersectRanges(RangeToken* const) {
     ThrowXML(RuntimeException, XMLExcepts::Regex_NotSupported);
 }
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
 
 /**
