@@ -56,6 +56,12 @@
 
 /*
  * $Log: TokenFactory.cpp,v $
+ * Revision 1.4  2002/12/24 17:59:07  tng
+ * Build with ICU 2.4
+ *
+ * Revision 1.3  2002/11/04 15:17:00  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.2  2002/03/18 19:29:53  knoaman
  * Change constant names to eliminate possible conflict with user defined ones.
  *
@@ -103,6 +109,7 @@
 #include <xercesc/util/regx/RangeTokenMap.hpp>
 #include <xercesc/util/regx/RegxDefs.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  TokenFactory: Constructors and Destructor
@@ -463,7 +470,7 @@ void TokenFactory::initializeRegistry() {
 
 /*
 #if defined (XML_USE_ICU_TRANSCODER)
-   #include <unicode/unicode.h>
+   #include <unicode/uchar.h>
 #endif
 
 #include <stdio.h>
@@ -481,22 +488,22 @@ void TokenFactory::printUnicode() {
     {
         fprintf(outFl
                 , "    , 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X\n"
-                , (unsigned int)Unicode::getType(index)
-                , (unsigned int)Unicode::getType(index+1)
-                , (unsigned int)Unicode::getType(index+2)
-                , (unsigned int)Unicode::getType(index+3)
-                , (unsigned int)Unicode::getType(index+4)
-                , (unsigned int)Unicode::getType(index+5)
-                , (unsigned int)Unicode::getType(index+6)
-                , (unsigned int)Unicode::getType(index+7)
-				, (unsigned int)Unicode::getType(index+8)
-                , (unsigned int)Unicode::getType(index+9)
-                , (unsigned int)Unicode::getType(index+10)
-                , (unsigned int)Unicode::getType(index+11)
-				, (unsigned int)Unicode::getType(index+12)
-                , (unsigned int)Unicode::getType(index+13)
-                , (unsigned int)Unicode::getType(index+14)
-                , (unsigned int)Unicode::getType(index+15));
+                , (unsigned int)u_charType(index)
+                , (unsigned int)u_charType(index+1)
+                , (unsigned int)u_charType(index+2)
+                , (unsigned int)u_charType(index+3)
+                , (unsigned int)u_charType(index+4)
+                , (unsigned int)u_charType(index+5)
+                , (unsigned int)u_charType(index+6)
+                , (unsigned int)u_charType(index+7)
+				, (unsigned int)u_charType(index+8)
+                , (unsigned int)u_charType(index+9)
+                , (unsigned int)u_charType(index+10)
+                , (unsigned int)u_charType(index+11)
+				, (unsigned int)u_charType(index+12)
+                , (unsigned int)u_charType(index+13)
+                , (unsigned int)u_charType(index+14)
+                , (unsigned int)u_charType(index+15));
     }
     fprintf(outFl, "};\n");
 
@@ -505,6 +512,8 @@ void TokenFactory::printUnicode() {
 }
 */
 
+XERCES_CPP_NAMESPACE_END
+
 /**
   * End of file TokenFactory.cpp
   */
