@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: DTDValidator.cpp,v 1.1.1.1 2002/02/01 22:22:45 peiyongz Exp $
+ * $Id: DTDValidator.cpp,v 1.11 2002/12/20 22:10:47 tng Exp $
  */
 
 
@@ -69,6 +69,8 @@
 #include <xercesc/internal/XMLScanner.hpp>
 #include <xercesc/validators/DTD/DTDValidator.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 // ---------------------------------------------------------------------------
 //  DTDValidator: Constructors and Destructor
 // ---------------------------------------------------------------------------
@@ -199,7 +201,9 @@ bool DTDValidator::requiresNamespaces() const
 
 void
 DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
-                                , const XMLCh* const    attrValue)
+                                , const XMLCh* const    attrValue
+                                , bool                  preValidation
+                                , const XMLElementDecl* elemDecl)
 {
     //
     //  Get quick refs to lost of of the stuff in the passed objects in
@@ -215,10 +219,13 @@ DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
     //
     //  If the default type is fixed, then make sure the passed value maps
     //  to the fixed value.
+    //  If during preContentValidation, the value we are validating is the fixed value itself
+    //  so no need to compare.
+    //  Only need to do this for regular attribute value validation
     //
-    if (defType == XMLAttDef::Fixed)
+    if (defType == XMLAttDef::Fixed && !preValidation)
     {
-        if (XMLString::compareString(attrValue, valueText))
+        if (!XMLString::equals(attrValue, valueText))
             emitError(XMLValid::NotSameAsFixedValue, fullName, attrValue, valueText);
     }
 
@@ -229,6 +236,8 @@ DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
     if (type == XMLAttDef::CData)
         return;
 
+
+
     // An empty string cannot be valid for any of the other types
     if (!attrValue[0])
     {
@@ -294,6 +303,8 @@ DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
 
     XMLCh* valPtr = pszTmpVal;
 
+    bool doNamespace = getScanner()->getDoNamespaces();
+
     while (true)
     {
         // Reset the trigger flags
@@ -307,7 +318,7 @@ DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
         if (firstNameChar)
         {
             // If its not, emit and error but try to keep going
-            if (!XMLReader::isFirstNameChar(*valPtr))
+            if (!getReaderMgr()->getCurrentReader()->isFirstNameChar(*valPtr))
                 emitError(XMLValid::AttrValNotName, fullName);
             valPtr++;
         }
@@ -319,7 +330,7 @@ DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
             //  If we hit a whitespace, its either a break between two
             //  or more values, or an error if we have a single value.
             //
-            if (XMLReader::isWhitespace(*valPtr))
+            if (getReaderMgr()->getCurrentReader()->isWhitespace(*valPtr))
             {
                 if (!multipleValues)
                 {
@@ -329,7 +340,15 @@ DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
                 break;
             }
 
-            if (!XMLReader::isNameChar(*valPtr))
+            // Now this attribute can be of type
+            //     ID, IDREF, IDREFS, ENTITY, ENTITIES, NOTATION, NMTOKEN, NMTOKENS, ENUMERATION
+            //  All these must be valid XMLName
+            // If namespace is enabled, colon is not allowed in the first 6
+
+            if (doNamespace && *valPtr == chColon && firstNameChar)
+                getScanner()->emitError(XMLErrs::ColonNotLegalWithNS);
+
+            if (!getReaderMgr()->getCurrentReader()->isNameChar(*valPtr))
             {
                 emitError(XMLValid::AttrValNotName, fullName);
                 return;
@@ -373,8 +392,11 @@ DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
                 //
                 if (type == XMLAttDef::ID)
                     find->setDeclared(true);
-                else
-                    find->setUsed(true);
+                else {
+                    if (!preValidation) {
+                        find->setUsed(true);
+                    }
+                }
             }
         }
          else if ((type == XMLAttDef::Entity) || (type == XMLAttDef::Entities))
@@ -384,7 +406,7 @@ DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
             //  general entity pool. If not there, then its an error. If its
             //  not an external unparsed entity, then its an error.
             //
-            const XMLEntityDecl* decl = getScanner()->getEntityDecl(pszTmpVal);
+            const XMLEntityDecl* decl = fDTDGrammar->getEntityDecl(pszTmpVal);
             if (decl)
             {
                 if (!decl->isUnparsed())
@@ -431,7 +453,8 @@ DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
 
 }
 
-void DTDValidator::preContentValidation(bool reuseGrammar)
+void DTDValidator::preContentValidation(bool reuseGrammar,
+                                        bool validateDefAttr)
 {
     //
     //  Lets enumerate all of the elements in the element decl pool
@@ -518,7 +541,7 @@ void DTDValidator::preContentValidation(bool reuseGrammar)
 
                 seenId = true;
             }
-             else if (curAttDef.getType() == XMLAttDef::Notation)
+             else if (curAttDef.getType() == XMLAttDef::Notation && curAttDef.getEnumeration())
             {
                 //
                 //  We need to verify that all of its possible values
@@ -569,12 +592,14 @@ void DTDValidator::preContentValidation(bool reuseGrammar)
             }
 
             // If it has a default/fixed value, then validate it
-            if (curAttDef.getValue())
+            if (validateDefAttr && curAttDef.getValue())
             {
                 validateAttrValue
                 (
                     &curAttDef
                     , curAttDef.getValue()
+                    , true
+                    , &curElem
                 );
             }
         }
@@ -584,7 +609,7 @@ void DTDValidator::preContentValidation(bool reuseGrammar)
     //  And enumerate all of the general entities. If any of them
     //  reference a notation, then make sure the notation exists.
     //
-    NameIdPoolEnumerator<DTDEntityDecl> entEnum = getScanner()->getEntityEnumerator();
+    NameIdPoolEnumerator<DTDEntityDecl> entEnum = fDTDGrammar->getEntityEnumerator();
     while (entEnum.hasMoreElements())
     {
         const DTDEntityDecl& curEntity = entEnum.nextElement();
@@ -612,3 +637,5 @@ void DTDValidator::postParseValidation()
     //  validator.
     //
 }
+
+XERCES_CPP_NAMESPACE_END
