@@ -57,6 +57,12 @@
 
 /*
  * $Log: GrammarResolver.cpp,v $
+ * Revision 1.5  2002/11/04 14:54:58  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.4  2002/09/24 19:48:39  tng
+ * Performance: use XMLString::equals instead of XMLString::compareString
+ *
  * Revision 1.3  2002/07/12 14:35:37  knoaman
  * Add an error message and use it in the scanner.
  *
@@ -88,6 +94,8 @@
 #include <xercesc/validators/schema/SchemaSymbols.hpp>
 #include <xercesc/validators/schema/SchemaGrammar.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 // ---------------------------------------------------------------------------
 //  GrammarResolver: Constructor and Destructor
 // ---------------------------------------------------------------------------
@@ -118,7 +126,7 @@ GrammarResolver::getDatatypeValidator(const XMLCh* const uriStr,
 
     DatatypeValidator* dv = 0;
 
-    if (XMLString::compareString(uriStr, SchemaSymbols::fgURI_SCHEMAFORSCHEMA) == 0) {
+    if (XMLString::equals(uriStr, SchemaSymbols::fgURI_SCHEMAFORSCHEMA)) {
 
         if (!fDataTypeReg) {
 
@@ -212,7 +220,7 @@ void GrammarResolver::cacheGrammars()
         keys.addElement(grammarKey);
         keyCount++;
     }
- 
+
     if (!fCachedGrammarRegistry)
         fCachedGrammarRegistry = new RefHashTableOf<Grammar>(29, true);
 
@@ -239,3 +247,5 @@ void GrammarResolver::cacheGrammarFromParse(const bool aValue) {
     fGrammarRegistry->removeAll();
     fGrammarRegistry->setAdoptElements(!fCacheGrammar);
 }
+
+XERCES_CPP_NAMESPACE_END
