@@ -56,6 +56,12 @@
 
 /*
  * $Log: MixedContentModel.cpp,v $
+ * Revision 1.3  2002/11/04 14:54:58  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.2  2002/09/24 19:48:39  tng
+ * Performance: use XMLString::equals instead of XMLString::compareString
+ *
  * Revision 1.1.1.1  2002/02/01 22:22:39  peiyongz
  * sane_include
  *
@@ -135,6 +141,8 @@
 #include <xercesc/validators/common/Grammar.hpp>
 #include <xercesc/validators/schema/SubstitutionGroupComparator.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 // ---------------------------------------------------------------------------
 //  MixedContentModel: Constructors and Destructor
 // ---------------------------------------------------------------------------
@@ -207,13 +215,13 @@ bool MixedContentModel::hasDups() const
                 continue;
 
             if (fDTD) {
-                if (!XMLString::compareString(curVal->getRawName(), fChildren[iIndex]->getRawName())) {
+                if (XMLString::equals(curVal->getRawName(), fChildren[iIndex]->getRawName())) {
                     return true;
                 }
             }
             else {
                 if ((curVal->getURI() == fChildren[iIndex]->getURI()) &&
-                    (!XMLString::compareString(curVal->getLocalPart(), fChildren[iIndex]->getLocalPart()))) {
+                    (XMLString::equals(curVal->getLocalPart(), fChildren[iIndex]->getLocalPart()))) {
                     return true;
                 }
             }
@@ -254,13 +262,13 @@ MixedContentModel::validateContent( QName** const         children
 
             if (type == ContentSpecNode::Leaf) {
                 if (fDTD) {
-                    if (XMLString::compareString(inChild->getRawName(), curChild->getRawName())) {
+                    if (!XMLString::equals(inChild->getRawName(), curChild->getRawName())) {
                         return outIndex;
                     }
                 }
                 else {
                     if ((inChild->getURI() != curChild->getURI()) ||
-                        (XMLString::compareString(inChild->getLocalPart(), curChild->getLocalPart()))) {
+                        (!XMLString::equals(inChild->getLocalPart(), curChild->getLocalPart()))) {
                         return outIndex;
                     }
                 }
@@ -300,13 +308,13 @@ MixedContentModel::validateContent( QName** const         children
 
                 if (type == ContentSpecNode::Leaf) {
                     if (fDTD) {
-                        if (!XMLString::compareString(inChild->getRawName(), curChild->getRawName())) {
+                        if (XMLString::equals(inChild->getRawName(), curChild->getRawName())) {
                             break;
                         }
                     }
                     else {
                         if ((inChild->getURI() == curChild->getURI()) &&
-                            (!XMLString::compareString(inChild->getLocalPart(), curChild->getLocalPart()))) {
+                            (XMLString::equals(inChild->getLocalPart(), curChild->getLocalPart()))) {
                             break;
                         }
                     }
@@ -479,3 +487,5 @@ MixedContentModel::buildChildList(  ContentSpecNode* const       curNode
     }
 }
 
+XERCES_CPP_NAMESPACE_END
+
