@@ -55,8 +55,35 @@
  */
 
 /*
- * $Id: AbstractNumericFacetValidator.cpp,v 1.2 2002/02/14 15:17:31 peiyongz Exp $
+ * $Id: AbstractNumericFacetValidator.cpp,v 1.11 2003/02/03 16:34:35 peiyongz Exp $
  * $Log: AbstractNumericFacetValidator.cpp,v $
+ * Revision 1.11  2003/02/03 16:34:35  peiyongz
+ * fix to misplaced derived and base value in error message.
+ *
+ * Revision 1.10  2003/02/02 23:55:40  peiyongz
+ * getFormattedString() added to return original and converted value.
+ *
+ * Revision 1.9  2003/01/30 21:56:22  tng
+ * Performance: call getRawData instead of toString
+ *
+ * Revision 1.8  2003/01/25 16:33:25  peiyongz
+ * wrong error code.
+ *
+ * Revision 1.7  2002/12/18 14:17:55  gareth
+ * Fix to bug #13438. When you eant a vector that calls delete[] on its members you should use RefArrayVectorOf.
+ *
+ * Revision 1.6  2002/11/26 22:19:15  peiyongz
+ * Schema Errata E2-16 maxExclusive
+ *
+ * Revision 1.5  2002/11/04 14:53:27  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.4  2002/09/24 19:44:40  tng
+ * Performance: use XMLString::equals instead of XMLString::compareString
+ *
+ * Revision 1.3  2002/04/01 20:17:46  peiyongz
+ * Bug#7551: Exceptions are caught by value, rather than by reference
+ *
  * Revision 1.2  2002/02/14 15:17:31  peiyongz
  * getEnumString()
  *
@@ -92,33 +119,28 @@
 #include <xercesc/validators/datatype/InvalidDatatypeFacetException.hpp>
 #include <xercesc/util/NumberFormatException.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 const int AbstractNumericFacetValidator::INDETERMINATE = 2;
 
 #define  REPORT_FACET_ERROR(val1, val2, except_code)    \
-  XMLCh* value1 = (val1)->toString();                   \
-  ArrayJanitor<XMLCh> jan1(value1);                     \
-  XMLCh* value2 = (val2)->toString();                   \
-  ArrayJanitor<XMLCh> jan2(value2);                     \
   ThrowXML2(InvalidDatatypeFacetException               \
           , except_code                                 \
-          , value2                                      \
-          , value1);
-
+          , val1->getFormattedString()                  \
+          , val2->getFormattedString());
 
 #define  FROM_BASE_VALUE_SPACE(val, facetFlag, except_code)   \
   if ((thisFacetsDefined & facetFlag) != 0)                   \
 {                                                             \
-    XMLCh* value1 = (val)->toString();                        \
-    ArrayJanitor<XMLCh> jan(value1);                          \
     try                                                       \
 {                                                             \
-        numBase->checkContent(value1, false);                 \
+        numBase->checkContent(val->getRawData(), false);      \
 }                                                             \
     catch ( XMLException& )                                   \
 {                                                             \
         ThrowXML1(InvalidDatatypeFacetException               \
                 , except_code                                 \
-                , value1);                                    \
+                , val->getRawData());                         \
 }                                                             \
 }
 
@@ -172,7 +194,7 @@ AbstractNumericFacetValidator::AbstractNumericFacetValidator(
 //
 //  P1. Enumeration
 //
-void AbstractNumericFacetValidator::init(RefVectorOf<XMLCh>* const enums)
+void AbstractNumericFacetValidator::init(RefArrayVectorOf<XMLCh>* const enums)
 {
 
     fStrEnumeration = enums; // save the literal value
@@ -213,62 +235,62 @@ void AbstractNumericFacetValidator::assignFacet()
         key = pair.getKey();
         value = pair.getValue();
 
-        if (XMLString::compareString(key, SchemaSymbols::fgELT_PATTERN) == 0)
+        if (XMLString::equals(key, SchemaSymbols::fgELT_PATTERN))
         {
             setPattern(value);
             if (getPattern())
                 setFacetsDefined(DatatypeValidator::FACET_PATTERN);
             // do not construct regex until needed
         }
-        else if (XMLString::compareString(key, SchemaSymbols::fgELT_MAXINCLUSIVE)==0)
+        else if (XMLString::equals(key, SchemaSymbols::fgELT_MAXINCLUSIVE))
         {
             try
             {
                 setMaxInclusive(value);
             }
-            catch (NumberFormatException)
+            catch (NumberFormatException&)
             {
                 ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_Invalid_MaxIncl, value);
             }
             setFacetsDefined(DatatypeValidator::FACET_MAXINCLUSIVE);
         }
-        else if (XMLString::compareString(key, SchemaSymbols::fgELT_MAXEXCLUSIVE)==0)
+        else if (XMLString::equals(key, SchemaSymbols::fgELT_MAXEXCLUSIVE))
         {
             try
             {
                 setMaxExclusive(value);
             }
-            catch (NumberFormatException)
+            catch (NumberFormatException&)
             {
                 ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_Invalid_MaxExcl, value);
             }
             setFacetsDefined(DatatypeValidator::FACET_MAXEXCLUSIVE);
         }
-        else if (XMLString::compareString(key, SchemaSymbols::fgELT_MININCLUSIVE)==0)
+        else if (XMLString::equals(key, SchemaSymbols::fgELT_MININCLUSIVE))
         {
             try
             {
                 setMinInclusive(value);
             }
-            catch (NumberFormatException)
+            catch (NumberFormatException&)
             {
                 ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_Invalid_MinIncl, value);
             }
             setFacetsDefined(DatatypeValidator::FACET_MININCLUSIVE);
         }
-        else if (XMLString::compareString(key, SchemaSymbols::fgELT_MINEXCLUSIVE)==0)
+        else if (XMLString::equals(key, SchemaSymbols::fgELT_MINEXCLUSIVE))
         {
             try
             {
                 setMinExclusive(value);
             }
-            catch (NumberFormatException)
+            catch (NumberFormatException&)
             {
                 ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_Invalid_MinExcl, value);
             }
             setFacetsDefined(DatatypeValidator::FACET_MINEXCLUSIVE);
         }
-        else if (XMLString::compareString(key, SchemaSymbols::fgATT_FIXED)==0)
+        else if (XMLString::equals(key, SchemaSymbols::fgATT_FIXED))
         {
             unsigned int val;
             bool         retStatus;
@@ -276,7 +298,7 @@ void AbstractNumericFacetValidator::assignFacet()
             {
                 retStatus = XMLString::textToBin(value, val);
             }
-            catch (RuntimeException)
+            catch (RuntimeException&)
             {
                 ThrowXML(InvalidDatatypeFacetException, XMLExcepts::FACET_internalError_fixed);
             }
@@ -440,15 +462,15 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             if (((baseFixed & DatatypeValidator::FACET_MAXINCLUSIVE) != 0) &&
                  (result != 0 ))
             {
-                REPORT_FACET_ERROR(baseMaxInclusive
-                                 , thisMaxInclusive
+                REPORT_FACET_ERROR(thisMaxInclusive
+                                 , baseMaxInclusive
                                  , XMLExcepts::FACET_maxIncl_base_fixed)
             }
 
             if (result == 1 || result == INDETERMINATE)
             {
-                REPORT_FACET_ERROR(baseMaxInclusive
-                                 , thisMaxInclusive
+                REPORT_FACET_ERROR(thisMaxInclusive
+                                 , baseMaxInclusive
                                  , XMLExcepts::FACET_maxIncl_base_maxIncl)
             }
 
@@ -459,8 +481,8 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMaxInclusive, baseMaxExclusive);
             if (result != -1 )
             {
-                REPORT_FACET_ERROR(baseMaxExclusive
-                                 , thisMaxInclusive
+                REPORT_FACET_ERROR(thisMaxInclusive
+                                 , baseMaxExclusive
                                  , XMLExcepts::FACET_maxIncl_base_maxExcl)
             }
         }
@@ -471,8 +493,8 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMaxInclusive, baseMinInclusive);
             if (result == -1 || result == INDETERMINATE)
             {
-                REPORT_FACET_ERROR(baseMinInclusive
-                                 , thisMaxInclusive
+                REPORT_FACET_ERROR(thisMaxInclusive
+                                 , baseMinInclusive
                                  , XMLExcepts::FACET_maxIncl_base_minIncl)
             }
         }
@@ -482,8 +504,8 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMaxInclusive, baseMinExclusive);
             if (result != 1 )
             {
-                REPORT_FACET_ERROR(baseMinExclusive
-                                 , thisMaxInclusive
+                REPORT_FACET_ERROR(thisMaxInclusive
+                                 , baseMinExclusive
                                  , XMLExcepts::FACET_maxIncl_base_minExcl)
             }
         }
@@ -505,17 +527,39 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             if (((baseFixed & DatatypeValidator::FACET_MAXEXCLUSIVE) != 0) &&
                  (result != 0 ))
             {
-                REPORT_FACET_ERROR(baseMaxExclusive
-                                 , thisMaxExclusive
+                REPORT_FACET_ERROR(thisMaxExclusive
+                                 , baseMaxExclusive
                                  , XMLExcepts::FACET_maxExcl_base_fixed)
              }
 
             if (result == 1 || result == INDETERMINATE)
             {
-                REPORT_FACET_ERROR(baseMaxExclusive
-                                 , thisMaxExclusive
+                REPORT_FACET_ERROR(thisMaxExclusive
+                                 , baseMaxExclusive
                                  , XMLExcepts::FACET_maxExcl_base_maxExcl)
             }
+
+            /**
+             * Schema Errata
+             * E2-16 maxExclusive
+             *
+             *   derived type's maxExclusive must either be
+             *   1) equal to base' maxExclusive or
+             *   2) from the base type value space
+             *
+             */
+            if (result != 0)
+            {
+                FROM_BASE_VALUE_SPACE(thisMaxExclusive
+                        , DatatypeValidator::FACET_MAXEXCLUSIVE
+                        , XMLExcepts::FACET_maxExcl_notFromBase)
+            }
+        }
+        else  // base has no maxExclusive
+        {
+            FROM_BASE_VALUE_SPACE(thisMaxExclusive
+                        , DatatypeValidator::FACET_MAXEXCLUSIVE
+                        , XMLExcepts::FACET_maxExcl_notFromBase)
         }
 
         if (( baseFacetsDefined & DatatypeValidator::FACET_MAXINCLUSIVE) != 0)
@@ -523,8 +567,8 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMaxExclusive, baseMaxInclusive);
             if (result == 1 || result == INDETERMINATE)
             {
-                REPORT_FACET_ERROR(baseMaxInclusive
-                                 , thisMaxExclusive
+                REPORT_FACET_ERROR(thisMaxExclusive
+                                 , baseMaxInclusive
                                  , XMLExcepts::FACET_maxExcl_base_maxIncl)
             }
         }
@@ -534,8 +578,8 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMaxExclusive, baseMinExclusive);
             if (result != 1)
             {
-                REPORT_FACET_ERROR(baseMinExclusive
-                                 , thisMaxExclusive
+                REPORT_FACET_ERROR(thisMaxExclusive
+                                 , baseMinExclusive
                                  , XMLExcepts::FACET_maxExcl_base_minExcl)
             }
         }
@@ -545,9 +589,9 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMaxExclusive, baseMinInclusive);
             if (result != 1)
             {
-                REPORT_FACET_ERROR(baseMinInclusive
-                                 , thisMaxExclusive
-                                 , XMLExcepts::FACET_maxExcl_base_minExcl)
+                REPORT_FACET_ERROR(thisMaxExclusive
+                                 , baseMinInclusive
+                                 , XMLExcepts::FACET_maxExcl_base_minIncl)
             }
         }
     }
@@ -567,17 +611,41 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             if (((baseFixed & DatatypeValidator::FACET_MINEXCLUSIVE) != 0) &&
                  (result != 0 ))
             {
-                REPORT_FACET_ERROR(baseMinExclusive
-                                 , thisMinExclusive
+                REPORT_FACET_ERROR(thisMinExclusive
+                                 , baseMinExclusive
                                  , XMLExcepts::FACET_minExcl_base_fixed)
             }
 
             if (result == -1 || result == INDETERMINATE)
             {
-                REPORT_FACET_ERROR(baseMinExclusive
-                                 , thisMinExclusive
+                REPORT_FACET_ERROR(thisMinExclusive
+                                 , baseMinExclusive
                                  , XMLExcepts::FACET_minExcl_base_minExcl)
             }
+
+            /**
+             * Schema Errata
+             * E2-16 maxExclusive
+             *
+             *   derived type's minExclusive must either be
+             *   1) equal to base' minxExclusive or
+             *   2) from the base type value space
+             *
+             *  Note: deduced from, not explicitly expressed in the Errata
+             */
+            if (result != 0)
+            {
+                FROM_BASE_VALUE_SPACE(thisMinExclusive
+                        , DatatypeValidator::FACET_MINEXCLUSIVE
+                        , XMLExcepts::FACET_minExcl_notFromBase)
+            }
+        }
+        else // base has no minExclusive
+        {
+
+            FROM_BASE_VALUE_SPACE(thisMinExclusive
+                        , DatatypeValidator::FACET_MINEXCLUSIVE
+                        , XMLExcepts::FACET_minExcl_notFromBase)
         }
 
         if (( baseFacetsDefined & DatatypeValidator::FACET_MAXINCLUSIVE) != 0)
@@ -585,8 +653,9 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMinExclusive, baseMaxInclusive);
             if (result == 1 || result == INDETERMINATE)
             {
-                REPORT_FACET_ERROR(baseMaxInclusive
-                                 , thisMinExclusive
+                REPORT_FACET_ERROR(thisMinExclusive
+                                 , baseMaxInclusive
+
                                  , XMLExcepts::FACET_minExcl_base_maxIncl)
             }
         }
@@ -596,8 +665,8 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMinExclusive, baseMinInclusive);
             if (result == -1 || result == INDETERMINATE)
             {
-                REPORT_FACET_ERROR(baseMinInclusive
-                                 , thisMinExclusive
+                REPORT_FACET_ERROR(thisMinExclusive
+                                 , baseMinInclusive
                                  , XMLExcepts::FACET_minExcl_base_minIncl)
             }
         }
@@ -607,8 +676,8 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMinExclusive, baseMaxExclusive);
             if (result != -1)
             {
-                REPORT_FACET_ERROR(baseMaxExclusive
-                                 , thisMinExclusive
+                REPORT_FACET_ERROR(thisMinExclusive
+                                 , baseMaxExclusive
                                  , XMLExcepts::FACET_minExcl_base_maxExcl)
             }
         }
@@ -631,15 +700,15 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             if (((baseFixed & DatatypeValidator::FACET_MININCLUSIVE) != 0) &&
                  (result != 0 ))
             {
-                REPORT_FACET_ERROR(baseMinInclusive
-                                 , thisMinInclusive
+                REPORT_FACET_ERROR(thisMinInclusive
+                                 , baseMinInclusive
                                  , XMLExcepts::FACET_minIncl_base_fixed)
             }
 
             if (result == -1 || result == INDETERMINATE)
             {
-                REPORT_FACET_ERROR(baseMinInclusive
-                                 , thisMinInclusive
+                REPORT_FACET_ERROR(thisMinInclusive
+                                 , baseMinInclusive
                                  , XMLExcepts::FACET_minIncl_base_minIncl)
             }
         }
@@ -649,8 +718,8 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMinInclusive, baseMaxInclusive);
             if (result == 1 || result == INDETERMINATE)
             {
-                REPORT_FACET_ERROR(baseMaxInclusive
-                                 , thisMinInclusive
+                REPORT_FACET_ERROR(thisMinInclusive
+                                 , baseMaxInclusive
                                  , XMLExcepts::FACET_minIncl_base_maxIncl)
             }
         }
@@ -660,8 +729,8 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMinInclusive, baseMinExclusive);
             if (result != 1)
             {
-                REPORT_FACET_ERROR(baseMinExclusive
-                                 , thisMinInclusive
+                REPORT_FACET_ERROR(thisMinInclusive
+                                 , baseMinExclusive
                                  , XMLExcepts::FACET_minIncl_base_minExcl)
             }
         }
@@ -671,8 +740,8 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMinInclusive, baseMaxExclusive);
             if (result != -1)
             {
-                REPORT_FACET_ERROR(baseMaxExclusive
-                                 , thisMinInclusive
+                REPORT_FACET_ERROR(thisMinInclusive
+                                 , baseMaxExclusive
                                  , XMLExcepts::FACET_minIncl_base_maxExcl)
             }
         }
@@ -702,18 +771,10 @@ void AbstractNumericFacetValidator::inspectFacetBase()
                         , DatatypeValidator::FACET_MAXINCLUSIVE
                         , XMLExcepts::FACET_maxIncl_notFromBase)
 
-    FROM_BASE_VALUE_SPACE(thisMaxExclusive
-                        , DatatypeValidator::FACET_MAXEXCLUSIVE
-                        , XMLExcepts::FACET_maxExcl_notFromBase)
-
     FROM_BASE_VALUE_SPACE(thisMinInclusive
                         , DatatypeValidator::FACET_MININCLUSIVE
                         , XMLExcepts::FACET_minIncl_notFromBase)
 
-    FROM_BASE_VALUE_SPACE(thisMinExclusive
-                        , DatatypeValidator::FACET_MINEXCLUSIVE
-                        , XMLExcepts::FACET_minExcl_notFromBase)
-
 } //end of inspectFacetBase
 
 //
@@ -787,11 +848,13 @@ void AbstractNumericFacetValidator::inheritFacet()
 
 }
 
-const RefVectorOf<XMLCh>* AbstractNumericFacetValidator::getEnumString() const
+const RefArrayVectorOf<XMLCh>* AbstractNumericFacetValidator::getEnumString() const
 {
 	return (fEnumerationInherited? getBaseValidator()->getEnumString() : fStrEnumeration );
 }
 
+XERCES_CPP_NAMESPACE_END
+
 /**
   * End of file AbstractNumericFacetValidator::cpp
   */
