@@ -56,6 +56,27 @@
 
 /*
  * $Log: AbstractStringValidator.cpp,v $
+ * Revision 1.9  2003/01/27 19:24:17  peiyongz
+ * normalize Base64 data before checking against enumeration.
+ *
+ * Revision 1.8  2003/01/24 23:18:34  peiyongz
+ * normalizeEnumeration() added to remove optional ws in Base64 data.
+ *
+ * Revision 1.7  2002/12/18 14:17:55  gareth
+ * Fix to bug #13438. When you eant a vector that calls delete[] on its members you should use RefArrayVectorOf.
+ *
+ * Revision 1.6  2002/11/04 14:53:27  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.5  2002/10/02 13:29:12  tng
+ * Since the compare function return int, so use XMLString::compareString instead of XMLString::equals there.
+ *
+ * Revision 1.4  2002/09/24 19:44:40  tng
+ * Performance: use XMLString::equals instead of XMLString::compareString
+ *
+ * Revision 1.3  2002/04/01 20:17:46  peiyongz
+ * Bug#7551: Exceptions are caught by value, rather than by reference
+ *
  * Revision 1.2  2002/02/14 15:17:31  peiyongz
  * getEnumString()
  *
@@ -103,6 +124,8 @@
 #include <xercesc/validators/datatype/InvalidDatatypeValueException.hpp>
 #include <xercesc/util/NumberFormatException.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 static const int BUF_LEN = 64;
 static XMLCh value1[BUF_LEN+1];
 static XMLCh value2[BUF_LEN+1];
@@ -154,11 +177,14 @@ AbstractStringValidator::AbstractStringValidator(
     // assigneAdditionalFacet(), inheritAdditionalFacet().
 }
 
-void AbstractStringValidator::init(RefVectorOf<XMLCh>*           const enums)
+void AbstractStringValidator::init(RefArrayVectorOf<XMLCh>*           const enums)
 {
 
     if (enums)
+    {
         setEnumeration(enums, false);
+        normalizeEnumeration();
+    }
 
     assignFacet();
     inspectFacet();
@@ -190,14 +216,14 @@ void AbstractStringValidator::assignFacet()
         key = pair.getKey();
         value = pair.getValue();
 
-        if (XMLString::compareString(key, SchemaSymbols::fgELT_LENGTH)==0)
+        if (XMLString::equals(key, SchemaSymbols::fgELT_LENGTH))
         {
             int val;
             try
             {
                 val = XMLString::parseInt(value);
             }
-            catch (NumberFormatException nfe)
+            catch (NumberFormatException&)
             {
                 ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_Invalid_Len, value);
             }
@@ -208,14 +234,14 @@ void AbstractStringValidator::assignFacet()
                 setLength(val);
                 setFacetsDefined(DatatypeValidator::FACET_LENGTH);
         }
-        else if (XMLString::compareString(key, SchemaSymbols::fgELT_MINLENGTH)==0)
+        else if (XMLString::equals(key, SchemaSymbols::fgELT_MINLENGTH))
         {
             int val;
             try
             {
                 val = XMLString::parseInt(value);
             }
-            catch (NumberFormatException nfe)
+            catch (NumberFormatException&)
             {
                 ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_Invalid_minLen, value);
             }
@@ -226,14 +252,14 @@ void AbstractStringValidator::assignFacet()
             setMinLength(val);
             setFacetsDefined(DatatypeValidator::FACET_MINLENGTH);
         }
-        else if (XMLString::compareString(key, SchemaSymbols::fgELT_MAXLENGTH)==0)
+        else if (XMLString::equals(key, SchemaSymbols::fgELT_MAXLENGTH))
         {
             int val;
             try
             {
                 val = XMLString::parseInt(value);
             }
-            catch (NumberFormatException nfe)
+            catch (NumberFormatException&)
             {
                 ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_Invalid_maxLen, value);
             }
@@ -244,14 +270,14 @@ void AbstractStringValidator::assignFacet()
             setMaxLength(val);
             setFacetsDefined(DatatypeValidator::FACET_MAXLENGTH);
         }
-        else if (XMLString::compareString(key, SchemaSymbols::fgELT_PATTERN)==0)
+        else if (XMLString::equals(key, SchemaSymbols::fgELT_PATTERN))
         {
             setPattern(value);
             if (getPattern())
                 setFacetsDefined(DatatypeValidator::FACET_PATTERN);
             // do not construct regex until needed
         }
-        else if (XMLString::compareString(key, SchemaSymbols::fgATT_FIXED)==0)
+        else if (XMLString::equals(key, SchemaSymbols::fgATT_FIXED))
         {
             unsigned int val;
             bool         retStatus;
@@ -259,7 +285,7 @@ void AbstractStringValidator::assignFacet()
             {
                 retStatus = XMLString::textToBin(value, val);
             }
-            catch (RuntimeException)
+            catch (RuntimeException&)
             {
                 ThrowXML(InvalidDatatypeFacetException, XMLExcepts::FACET_internalError_fixed);
             }
@@ -273,7 +299,7 @@ void AbstractStringValidator::assignFacet()
             //no setFacetsDefined here
         }
         //
-        // else if (XMLString::compareString(key, SchemaSymbols::fgELT_SPECIAL_TOKEN)==0)
+        // else if (XMLString::equals(key, SchemaSymbols::fgELT_SPECIAL_TOKEN))
         // TODO
         //
         // Note: whitespace is taken care of by TraverseSchema.
@@ -640,11 +666,15 @@ void AbstractStringValidator::checkContent( const XMLCh* const content, bool asB
     if ((thisFacetsDefined & DatatypeValidator::FACET_ENUMERATION) != 0 &&
         (getEnumeration() != 0))
     {
+        XMLCh* normContent = XMLString::replicate(content);
+        ArrayJanitor<XMLCh>  jan(normContent);
+        normalizeContent(normContent);
+
         int i=0;
         int enumLength = getEnumeration()->size();
         for ( ; i < enumLength; i++)
         {
-            if (XMLString::compareString(content, getEnumeration()->elementAt(i))==0)
+            if (XMLString::equals(normContent, getEnumeration()->elementAt(i)))
                 break;
         }
 
@@ -656,11 +686,25 @@ void AbstractStringValidator::checkContent( const XMLCh* const content, bool asB
 
 }
 
-const RefVectorOf<XMLCh>* AbstractStringValidator::getEnumString() const
+const RefArrayVectorOf<XMLCh>* AbstractStringValidator::getEnumString() const
 {
 	return getEnumeration();
 }
 
+void AbstractStringValidator::normalizeEnumeration()
+{
+    // default implementation: do nothing
+    return;
+}
+
+void AbstractStringValidator::normalizeContent(XMLCh* const) const
+{
+    // default implementation: do nothing
+    return;
+}
+
+XERCES_CPP_NAMESPACE_END
+
 /**
   * End of file AbstractStringValidator.cpp
   */
