@@ -56,6 +56,15 @@
 
 /*
  * $Log: BooleanDatatypeValidator.cpp,v $
+ * Revision 1.6  2002/12/18 14:17:55  gareth
+ * Fix to bug #13438. When you eant a vector that calls delete[] on its members you should use RefArrayVectorOf.
+ *
+ * Revision 1.5  2002/11/04 14:53:28  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.4  2002/09/24 19:44:40  tng
+ * Performance: use XMLString::equals instead of XMLString::compareString
+ *
  * Revision 1.3  2002/08/23 20:16:50  tng
  * Memory leak fix: enums is not deleted if an error occurred.
  *
@@ -96,6 +105,8 @@
 #include <xercesc/validators/datatype/InvalidDatatypeFacetException.hpp>
 #include <xercesc/validators/datatype/InvalidDatatypeValueException.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 const int   ARRAYSIZE = 4;
 const XMLCh fgValueSpace[][32] =
 {
@@ -111,7 +122,7 @@ const XMLCh fgValueSpace[][32] =
 BooleanDatatypeValidator::BooleanDatatypeValidator(
                           DatatypeValidator*            const baseValidator
                         , RefHashTableOf<KVStringPair>* const facets
-                        , RefVectorOf<XMLCh>*           const enums
+                        , RefArrayVectorOf<XMLCh>*           const enums
                         , const int                           finalSet)
 :DatatypeValidator(baseValidator, facets, finalSet, DatatypeValidator::Boolean)
 {
@@ -138,7 +149,7 @@ BooleanDatatypeValidator::BooleanDatatypeValidator(
             key = pair.getKey();
             value = pair.getValue();
 
-            if (XMLString::compareString(key, SchemaSymbols::fgELT_PATTERN)==0)
+            if (XMLString::equals(key, SchemaSymbols::fgELT_PATTERN))
             {
                 setPattern(value);
                 setFacetsDefined(DatatypeValidator::FACET_PATTERN);
@@ -194,7 +205,7 @@ void BooleanDatatypeValidator::checkContent( const XMLCh* const content, bool as
     int   i = 0;
     for ( ; i < ARRAYSIZE; i++ )
     {
-        if ( XMLString::compareString(content, fgValueSpace[i]) ==0 )
+        if ( XMLString::equals(content, fgValueSpace[i]))
             break;
     }
 
@@ -211,30 +222,32 @@ int BooleanDatatypeValidator::compare(const XMLCh* const lValue
     // 1 == true
     // 0 == false
 
-    if (( XMLString::compareString(lValue, fgValueSpace[0]) ==0 ) ||
-        ( XMLString::compareString(lValue, fgValueSpace[2]) ==0 ) )
+    if (XMLString::equals(lValue, fgValueSpace[0])||
+        XMLString::equals(lValue, fgValueSpace[2]))
     {
-        if (( XMLString::compareString(rValue, fgValueSpace[0]) ==0 ) ||
-            ( XMLString::compareString(rValue, fgValueSpace[2]) ==0 ) )
+        if (XMLString::equals(rValue, fgValueSpace[0]) ||
+            XMLString::equals(rValue, fgValueSpace[2]))
             return 0;
     }
     else
-    if (( XMLString::compareString(lValue, fgValueSpace[1]) ==0 ) ||
-        ( XMLString::compareString(lValue, fgValueSpace[3]) ==0 ) )
+    if (XMLString::equals(lValue, fgValueSpace[1]) ||
+        XMLString::equals(lValue, fgValueSpace[3]))
     {
-        if (( XMLString::compareString(rValue, fgValueSpace[1]) ==0 ) ||
-            ( XMLString::compareString(rValue, fgValueSpace[3]) ==0 ) )
+        if (XMLString::equals(rValue, fgValueSpace[1]) ||
+            XMLString::equals(rValue, fgValueSpace[3]))
             return 0;
     }
 
     return 1;
 }
 
-const RefVectorOf<XMLCh>* BooleanDatatypeValidator::getEnumString() const
+const RefArrayVectorOf<XMLCh>* BooleanDatatypeValidator::getEnumString() const
 {
 	return 0;
 }
 
+XERCES_CPP_NAMESPACE_END
+
 /**
   * End of file BooleanDatatypeValidator.cpp
   */
