@@ -56,6 +56,33 @@
 
 /*
  * $Log: DatatypeValidatorFactory.cpp,v $
+ * Revision 1.11  2003/01/29 19:51:48  gareth
+ * type names / uris are now added to DatatypeValidators.
+ *
+ * Revision 1.10  2003/01/03 16:39:45  tng
+ * Performance: no need to create that many members for fBuiltInRegistry
+ *
+ * Revision 1.9  2002/12/18 14:17:55  gareth
+ * Fix to bug #13438. When you eant a vector that calls delete[] on its members you should use RefArrayVectorOf.
+ *
+ * Revision 1.8  2002/12/02 16:01:24  gareth
+ * Fix to bug #12188. NCNAME validator now has a base validator of NAME. Patch by Peter Volchek.
+ *
+ * Revision 1.7  2002/12/02 13:25:51  gareth
+ * Fix for bug #12188. Create NMTOKEN, ID, IDREF, ENTITY, NAME, NCNAME with appropriate base types. Some reordering of creation was required where dependencies resulted.
+ *
+ * Revision 1.6  2002/11/21 23:43:19  peiyongz
+ * Schema Errata: E2-25 language
+ *
+ * Revision 1.5  2002/11/04 14:53:28  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.4  2002/05/08 12:25:30  tng
+ * Schema Fix: re-add the ID, IDREF ... datatype validators only if they were not there.
+ *
+ * Revision 1.3  2002/05/07 20:00:07  tng
+ * Schema Fix: No need to re-add the ID, IDREF ... datatype validators if the fUserDefinedRegistry already exists.
+ *
  * Revision 1.2  2002/02/25 21:28:26  tng
  * Schema Fix: Thread-safe the built-in datatype validator factory.
  *
@@ -184,6 +211,9 @@
 #include <xercesc/util/PlatformUtils.hpp>
 #include <xercesc/util/XMLRegisterCleanup.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
+
 // ---------------------------------------------------------------------------
 //  DatatypeValidatorFactory: Local const data
 // ---------------------------------------------------------------------------
@@ -192,18 +222,16 @@ const XMLCh fgTokPattern[] =
     chBackSlash, chLatin_c, chPlus, chNull
 };
 
-//"([a-zA-Z]{2}|[iI]-[a-zA-Z]+|[xX]-[a-zA-Z]+)(-[a-zA-Z]+)*"
+// http://www.w3.org/2001/05/xmlschema-errata#Errata2
+// E2-25
+//"([a-zA-Z]{1,8})(-[a-zA-Z0-9]{1,8})*"
 const XMLCh fgLangPattern[] =
 {
-    chOpenParen, chOpenSquare, chLatin_a, chDash, chLatin_z, chLatin_A, chDash,
-    chLatin_Z, chCloseSquare, chOpenCurly, chDigit_2, chCloseCurly, chPipe,
-    chOpenSquare, chLatin_i, chLatin_I, chCloseSquare, chDash, chOpenSquare,
-    chLatin_a, chDash, chLatin_z, chLatin_A, chDash, chLatin_Z, chCloseSquare,
-    chPlus, chPipe, chOpenSquare, chLatin_x, chLatin_X, chCloseSquare, chDash,
-    chOpenSquare, chLatin_a, chDash, chLatin_z, chLatin_A, chDash, chLatin_Z,
-	chCloseSquare, chPlus, chCloseParen, chOpenParen, chDash, chOpenSquare,
-    chLatin_a, chDash, chLatin_z, chLatin_A, chDash, chLatin_Z, chCloseSquare,
-    chPlus, chCloseParen, chAsterisk, chNull
+    chOpenParen,   chOpenSquare,  chLatin_a, chDash, chLatin_z, chLatin_A, chDash, chLatin_Z,
+    chCloseSquare, chOpenCurly, chDigit_1, chComma, chDigit_8, chCloseCurly, chCloseParen,
+    chOpenParen,   chDash, chOpenSquare, chLatin_a, chDash, chLatin_z, chLatin_A, chDash, chLatin_Z,
+	chDigit_0,     chDash, chDigit_9, chCloseSquare, chOpenCurly, chDigit_1, chComma, chDigit_8,
+    chCloseCurly,  chCloseParen, chAsterisk, chNull
 };
 
 //"\\i\\c*"
@@ -377,10 +405,9 @@ void DatatypeValidatorFactory::reinitRegistry() {
 void DatatypeValidatorFactory::expandRegistryToFullSchemaSet()
 {
     static XMLRegisterCleanup builtInRegistryCleanup;
-
     // Initialize common Schema/DTD Datatype validator set if not initialized
     if (fBuiltInRegistry == 0) {
-        RefHashTableOf<DatatypeValidator>* t = new RefHashTableOf<DatatypeValidator>(109);
+        RefHashTableOf<DatatypeValidator>* t = new RefHashTableOf<DatatypeValidator>(29);
         if (XMLPlatformUtils::compareAndSwap((void **)&fBuiltInRegistry, t, 0) != 0)
         {
             delete t;
@@ -389,53 +416,85 @@ void DatatypeValidatorFactory::expandRegistryToFullSchemaSet()
         {
             builtInRegistryCleanup.registerCleanup(reinitRegistry);
 
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_STRING,
-                       new StringDatatypeValidator());
-            fBuiltInRegistry->put((void*) XMLUni::fgNotationString,
-                       new NOTATIONDatatypeValidator());
+            DatatypeValidator *dv = new StringDatatypeValidator(); 
+            dv->setTypeName(SchemaSymbols::fgDT_STRING, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_STRING, dv);
+            
+            dv = new NOTATIONDatatypeValidator();
+            dv->setTypeName(XMLUni::fgNotationString, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) XMLUni::fgNotationString, dv);
 
+            dv = new AnySimpleTypeDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_ANYSIMPLETYPE, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_ANYSIMPLETYPE, dv);
 
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_ANYSIMPLETYPE,
-                           new AnySimpleTypeDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_BOOLEAN,
-                           new BooleanDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DECIMAL,
-                           new DecimalDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_HEXBINARY,
-                           new HexBinaryDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_BASE64BINARY,
-                           new Base64BinaryDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DOUBLE,
-                           new DoubleDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_FLOAT,
-                           new FloatDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_ANYURI,
-                           new AnyURIDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_QNAME,
-                           new QNameDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_NAME,
-                           new NameDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_NCNAME,
-                           new NCNameDatatypeValidator());
+            dv = new BooleanDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_BOOLEAN, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_BOOLEAN, dv);
 
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DATETIME,
-                           new DateTimeDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DATE,
-                           new DateDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_TIME,
-                           new TimeDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DAY,
-                           new DayDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_MONTH,
-                           new MonthDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_MONTHDAY,
-                           new MonthDayDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_YEAR,
-                           new YearDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_YEARMONTH,
-                           new YearMonthDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DURATION,
-                           new DurationDatatypeValidator());
+            dv = new DecimalDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_DECIMAL, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DECIMAL, dv);
+
+            dv = new HexBinaryDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_HEXBINARY, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_HEXBINARY, dv);
+
+            dv = new Base64BinaryDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_BASE64BINARY, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_BASE64BINARY, dv);
+
+            dv = new DoubleDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_DOUBLE, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DOUBLE, dv);
+
+            dv = new FloatDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_FLOAT, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_FLOAT, dv);
+
+            dv = new AnyURIDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_ANYURI, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_ANYURI, dv);
+
+            dv = new QNameDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_QNAME, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_QNAME, dv);
+
+            dv = new DateTimeDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_DATETIME, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DATETIME, dv);
+
+            dv = new DateDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_DATE, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DATE, dv);
+
+            dv = new TimeDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_TIME, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_TIME, dv);
+
+            dv = new DayDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_DAY, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DAY, dv);
+
+            dv = new MonthDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_MONTH, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_MONTH, dv);
+
+            dv = new MonthDayDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_MONTHDAY, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_MONTHDAY, dv);
+
+            dv = new YearDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_YEAR, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_YEAR, dv);
+
+            dv = new YearMonthDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_YEARMONTH, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_YEARMONTH, dv);
+
+            dv = new DurationDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_DURATION, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DURATION, dv);
 
             // REVISIT
             // We are creating a lot of Hashtables for the facets of the different
@@ -444,31 +503,16 @@ void DatatypeValidatorFactory::expandRegistryToFullSchemaSet()
 
             RefHashTableOf<KVStringPair>* facets = new RefHashTableOf<KVStringPair>(3);
 
-            // Create 'NMTOKEN' datatype validator
-            facets->put((void*) SchemaSymbols::fgELT_PATTERN ,
-                            new KVStringPair(SchemaSymbols::fgELT_PATTERN,fgTokPattern));
-            facets->put((void*) SchemaSymbols::fgELT_WHITESPACE,
-                            new KVStringPair(SchemaSymbols::fgELT_WHITESPACE, SchemaSymbols::fgWS_COLLAPSE));
-
-            createDatatypeValidator(XMLUni::fgNmTokenString,
-                            getDatatypeValidator(SchemaSymbols::fgDT_STRING),facets, 0, false, 0, false);
-
-            // Create 'NMTOKENS' datatype validator
-            createDatatypeValidator(XMLUni::fgNmTokensString,
-        	                 getDatatypeValidator(XMLUni::fgNmTokenString), 0, 0, true, 0, false);
-
             // Create 'normalizedString' datatype validator
-            facets = new RefHashTableOf<KVStringPair>(3);
             facets->put((void*) SchemaSymbols::fgELT_WHITESPACE,
                         new KVStringPair(SchemaSymbols::fgELT_WHITESPACE, SchemaSymbols::fgWS_REPLACE));
 
             createDatatypeValidator(SchemaSymbols::fgDT_NORMALIZEDSTRING,
-                        getDatatypeValidator(SchemaSymbols::fgDT_STRING),
-    				    facets, 0, false, 0, false);
+                                    getDatatypeValidator(SchemaSymbols::fgDT_STRING),
+                                    facets, 0, false, 0, false);
 
             // Create 'token' datatype validator
             facets = new RefHashTableOf<KVStringPair>(3);
-
             facets->put((void*) SchemaSymbols::fgELT_WHITESPACE,
                         new KVStringPair(SchemaSymbols::fgELT_WHITESPACE, SchemaSymbols::fgWS_COLLAPSE));
 
@@ -476,6 +520,31 @@ void DatatypeValidatorFactory::expandRegistryToFullSchemaSet()
                           getDatatypeValidator(SchemaSymbols::fgDT_NORMALIZEDSTRING),
                           facets, 0, false, 0, false);
 
+
+            dv = new NameDatatypeValidator(getDatatypeValidator(SchemaSymbols::fgDT_TOKEN), 0, 0, 0);
+            dv->setTypeName(SchemaSymbols::fgDT_NAME, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_NAME, dv);
+
+
+            dv = new NCNameDatatypeValidator(getDatatypeValidator(SchemaSymbols::fgDT_NAME), 0, 0, 0);
+            dv->setTypeName(SchemaSymbols::fgDT_NCNAME, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_NCNAME, dv);
+
+            // Create 'NMTOKEN' datatype validator
+            facets = new RefHashTableOf<KVStringPair>(3);
+
+            facets->put((void*) SchemaSymbols::fgELT_PATTERN ,
+                            new KVStringPair(SchemaSymbols::fgELT_PATTERN,fgTokPattern));
+            facets->put((void*) SchemaSymbols::fgELT_WHITESPACE,
+                            new KVStringPair(SchemaSymbols::fgELT_WHITESPACE, SchemaSymbols::fgWS_COLLAPSE));
+
+            createDatatypeValidator(XMLUni::fgNmTokenString,
+                            getDatatypeValidator(SchemaSymbols::fgDT_TOKEN),facets, 0, false, 0, false);
+
+            // Create 'NMTOKENS' datatype validator
+            createDatatypeValidator(XMLUni::fgNmTokensString,
+        	                 getDatatypeValidator(XMLUni::fgNmTokenString), 0, 0, true, 0, false);
+
             // Create 'language' datatype validator
             facets = new RefHashTableOf<KVStringPair>(3);
 
@@ -630,25 +699,32 @@ void DatatypeValidatorFactory::expandRegistryToFullSchemaSet()
     //   and cannot be shared across threads
     // So instead of storing them in the static fBuiltInRegistry,
     //   store them in local data fUserDefinedRegistry
-    if (fUserDefinedRegistry == 0) {
+    if (fUserDefinedRegistry == 0)
         fUserDefinedRegistry = new RefHashTableOf<DatatypeValidator>(29);
+
+    if (!getDatatypeValidator(XMLUni::fgIDRefsString)) {
+
+        DatatypeValidator *dv = new IDDatatypeValidator(getDatatypeValidator(SchemaSymbols::fgDT_NCNAME), 0, 0, 0);
+        dv->setTypeName(XMLUni::fgIDString, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+        fUserDefinedRegistry->put((void*) XMLUni::fgIDString, dv);
+
+        dv = new IDREFDatatypeValidator(getDatatypeValidator(SchemaSymbols::fgDT_NCNAME), 0, 0, 0);
+        dv->setTypeName(XMLUni::fgIDRefString, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+        fUserDefinedRegistry->put((void*) XMLUni::fgIDRefString, dv);
+
+        dv = new ENTITYDatatypeValidator(getDatatypeValidator(SchemaSymbols::fgDT_NCNAME), 0, 0, 0);
+        dv->setTypeName(XMLUni::fgEntityString, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+        fUserDefinedRegistry->put((void*) XMLUni::fgEntityString, dv);
+
+        // Create 'IDREFS' datatype validator
+    	createDatatypeValidator(XMLUni::fgIDRefsString,
+                        getDatatypeValidator(XMLUni::fgIDRefString), 0, 0, true, 0, true);
+
+        // Create 'ENTITIES' datatype validator
+        createDatatypeValidator(XMLUni::fgEntitiesString,
+    		            getDatatypeValidator(XMLUni::fgEntityString), 0, 0, true, 0, true);
     }
 
-    fUserDefinedRegistry->put((void*) XMLUni::fgIDString,
-                       new IDDatatypeValidator());
-    fUserDefinedRegistry->put((void*) XMLUni::fgIDRefString,
-                       new IDREFDatatypeValidator());
-    fUserDefinedRegistry->put((void*) XMLUni::fgEntityString,
-                       new ENTITYDatatypeValidator());
-
-    // Create 'IDREFS' datatype validator
-	 createDatatypeValidator(XMLUni::fgIDRefsString,
-                    getDatatypeValidator(XMLUni::fgIDRefString), 0, 0, true, 0, true);
-
-    // Create 'ENTITIES' datatype validator
-    createDatatypeValidator(XMLUni::fgEntitiesString,
-		            getDatatypeValidator(XMLUni::fgEntityString), 0, 0, true, 0, true);
-
 }
 
 // ---------------------------------------------------------------------------
@@ -658,7 +734,7 @@ DatatypeValidator*
 DatatypeValidatorFactory::createDatatypeValidator(const XMLCh* const typeName,
 		                                          DatatypeValidator* const baseValidator,
                                                   RefHashTableOf<KVStringPair>* const facets,
-                                                  RefVectorOf<XMLCh>* const enums,
+                                                  RefArrayVectorOf<XMLCh>* const enums,
                                                   const bool derivedByList,
                                                   const int finalSet,
                                                   const bool userDefined)
@@ -708,6 +784,7 @@ DatatypeValidatorFactory::createDatatypeValidator(const XMLCh* const typeName,
         else {
             fBuiltInRegistry->put((void *)typeName, datatypeValidator);
         }
+        datatypeValidator->setTypeName(typeName);
     }
 
     return datatypeValidator;
@@ -740,11 +817,14 @@ DatatypeValidatorFactory::createDatatypeValidator(const XMLCh* const typeName,
         else {
             fBuiltInRegistry->put((void *)typeName, datatypeValidator);
         }
+        datatypeValidator->setTypeName(typeName);
     }
 
     return datatypeValidator;
 }
 
+XERCES_CPP_NAMESPACE_END
+
 /**
   * End of file DatatypeValidatorFactory.cpp
   */
