@@ -56,6 +56,21 @@
 
 /*
  * $Log: DecimalDatatypeValidator.cpp,v $
+ * Revision 1.6  2003/01/30 21:56:22  tng
+ * Performance: call getRawData instead of toString
+ *
+ * Revision 1.5  2002/12/18 14:17:55  gareth
+ * Fix to bug #13438. When you eant a vector that calls delete[] on its members you should use RefArrayVectorOf.
+ *
+ * Revision 1.4  2002/11/04 14:53:28  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.3  2002/09/24 19:44:40  tng
+ * Performance: use XMLString::equals instead of XMLString::compareString
+ *
+ * Revision 1.2  2002/04/01 20:17:46  peiyongz
+ * Bug#7551: Exceptions are caught by value, rather than by reference
+ *
  * Revision 1.1.1.1  2002/02/01 22:22:41  peiyongz
  * sane_include
  *
@@ -125,6 +140,8 @@
 #include <xercesc/validators/datatype/InvalidDatatypeValueException.hpp>
 #include <xercesc/util/NumberFormatException.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 static const int BUF_LEN = 64;
 static XMLCh value1[BUF_LEN+1];
 static XMLCh value2[BUF_LEN+1];
@@ -141,7 +158,7 @@ DecimalDatatypeValidator::DecimalDatatypeValidator()
 DecimalDatatypeValidator::DecimalDatatypeValidator(
                           DatatypeValidator*            const baseValidator
                         , RefHashTableOf<KVStringPair>* const facets
-                        , RefVectorOf<XMLCh>*           const enums
+                        , RefArrayVectorOf<XMLCh>*           const enums
                         , const int                           finalSet)
 :AbstractNumericValidator(baseValidator, facets, finalSet, DatatypeValidator::Decimal)
 , fTotalDigits(0)
@@ -169,7 +186,7 @@ int DecimalDatatypeValidator::compare(const XMLCh* const lValue
 
 DatatypeValidator* DecimalDatatypeValidator::newInstance(
                                       RefHashTableOf<KVStringPair>* const facets
-                                    , RefVectorOf<XMLCh>*           const enums
+                                    , RefArrayVectorOf<XMLCh>*           const enums
                                     , const int                           finalSet)
 {
     return (DatatypeValidator*) new DecimalDatatypeValidator(this, facets, enums, finalSet);
@@ -192,14 +209,14 @@ DecimalDatatypeValidator::DecimalDatatypeValidator(DatatypeValidator*
 void DecimalDatatypeValidator::assignAdditionalFacet(const XMLCh* const key
                                                    , const XMLCh* const value)
 {
-    if (XMLString::compareString(key, SchemaSymbols::fgELT_TOTALDIGITS)==0)
+    if (XMLString::equals(key, SchemaSymbols::fgELT_TOTALDIGITS))
     {
         int val;
         try
         {
             val = XMLString::parseInt(value);
         }
-        catch (NumberFormatException)
+        catch (NumberFormatException&)
         {
             ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_Invalid_TotalDigit, value);
         }
@@ -211,14 +228,14 @@ void DecimalDatatypeValidator::assignAdditionalFacet(const XMLCh* const key
         setTotalDigits(val);
         setFacetsDefined(DatatypeValidator::FACET_TOTALDIGITS);
     }
-    else if (XMLString::compareString(key, SchemaSymbols::fgELT_FRACTIONDIGITS)==0)
+    else if (XMLString::equals(key, SchemaSymbols::fgELT_FRACTIONDIGITS))
     {
         int val;
         try
         {
             val = XMLString::parseInt(value);
         }
-        catch (NumberFormatException)
+        catch (NumberFormatException&)
         {
             ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_Invalid_FractDigit, value);
         }
@@ -246,23 +263,23 @@ void DecimalDatatypeValidator::inheritAdditionalFacet()
     if (!numBase)
         return;
 
-    int thisFacetsDefined = getFacetsDefined();  
+    int thisFacetsDefined = getFacetsDefined();
     int baseFacetsDefined = numBase->getFacetsDefined();
 
-    // inherit totalDigits         
-    if ((( baseFacetsDefined & DatatypeValidator::FACET_TOTALDIGITS) != 0) &&          
-        (( thisFacetsDefined & DatatypeValidator::FACET_TOTALDIGITS) == 0) )              
-    {          
-        setTotalDigits(numBase->fTotalDigits);              
-        setFacetsDefined(DatatypeValidator::FACET_TOTALDIGITS);              
+    // inherit totalDigits
+    if ((( baseFacetsDefined & DatatypeValidator::FACET_TOTALDIGITS) != 0) &&
+        (( thisFacetsDefined & DatatypeValidator::FACET_TOTALDIGITS) == 0) )
+    {
+        setTotalDigits(numBase->fTotalDigits);
+        setFacetsDefined(DatatypeValidator::FACET_TOTALDIGITS);
     }
-          
-    // inherit fractionDigits          
-    if ((( baseFacetsDefined & DatatypeValidator::FACET_FRACTIONDIGITS) != 0) &&          
-        (( thisFacetsDefined & DatatypeValidator::FACET_FRACTIONDIGITS) == 0) )              
-    {          
-        setFractionDigits(numBase->fFractionDigits);              
-        setFacetsDefined(DatatypeValidator::FACET_FRACTIONDIGITS);              
+
+    // inherit fractionDigits
+    if ((( baseFacetsDefined & DatatypeValidator::FACET_FRACTIONDIGITS) != 0) &&
+        (( thisFacetsDefined & DatatypeValidator::FACET_FRACTIONDIGITS) == 0) )
+    {
+        setFractionDigits(numBase->fFractionDigits);
+        setFacetsDefined(DatatypeValidator::FACET_FRACTIONDIGITS);
     }
 }
 
@@ -323,7 +340,7 @@ void DecimalDatatypeValidator::checkAdditionalFacetConstraintsBase() const
                                  , XMLExcepts::FACET_totalDigit_base_fixed
                                  , value1
                                  , value2);
-        }                    
+        }
     }
 
     if (( thisFacetsDefined & DatatypeValidator::FACET_FRACTIONDIGITS) != 0)
@@ -416,7 +433,7 @@ void DecimalDatatypeValidator::setEnumeration()
             {
                 numBase->checkContent(fStrEnumeration->elementAt(i), false);
             }
-        }              
+        }
         catch (XMLException&)
         {
             ThrowXML1(InvalidDatatypeFacetException
@@ -427,11 +444,11 @@ void DecimalDatatypeValidator::setEnumeration()
 
     // We put the this->checkContent in a separate loop
     // to not block original message with in that method.
-    // 
+    //
     for ( i = 0; i < enumLength; i++)
     {
         checkContent(fStrEnumeration->elementAt(i), false);
-    }              
+    }
 
     fEnumeration = new RefVectorOf<XMLNumber>(enumLength, true);
     fEnumerationInherited = false;
@@ -439,7 +456,7 @@ void DecimalDatatypeValidator::setEnumeration()
     for ( i = 0; i < enumLength; i++)
     {
         fEnumeration->insertElementAt(new XMLBigDecimal(fStrEnumeration->elementAt(i)), i);
-    }              
+    }
 
 }
 
@@ -507,14 +524,12 @@ void DecimalDatatypeValidator::checkContent( const XMLCh* const content, bool as
         if ( (thisFacetsDefined & DatatypeValidator::FACET_FRACTIONDIGITS) != 0 )
         {
             if ( theData->getScale() > fFractionDigits )
-            {
-                XMLCh* value = theData->toString();
-                ArrayJanitor<XMLCh> jan(value);
+            {                
                 XMLString::binToText(theData->getScale(), value1, BUF_LEN, 10);
                 XMLString::binToText(fFractionDigits, value2, BUF_LEN, 10);
                 ThrowXML3(InvalidDatatypeFacetException
                                  , XMLExcepts::VALUE_exceed_fractDigit
-                                 , value
+                                 , theData->getRawData()
                                  , value1
                                  , value2);
             }
@@ -523,14 +538,12 @@ void DecimalDatatypeValidator::checkContent( const XMLCh* const content, bool as
         if ( (thisFacetsDefined & DatatypeValidator::FACET_TOTALDIGITS) != 0 )
         {
             if ( theData->getTotalDigit() > fTotalDigits )
-            {
-                XMLCh* value = theData->toString();
-                ArrayJanitor<XMLCh> jan(value);
+            {                
                 XMLString::binToText(theData->getTotalDigit(), value1, BUF_LEN, 10);
                 XMLString::binToText(fTotalDigits, value2, BUF_LEN, 10);
                 ThrowXML3(InvalidDatatypeFacetException
                                  , XMLExcepts::VALUE_exceed_totalDigit
-                                 , value
+                                 , theData->getRawData()
                                  , value1
                                  , value2);
             }
@@ -544,6 +557,8 @@ void DecimalDatatypeValidator::checkContent( const XMLCh* const content, bool as
 
 }
 
+XERCES_CPP_NAMESPACE_END
+
 /**
   * End of file DecimalDatatypeValidator::cpp
   */
