@@ -56,6 +56,15 @@
 
 /*
  * $Log: ComplexTypeInfo.cpp,v $
+ * Revision 1.6  2003/01/29 19:57:30  gareth
+ * API to retrive local and uri part of the type name
+ *
+ * Revision 1.5  2002/12/19 14:04:05  gareth
+ * get/set methods to see if the represented type is anonymous. Patch by Peter Volchek.
+ *
+ * Revision 1.4  2002/11/04 14:49:41  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.3  2002/03/21 15:34:40  knoaman
  * Add support for reporting line/column numbers of schema errors.
  *
@@ -162,6 +171,8 @@
 #include <xercesc/validators/common/SimpleContentModel.hpp>
 #include <xercesc/validators/schema/XSDLocator.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 // ---------------------------------------------------------------------------
 //  ComplexTypeInfo: Constructors and Destructor
 // ---------------------------------------------------------------------------
@@ -177,6 +188,8 @@ ComplexTypeInfo::ComplexTypeInfo()
     , fElementId(XMLElementDecl::fgInvalidElemId)
     , fContentType(SchemaElementDecl::Empty)
     , fTypeName(0)
+    , fTypeLocalName(0)
+    , fTypeUri(0)
     , fBaseDatatypeValidator(0)
     , fDatatypeValidator(0)
     , fBaseComplexTypeInfo(0)
@@ -192,6 +205,7 @@ ComplexTypeInfo::ComplexTypeInfo()
     , fContentSpecOrgURISize(16)
     , fSpecNodesToDelete(0)
     , fLocator(0)
+    , fAnonymous(false)
 {
 
 }
@@ -200,6 +214,8 @@ ComplexTypeInfo::ComplexTypeInfo()
 ComplexTypeInfo::~ComplexTypeInfo()
 {
     delete [] fTypeName;
+    delete [] fTypeLocalName;
+    delete [] fTypeUri;
 
     if (fAdoptContentSpec) {
         delete fContentSpec;
@@ -744,6 +760,7 @@ void ComplexTypeInfo::resizeContentSpecOrgURI() {
     fContentSpecOrgURISize = newSize;
 }
 
+XERCES_CPP_NAMESPACE_END
 
 /**
   * End of file ComplexTypeInfo.cpp
