@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: ComplexTypeInfo.hpp,v 1.4 2002/04/01 15:47:06 knoaman Exp $
+ * $Id: ComplexTypeInfo.hpp,v 1.11 2003/02/05 09:00:18 gareth Exp $
  */
 
 #if !defined(COMPLEXTYPEINFO_HPP)
@@ -79,6 +79,8 @@
 #include <xercesc/framework/XMLContentModel.hpp>
 #include <xercesc/validators/schema/SchemaAttDef.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 
 // ---------------------------------------------------------------------------
 //  Forward Declarations
@@ -130,6 +132,13 @@ public:
     XMLContentModel*         getContentModel(const bool checkUPA = false);
     const XMLCh*             getFormattedContentModel ()   const;
     XSDLocator*              getLocator() const;
+    const XMLCh*             getTypeLocalName() const;
+    const XMLCh*             getTypeUri() const;
+
+    /**
+     * returns true if this type is anonymous
+     **/
+    bool getAnonymous() const;
 
     // -----------------------------------------------------------------------
     //  Setter methods
@@ -155,6 +164,11 @@ public:
     void setContentModel(XMLContentModel* const newModelToAdopt);
     void setLocator(XSDLocator* const aLocator);
 
+    /**
+     * sets this type to be anonymous
+     **/
+    void setAnonymous();
+
     // -----------------------------------------------------------------------
     //  Helper methods
     // -----------------------------------------------------------------------
@@ -210,21 +224,24 @@ private:
     unsigned int                       fElementId;
     int                                fContentType;
     XMLCh*                             fTypeName;
+    XMLCh*                             fTypeLocalName;
+    XMLCh*                             fTypeUri;
     DatatypeValidator*                 fBaseDatatypeValidator;
     DatatypeValidator*                 fDatatypeValidator;
     ComplexTypeInfo*                   fBaseComplexTypeInfo;
     ContentSpecNode*                   fContentSpec;
     SchemaAttDef*                      fAttWildCard;
-    RefHash2KeysTableOf<SchemaAttDef>* fAttDefs;
     SchemaAttDefList*                  fAttList;
     RefVectorOf<SchemaElementDecl>*    fElements;
+    RefVectorOf<ContentSpecNode>*      fSpecNodesToDelete;
+    RefHash2KeysTableOf<SchemaAttDef>* fAttDefs;
     XMLContentModel*                   fContentModel;
     XMLCh*                             fFormattedModel;
     unsigned int*                      fContentSpecOrgURI;
     unsigned int                       fUniqueURI;
     unsigned int                       fContentSpecOrgURISize;
-    RefVectorOf<ContentSpecNode>*      fSpecNodesToDelete;
     XSDLocator*                        fLocator;
+    bool                               fAnonymous;
 };
 
 // ---------------------------------------------------------------------------
@@ -290,7 +307,6 @@ inline unsigned int ComplexTypeInfo::elementCount() const {
 }
 
 inline XMLCh* ComplexTypeInfo::getTypeName() const {
-
     return fTypeName;
 }
 
@@ -377,6 +393,34 @@ inline XSDLocator* ComplexTypeInfo::getLocator() const
     return fLocator;
 }
 
+inline bool ComplexTypeInfo::getAnonymous() const {
+    return fAnonymous;
+}
+
+inline const XMLCh* ComplexTypeInfo::getTypeLocalName() const
+{
+    if(!fTypeLocalName) {
+        int index = XMLString::indexOf(fTypeName, chComma);
+        int length = XMLString::stringLen(fTypeName);
+        XMLCh *tName = new XMLCh[length - index + 1];
+        XMLString::subString(tName, fTypeName, index + 1, length);
+        ((ComplexTypeInfo *)this)->fTypeLocalName = tName;
+    }
+
+    return fTypeLocalName;
+}
+
+inline const XMLCh* ComplexTypeInfo::getTypeUri() const
+{
+    if(!fTypeUri) {
+        int index = XMLString::indexOf(fTypeName, chComma);
+        XMLCh *uri = new XMLCh[index + 1];
+        XMLString::subString(uri, fTypeName, 0, index);
+        ((ComplexTypeInfo *)this)->fTypeUri = uri;
+    }
+
+   return fTypeUri;
+}
 
 // ---------------------------------------------------------------------------
 //  ComplexTypeInfo: Setter methods
@@ -434,9 +478,11 @@ ComplexTypeInfo::setContentType(const int contentType) {
 
 inline void ComplexTypeInfo::setTypeName(const XMLCh* const typeName) {
 
-    if (fTypeName != 0) {
-        delete [] fTypeName;
-    }
+    delete [] fTypeName;
+    delete [] fTypeLocalName;
+    fTypeLocalName = 0;
+    delete [] fTypeUri;
+    fTypeUri = 0;
 
     fTypeName = XMLString::replicate(typeName);
 }
@@ -488,6 +534,11 @@ ComplexTypeInfo::setContentModel(XMLContentModel* const newModelToAdopt)
 }
 
 
+
+inline void ComplexTypeInfo::setAnonymous() {
+    fAnonymous = true;
+}
+
 // ---------------------------------------------------------------------------
 //  ComplexTypeInfo: Helper methods
 // ---------------------------------------------------------------------------
@@ -510,8 +561,7 @@ inline bool ComplexTypeInfo::contains(const XMLCh* const attName) {
 
     while (enumDefs.hasMoreElements()) {
 
-        if (XMLString::compareString(attName,
-                enumDefs.nextElement().getAttName()->getLocalPart()) == 0) {
+        if (XMLString::equals(attName, enumDefs.nextElement().getAttName()->getLocalPart())) {
             return true;
         }
     }
@@ -519,6 +569,8 @@ inline bool ComplexTypeInfo::contains(const XMLCh* const attName) {
     return false;
 }
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
 
 /**
