@@ -56,6 +56,28 @@
 
 /*
  * $Log: GeneralAttributeCheck.cpp,v $
+ * Revision 1.12  2003/01/17 15:35:57  knoaman
+ * Fix for attribute checking of schema declarations.
+ *
+ * Revision 1.11  2003/01/15 18:33:11  knoaman
+ * Make sure that the namespace of schema declarations is the schema namespace.
+ *
+ * Revision 1.10  2002/12/10 16:58:22  knoaman
+ * Schema Errata E1-16.
+ *
+ * Revision 1.9  2002/11/04 14:49:41  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.8  2002/09/27 13:22:52  tng
+ * [Bug 13073] GeneralAttributeCheck.cpp : compilation fails with Sun C++ 4.2 on Solaris2.7 system.
+ *
+ * Revision 1.7  2002/09/26 21:08:14  tng
+ * [Bug12849] comparison is always false warning.  Patch from Gareth Reakes
+ *
+ * Revision 1.6  2002/09/24 20:18:14  tng
+ * Performance: use XMLString::equals instead of XMLString::compareString
+ * and check for null string directly isntead of calling XMLString::stringLen
+ *
  * Revision 1.5  2002/05/27 19:54:33  knoaman
  * Performance: use pre-built element-attribute map table.
  *
@@ -138,6 +160,8 @@
 #include <xercesc/util/XMLRegisterCleanup.hpp>
 #include <xercesc/validators/datatype/DatatypeValidatorFactory.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 // ---------------------------------------------------------------------------
 //  Local const data
 // ---------------------------------------------------------------------------
@@ -318,12 +342,22 @@ GeneralAttributeCheck::checkAttributes(const DOMElement* const elem,
                                        TraverseSchema* const schema,
                                        const bool isTopLevel) {
 
-    if (elem == 0 || !fAttMap || elemContext<0 || elemContext>=E_Count) {
+    if (elem == 0 || !fAttMap || elemContext>=E_Count) {
         return;
     }
 
+    const XMLCh* elemName = elem->getLocalName();
+    if (!XMLString::equals(SchemaSymbols::fgURI_SCHEMAFORSCHEMA, elem->getNamespaceURI())) {
+        schema->reportSchemaError
+        (
+            elem
+            , XMLUni::fgXMLErrDomain
+            , XMLErrs::ELTSchemaNS
+            , elemName
+        );
+    }
+
     const XMLCh*     contextStr = (isTopLevel) ? fgGlobal : fgLocal;
-    const XMLCh*     elemName = elem->getLocalName();
     DOMNamedNodeMap* eltAttrs = elem->getAttributes();
     int              attrCount = eltAttrs->getLength();
     XMLByte          attList[A_Count];
@@ -345,13 +379,13 @@ GeneralAttributeCheck::checkAttributes(const DOMElement* const elem,
         // for attributes with namespace prefix
         const XMLCh* attrURI = attribute->getNamespaceURI();
 
-        if (attrURI != 0 && XMLString::stringLen(attrURI) != 0) {
+        if (attrURI != 0 && *attrURI) {
 
             // attributes with schema namespace are not allowed
             // and not allowed on "documentation" and "appInfo"
-            if (!XMLString::compareString(attrURI, SchemaSymbols::fgURI_SCHEMAFORSCHEMA) ||
-                !XMLString::compareString(elemName, SchemaSymbols::fgELT_APPINFO) ||
-                !XMLString::compareString(elemName, SchemaSymbols::fgELT_DOCUMENTATION)) {
+            if (XMLString::equals(attrURI, SchemaSymbols::fgURI_SCHEMAFORSCHEMA) ||
+                XMLString::equals(elemName, SchemaSymbols::fgELT_APPINFO) ||
+                XMLString::equals(elemName, SchemaSymbols::fgELT_DOCUMENTATION)) {
 
                 schema->reportSchemaError(elem, XMLUni::fgXMLErrDomain,
                     XMLErrs::AttributeDisallowed, attName, contextStr, elemName);
@@ -395,15 +429,17 @@ GeneralAttributeCheck::checkAttributes(const DOMElement* const elem,
             continue;
         }
 
-        const XMLCh* attrVal = attribute->getNodeValue();
-
         if (fgElemAttTable[elemContext][attNameId] & Att_Mask) {
 
             attList[attNameId] = 1;
-
-            if (XMLString::stringLen(attrVal)) {
-                validate(elem, attName, attrVal, fgElemAttTable[elemContext][attNameId] & DV_Mask, schema);
-            }
+            validate
+            (
+                elem
+                , attName
+                , attribute->getNodeValue()
+                , fgElemAttTable[elemContext][attNameId] & DV_Mask
+                , schema
+            );
         }
         else {
             schema->reportSchemaError(elem, XMLUni::fgXMLErrDomain,
@@ -435,46 +471,46 @@ void GeneralAttributeCheck::validate(const DOMElement* const elem,
 
     switch (dvIndex) {
     case DV_Form:
-        if (XMLString::compareString(attValue, SchemaSymbols::fgATTVAL_QUALIFIED) != 0
-            && XMLString::compareString(attValue, SchemaSymbols::fgATTVAL_UNQUALIFIED) != 0) {
+        if (!XMLString::equals(attValue, SchemaSymbols::fgATTVAL_QUALIFIED)
+            && !XMLString::equals(attValue, SchemaSymbols::fgATTVAL_UNQUALIFIED)) {
             isInvalid = true;
         }
         break;
     case DV_MaxOccurs:
             // maxOccurs = (nonNegativeInteger | unbounded)
-        if (XMLString::compareString(attValue, fgUnbounded) != 0) {
+        if (!XMLString::equals(attValue, fgUnbounded)) {
             dv = fNonNegIntDV;
         }
         break;
     case DV_MaxOccurs1:
-        if (XMLString::compareString(attValue, fgValueOne) != 0) {
+        if (!XMLString::equals(attValue, fgValueOne)) {
             isInvalid = true;
         }
         break;
     case DV_MinOccurs1:
-        if (XMLString::compareString(attValue, fgValueZero) != 0
-            && XMLString::compareString(attValue, fgValueOne) != 0) {
+        if (!XMLString::equals(attValue, fgValueZero)
+            && !XMLString::equals(attValue, fgValueOne)) {
             isInvalid = true;
         }
         break;
     case DV_ProcessContents:
-        if (XMLString::compareString(attValue, SchemaSymbols::fgATTVAL_SKIP) != 0
-            && XMLString::compareString(attValue, SchemaSymbols::fgATTVAL_LAX) != 0
-            && XMLString::compareString(attValue, SchemaSymbols::fgATTVAL_STRICT) != 0) {
+        if (!XMLString::equals(attValue, SchemaSymbols::fgATTVAL_SKIP)
+            && !XMLString::equals(attValue, SchemaSymbols::fgATTVAL_LAX)
+            && !XMLString::equals(attValue, SchemaSymbols::fgATTVAL_STRICT)) {
             isInvalid = true;
         }
         break;
     case DV_Use:
-        if (XMLString::compareString(attValue, SchemaSymbols::fgATTVAL_OPTIONAL) != 0
-            && XMLString::compareString(attValue, SchemaSymbols::fgATTVAL_PROHIBITED) != 0
-            && XMLString::compareString(attValue, SchemaSymbols::fgATTVAL_REQUIRED) != 0) {
+        if (!XMLString::equals(attValue, SchemaSymbols::fgATTVAL_OPTIONAL)
+            && !XMLString::equals(attValue, SchemaSymbols::fgATTVAL_PROHIBITED)
+            && !XMLString::equals(attValue, SchemaSymbols::fgATTVAL_REQUIRED)) {
             isInvalid = true;
         }
         break;
     case DV_WhiteSpace:
-        if (XMLString::compareString(attValue, SchemaSymbols::fgWS_PRESERVE) != 0
-            && XMLString::compareString(attValue, SchemaSymbols::fgWS_REPLACE) != 0
-            && XMLString::compareString(attValue, SchemaSymbols::fgWS_COLLAPSE) != 0) {
+        if (!XMLString::equals(attValue, SchemaSymbols::fgWS_PRESERVE)
+            && !XMLString::equals(attValue, SchemaSymbols::fgWS_REPLACE)
+            && !XMLString::equals(attValue, SchemaSymbols::fgWS_COLLAPSE)) {
             isInvalid = true;
         }
         break;
@@ -550,7 +586,7 @@ void GeneralAttributeCheck::initCharFlagTable()
     attList[E_All][A_ID] = Att_Optional | DV_ID;
     attList[E_All][A_MaxOccurs] = Att_Optional | DV_MaxOccurs1;
     attList[E_All][A_MinOccurs] = Att_Optional | DV_MinOccurs1;
-    
+
     // "annotation"
     attList[E_Annotation][A_ID] = Att_Optional | DV_ID;
 
@@ -741,7 +777,7 @@ void GeneralAttributeCheck::initCharFlagTable()
     // "notation"
     attList[E_Notation][A_ID] = Att_Optional | DV_ID;
     attList[E_Notation][A_Name] = Att_Required;
-    attList[E_Notation][A_Public] = Att_Required;
+    attList[E_Notation][A_Public] = Att_Optional;
     attList[E_Notation][A_System] = Att_Optional | DV_AnyURI;
 
     // "pattern"
@@ -825,7 +861,7 @@ void GeneralAttributeCheck::initCharFlagTable()
 #endif
 
 
-unsigned short GeneralAttributeCheck::fgElemAttTable[E_Count][A_Count] =
+unsigned short GeneralAttributeCheck::fgElemAttTable[GeneralAttributeCheck::E_Count][GeneralAttributeCheck::A_Count] =
 {
   { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 34, 0, 258, 0, 514, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
   { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 34, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
@@ -917,6 +953,8 @@ const XMLCh* GeneralAttributeCheck::fAttNames[A_Count] =
     SchemaSymbols::fgATT_XPATH,
 };
 
+XERCES_CPP_NAMESPACE_END
+
 /**
   * End of file GeneralAttributeCheck.cpp
   */
