@@ -56,6 +56,12 @@
 
 /*
  * $Log: SchemaElementDecl.cpp,v $
+ * Revision 1.7  2003/01/29 19:47:16  gareth
+ * added DOMTypeInfo and some PSVI methods
+ *
+ * Revision 1.6  2002/11/04 14:49:41  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.5  2002/07/12 15:17:48  knoaman
  * For a given global element, store info about a substitution group element
  * as a SchemaElementDecl and not as a string.
@@ -140,6 +146,8 @@
 #include <xercesc/validators/schema/SchemaElementDecl.hpp>
 #include <xercesc/validators/schema/identity/IdentityConstraint.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 // ---------------------------------------------------------------------------
 //  SchemaElementDecl: Constructors and Destructor
 // ---------------------------------------------------------------------------
@@ -154,10 +162,16 @@ SchemaElementDecl::SchemaElementDecl() :
     , fDefaultValue(0)
     , fComplexTypeInfo(0)
     , fXsiComplexTypeInfo(0)
+    , fXsiSimpleTypeInfo(0)
     , fAttDefs(0)
     , fIdentityConstraints(0)
     , fAttWildCard(0)
     , fSubstitutionGroupElem(0)
+    , fValidity(PSVIDefs::UNKNOWN)
+    , fValidation(PSVIDefs::NONE)
+    , fSeenValidation(false)
+    , fSeenNoValidation(false)
+    , fHadContent(false)
 {
 }
 
@@ -176,10 +190,16 @@ SchemaElementDecl::SchemaElementDecl(const XMLCh* const                  prefix
     , fDefaultValue(0)
     , fComplexTypeInfo(0)
     , fXsiComplexTypeInfo(0)
+    , fXsiSimpleTypeInfo(0)
     , fAttDefs(0)
     , fIdentityConstraints(0)
     , fAttWildCard(0)
     , fSubstitutionGroupElem(0)
+    , fValidity(PSVIDefs::UNKNOWN)
+    , fValidation(PSVIDefs::NONE)
+    , fSeenValidation(false)
+    , fSeenNoValidation(false)
+    , fHadContent(false)
 {
     setElementName(prefix, localPart, uriId);
 }
@@ -197,10 +217,16 @@ SchemaElementDecl::SchemaElementDecl(const QName* const                  element
     , fDefaultValue(0)
     , fComplexTypeInfo(0)
     , fXsiComplexTypeInfo(0)
+    , fXsiSimpleTypeInfo(0)
     , fAttDefs(0)
     , fIdentityConstraints(0)
     , fAttWildCard(0)
     , fSubstitutionGroupElem(0)
+    , fValidity(PSVIDefs::UNKNOWN)
+    , fValidation(PSVIDefs::NONE)
+    , fSeenValidation(false)
+    , fSeenNoValidation(false)
+    , fHadContent(false)
 {
     setElementName(elementName);
 }
@@ -271,7 +297,7 @@ XMLAttDefList& SchemaElementDecl::getAttDefList() const
     if (fXsiComplexTypeInfo) {
         return fXsiComplexTypeInfo->getAttDefList();
     }
-    else if (!fComplexTypeInfo) 
+    else if (!fComplexTypeInfo)
 	{
         ThrowXML(RuntimeException, XMLExcepts::DV_InvalidOperation);
     }
@@ -385,3 +411,4 @@ SchemaAttDef* SchemaElementDecl::getAttDef(const XMLCh* const baseName, const in
     return 0;
 }
 
+XERCES_CPP_NAMESPACE_END
