@@ -56,6 +56,22 @@
 
 /*
  * $Log: SchemaElementDecl.hpp,v $
+ * Revision 1.8  2003/01/29 19:47:16  gareth
+ * added DOMTypeInfo and some PSVI methods
+ *
+ * Revision 1.7  2002/11/04 14:49:41  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.6  2002/07/12 15:17:48  knoaman
+ * For a given global element, store info about a substitution group element
+ * as a SchemaElementDecl and not as a string.
+ *
+ * Revision 1.5  2002/04/01 15:47:06  knoaman
+ * Move Element Consistency checking (ref to global declarations) to SchemaValidator.
+ *
+ * Revision 1.4  2002/03/21 16:31:43  knoaman
+ * Remove data/methods from SchemaElementDecl that are not used.
+ *
  * Revision 1.3  2002/03/04 15:09:50  knoaman
  * Fix for bug 6834.
  *
@@ -123,10 +139,14 @@
 #include <xercesc/validators/common/Grammar.hpp>
 #include <xercesc/validators/schema/ComplexTypeInfo.hpp>
 #include <xercesc/validators/schema/identity/IdentityConstraint.hpp>
+#include <xercesc/validators/datatype/DatatypeValidator.hpp>
+#include <xercesc/validators/datatype/UnionDatatypeValidator.hpp>
+#include <xercesc/validators/schema/PSVIDefs.hpp>
+
+XERCES_CPP_NAMESPACE_BEGIN
 
 class ContentSpecNode;
 class SchemaAttDefList;
-class DatatypeValidator;
 
 //
 //  This class is a derivative of the basic element decl. This one implements
@@ -135,6 +155,7 @@ class DatatypeValidator;
 class VALIDATORS_EXPORT SchemaElementDecl : public XMLElementDecl
 {
 public :
+
     // -----------------------------------------------------------------------
     //  Class specific types
     //
@@ -213,14 +234,96 @@ public :
     ModelTypes getModelType() const;
     DatatypeValidator* getDatatypeValidator() const;
     int getEnclosingScope() const;
-    int getDefinedScope() const;
     int getFinalSet() const;
     int getBlockSet() const;
     int getMiscFlags() const;
     XMLCh* getDefaultValue() const;
-    XMLCh* getSubstitutionGroupName() const;
-    XMLCh* getTypeFromAnotherSchemaURI() const;
     ComplexTypeInfo* getComplexTypeInfo() const;
+    virtual bool isGlobalDecl() const;
+    SchemaElementDecl* getSubstitutionGroupElem() const;
+
+
+    // ----------------------------------------------------------------------
+    // Partial implementation of PSVI
+    // The values these methods return are only accurate until the cleanUp method
+    // is called (in the end tag part of the scanner you are using)
+    // note that some of this information has dependancies. For example,
+    // if something is not valid then the information returned by the other 
+    // calls may be meaningless
+    // See http://www.w3.org/TR/xmlschema-1/ for detailed information
+    // ----------------------------------------------------------------------
+
+
+    /**
+     * The appropriate case among the following:
+     * 1 If it was strictly assessed, then the appropriate case among the following:
+     * 1.1 If all of the following are true
+     *    1.1.1
+     *    1.1.1.1 clause 1.1 of Schema-Validity Assessment (Element) (3.3.4) applied and the item was valid as defined by Element Locally Valid (Element) (3.3.4);
+     *    1.1.1.2 clause 1.2 of Schema-Validity Assessment (Element) (3.3.4) applied and the item was valid as defined by Element Locally Valid (Type) (3.3.4).
+     *    1.1.2 Neither its [children] nor its [attributes] contains an information item (element or attribute respectively) whose [validity] is invalid.
+     *    1.1.3 Neither its [children] nor its [attributes] contains an information item (element or attribute respectively) with a context-determined declaration of mustFind whose [validity] is unknown.
+    , then valid;
+    *    1.2 otherwise invalid.
+    *    2 otherwise notKnown.
+    */
+    PSVIDefs::Validity getValidity() const;
+
+
+    /**
+     * The appropriate case among the following:
+     * 1 If it was strictly assessed and neither its [children] nor its [attributes] contains an information item (element or attribute respectively) whose [validation attempted] is not full, then full;
+     * 2 If it was not strictly assessed and neither its [children] nor its [attributes] contains an information item (element or attribute respectively) whose [validation attempted] is not none, then none;
+     *3 otherwise partial.
+     */
+    PSVIDefs::Validation getValidationAttempted() const;
+
+
+    /**
+     * @return the complexity. simple or complex, depending on the type definition.
+     */
+    PSVIDefs::Complexity getTypeType() const;
+
+    /**
+     * The target namespace of the type definition.
+     */
+    const XMLCh* getTypeUri() const;
+
+    /**
+     * The {name} of the type definition, if it is not absent. 
+     */
+    const XMLCh* getTypeName() const;
+
+    /**
+     * true if the {name} of the type definition is absent, otherwise false.
+     */
+    bool getTypeAnonymous() const;
+
+    /**
+     * If this method returns true and validity is VALID then the next three 
+     * produce accurate results
+     * @return true if the element is validated using a union type
+     */
+    bool isTypeDefinitionUnion() const;
+
+    /**
+     * The {target namespace} of the actual member type definition.
+     */
+    const XMLCh* getMemberTypeUri() const;
+
+    /**
+     * @return true if the {name} of the actual member type definition is absent, otherwise false.
+     */
+    bool getMemberTypeAnonymous() const;
+
+    /**
+     * @return the {name} of the actual member type definition, if it is not absent. 
+     */
+    const XMLCh* getMemberTypeName() const;
+
+
+    virtual const XMLCh* getDOMTypeInfoUri() const;
+    virtual const XMLCh* getDOMTypeInfoName() const;
 
 
     // -----------------------------------------------------------------------
@@ -229,16 +332,26 @@ public :
     void setModelType(const SchemaElementDecl::ModelTypes toSet);
     void setDatatypeValidator(DatatypeValidator* newDatatypeValidator);
     void setEnclosingScope(const int enclosingScope);
-    void setDefinedScope(const int definedScope);
     void setFinalSet(const int finalSet);
     void setBlockSet(const int blockSet);
     void setMiscFlags(const int flags);
     void setDefaultValue(const XMLCh* const value);
-    void setSubstitutionGroupName(const XMLCh* const name);
-    void setTypeFromAnotherSchemaURI(const XMLCh* const uriStr);
     void setComplexTypeInfo(ComplexTypeInfo* const typeInfo);
     void setXsiComplexTypeInfo(ComplexTypeInfo* const typeInfo);
+    void setXsiSimpleTypeInfo(const DatatypeValidator* const dtv);
     void setAttWildCard(SchemaAttDef* const attWildCard);
+    void setSubstitutionGroupElem(SchemaElementDecl* const elemDecl);
+    void setValidity(PSVIDefs::Validity valid);
+    void setValidationAttempted(PSVIDefs::Validation validation);
+    
+    //called when element content of this element was validated
+    void updateValidityFromElement(const XMLElementDecl *decl, Grammar::GrammarType eleGrammar);
+    
+    //called when attribute content of this element was validated    
+    void updateValidityFromAttribute(const SchemaAttDef *def);
+
+    //cleans up inbetween uses of the SchemaElementDecl. Resets xsiType, Validity etc.
+    void reset();
 
     // -----------------------------------------------------------------------
     //  IC methods
@@ -261,9 +374,6 @@ private :
     //  fEnclosingScope
     //      The enclosing scope where this element is declared.
     //
-    //  fDefinedScope
-    //      The Complex Element that this element belongs to.
-    //
     //  fFinalSet
     //      The value set of the 'final' attribute.
     //
@@ -276,12 +386,6 @@ private :
     //  fDefaultValue
     //      The defalut/fixed value
     //
-    //  fSubstitutionGroupName
-    //      The substitution group full name ("uristring','local")
-    //
-    //  fTypeFromAnotherSchemaURI
-    //      The URI of type if it belongs to a different schema
-    //
     //  fComplexTypeInfo
     //      Stores complex type information
     //      (no need to delete - handled by schema grammar)
@@ -296,28 +400,56 @@ private :
     //      Temporary store the xsi:type ComplexType here for validation
     //      If it presents, then it takes precedence than its own fComplexTypeInfo.
     //
+    //  fXsiSimpleTypeInfo
+    //      Temporary store the xsi:type SimpleType here for validation
+    //      If it present then the information from it will be returned rather than fDatatypeValidator
+    //
     //  fIdentityConstraints
     //      Store information about an element identity constraints.
     //
     //  fAttWildCard
     //      Store wildcard attribute in the case of an element with a type of
     //      'anyType'.
+    //
+    //  fSubstitutionGroupElem
+    //      The substitution group element declaration.
+    //
+    //  fValidity
+    //      After this attr has been validated this is its validity
+    //
+    //  fValidation
+    //      The type of validation that happened to this attr
+    //
+    //  fSeenValidation
+    //      set to true when a piece of content of this element is validated 
+    //
+    //  fSeenNoValidation
+    //      set to true when a piece of content of this element is laxly or skip validated
+    //
+    //  fHadContent
+    //      true when this element actually had content.
+    // -----------------------------------------------------------------------
+
     // -----------------------------------------------------------------------
     ModelTypes                         fModelType;
     DatatypeValidator*                 fDatatypeValidator;
     int                                fEnclosingScope;
-    int                                fDefinedScope;
     int                                fFinalSet;
     int                                fBlockSet;
     int                                fMiscFlags;
     XMLCh*                             fDefaultValue;
-    XMLCh*                             fSubstitutionGroupName;
-    XMLCh*                             fTypeFromAnotherSchemaURI;
     ComplexTypeInfo*                   fComplexTypeInfo;
     RefHash2KeysTableOf<SchemaAttDef>* fAttDefs;
     ComplexTypeInfo*                   fXsiComplexTypeInfo;
+    const DatatypeValidator*           fXsiSimpleTypeInfo;
     RefVectorOf<IdentityConstraint>*   fIdentityConstraints;
     SchemaAttDef*                      fAttWildCard;
+    SchemaElementDecl*                 fSubstitutionGroupElem;
+    PSVIDefs::Validity                 fValidity;
+    PSVIDefs::Validation               fValidation;
+    bool                               fSeenValidation;
+    bool                               fSeenNoValidation;
+    bool                               fHadContent;
 };
 
 // ---------------------------------------------------------------------------
@@ -400,11 +532,6 @@ inline int SchemaElementDecl::getEnclosingScope() const
     return fEnclosingScope;
 }
 
-inline int SchemaElementDecl::getDefinedScope() const
-{
-    return fDefinedScope;
-}
-
 inline int SchemaElementDecl::getFinalSet() const
 {
     return fFinalSet;
@@ -425,17 +552,6 @@ inline XMLCh* SchemaElementDecl::getDefaultValue() const
     return fDefaultValue;
 }
 
-inline XMLCh* SchemaElementDecl::getSubstitutionGroupName() const
-{
-
-    return fSubstitutionGroupName;
-}
-
-inline XMLCh* SchemaElementDecl::getTypeFromAnotherSchemaURI() const {
-
-    return fTypeFromAnotherSchemaURI;
-}
-
 inline ComplexTypeInfo* SchemaElementDecl::getComplexTypeInfo() const
 {
     if (fXsiComplexTypeInfo) {
@@ -469,6 +585,167 @@ inline SchemaAttDef* SchemaElementDecl::getAttWildCard() {
     return fAttWildCard;
 }
 
+inline bool SchemaElementDecl::isGlobalDecl() const {
+
+    return (fEnclosingScope == Grammar::TOP_LEVEL_SCOPE);
+}
+
+inline SchemaElementDecl*
+SchemaElementDecl::getSubstitutionGroupElem() const {
+
+    return fSubstitutionGroupElem;
+}
+
+inline const XMLCh* SchemaElementDecl::getTypeName() const {
+    if (fXsiComplexTypeInfo)
+        return fXsiComplexTypeInfo->getTypeLocalName();
+    else if (fComplexTypeInfo) 
+        return fComplexTypeInfo->getTypeLocalName();
+    else if(fXsiSimpleTypeInfo)
+        return fXsiSimpleTypeInfo->getTypeLocalName();
+    else if(fDatatypeValidator)
+        return fDatatypeValidator->getTypeLocalName();
+
+    //its anyType if we have not done validation on it
+    if(getValidationAttempted() == PSVIDefs::NONE)
+        return SchemaSymbols::fgATTVAL_ANYTYPE;
+
+
+    return 0;
+}
+
+inline PSVIDefs::Complexity SchemaElementDecl::getTypeType() const {
+    if(getModelType() == Simple) {
+        return PSVIDefs::SIMPLE;
+    }
+    else {
+        return PSVIDefs::COMPLEX;
+    }
+}
+
+
+inline const XMLCh* SchemaElementDecl::getTypeUri() const {
+    if (fXsiComplexTypeInfo)
+        return fXsiComplexTypeInfo->getTypeUri();
+    else if (fComplexTypeInfo)
+        return fComplexTypeInfo->getTypeUri();
+    else if(fXsiSimpleTypeInfo)
+        return fXsiSimpleTypeInfo->getTypeUri();
+    else if(fDatatypeValidator)
+        return fDatatypeValidator->getTypeUri();
+
+    //its anyType if we have not done validation on it
+    if(getValidationAttempted() == PSVIDefs::NONE)
+        return SchemaSymbols::fgURI_SCHEMAFORSCHEMA;
+
+
+    return 0;
+}
+
+inline const XMLCh* SchemaElementDecl::getMemberTypeName() const {
+    if(fXsiSimpleTypeInfo && fXsiSimpleTypeInfo->getType() == DatatypeValidator::Union)
+        return ((UnionDatatypeValidator*)fXsiSimpleTypeInfo)->getMemberTypeName();
+    else if(fDatatypeValidator && fDatatypeValidator->getType() == DatatypeValidator::Union)
+        return ((UnionDatatypeValidator*)fDatatypeValidator)->getMemberTypeName();
+    return 0;
+}
+
+inline const XMLCh* SchemaElementDecl::getMemberTypeUri() const {
+    if(fXsiSimpleTypeInfo && fXsiSimpleTypeInfo->getType() == DatatypeValidator::Union)
+        return ((UnionDatatypeValidator*)fXsiSimpleTypeInfo)->getMemberTypeUri();
+    if(fDatatypeValidator && fDatatypeValidator->getType() == DatatypeValidator::Union)
+        return ((UnionDatatypeValidator*)fDatatypeValidator)->getMemberTypeUri();
+
+    return 0;
+}
+
+inline bool SchemaElementDecl::getMemberTypeAnonymous() const {
+    if(fXsiSimpleTypeInfo && fXsiSimpleTypeInfo->getType() == DatatypeValidator::Union)
+        return ((UnionDatatypeValidator*)fXsiSimpleTypeInfo)->getMemberTypeAnonymous();
+    else if(fDatatypeValidator && fDatatypeValidator->getType() == DatatypeValidator::Union)
+        return ((UnionDatatypeValidator*)fDatatypeValidator)->getMemberTypeAnonymous();
+    return false;
+}
+
+inline bool SchemaElementDecl::isTypeDefinitionUnion() const {
+   if(fXsiSimpleTypeInfo && fXsiSimpleTypeInfo->getType() == DatatypeValidator::Union ||
+      fDatatypeValidator && fDatatypeValidator->getType() == DatatypeValidator::Union)
+       return true;
+    return false;
+}
+
+inline PSVIDefs::Validity SchemaElementDecl::getValidity() const {
+    return fValidity;
+}
+
+inline PSVIDefs::Validation SchemaElementDecl::getValidationAttempted() const {
+    if(!fHadContent)
+        return fValidation;
+
+    if(!fSeenNoValidation && fSeenValidation)
+        return PSVIDefs::FULL;
+    else if(fSeenNoValidation && !fSeenValidation)
+        return PSVIDefs::NONE;
+    else
+        return PSVIDefs::PARTIAL;
+}
+
+inline bool SchemaElementDecl::getTypeAnonymous() const {
+    
+    //REVISIT - since xsi type have to be accessed through names 
+    //presumeably they cannot be anonymous
+    
+    if (fXsiComplexTypeInfo) {
+        return fXsiComplexTypeInfo->getAnonymous();
+    }
+    else if (fComplexTypeInfo) {
+        return fComplexTypeInfo->getAnonymous();
+    }
+    else if(fXsiSimpleTypeInfo) {
+        return fXsiSimpleTypeInfo->getAnonymous();
+    }
+    else if(fDatatypeValidator){
+        return fDatatypeValidator->getAnonymous();
+    }
+
+    return true;
+}
+
+inline const XMLCh* SchemaElementDecl::getDOMTypeInfoName() const {
+    if(fValidity != PSVIDefs::VALID) {
+        if(getTypeType() == PSVIDefs::SIMPLE)
+            return SchemaSymbols::fgDT_ANYSIMPLETYPE;
+        else
+            return SchemaSymbols::fgATTVAL_ANYTYPE;
+    }
+
+    if(getTypeAnonymous() || getMemberTypeAnonymous())
+        return 0;
+    if(fDatatypeValidator && fDatatypeValidator->getType() == DatatypeValidator::Union)
+        return ((UnionDatatypeValidator*)fDatatypeValidator)->getMemberTypeName();
+    if(fXsiSimpleTypeInfo && fXsiSimpleTypeInfo->getType() == DatatypeValidator::Union)
+        return ((UnionDatatypeValidator*)fXsiSimpleTypeInfo)->getMemberTypeName();
+    return getTypeName();
+}
+
+inline const XMLCh* SchemaElementDecl::getDOMTypeInfoUri() const {
+
+    if(fValidity != PSVIDefs::VALID)
+        return SchemaSymbols::fgURI_SCHEMAFORSCHEMA;
+
+    if(getTypeAnonymous() || getMemberTypeAnonymous())
+        return 0;
+
+    if(fDatatypeValidator && fDatatypeValidator->getType() == DatatypeValidator::Union)
+        return ((UnionDatatypeValidator*)fDatatypeValidator)->getMemberTypeUri();
+
+    if(fXsiSimpleTypeInfo && fXsiSimpleTypeInfo->getType() == DatatypeValidator::Union)
+        return ((UnionDatatypeValidator*)fXsiSimpleTypeInfo)->getMemberTypeUri();
+
+
+    return getTypeUri();
+}
+
 // ---------------------------------------------------------------------------
 //  SchemaElementDecl: Setter methods
 // ---------------------------------------------------------------------------
@@ -488,11 +765,6 @@ inline void SchemaElementDecl::setEnclosingScope(const int newEnclosingScope)
     fEnclosingScope = newEnclosingScope;
 }
 
-inline void SchemaElementDecl::setDefinedScope(const int newDefinedScope)
-{
-    fDefinedScope = newDefinedScope;
-}
-
 inline void SchemaElementDecl::setFinalSet(const int finalSet)
 {
     fFinalSet = finalSet;
@@ -517,25 +789,6 @@ inline void SchemaElementDecl::setDefaultValue(const XMLCh* const value)
     fDefaultValue = XMLString::replicate(value);
 }
 
-inline void SchemaElementDecl::setSubstitutionGroupName(const XMLCh* const name)
-{
-    if (fSubstitutionGroupName) {
-        delete [] fSubstitutionGroupName;
-    }
-
-    fSubstitutionGroupName = XMLString::replicate(name);
-}
-
-inline void
-SchemaElementDecl::setTypeFromAnotherSchemaURI(const XMLCh* const uriStr) {
-
-    if (fTypeFromAnotherSchemaURI) {
-        delete [] fTypeFromAnotherSchemaURI;
-    }
-
-    fTypeFromAnotherSchemaURI = XMLString::replicate(uriStr);
-}
-
 inline void
 SchemaElementDecl::setComplexTypeInfo(ComplexTypeInfo* const typeInfo)
 {
@@ -548,6 +801,12 @@ SchemaElementDecl::setXsiComplexTypeInfo(ComplexTypeInfo* const typeInfo)
     fXsiComplexTypeInfo = typeInfo;
 }
 
+inline void
+SchemaElementDecl::setXsiSimpleTypeInfo(const DatatypeValidator* const dtv)
+{
+    fXsiSimpleTypeInfo = dtv;
+}
+
 inline void
 SchemaElementDecl::setAttWildCard(SchemaAttDef* const attWildCard) {
 
@@ -557,6 +816,76 @@ SchemaElementDecl::setAttWildCard(SchemaAttDef* const attWildCard) {
     fAttWildCard = attWildCard;
 }
 
+inline void
+SchemaElementDecl::setSubstitutionGroupElem(SchemaElementDecl* const elemDecl) {
+
+    fSubstitutionGroupElem = elemDecl;
+}
+
+inline void SchemaElementDecl::setValidity(PSVIDefs::Validity valid) {
+    fValidity = valid;
+}
+
+inline void SchemaElementDecl::setValidationAttempted(PSVIDefs::Validation validation) {
+    fValidation = validation;
+}
+
+inline void SchemaElementDecl::updateValidityFromAttribute(const SchemaAttDef *def) {
+
+    PSVIDefs::Validation curValAttemted = def->getValidationAttempted();
+    PSVIDefs::Validity curVal = def->getValidity();
+        
+    if(curValAttemted == PSVIDefs::NONE || curValAttemted == PSVIDefs::PARTIAL) {
+        fSeenNoValidation = true;
+        fValidity = PSVIDefs::UNKNOWN;
+    }
+    else {
+        fSeenValidation = true;
+    }
+        
+    if(curVal == PSVIDefs::INVALID)
+        fValidity = PSVIDefs::INVALID;
+
+    fHadContent = true;
+}
+
+inline void SchemaElementDecl::updateValidityFromElement(const XMLElementDecl *decl, Grammar::GrammarType eleGrammar) {
+
+    if (eleGrammar == Grammar::SchemaGrammarType) {                    
+        PSVIDefs::Validation curValAttemted = ((SchemaElementDecl *)decl)->getValidationAttempted();
+        PSVIDefs::Validity curVal = ((SchemaElementDecl *)decl)->getValidity();
+        
+        if(curValAttemted == PSVIDefs::NONE || curValAttemted == PSVIDefs::PARTIAL) {
+            fSeenNoValidation = true;
+            fValidity = PSVIDefs::UNKNOWN;
+        }
+        else {
+            fSeenValidation = true;
+        }
+        
+        if(curVal == PSVIDefs::INVALID)
+            fValidity = PSVIDefs::INVALID;
+    }
+
+    fHadContent = true;
+
+}
+
+inline void SchemaElementDecl::reset() {
+    if(fXsiSimpleTypeInfo && fXsiSimpleTypeInfo->getType() == DatatypeValidator::Union)
+        ((UnionDatatypeValidator *)fXsiSimpleTypeInfo)->reset();
+    if(fDatatypeValidator && fDatatypeValidator->getType() == DatatypeValidator::Union)
+        ((UnionDatatypeValidator *)fDatatypeValidator)->reset();
+
+    setXsiSimpleTypeInfo(0);
+    setXsiComplexTypeInfo(0);
+    fValidity = PSVIDefs::UNKNOWN;
+    fValidation = PSVIDefs::NONE;    
+    fSeenValidation = false;
+    fSeenNoValidation = false;
+    fHadContent = false;
+}
+
 // ---------------------------------------------------------------------------
 //  SchemaElementDecl: IC methods
 // ---------------------------------------------------------------------------
@@ -589,4 +918,6 @@ SchemaElementDecl::getIdentityConstraintAt(unsigned int index) const {
     return 0;
 }
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
