@@ -56,6 +56,28 @@
 
 /*
  * $Log: SchemaValidator.hpp,v $
+ * Revision 1.13  2003/01/20 19:04:48  knoaman
+ * Fix for particle derivation checking.
+ *
+ * Revision 1.12  2003/01/13 20:16:51  knoaman
+ * [Bug 16024] SchemaSymbols.hpp conflicts C++ Builder 6 dir.h
+ *
+ * Revision 1.11  2003/01/09 22:34:54  tng
+ * [Bug 14955] error validating parser.
+ *
+ * Revision 1.10  2002/11/07 21:57:37  tng
+ * Fix the following Schema Test Failures:
+ * 1. Typo when comparing miscFlags with FIXED
+ * 2. If xsi:type is specified, need to validate using that xsitype validator even if the type was any
+ * 3. Need to check ID/IDREFs for element value
+ * 4. Need to duplicate attribute id for wildcard scenario.
+ *
+ * Revision 1.9  2002/11/04 14:49:42  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.8  2002/09/04 18:17:41  tng
+ * Do not set IDREF to used during prevalidation.
+ *
  * Revision 1.7  2002/07/11 18:55:45  knoaman
  * Add a flag to the preContentValidation method to indicate whether to validate
  * default/fixed attributes or not.
@@ -124,6 +146,8 @@
 #include <xercesc/validators/schema/SchemaGrammar.hpp>
 #include <xercesc/validators/schema/XSDErrorReporter.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 //
 //  This is a derivative of the abstract validator interface. This class
 //  implements a validator that supports standard XML Schema semantics.
@@ -139,20 +163,6 @@ public:
     SchemaValidator(XMLErrorReporter* const errReporter = 0);
     virtual ~SchemaValidator();
 
-    // -----------------------------------------------------------------------
-    //  Setter methods
-    // -----------------------------------------------------------------------
-    void setGrammarResolver(GrammarResolver* grammarResolver);
-
-    void setXsiType(const XMLCh* const        prefix
-      , const XMLCh* const        localPart
-       , const unsigned int        uriId);
-
-    void setNillable(bool isNil);
-    void setErrorReporter(XMLErrorReporter* const errorReporter);
-    void setExitOnFirstFatal(const bool newValue);
-    void setDatatypeBuffer(const XMLCh* const value);
-
     // -----------------------------------------------------------------------
     //  Implementation of the XMLValidator interface
     // -----------------------------------------------------------------------
@@ -182,6 +192,8 @@ public:
     (
         const   XMLAttDef*                  attDef
         , const XMLCh* const                attrValue
+        , bool                              preValidation = false
+        , const XMLElementDecl*             elemDecl = 0
     );
 
     virtual void validateElement
@@ -207,6 +219,24 @@ public:
     // -----------------------------------------------------------------------
     void normalizeWhiteSpace(DatatypeValidator* dV, const XMLCh* const value, XMLBuffer& toFill);
 
+    // -----------------------------------------------------------------------
+    //  Setter methods
+    // -----------------------------------------------------------------------
+    void setGrammarResolver(GrammarResolver* grammarResolver);
+
+    void setXsiType(const XMLCh* const        prefix
+      , const XMLCh* const        localPart
+       , const unsigned int        uriId);
+
+    void setNillable(bool isNil);
+    void setErrorReporter(XMLErrorReporter* const errorReporter);
+    void setExitOnFirstFatal(const bool newValue);
+    void setDatatypeBuffer(const XMLCh* const value);
+
+    // -----------------------------------------------------------------------
+    //  Getter methods
+    // -----------------------------------------------------------------------
+    ComplexTypeInfo* getCurrentTypeInfo() const;
 
 private:
     // -----------------------------------------------------------------------
@@ -226,7 +256,8 @@ private:
                                    const int derivedScope,
                                    ContentSpecNode* const baseNode,
                                    const int baseScope,
-                                   const ComplexTypeInfo* const baseInfo = 0);
+                                   const ComplexTypeInfo* const baseInfo = 0,
+                                   const bool toCheckOccurrence = true);
     ContentSpecNode* checkForPointlessOccurrences(ContentSpecNode* const specNode,
                                                   const ContentSpecNode::NodeTypes nodeType,
                                                   ValueVectorOf<ContentSpecNode*>* const nodes);
@@ -235,7 +266,8 @@ private:
                         ValueVectorOf<ContentSpecNode*>* const nodes);
     bool isOccurrenceRangeOK(const int min1, const int max1, const int min2, const int max2);
     void checkNSCompat(const ContentSpecNode* const derivedSpecNode,
-                       const ContentSpecNode* const baseSpecNode);
+                       const ContentSpecNode* const baseSpecNode,
+                       const bool toCheckOccurence);
     bool wildcardEltAllowsNamespace(const ContentSpecNode* const baseSpecNode,
                                     const unsigned int derivedURI);
     void checkNameAndTypeOK(SchemaGrammar* const currentGrammar,
@@ -280,7 +312,8 @@ private:
                                         const ContentSpecNode* const derivedSpecNode,
                                         ValueVectorOf<ContentSpecNode*>* const derivedNodes,
                                         const int derivedScope,
-                                        ContentSpecNode* const baseSpecNode);
+                                        ContentSpecNode* const baseSpecNode,
+                                        const bool toCheckOccurence);
     void checkRecurseUnordered(SchemaGrammar* const currentGrammar,
                                const ContentSpecNode* const derivedSpecNode,
                                ValueVectorOf<ContentSpecNode*>* const derivedNodes,
@@ -326,6 +359,9 @@ private:
     //  fTrailing
     //      Previous chunk had a trailing space
     //
+    //  fSeenId
+    //      Indicate if an attribute of ID type has been seen already, reset per element.
+    //
     //  fSchemaErrorReporter
     //      Report schema process errors
     //
@@ -341,6 +377,7 @@ private:
 
     XMLBuffer fDatatypeBuffer;
     bool fTrailing;
+    bool fSeenId;
     XSDErrorReporter fSchemaErrorReporter;
     ValueStackOf<ComplexTypeInfo*>* fTypeStack;
 };
@@ -375,6 +412,15 @@ inline void SchemaValidator::setDatatypeBuffer(const XMLCh* const value)
     fDatatypeBuffer.append(value);
 }
 
+// ---------------------------------------------------------------------------
+//  SchemaValidator: Getter methods
+// ---------------------------------------------------------------------------
+inline ComplexTypeInfo* SchemaValidator::getCurrentTypeInfo() const {
+    if (fTypeStack->empty())
+        return 0;
+    return fTypeStack->peek();
+}
+
 // ---------------------------------------------------------------------------
 //  Virtual interface
 // ---------------------------------------------------------------------------
@@ -417,11 +463,13 @@ SchemaValidator::isOccurrenceRangeOK(const int min1, const int max1,
                                      const int min2, const int max2) {
 
     if (min1 >= min2 &&
-        (max2 == SchemaSymbols::UNBOUNDED ||
-         (max1 != SchemaSymbols::UNBOUNDED && max1 <= max2))) {
+        (max2 == SchemaSymbols::XSD_UNBOUNDED ||
+         (max1 != SchemaSymbols::XSD_UNBOUNDED && max1 <= max2))) {
         return true;
     }
     return false;
 }
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
