@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: TraverseSchema.hpp,v 1.13 2002/08/16 15:46:17 knoaman Exp $
+ * $Id: TraverseSchema.hpp,v 1.21 2003/01/15 15:32:15 knoaman Exp $
  */
 
 #if !defined(TRAVERSESCHEMA_HPP)
@@ -83,6 +83,8 @@
 #include <xercesc/validators/schema/GeneralAttributeCheck.hpp>
 #include <xercesc/validators/schema/XSDErrorReporter.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 // ---------------------------------------------------------------------------
 //  Forward Declarations
 // ---------------------------------------------------------------------------
@@ -126,11 +128,11 @@ public:
     ~TraverseSchema();
 
 private:
-   	// This enumeration is defined here for compatibility with the CodeWarrior
-   	// compiler, which apparently doesn't like to accept default parameter
-   	// arguments that it hasn't yet seen. The Not_All_Context argument is
-   	// used in the declaration of checkMinMax, below.
-   	//
+  	 // This enumeration is defined here for compatibility with the CodeWarrior
+  	 // compiler, which apparently doesn't like to accept default parameter
+  	 // arguments that it hasn't yet seen. The Not_All_Context argument is
+  	 // used in the declaration of checkMinMax, below.
+  	 //
     // Flags indicate any special restrictions on minOccurs and maxOccurs
     // relating to "all".
     //    Not_All_Context    - not processing an <all>
@@ -185,7 +187,7 @@ private:
                                                   const bool isMixed);
     DatatypeValidator*  traverseSimpleTypeDecl(const DOMElement* const childElem,
                                                const bool topLevel = true,
-                                               int baseRefContext = SchemaSymbols::EMPTY_SET);
+                                               int baseRefContext = SchemaSymbols::XSD_EMPTYSET);
     int                 traverseComplexTypeDecl(const DOMElement* const childElem,
                                                 const bool topLevel = true,
                                                 const XMLCh* const recursingTypeName = 0);
@@ -242,7 +244,7 @@ private:
                            const int errorCode);
     void reportSchemaError(const XSDLocator* const aLocator,
                            const XMLCh* const msgDomain,
-                           const int errorCode, 
+                           const int errorCode,
                            const XMLCh* const text1,
                            const XMLCh* const text2 = 0,
                            const XMLCh* const text3 = 0,
@@ -252,7 +254,7 @@ private:
                            const int errorCode);
     void reportSchemaError(const DOMElement* const elem,
                            const XMLCh* const msgDomain,
-                           const int errorCode, 
+                           const int errorCode,
                            const XMLCh* const text1,
                            const XMLCh* const text2 = 0,
                            const XMLCh* const text3 = 0,
@@ -281,13 +283,13 @@ private:
       * Parameters:
       *   rootElem - top element for a given type declaration
       *   contentElem - content must be annotation? or some other simple content
-      *   isEmpty: - true if (annotation?, smth_else), false if (annotation?) 
+      *   isEmpty: - true if (annotation?, smth_else), false if (annotation?)
       *
       * Check for Annotation if it is present, traverse it. If a sibling is
       * found and it is not an annotation return it, otherwise return 0.
       * Used by traverseSimpleTypeDecl.
       */
-    DOMElement* checkContent(const DOMElement* const rootElem, 
+    DOMElement* checkContent(const DOMElement* const rootElem,
                                DOMElement* const contentElem,
                                const bool isEmpty);
 
@@ -307,7 +309,7 @@ private:
       * Return a dataype validator if valid type, otherwise 0.
       */
     DatatypeValidator* checkForSimpleTypeValidator(const DOMElement* const content,
-                                                   int baseRefContext = SchemaSymbols::EMPTY_SET);
+                                                   int baseRefContext = SchemaSymbols::XSD_EMPTYSET);
 
     /**
       * Process complexType content of an element
@@ -433,6 +435,9 @@ private:
                                   const XMLCh* const elemName,
                                   const bool toEmit = true);
 
+    bool isSubstitutionGroupCircular(SchemaElementDecl* const elemDecl,
+                                     SchemaElementDecl* const subsElemDecl);
+
     /**
       * Create a 'SchemaElementDecl' object and add it to SchemaGrammar
       */
@@ -554,13 +559,13 @@ private:
                                 XercesAttGroupInfo* const toAttGroup,
                                 ComplexTypeInfo* const typeInfo);
 
-    const XMLCh* getTargetNamespaceString(const DOMElement* const elem);
+    void checkForEmptyTargetNamespace(const DOMElement* const elem);
 
     /**
       * Attribute wild card intersection.
       *
       * Note:
-      *    The first parameter will be the result of the intersection, so 
+      *    The first parameter will be the result of the intersection, so
       *    we need to make sure that first parameter is a copy of the
       *    actual attribute definition we need to intersect with.
       *
@@ -574,7 +579,7 @@ private:
       * Attribute wild card union.
       *
       * Note:
-      *    The first parameter will be the result of the union, so 
+      *    The first parameter will be the result of the union, so
       *    we need to make sure that first parameter is a copy of the
       *    actual attribute definition we need to intersect with.
       *
@@ -643,12 +648,12 @@ private:
                                     SchemaInfo* const redefiningSchemaInfo);
 
 	/**
-      * This function looks among the children of 'redefineChildElem' for a 
+      * This function looks among the children of 'redefineChildElem' for a
       * component of type 'redefineChildComponentName'. If it finds one, it
       * evaluates whether its ref attribute contains a reference to
       * 'refChildTypeName'. If it does, it returns 1 + the value returned by
       * calls to itself on all other children.  In all other cases it returns
-      * 0 plus the sum of the values returned by calls to itself on 
+      * 0 plus the sum of the values returned by calls to itself on
       * redefineChildElem's children. It also resets the value of ref so that
       * it will refer to the renamed type from the schema being redefined.
       */
@@ -744,7 +749,8 @@ private:
     RefHashTableOf<ComplexTypeInfo>*               fComplexTypeRegistry;
     RefHashTableOf<XercesGroupInfo>*               fGroupRegistry;
     RefHashTableOf<XercesAttGroupInfo>*            fAttGroupRegistry;
-    RefHash2KeysTableOf<SchemaInfo>*               fSchemaInfoList;
+    RefHashTableOf<ElemVector>*                    fIC_ElementsNS;
+    RefHashTableOf<SchemaInfo>*                    fPreprocessedNodes;
     SchemaInfo*                                    fSchemaInfo;
     XercesGroupInfo*                               fCurrentGroupInfo;
     XercesAttGroupInfo*                            fCurrentAttGroupInfo;
@@ -753,18 +759,17 @@ private:
     ValueVectorOf<unsigned int>*                   fCurrentGroupStack;
     ValueVectorOf<unsigned int>*                   fIC_NamespaceDepth;
     ValueVectorOf<SchemaElementDecl*>*             fIC_Elements;
-    GeneralAttributeCheck                          fAttributeCheck;
-    RefHash2KeysTableOf<XMLCh>*                    fGlobalDeclarations;
+    ValueVectorOf<const DOMElement*>*              fDeclStack;
+    ValueVectorOf<unsigned int>**                  fGlobalDeclarations;
+    RefHashTableOf<ValueVectorOf<DOMElement*> >*   fIC_NodeListNS;
+    RefHashTableOf<ValueVectorOf<unsigned int> >*  fIC_NamespaceDepthNS;
     RefHash2KeysTableOf<XMLCh>*                    fNotationRegistry;
     RefHash2KeysTableOf<XMLCh>*                    fRedefineComponents;
     RefHash2KeysTableOf<IdentityConstraint>*       fIdentityConstraintNames;
-    RefHash2KeysTableOf<SchemaElementDecl>*        fSubstitutionGroups;
     RefHash2KeysTableOf<ElemVector>*               fValidSubstitutionGroups;
-    RefHashTableOf<ValueVectorOf<DOMElement*> >* fIC_NodeListNS;
-    RefHashTableOf<ElemVector>*                    fIC_ElementsNS;
-    RefHashTableOf<ValueVectorOf<unsigned int> >*  fIC_NamespaceDepthNS;
+    RefHash2KeysTableOf<SchemaInfo>*               fSchemaInfoList;
+    GeneralAttributeCheck                          fAttributeCheck;
     XSDDOMParser*                                  fParser;
-    RefHashTableOf<SchemaInfo>*                    fPreprocessedNodes;
     XSDErrorReporter                               fXSDErrorReporter;
     XSDLocator*                                    fLocator;
 
@@ -816,7 +821,7 @@ TraverseSchema::isValidRefDeclaration(const DOMElement* const elem) {
              || XMLString::stringLen(elem->getAttribute(SchemaSymbols::fgATT_BLOCK)) != 0
              || XMLString::stringLen(elem->getAttribute(SchemaSymbols::fgATT_FINAL)) != 0
              || XMLString::stringLen(elem->getAttribute(SchemaSymbols::fgATT_TYPE)) != 0
-             || XMLString::stringLen(elem->getAttribute(SchemaSymbols::fgATT_DEFAULT)) != 0 
+             || XMLString::stringLen(elem->getAttribute(SchemaSymbols::fgATT_DEFAULT)) != 0
              || XMLString::stringLen(elem->getAttribute(SchemaSymbols::fgATT_FIXED)) != 0
              || XMLString::stringLen(elem->getAttribute(SchemaSymbols::fgATT_SUBSTITUTIONGROUP)) != 0);
 }
@@ -840,7 +845,7 @@ const XMLCh* TraverseSchema::getElementAttValue(const DOMElement* const elem,
         XMLCh* bufValue = fBuffer.getRawBuffer();
         XMLString::trim(bufValue);
 
-        if (!XMLString::stringLen(bufValue)) {
+        if (!bufValue || !*bufValue) {
             return XMLUni::fgZeroLenString;
         }
 
@@ -850,24 +855,22 @@ const XMLCh* TraverseSchema::getElementAttValue(const DOMElement* const elem,
     return attValue;
 }
 
-inline const XMLCh* 
-TraverseSchema::getTargetNamespaceString(const DOMElement* const elem) {
+inline void
+TraverseSchema::checkForEmptyTargetNamespace(const DOMElement* const elem) {
 
     const XMLCh* targetNS = getElementAttValue(elem, SchemaSymbols::fgATT_TARGETNAMESPACE);
 
-    if (targetNS && XMLString::stringLen(targetNS) == 0) {
+    if (targetNS && !*targetNS) {
         reportSchemaError(elem, XMLUni::fgXMLErrDomain, XMLErrs::InvalidTargetNSValue);
     }
-
-    return targetNS;
 }
 
 inline bool TraverseSchema::isBaseFromAnotherSchema(const XMLCh* const baseURI)
 {
-    if (XMLString::compareString(baseURI,fTargetNSURIString) != 0
-        && XMLString::compareString(baseURI, SchemaSymbols::fgURI_SCHEMAFORSCHEMA) != 0
-        && XMLString::stringLen(baseURI) != 0) {
-        //REVISIT, !!!! a hack: for schema that has no 
+    if (!XMLString::equals(baseURI,fTargetNSURIString)
+        && !XMLString::equals(baseURI, SchemaSymbols::fgURI_SCHEMAFORSCHEMA)
+        && (baseURI && *baseURI)) {
+        //REVISIT, !!!! a hack: for schema that has no
         //target namespace, e.g. personal-schema.xml
         return true;
     }
@@ -879,9 +882,9 @@ inline bool TraverseSchema::isAttrOrAttrGroup(const DOMElement* const elem) {
 
     const XMLCh* elementName = elem->getLocalName();
 
-    if (!XMLString::compareString(elementName, SchemaSymbols::fgELT_ATTRIBUTE) ||
-        !XMLString::compareString(elementName, SchemaSymbols::fgELT_ATTRIBUTEGROUP) ||
-        !XMLString::compareString(elementName, SchemaSymbols::fgELT_ANYATTRIBUTE)) {
+    if (XMLString::equals(elementName, SchemaSymbols::fgELT_ATTRIBUTE) ||
+        XMLString::equals(elementName, SchemaSymbols::fgELT_ATTRIBUTEGROUP) ||
+        XMLString::equals(elementName, SchemaSymbols::fgELT_ANYATTRIBUTE)) {
         return true;
     }
 
@@ -908,7 +911,7 @@ inline void TraverseSchema::popCurrentTypeNameStack() {
     }
 }
 
-inline void 
+inline void
 TraverseSchema::copyWildCardData(const SchemaAttDef* const srcWildCard,
                                  SchemaAttDef* const destWildCard) {
 
@@ -944,6 +947,9 @@ inline void TraverseSchema::normalizeURI(const XMLCh* const systemURI,
             pszSrc += 3;
             normalizedURI.append(chSpace);
         }
+        else if (*pszSrc == 0xFFFF) { //escaped character
+            pszSrc++;
+        }
         else {
             normalizedURI.append(*pszSrc);
             pszSrc++;
@@ -951,6 +957,8 @@ inline void TraverseSchema::normalizeURI(const XMLCh* const systemURI,
     }
 }
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
 
 /**
