@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: XercesGroupInfo.hpp,v 1.3 2002/04/01 15:47:06 knoaman Exp $
+ * $Id: XercesGroupInfo.hpp,v 1.4 2002/11/04 14:49:42 tng Exp $
  */
 
 #if !defined(XERCESGROUPINFO_HPP)
@@ -74,6 +74,7 @@
 #include <xercesc/util/RefVectorOf.hpp>
 #include <xercesc/validators/schema/SchemaElementDecl.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  Forward Declarations
@@ -149,13 +150,13 @@ inline ContentSpecNode* XercesGroupInfo::getContentSpec() const {
     return fContentSpec;
 }
 
-inline SchemaElementDecl* 
+inline SchemaElementDecl*
 XercesGroupInfo::elementAt(const unsigned int index) {
 
     return fElements->elementAt(index);
 }
 
-inline const SchemaElementDecl* 
+inline const SchemaElementDecl*
 XercesGroupInfo::elementAt(const unsigned int index) const {
 
     return fElements->elementAt(index);
@@ -205,6 +206,8 @@ inline void XercesGroupInfo::setCheckElementConsistency(const bool aValue) {
     fCheckElementConsistency = aValue;
 }
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
 
 /**
