@@ -56,6 +56,18 @@
 
 /*
  * $Log: XercesXPath.cpp,v $
+ * Revision 1.5  2002/12/20 22:10:48  tng
+ * XML 1.1
+ *
+ * Revision 1.4  2002/12/04 18:21:23  knoaman
+ * Identity constraint fix.
+ *
+ * Revision 1.3  2002/11/04 14:47:42  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.2  2002/10/30 21:52:01  tng
+ * [Bug 13641] compiler-generated copy-constructor for QName doesn't do the right thing.
+ *
  * Revision 1.1.1.1  2002/02/01 22:22:51  peiyongz
  * sane_include
  *
@@ -83,11 +95,13 @@
 #include <xercesc/internal/XMLReader.hpp>
 #include <xercesc/util/RuntimeException.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 
 // ---------------------------------------------------------------------------
 //  Static data
 // ---------------------------------------------------------------------------
-const XMLByte XPathScanner::fASCIICharMap[128] = 
+const XMLByte XPathScanner::fASCIICharMap[128] =
 {
     0,  0,  0,  0,  0,  0,  0,  0,  0,  2,  2,  0,  0,  2,  0,  0,
     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
@@ -111,7 +125,7 @@ XercesNodeTest::XercesNodeTest(const short aType)
 
 XercesNodeTest::XercesNodeTest(const QName* const qName)
     : fType(QNAME)
-    , fName(new QName(qName))
+    , fName(new QName(*qName))
 {
 }
 
@@ -126,7 +140,7 @@ XercesNodeTest::XercesNodeTest(const XMLCh* const prefix,
 
 XercesNodeTest::XercesNodeTest(const XercesNodeTest& other)
     : fType(other.fType)
-    , fName(new QName(other.fName))
+    , fName(new QName(*other.fName))
 {
 }
 
@@ -323,7 +337,7 @@ void XercesXPath::checkForSelectedAttributes() {
         if (stepSize) {
             if (locPath->getStep(stepSize - 1)->getAxisType() == XercesStep::ATTRIBUTE) {
                 ThrowXML(XPathException, XMLExcepts::XPath_NoAttrSelector);
-            }        
+            }
 		}
     }
 }
@@ -451,7 +465,7 @@ void XercesXPath::parseExpression(XMLStringPool* const stringPool,
                 firstTokenOfLocationPath=false;
                 break;
             }
-        case XercesXPath::EXPRTOKEN_DOUBLE_COLON: 
+        case XercesXPath::EXPRTOKEN_DOUBLE_COLON:
             {
                 // should never have a bare double colon
                 ThrowXML(XPathException, XMLExcepts::XPath_NoDoubleColon);
@@ -554,21 +568,21 @@ void XercesXPath::parseExpression(XMLStringPool* const stringPool,
             }
         case XercesXPath::EXPRTOKEN_OPERATOR_DOUBLE_SLASH:
             {
-                ThrowXML(XPathException, XMLExcepts::XPath_NoDoubleForwardSlash); 
+                ThrowXML(XPathException, XMLExcepts::XPath_NoDoubleForwardSlash);
             }
         case XercesXPath::EXPRTOKEN_OPERATOR_SLASH:
             {
                 if (i == 0) {
-                    ThrowXML(XPathException, XMLExcepts::XPath_NoForwardSlashAtStart); 
+                    ThrowXML(XPathException, XMLExcepts::XPath_NoForwardSlashAtStart);
                 }
 
                 // keep on truckin'
                 if (firstTokenOfLocationPath) {
-                    ThrowXML(XPathException, XMLExcepts::XPath_NoSelectionOfRoot); 
+                    ThrowXML(XPathException, XMLExcepts::XPath_NoSelectionOfRoot);
                 }
 
                 if (i == tokenCount - 1) {
-                    ThrowXML(XPathException, XMLExcepts::XPath_ExpectedStep3); 
+                    ThrowXML(XPathException, XMLExcepts::XPath_ExpectedStep3);
                 }
 
                 firstTokenOfLocationPath=false;
@@ -676,7 +690,7 @@ bool XPathScanner::scanExpression(const XMLCh* const data,
 
         ch = data[currentOffset];
 
-        while (XMLReader::isWhitespace(ch)) {
+        while (XMLChar1_0::isWhitespace(ch)) {
 
             if (++currentOffset == endOffset) {
                 break;
@@ -742,13 +756,27 @@ bool XPathScanner::scanExpression(const XMLCh* const data,
                 addToken(tokens, XercesXPath::EXPRTOKEN_PERIOD);
                 starIsMultiplyOperator = true;
                 currentOffset++;
-            } else {                    // '.'
+            } else if (ch == chPipe) { // '|'
+                addToken(tokens, XercesXPath::EXPRTOKEN_PERIOD);
+                starIsMultiplyOperator = true;
+                currentOffset++;
+            } else if (XMLChar1_0::isWhitespace(ch)) {
+                do {
+                    if (++currentOffset == endOffset)
+                        break;
+
+                    ch = data[currentOffset];
+                } while (XMLChar1_0::isWhitespace(ch));
+
+                if (currentOffset == endOffset || ch == chPipe) {
+				    addToken(tokens, XercesXPath::EXPRTOKEN_PERIOD);
+                    starIsMultiplyOperator = true;
+                    break;
+                }
+            } else {
                 ThrowXML(XPathException, XMLExcepts::XPath_InvalidChar);
             }
 
-            if (currentOffset == endOffset) {
-                break;
-            }
             break;
         case CHARTYPE_ATSIGN:           // '@'
             addToken(tokens, XercesXPath::EXPRTOKEN_ATSIGN);
@@ -878,7 +906,7 @@ bool XPathScanner::scanExpression(const XMLCh* const data,
                 while (ch != qchar) {
                     if (++currentOffset == endOffset) {
                         return false; // REVISIT
-                    } 
+                    }
 
                     ch = data[currentOffset];
                 }
@@ -1005,7 +1033,7 @@ bool XPathScanner::scanExpression(const XMLCh* const data,
             nameOffset = currentOffset;
             currentOffset = scanNCName(data, endOffset, currentOffset);
             if (currentOffset == nameOffset) {
-                return false; // REVISIT            
+                return false; // REVISIT
 			}
 
             if (currentOffset < endOffset) {
@@ -1063,7 +1091,7 @@ bool XPathScanner::scanExpression(const XMLCh* const data,
             //
             // [39] ExprWhitespace ::= S
             //
-            while (XMLReader::isWhitespace(ch)) {
+            while (XMLChar1_0::isWhitespace(ch)) {
                 if (++currentOffset == endOffset) {
                     break;
                 }
@@ -1199,7 +1227,7 @@ int XPathScanner::scanNCName(const XMLCh* const data,
 
     XMLCh ch = data[currentOffset];
 
-    if (!XMLReader::isXMLLetter(ch) && ch != chUnderscore) {
+    if (!XMLChar1_0::isXMLLetter(ch) && ch != chUnderscore) {
         return currentOffset;
     }
 
@@ -1207,7 +1235,7 @@ int XPathScanner::scanNCName(const XMLCh* const data,
 
         ch = data[currentOffset];
 
-        if (ch == chColon || !XMLReader::isNameChar(ch)) {
+        if (ch == chColon || !XMLChar1_0::isNameChar(ch)) {
             break;
         }
     }
@@ -1304,6 +1332,8 @@ void XPathScannerForSchema::addToken(ValueVectorOf<int>* const tokens,
     ThrowXML(XPathException, XMLExcepts::XPath_TokenNotSupported);
 }
 
+XERCES_CPP_NAMESPACE_END
+
 /**
   * End of file XercesPath.cpp
   */
