@@ -60,7 +60,7 @@
 //
 
 /*
- * $Id: DOMMemTest.cpp,v 1.39 2002/08/08 14:16:14 tng Exp $
+ * $Id: DOMMemTest.cpp,v 1.42 2003/02/05 18:55:19 tng Exp $
  */
 
 #include <stdio.h>
@@ -72,12 +72,18 @@
 #include <xercesc/util/XMLUniDefs.hpp>
 
 
+XERCES_CPP_NAMESPACE_USE
+
+bool errorOccurred = false;
+
 #define TASSERT(c) tassert((c), __FILE__, __LINE__)
 
 void tassert(bool c, const char *file, int line)
 {
-    if (!c)
+    if (!c) {
         printf("Failure.  Line %d,   file %s\n", line, file);
+        errorOccurred = true;
+    }
 };
 
 
@@ -86,14 +92,18 @@ void tassert(bool c, const char *file, int line)
     try {                                                           \
     operation;                                                      \
     printf(" Error: no exception thrown at line %d\n", __LINE__);   \
-}                                                                   \
-    catch (DOMException &e) {                                  \
-    if (e.code != expected_exception)                       \
-    printf(" Wrong exception code: %d at line %d\n", e.code, __LINE__); \
-}                                                                 \
+    errorOccurred = true;                                           \
+    }                                                               \
+    catch (DOMException &e) {                                       \
+    if (e.code != expected_exception) {                             \
+        printf(" Wrong exception code: %d at line %d\n", e.code, __LINE__); \
+        errorOccurred = true;                                       \
+    }                                                               \
+    }                                                               \
     catch (...)   {                                                 \
-    printf(" Wrong exception thrown at line %d\n", __LINE__);       \
-}                                                                   \
+        printf(" Wrong exception thrown at line %d\n", __LINE__);   \
+        errorOccurred = true;                                       \
+    }                                                               \
 }
 
 // ---------------------------------------------------------------------------
@@ -114,7 +124,7 @@ public :
 
     ~XStr()
     {
-        delete [] fUnicodeForm;
+        XMLString::release(&fUnicodeForm);
     }
 
 
@@ -1482,7 +1492,7 @@ int  mymain()
         char *pMessage = XMLString::transcode(toCatch.getMessage());
         fprintf(stderr, "Error during XMLPlatformUtils::Initialize(). \n"
             "  Message is: %s\n", pMessage);
-        delete [] pMessage;
+        XMLString::release(&pMessage);
         return -1;
     }
 
@@ -1503,7 +1513,13 @@ int  main() {
    for (int i = 0; i<3; i++)
         mymain();
 
+    if (errorOccurred) {
+        printf("Test Failed\n");
+        return 4;
+    }
+
     printf("Test Run Successfully\n");
+
     return 0;
 }
 
