@@ -57,6 +57,18 @@
 
 /*
  * $Log: Xlat.cpp,v $
+ * Revision 1.16  2002/12/12 23:40:39  peiyongz
+ * normlize locale string.
+ *
+ * Revision 1.15  2002/11/12 17:24:58  tng
+ * DOM Message: add new domain for DOM Messages.
+ *
+ * Revision 1.14  2002/11/04 15:24:50  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.13  2002/09/30 22:09:58  peiyongz
+ * To generate icu resource file (in text) for error message.
+ *
  * Revision 1.12  2002/07/04 17:40:07  tng
  * Use new DOM in Xlat.
  *
@@ -402,7 +414,7 @@ extern "C" int wmain(int argC, XMLCh** argV)
 
     {
         //  Nest entire code in an inner block.
-        
+
         DOMDocument* srcDoc;
         const unsigned int bufSize = 4095;
         XMLCh tmpFileBuf[bufSize + 1];
@@ -457,8 +469,12 @@ extern "C" int wmain(int argC, XMLCh** argV)
                     formatter = new MsgCatFormatter;
                     break;
 
+                case OutFormat_ResBundle:
+                    formatter = new ICUResBundFormatter;
+                    break;
+
                 default :
-                    wprintf(L"Uknown formatter type enum\n\n");
+                    wprintf(L"Unknown formatter type enum\n\n");
                     throw ErrReturn_Internal;
             }
 
@@ -489,11 +505,33 @@ extern "C" int wmain(int argC, XMLCh** argV)
             //
             const unsigned int count = msgSetList->getLength();
 
+            //
+            // Normalize locale string
+            //
+            // locale = ll[[_CC][_VARIANT]]
+            // where ll          is language code
+            //       CC          is country code
+            //       VARIANT     is variant code
+            //
+            XMLCh normalizedLocale[256];
+
+            normalizedLocale[0] = localeStr[0];
+            normalizedLocale[1] = localeStr[1];
+            normalizedLocale[2] = 0;
+            XMLString::lowerCase(normalizedLocale);
+
+            if (XMLString::stringLen(localeStr) > 2)
+            {
+                XMLString::catString(&(normalizedLocale[2]), &(localeStr[2]));
+                XMLString::upperCase(&(normalizedLocale[2]));
+            }
+
             //
             //  Ok, its good enough to get started. So lets call the start output
             //  method on the formatter.
             //
-            formatter->startOutput(localeStr, gOutPath);
+    
+            formatter->startOutput(normalizedLocale, gOutPath);
 
             //
             //  For each message domain element, we call start and end domain
@@ -537,6 +575,11 @@ extern "C" int wmain(int argC, XMLCh** argV)
                 {
                     headerName = L"XMLExceptMsgs.hpp";
                     errNameSpace = L"XMLExcepts";
+                }
+                 else if (!XMLString::compareString(domainStr, XMLUni::fgXMLDOMMsgDomain))
+                {
+                    headerName = L"XMLDOMMsg.hpp";
+                    errNameSpace = L"XMLDOMMsg";
                 }
                  else
                 {
@@ -572,8 +615,13 @@ extern "C" int wmain(int argC, XMLCh** argV)
 
                 // If its not the exception domain, then we need a header included
                 if (XMLString::compareString(domainStr, XMLUni::fgExceptDomain))
-                    fwprintf(outHeader, L"#include <xercesc/framework/XMLErrorReporter.hpp>\n\n");
+                    fwprintf(outHeader, L"#include <xercesc/framework/XMLErrorReporter.hpp>\n");
 
+                //  Write out the namespace declaration
+                fwprintf(outHeader, L"#include <xercesc/util/XercesDefs.hpp>\n\n");
+                fwprintf(outHeader, L"XERCES_CPP_NAMESPACE_BEGIN\n\n");
+
+                //  Now the message codes
                 fwprintf(outHeader, L"class %s\n{\npublic :\n    enum Codes\n    {\n", errNameSpace);
 
                 // Tell the formatter that a new domain is starting
@@ -740,8 +788,11 @@ extern "C" int wmain(int argC, XMLCh** argV)
                     );
                 }
 
-                // And close out the class declaration and the header file
-                fwprintf(outHeader, L"};\n#endif\n\n");
+
+                // And close out the class declaration, the namespace declaration and the header file
+                fwprintf(outHeader, L"};\n\n");
+                fwprintf(outHeader, L"XERCES_CPP_NAMESPACE_END\n\n");
+                fwprintf(outHeader, L"#endif\n\n");
                 fclose(outHeader);
             }
 
